# ALM1000 Vector Volt Meter-1.0.py(w)  (12-15-2015)
# For Python version 2.7.8 or greater
# With external module libpysmu (for Python version 2.7);
# NUMPY module (for Python version 2.7)
# Created by D Mercer
import atexit
from collections import defaultdict
from operator import add
import libpysmu
import math
import time
import struct
import numpy.fft

from time import gmtime, strftime
import tkFont
from Tkinter import *
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
from tkSimpleDialog import askstring
from tkMessageBox import *
import webbrowser
#
TBicon = """
R0lGODlhIAAgAHAAACH5BAEAAAIALAAAAAAgACAAgQAAAP///wAAAAAAAAJJhI+py+0PYwtBWkDp
hTnv2XlfEobjUZZnmn4se72vJMtcbYN4ruz44uORgiodsfI4Im++2M5VW81OmBbVULxiRVrUsgsO
i8fUAgA7
"""

root=Tk()
root.title("ALICE Vector Volt Meter-1.0.py(w) (12-15-2015)")
img = PhotoImage(data=TBicon)
root.call('wm', 'iconphoto', root._w, img)

root.minsize(100, 100)
# Values that can be modified
GRWN = 400                  # Width of the grid 400 default
GRHN = 400                  # Height of the grid 400 default
X0L = 37                    # Left top X value of grid
Y0T = 25                    # Left top Y value of grid

SAMPLErate = 100000         # Sample rate of the ALM1000 system 100000
TRACEmode = IntVar(0)       # 1 normal mode, 2 max hold mode, 3 average mode
TRACEmode.set(1)
TRACEaverage = IntVar(0)           # Number of average sweeps for average mode
TRACEaverage.set(8)
TRACEreset = True           # True for first new trace, reset max hold and averageing 
MinSamples = 64
MaxSamples = 65536
DevID = "m1k"
ResScalediv = (10, 20, 50, 100, 200, 500, 1000, 2000, 5000)
# ZEROstuffing = 1          # ZEROstuffing for signal interpolation between frequency samples (0=none)

# Colors that can be modified

COLORframes = "#000080"   # Color = "#rrggbb" rr=red gg=green bb=blue, Hexadecimal values 00 - ff
COLORcanvas = "#000000"   # 100% black
COLORgrid = "#808080"     # 50% Gray
COLORzeroline = "#0000ff" # 100% blue
COLORtrace1 = "#00ff00"   # 100% green
COLORtrace2 = "#ff8000"   # 100% orange
COLORtrace3 = "#00ffff"   # 100% cyan
COLORtrace4 = "#ffff00"   # 100% yellow
COLORtrace5 = "#ff00ff"   # 100% magenta
COLORtrace6 = "#ff0000"   # 100% red
COLORtraceR1 = "#008000"   # 50% green
COLORtraceR2 = "#804000"   # 50% orange
COLORtraceR3 = "#008080"   # 50% cyan
COLORtraceR4 = "#808000"   # 50% yellow
COLORtraceR5 = "#800080"   # 50% magenta
COLORtext = "#ffffff"     # 100% white

# Button sizes that can be modified
Buttonwidth1 = 12
Buttonwidth2 = 8

# Initialisation of general variables
STARTfrequency = IntVar(0)  # Startfrequency
STARTfrequency.set(0) 
STOPfrequency = IntVar(0)   # Stopfrequency
STOPfrequency.set(45000)
ZEROstuffing = IntVar(0) # The zero stuffing value is 2 ** ZERO stuffing, calculated on initialize
ZEROstuffing.set(1)

# Other global variables required in various routines
GRW = GRWN                      # Initialize GRW
GRH = GRHN                      # Initialize GRH

CANVASwidth = 170 + GRW + 2 * X0L     # The canvas width
CANVASheight = GRH + Y0T + 10         # The canvas height

ADsignal1 = []                  # Analog trace channels

FFTbandwidth = 0                # The FFT bandwidth
FFTresultA = []                 # FFT result CHA
PhaseA = []
FFTresultB = []                 # FFT result CHB
PhaseB = []
# FFTwindow = 5             # FFTwindow 0=None (rectangular B=1), 1=Cosine (B=1.24), 2=Triangular non-zero endpoints (B=1.33),
                                # 3=Hann (B=1.5), 4=Blackman (B=1.73), 5=Nuttall (B=2.02), 6=Flat top (B=3.77)
FFTwindowname = "--"            # The FFT window name
FFTmemoryA = numpy.ones(1)       # The memory for averaging
PhaseMemoryA = numpy.ones(1)
FFTmemoryB = numpy.ones(1)       # The memory for averaging
PhaseMemoryB = numpy.ones(1)

RUNstatus = 0                   # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
PowerStatus = 1
STARTsample = 0                 # The start sample used for the display, be calculated on initialize
STOPsample = 0                  # The stop sample used for the display, be calculated on initialize

SMPfftpwrTwo = IntVar(0)        # The power of two of SMPfft
SMPfftpwrTwo.set(14)
SMPfft = 2 ** SMPfftpwrTwo.get()    # Initialize

FFTwindowshape = numpy.ones(SMPfft) # The FFT window curve

#
PeakfreqA = PeakfreqB = PeakfreqM = 0
PeakphaseA = 0
PeakphaseB = 0
PeakSample = 0
PeakRelPhase = 0
# AWG variables
AWGAAmplvalue = 0.0
AWGAOffsetvalue = 0.0
AWGAFreqvalue = 0.0
AWGAPhasevalue = 0
AWGAdelayvalue = 0
AWGAWave = 'dc'
AWGBAmplvalue = 0.0
AWGBOffsetvalue = 0.0
AWGBFreqvalue = 0.0
AWGBPhasevalue = 0
AWGBdelayvalue = 0
AWGBWave = 'dc'
# Calibration coefficients
CHAVGain = CHBVGain = 1.025
CHAVOffset = CHBVOffset = 0.0

# ====== define libsmu funcions ======
class Smu(object):
    def __init__(self):
        atexit.register(libpysmu.cleanup)
        libpysmu.setup()
        self.load_channels()

    def load_channels(self):
        self.devices = libpysmu.get_dev_info()
        self.serials = {i: v[0] for i, v in enumerate(self.devices)}
        self.devices = [x[1] for x in self.devices]

        names = (chr(x) for x in xrange(65,91))
        channels = {names.next(): (i, v) for i, d in enumerate(self.devices)
                    for k, v in d.items()}
        self.channels = {k: Channel(k, self.serials[v[0]], v[1]) for k, v in channels.items()}

        device_channels = defaultdict(list)
        for k, v in channels.items():
            device_channels[v[0]].append(Channel(k, *v))
        self.devices = {i: Device(self.serials[i], device_channels[i]) for i, v
                        in enumerate(self.devices)}

    def ctrl_transfer(self, device, bm_request_type, b_request, wValue, wIndex,
                      data, wLength, timeout):
        data = str(data)
        if bm_request_type&0x80 == 0x80:
            if data == '0':
                data = '\x00'*wLength
        else:
            wLength = 0
        ret = libpysmu.ctrl_transfer(device, bm_request_type, b_request, wValue,
                                   wIndex, data, wLength, timeout)
        if bm_request_type&0x80 == 0x80:
            return map(ord, data)
        else:
            return ret

    def __repr__(self):
        return 'Devices: ' + str(self.devices)


class Device(object):
    def __init__(self, serial, channels):
        self.serial = serial
        self.channels = channels

    def get_samples(self, n_samples):
        """query device for a list of samples from all channels

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from all the device's channels
        """
        return libpysmu.get_all_inputs(self.serial, n_samples)

    @property
    def samples(self):
        """iterator for samples from the device run in continuous mode"""
        return libpysmu.iterate_inputs(self.serial)


class Channel(object):
    def __init__(self, chan, dev_serial, signals):
        self.dev = dev_serial
        self.chan = ord(chan) - 65
        self.signals = {v: i for i, v in enumerate(signals)}

    def set_mode(self, mode):
        if mode == 'v' or mode == 'V':
            libpysmu.set_mode(self.dev, self.chan, 1)
            self.mode = 1
        elif mode == 'i' or mode == 'I':
            libpysmu.set_mode(self.dev, self.chan, 2)
            self.mode = 2
        elif mode == 'd' or mode == 'D':
            libpysmu.set_mode(self.dev, self.chan, 0)
            self.mode = 0
        else:
            raise ValueError('invalid mode')

    def arbitrary(self, *args, **kwargs):
        repeat = 0
        if 'repeat' in map(str.lower, kwargs.keys()):
            repeat = 1
        wave = map(float, reduce(add, [[s]*100*n for s, n in args]))
        return libpysmu.set_output_buffer(wave, self.dev, self.chan, self.mode, repeat)
    
    def get_samples(self, n_samples):
        """query channel for samples

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from the channel
        """
        return libpysmu.get_inputs(self.dev, self.chan, n_samples)

    def constant(self, val):
        """set output to a constant waveform"""
        return libpysmu.set_output_constant(self.dev, self.chan, self.mode, val)

    def square(self, midpoint, peak, period, phase, duty_cycle):
        """set output to a square waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 1, midpoint, peak, period, phase, duty_cycle)

    def sawtooth(self, midpoint, peak, period, phase):
        """set output to a sawtooth waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 2, midpoint, peak, period, phase, 42)

    def stairstep(self, midpoint, peak, period, phase):
        """set output to a stairstep waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 3, midpoint, peak, period, phase, 42)

    def sine(self, midpoint, peak, period, phase):
        """set output to a sinusoidal waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 4, midpoint, peak, period, phase, 42)

    def triangle(self, midpoint, peak, period, phase):
        """set output to a triangle waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 5, midpoint, peak, period, phase, 42)

    def __repr__(self):
        return 'Device: ' + str(self.dev) + '\nChannel: ' + str(self.chan) + '\nSignals: ' + str(self.signals)
#
def BSaveScreen():
    global CANVASwidth, CANVASheight
    global COLORtext
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # temp change text color to black
    COLORtext = "#000000"
    UpdateScreen()
    # save postscript file
    ca.postscript(file=filename, height=CANVASheight, width=CANVASwidth, colormode='color')
    # 
    COLORtext = "#ffffff"
    UpdateScreen()
# ======= Start command routines =================
def Bnot():
    print "Routine not made yet"

def donothing():
    global CutDC
    
def BSaveConfig():
    global CutDC
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry
    global AWGAShape, AWGATerm, AWGAMode
    global AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global FFTwindow, TRACEmode, TRACEaverage
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
# open Config file for Write
    filename = asksaveasfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename, "w")
    #
    ConfgFile.write('FFTwindow.set(' + str(FFTwindow.get()) + ')\n')
    ConfgFile.write('SMPfftpwrTwo.set(' + str(SMPfftpwrTwo.get()) + ')\n')
    ConfgFile.write('STARTfrequency.set(' + str(STARTfrequency.get()) + ')\n')
    ConfgFile.write('STOPfrequency.set(' + str(STOPfrequency.get()) + ')\n')
    ConfgFile.write('ZEROstuffing.set(' + str(ZEROstuffing.get()) + ')\n')
    #
    ConfgFile.write('TRACEmode.set(' + str(TRACEmode.get()) + ')\n')
    ConfgFile.write('TRACEaverage.set(' + str(TRACEaverage.get()) + ')\n')
    ConfgFile.write('CutDC.set(' + str(CutDC.get()) + ')\n')
    #
    ConfgFile.write('AWGAMode.set('+ str(AWGAMode.get()) + ')\n')
    ConfgFile.write('AWGATerm.set('+ str(AWGATerm.get()) + ')\n')
    ConfgFile.write('AWGAAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGAAmplEntry.insert(4, ' + AWGAAmplEntry.get() + ')\n')
    ConfgFile.write('AWGAOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGAOffsetEntry.insert(4, ' + AWGAOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGAFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGAFreqEntry.insert(4, ' + AWGAFreqEntry.get() + ')\n')
    ConfgFile.write('AWGAShape.set(' + str(AWGAShape.get()) + ')\n')
    #
    ConfgFile.write('AWGBMode.set('+ str(AWGBMode.get()) + ')\n')
    ConfgFile.write('AWGBTerm.set('+ str(AWGBTerm.get()) + ')\n')
    ConfgFile.write('AWGBShape.set(' + str(AWGBShape.get()) + ')\n')
    #
    ConfgFile.write('AWGSync.set(' + str(AWGSync.get()) + ')\n')
    #
    ConfgFile.write('CHAVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    ConfgFile.write('CHBVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    ConfgFile.write('CHAVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    ConfgFile.close()

def BLoadConfig():
    global CutDC
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode
    global AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global FFTwindow, DBdivindex, DBlevel, TRACEmode, TRACEaverage, Vdiv
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
    # Read configuration values from file
    filename = askopenfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename)
    for line in ConfgFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skipping " + line.rstrip()
    ConfgFile.close()
    #
def BSaveCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global DevID

    devidstr = DevID[17:31]
    filename = devidstr + "_V.cal"
    CalFile = open(filename, "w")
    #
    CalFile.write('CHAVGainEntry.delete(0,END)\n')
    CalFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    CalFile.write('CHBVGainEntry.delete(0,END)\n')
    CalFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    CalFile.write('CHAVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    CalFile.write('CHBVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    CalFile.close()

def BLoadCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global DevID

    devidstr = DevID[17:31]
    filename = devidstr + "_V.cal"
    CalFile = open(filename)
    for line in CalFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skipping " + line.rstrip()
    CalFile.close()
    
def BHelp():
    # open a URL, in this case, the ALICE VVM user's guide
    url = "https://wiki.analog.com/university/tools/alice-vvm-1.0-users-guide"
    webbrowser.open(url,new=2)
    
def Bcloseexit():
    global RUNstatus
    
    RUNstatus = 0
    # close ALM and exit
    root.destroy()
    exit()
    
def BStart():
    global RUNstatus, LoopNum, PowerStatus, devx, PwrBt, DevID
    
    if DevID == "No Device":
        showwarning("WARNING","No Device Plugged In!")
        return()
    else:
        if PowerStatus == 0:
            PowerStatus = 1
            PwrBt.config(bg="green",text="PWR-On")
            devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power    
        if RUNstatus == 0:
            RUNstatus = 1
            LoopNum = 1
    #UpdateScreen()          # Always Update

def BStop():
    global RUNstatus
    
    if (RUNstatus == 1):
        RUNstatus = 0
    elif (RUNstatus == 2):
        RUNstatus = 3
    elif (RUNstatus == 3):
        RUNstatus = 3
    elif (RUNstatus == 4):
        RUNstatus = 3
    UpdateScreen()          # Always Update
    
def BPower():
    global RUNstatus, PowerStatus, devx, PwrBt

    if (RUNstatus == 1):
        BStop()
    if PowerStatus == 1:
        PowerStatus = 0
        PwrBt.config(bg="red",text="PWR-Off")
        devx.ctrl_transfer(DevID, 0x40, 0x50, 49, 0, 0, 0, 100) # turn off analog power
    else:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power

def BSetup():
    global RUNstatus
    global SAMPLErate
    global T1line
    global ZEROstuffing
    
    if (RUNstatus != 0):
        showwarning("WARNING","Pause program first")
        return()
    
    s = askstring("Zero stuffing","For better interpolation of levels between frequency samples.\nBut increases processing time!\n\nValue: " + str(ZEROstuffing.get()) + "\n\nNew value:\n(0-5, 0 is no zero stuffing)")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        if v < 0:
            v = 0
        if v > 5:
            v = 5
        ZEROstuffing.set(v)

    UpdateScreen()          # Always Update    

def Bsamples1():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
    
    if (SMPfftpwrTwo.get() > 6):  # Min 64
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() - 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def Bsamples2():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
      
    if (SMPfftpwrTwo.get() < 16): # Max 65536
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() + 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

# =========== Awg functions ==================
def BAWGAAmpl(temp):
    global AWGAAmplEntry, AWGAAmplvalue

    try:
        AWGAAmplvalue = float(eval(AWGAAmplEntry.get()))
    except:
        AWGAAmplEntry.delete(0,"end")
        AWGAAmplEntry.insert(0, AWGAAmplvalue)
    #UpdateAWGA()
    
def BAWGAOffset(temp):
    global AWGAOffsetEntry, AWGAOffsetvalue

    try:
        AWGAOffsetvalue = float(eval(AWGAOffsetEntry.get()))
    except:
        AWGAOffsetEntry.delete(0,"end")
        AWGAOffsetEntry.insert(0, AWGAOffsetvalue)
    #UpdateAWGA()
    
def BAWGAFreq(temp):
    global AWGAFreqEntry, AWGAFreqvalue

    try:
        AWGAFreqvalue = float(eval(AWGAFreqEntry.get()))
    except:
        AWGAFreqEntry.delete(0,"end")
        AWGAFreqEntry.insert(0, AWGAFreqvalue)

    if AWGAFreqvalue > 45000: # max freq is 45KHz
        AWGAFreqvalue = 45000
    #UpdateAWGA()

def BAWGAShape():
    global AWGAShape, AWGAWave

    if AWGAShape.get() == 0:
        AWGAWave = 'dc'
    if AWGAShape.get() == 1:
        AWGAWave = 'sine' 
    if AWGAShape.get() == 2:
        AWGAWave = 'triangle'
    if AWGAShape.get() == 3:
        AWGAWave = 'sawtooth'
    if AWGAShape.get() == 4:
        AWGAWave = 'square'
    if AWGAShape.get() == 5:
        AWGAWave = 'stairstep'
    if AWGAShape.get() == 6:
        AWGAWave = 'arbitrary'
    # UpdateAWGA() 

def UpdateAWGA():
    global AWGAAmplvalue, AWGAOffsetvalue
    global AWGAFreqvalue, AWGAPhasevalue
    global AWGAWave, AWGAMode, AWGATerm
    global CHA, SAMPLErate, DevID, devx

    temp = 0
    BAWGAAmpl(temp)
    BAWGAOffset(temp)
    if FSweepMode.get() != 1:
        BAWGAFreq(temp)
    BAWGAShape()
    AWGAPhasevalue = 90
    if AWGAFreqvalue > 0.0:
        AWGAperiodvalue = SAMPLErate/AWGAFreqvalue
    else:
        AWGAperiodvalue = 0.0

    if AWGAPhasevalue > 0:
        AWGAdelayvalue = AWGAperiodvalue * AWGAPhasevalue / 360
    else:
        AWGAdelayvalue = 0

    if AWGATerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
    elif AWGATerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 33, 0, 0, 0, 100) # set GND switch to closed
    elif AWGATerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 32, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
        
    if AWGAWave == 'dc':
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue)
            # 
        elif AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)
            # 
        elif AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)
            #
        elif AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)

    else:
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
     
        if AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)          
        else:
            if AWGAWave == 'sine':
                CHA.sine(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'triangle':
                CHA.triangle(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'sawtooth':
                CHA.sawtooth(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'stairstep':
                CHA.stairstep(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'arbitrary':
                CHA.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGBShape():
    global AWGBShape, AWGBWave
    
    if AWGBShape.get() == 0:
        AWGBWave = 'dc'
    if AWGBShape.get() == 1:
        AWGBWave = 'sine'
    if AWGBShape.get() == 2:
        AWGBWave = 'triangle'
    if AWGBShape.get() == 3:
        AWGBWave = 'sawtooth'
    if AWGBShape.get() == 4:
        AWGBWave = 'square'
    if AWGBShape.get() == 5:
        AWGBWave = 'stairstep'
    if AWGBShape.get() == 6:
        AWGBWave = 'arbitrary'        
    # UpdateAWGB()
    
def UpdateAWGB():
    global AWGBWave, AWGBMode, AWGBTerm
    global CHB, SAMPLErate, DevID, devx

    temp = 0
    BAWGBShape()
        
    if AWGBTerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
    elif AWGBTerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 38, 0, 0, 0, 100) # set GND switch to closed
    elif AWGBTerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 37, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
        
    if AWGBWave == 'dc':
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            CHB.constant(AWGBOffsetvalue)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            CHB.constant(AWGBOffsetvalue/1000)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            CHB.constant(0.0) #
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            CHB.constant(AWGBOffsetvalue/1000)
    else:
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        if AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            CHB.constant(0.0)
        else:
            if AWGBWave == 'sine':
                CHB.sine(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'triangle':
                CHB.triangle(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'sawtooth':
                CHB.sawtooth(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'stairstep':
                CHB.stairstep(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'arbitrary':
                CHB.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGEnab():
    global AWGAMode, AWGBMode, AWGSync
    global CHA, CHB

    if  AWGAMode.get() == 2:
        CHA.set_mode('d')
        CHA.constant(0.0) # 
    else:
        UpdateAWGA()

    if  AWGBMode.get() == 2:
        CHB.set_mode('d')
        CHB.constant(0.0) # 
    else:
        UpdateAWGB()
              
#    if AWGSync.get() > 0:
#    else:
#        if AWG1Enab.get() > 0:     
#        else:     
#        if AWG2Enab.get() > 0:         
#        else:
            
def BAWGSync():
    global AWGSync

#    if AWGSync.get() > 0:     
#    else:

# ============================================ Main routine ====================================================
def GetDigIn(): # Read digital inputs and update digital inputs display
    global DI0text, DI1text, DI2text, DI3text
    global DO0, DO1, DO2, DO3
    global DevID, devx

    if DO0.get() == "x" or DO0.get() == "X":
       Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 0, 0, 0, 1, 100)
       DI0text.set(str(Dval))
    if DO1.get() == "x" or DO1.get() == "X":
       Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 1, 0, 0, 1, 100)
       DI1text.set(str(Dval))
    if DO2.get() == "x" or DO2.get() == "X":
       Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 2, 0, 0, 1, 100)
       DI2text.set(str(Dval))
    if DO3.get() == "x" or DO3.get() == "X":
       Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 3, 0, 0, 1, 100)
       DI3text.set(str(Dval))

def SetDigOut(temp): # set digital output ( return typed in entry window )
    global DI0text, DI1text, DI2text, DI3text
    global DO0, DO1, DO2, DO3
    global DevID, devx    

    if DO0.get() == "0":
        devx.ctrl_transfer(DevID, 0x40, 0x50, 0, 0, 0, 0, 100)
        DI0text.set("0")
    if DO0.get() == "1":
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100)
        DI0text.set("1")
    if DO1.get() == "0":
        devx.ctrl_transfer(DevID, 0x40, 0x50, 1, 0, 0, 0, 100)
        DI1text.set("0")
    if DO1.get() == "1":
        devx.ctrl_transfer(DevID, 0x40, 0x51, 1, 0, 0, 0, 100)
        DI1text.set("1")
    if DO2.get() == "0":
        devx.ctrl_transfer(DevID, 0x40, 0x50, 2, 0, 0, 0, 100)
        DI2text.set("0")
    if DO2.get() == "1":
        devx.ctrl_transfer(DevID, 0x40, 0x51, 2, 0, 0, 0, 100)
        DI2text.set("1")
    if DO3.get() == "0":
        devx.ctrl_transfer(DevID, 0x40, 0x50, 3, 0, 0, 0, 100)
        DI3text.set("0")
    if DO3.get() == "1":
        devx.ctrl_transfer(DevID, 0x40, 0x51, 3, 0, 0, 0, 100)
        DI3text.set("1")

def Analog_In():   # Read the analog input from the ALM1000 and store the data into the arrays
    global ADsignal1, FFTBuffA, FFTBuffB
    global AWGSync, AWGAMode, AWGBMode
    global AWGAFreqvalue, AWGBFreqvalue
    global STARTfrequency, STOPfrequency
    global CHA, CHB, Both, devx
    global RUNstatus
    global SAMPLErate
    global SMPfft, OverRangeFlagA, OverRangeFlagB
    global DCA, DCB
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    
    while (True):                                           # Main loop
        # RUNstatus = 1 
        if (RUNstatus == 1) or (RUNstatus == 2):
            # Do input probe Calibration CH1VGain, CH2VGain, CH1VOffset, CH2VOffset
            try:
                InOffA = float(eval(CHAVOffsetEntry.get()))
            except:
                CHAVOffsetEntry.delete(0,END)
                CHAVOffsetEntry.insert(0, InOffA)
            try:
                InGainA = float(eval(CHAVGainEntry.get()))
            except:
                CHAVGainEntry.delete(0,END)
                CHAVGainEntry.insert(0, InGainA)
            try:
                InOffB = float(eval(CHBVOffsetEntry.get()))
            except:
                CHBVOffsetEntry.delete(0,END)
                CHBVOffsetEntry.insert(0, InOffB)
            try:
                InGainB = float(eval(CHBVGainEntry.get()))
            except:
                CHBVGainEntry.delete(0,END)
                CHBVGainEntry.insert(0, InGainB)
            #
            INITIALIZEstart()
            # Starting acquisition This is a HACK to get around non-continous AWG mode!
            # restat AWGs if indicated
            if AWGSync.get() > 0:
                BAWGEnab()
            # 
            GetDigIn() # Update digital inputs display after AWGs have started
            SHOWsamples = SMPfft + 128 # get 128 extra samples
            time.sleep(0.005)
            ADsignal1 = Both.get_samples(SHOWsamples) # get samples for both channel A and B
            # waite to finish then return to open termination
            devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set CHA 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set CHA GND switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
            if AWGAMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            if AWGBMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            
            FFTBuffA = [] # Clear the FFTBuff array for trace A
            FFTBuffB = [] # Clear the FFTBuff array for trace B
            OverRangeFlagA = OverRangeFlagB = 0 # Clear over range flags
            index = 127 # skip first 128 samples
            while index < SHOWsamples:
                SPA = ADsignal1[0][index]
                VAdata = float(SPA[0])
                VA = (VAdata - InOffA) * InGainA
                FFTBuffA.append(VA)
                SPB = ADsignal1[1][index]
                VBdata = float(SPB[0])
                VB = (VBdata - InOffB) * InGainB
                FFTBuffB.append(VB)
                if VAdata >= 5.0 or VAdata <= 0.0:
                    OverRangeFlagA = 1

                if VBdata >= 5.0 or VBdata <= 0.0:
                    OverRangeFlagB = 1
                index = index + 1

            DCA = numpy.average(FFTBuffA)
            DCB = numpy.average(FFTBuffB)
            if CutDC.get() == 1:
                FFTBuffA = FFTBuffA - DCA
                FFTBuffB = FFTBuffB - DCB
            UpdateAll()                         # Update Data, trace and screen

        # RUNstatus = 3: Stop
        # RUNstatus = 4: Stop and restart
        if (RUNstatus == 3) or (RUNstatus == 4):
            if RUNstatus == 3:
                RUNstatus = 0                   # Status is stopped 
            if RUNstatus == 4:          
                RUNstatus = 1                   # Status is (re)start
            UpdateScreen()                      # UpdateScreen() call
        # Update tasks and screens by TKinter 
        root.update_idletasks()
        root.update()                           # update screens

def UpdateAll():        # Update Data, trace and screen
    global FFTBuffA, FFTBuffB
    global SMPfft

    if len(FFTBuffA) < SMPfft and len(FFTBuffB) < SMPfft:
        return
    
    DoFFT()             # Fast Fourier transformation
    MakeTrace()         # Update the traces
    UpdateScreen()      # Update the screen 

def UpdateTrace():      # Update trace and screen
    MakeTrace()         # Update traces
    UpdateScreen()      # Update the screen

def UpdateScreen():     # Update screen with trace and text
    MakeScreen()        # Update the screen
    root.update()       # Activate updated screens    

def DoFFT():            # Fast Fourier transformation
    global FFTBuffA, FFTBuffB
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global FFTwindowshape, FFTbandwidth
    global SAMPLErate, STARTfrequency
    global SMPfft, LoopNum
    # global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample
    global TRACEaverage, TRACEmode, FSweepMode
    global TRACEreset
    global ZEROstuffing
    
    # T1 = time.time()              # For time measurement of FFT routine
    
    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffA[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Analog level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryA = FFTresultA
    if TRACEmode.get() == 3:
        PhaseMemoryA = PhaseA

    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseA = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (V*V) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseA = PhaseA[:le]
    PhaseA = numpy.degrees(PhaseA)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultA = Totalcorr * ALL

    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffB[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Analog level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryB = FFTresultB
    if TRACEmode.get() == 3:
        PhaseMemoryB = PhaseB
    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseB = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (U*U) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseB = PhaseB[:le]
    PhaseB = numpy.degrees(PhaseB)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultB = Totalcorr * ALL
    TRACEsize = len(FFTresultB)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum == 1:
        PhaseMemoryB = PhaseB
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
#
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultB = FFTmemoryB + (FFTresultB - FFTmemoryB) / TRACEaverage.get()
        PhaseB = PhaseMemoryB +(PhaseB - PhaseMemoryB) / TRACEaverage.get()
# 
    TRACEsize = len(FFTresultA)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum == 1:
        PhaseMemoryA = PhaseA
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
#
        FFTresultA = numpy.maximum(FFTresultA, FFTmemoryA)
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultA = FFTmemoryA + (FFTresultA - FFTmemoryA) / TRACEaverage.get()
        PhaseA = PhaseMemoryA +(PhaseA - PhaseMemoryA) / TRACEaverage.get()
#
    if TRACEmode.get() == 2 and TRACEreset == False:
        FFTresultB = numpy.maximum(FFTresultB, FFTmemoryB)

    TRACEreset = False          # Trace reset done

    # T2 = time.time()
    # print (T2 - T1)           # For time measurement of FFT routine
#
def DoImpedance():

# Input Variables
    global PeakdbA, PeakdbB, PeakRelPhase
    #(VZ/VA)from vector voltmeter 
    # global VVangle # angle in degrees between VZ and VA 
    global RsystemEntry # resistance of series resistor or power divider  
# Computed outputs 
    # global VVangleCosine # cosine of vector voltmeter angle 
    global ImpedanceMagnitude # in ohms 
    global ImpedanceAngle # in degrees 
    global ImpedanceRseries # in ohms 
    global ImpedanceXseries # in ohms 
    # global temp # temporary variable
    
    DEG2RAD = (3.141592654 / 180.0)
    SMALL = 1E-20
    try:
        ResValue = float(RsystemEntry.get())
    except:
        ResValue = 1000.0
        
    VVvoltageRatio = math.pow(10,((PeakdbB-PeakdbA)/20)) # VZ/VA 
    VVangleCosine = math.cos(math.radians(PeakRelPhase))
    Temp1 = ResValue * VVvoltageRatio
    Temp2 = 1.0 + VVvoltageRatio**2 - (2.0 * VVvoltageRatio * VVangleCosine)
    if(Temp2 < SMALL):
        Temp2 = SMALL # This handles negative and too small positive
        
    Temp3 = math.sqrt(Temp2)  # VI/VA
    Temp4 = ResValue * (1-(VVvoltageRatio * VVangleCosine))

    ImpedanceMagnitude = (Temp1/Temp3) #  # Zx
    ImpedanceRseries = (Temp4/Temp2) - ResValue # Rx
    temp = ImpedanceMagnitude**2 - ImpedanceRseries**2
    if(temp < 0.0):
        temp = 0.0
    ImpedanceXseries = math.sqrt(temp) # Xx
    if(PeakRelPhase < 0.0):
        ImpedanceXseries = -ImpedanceXseries
    ImpedanceAngle = math.atan2(ImpedanceXseries, ImpedanceRseries) / DEG2RAD
    
def MakeTrace():        # Update the grid and trace
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM, PeakphaseA, PeakphaseB 
    global PeakfreqA, PeakfreqB, PeakRelPhase
    global DBdivindex   # Index value
    global DBdivlist    # dB per division list
    global DBlevel      # Reference level
    global GRH          # Screenheight
    global GRW          # Screenwidth
    global SAMPLErate
    global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample, LoopNum, FSweepMode
    global TRACEmode
    global T1Vline, T2Vline, TMline, T1Pline, T2Pline
    global Vdiv         # Number of vertical divisions
    global X0L          # Left top X value
    global Y0T          # Left top Y value 
    global ImpedanceMagnitude # in ohms 
    global ImpedanceAngle # in degrees 
    global ImpedanceRseries # in ohms 
    global ImpedanceXseries # in ohms

    # Set the TRACEsize variable
    TRACEsize = len(FFTresultA)     # Set the trace length
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    # Horizontal conversion factors (frequency Hz) and border limits
    STARTsample = float(STARTfrequency.get()) / Fsample      # First sample in FFTresult[] that is used
    STARTsample = int(math.ceil(STARTsample))               # First within screen range

    STOPsample = float(STOPfrequency.get()) / Fsample        # Last sample in FFTresult[] that is used
    STOPsample = int(math.floor(STOPsample))                # Last within screen range, math.floor actually not necessary, part of int

    MAXsample = TRACEsize                                   # Just an out of range check
    if STARTsample > (MAXsample - 1):
        STARTsample = MAXsample - 1

    if STOPsample > MAXsample:
        STOPsample = MAXsample

    n = STARTsample
    PeakfreqA = PeakfreqB = PeakfreqM = F = n * Fsample
    PeakphaseA = PhaseA[n]
    PeakphaseB = PhaseB[n]
    PeakSample = n

    PeakdbA = (10 * math.log10(float(FFTresultA[n])) + 17)
    
    PeakdbB = (10 * math.log10(float(FFTresultB[n])) + 17)

    PeakMdb = PeakdbA - PeakdbB
    
    while n <= STOPsample:
        F = n * Fsample
        try:
            dbA = (10 * math.log10(float(FFTresultA[n])) + 17)   # Convert power to DBs, except for log(0) error
        except:
            dbA = -200
        if dbA > PeakdbA:
            PeakdbA = dbA
            PeakfreqA = F
            PeakphaseA = PhaseA[n]
            PeakSample = n

        try:
            dbB = (10 * math.log10(float(FFTresultB[n])) + 17) # Add 17 dB for max value of +10 dB ALSO in CSV file routine! 
        except:
            dbB = -200
        if dbB > PeakdbB:
            PeakdbB = dbB
            PeakfreqB = F
            PeakphaseB = PhaseB[n]

        RelPhase = PhaseA[n]-PhaseB[n]
        if RelPhase > 180:
            RelPhase = RelPhase - 360
        elif RelPhase < -180:
            RelPhase = RelPhase + 360
        PhErr = 0.0018 * n * Fsample # calculate pahse error due half sample period offset
        RelPhase = RelPhase + PhErr - 12
        n = n + 1

    PeakRelPhase = PeakphaseB-PeakphaseA
    if PeakRelPhase > 180:
        PeakRelPhase = PeakRelPhase - 360
    elif PeakRelPhase < -180:
        PeakRelPhase = PeakRelPhase + 360
    PhErr = 0.0018 * PeakSample * Fsample # calculate pahse error due half sample period offset
    PeakRelPhase = PeakRelPhase - PhErr # - 12
    DoImpedance()

def MakeScreen():       # Update the screen with traces and text
    global CANVASheight, CANVASwidth
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB, PeakfreqRA, PeakfreqRB
    global PeakxRA, PeakyRA, PeakxRB, PeakyRB, PeakdbRA, PeakdbRB
    global PeakxRM, PeakyRM, PeakRMdb, PeakfreqRM
    global PeakphaseA, PeakphaseB, PeakRelPhase
    global COLORgrid    # The colors
    global COLORsignalband, COLORtext
    global COLORtrace1, COLORtrace2
    global ResScale   # Ohms per div 
    global FFTwindow, FFTbandwidth, ZEROstuffing, FFTwindowname
    global X0L          # Left top X value
    global Y0T          # Left top Y value
    global GRW          # Screenwidth
    global GRH          # Screenheight
    global RUNstatus    # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
    global SAMPLErate, OverRangeFlagA, OverRangeFlagB
    global SMPfft       # number of FFT samples
    global STARTfrequency, STOPfrequency
    global TRACEaverage # Number of traces for averageing
    global TRACEmode    # 1 normal 2 max 3 average
    global Vdiv         # Number of vertical divisions
    global ImpedanceMagnitude # in ohms 
    global ImpedanceAngle # in degrees 
    global ImpedanceRseries # in ohms 
    global ImpedanceXseries # in ohms

    # Delete all items on the screen
    de = ca.find_enclosed ( -1000, -1000, CANVASwidth+1000, CANVASheight+1000 )    
    for n in de: 
        ca.delete(n)
 
    # Draw circular grid lines
    i = 1
    xcenter = GRW/2
    ycenter = GRH/2 
    Radius = (GRW-X0L)/11
    OhmsperPixel = float(ResScale.get())/Radius
    TRadius = Radius * 5
    x1 = X0L
    x2 = X0L + GRW
    xright = 10 + xcenter + ( 5 * Radius )
    while (i <= 5):
        x0 = xcenter - ( i * Radius )
        x1 = xcenter + ( i * Radius )
        y0 = ycenter - ( i * Radius )
        y1 = ycenter + ( i * Radius )
        ResTxt = float(ResScale.get()) * i
        ca.create_oval ( x0, y0, x1, y1, outline=COLORgrid, width=2)
        ca.create_line(xcenter, y0, xright, y0, fill=COLORgrid, width=1, dash=(4,3))
        ca.create_text(xright, y0, text=str(ResTxt), fill=COLORgrid, anchor="w", font=("arial", 10 ))
        # 
        i = i + 1
    ca.create_line(xcenter, y0, xcenter, y1, fill=COLORgrid, width=2)
    ca.create_line(x0, ycenter, x1, ycenter, fill=COLORgrid, width=2)
    RAngle = math.radians(45)
    y = TRadius*math.sin(RAngle)
    x = TRadius*math.cos(RAngle)
    ca.create_line(xcenter-x, ycenter-y, xcenter+x, ycenter+y, fill=COLORgrid, width=2)
    ca.create_line(xcenter+x, ycenter-y, xcenter-x, ycenter+y, fill=COLORgrid, width=2)
    ca.create_text(x0, ycenter, text="180", fill=COLORgrid, anchor="e", font=("arial", 10 ))
    ca.create_text(x1, ycenter, text="0.0", fill=COLORgrid, anchor="w", font=("arial", 10 ))
    ca.create_text(xcenter, y0, text="90", fill=COLORgrid, anchor="s", font=("arial", 10 ))
    ca.create_text(xcenter, y1, text="-90", fill=COLORgrid, anchor="n", font=("arial", 10 ))
    # Draw traces
    x1 = xcenter + ( ImpedanceRseries / OhmsperPixel )
    if x1 > 1500:
        x1 = xright
    elif x1 < -500:
        x1 = xcenter - xright
    ca.create_line(xcenter, ycenter, x1, ycenter, fill=COLORtrace1, width=3)
    y1 = ycenter - ( ImpedanceXseries / OhmsperPixel )
    if y1 > 1500:
        y1 = xright
    elif y1 < -500:
        y1 = ycenter - xright
    xmag = x1
    ymag = y1
    ca.create_line(xcenter, ycenter, xcenter, y1, fill=COLORtrace6, width=3)
    MagRadius = ImpedanceMagnitude / OhmsperPixel
    y1 = ycenter - MagRadius*math.sin(math.radians(ImpedanceAngle))
    if y1 > 1500:
        y1 = xright
    elif y1 < -500:
        y1 = ycenter - xright
    x1 = xcenter + MagRadius*math.cos(math.radians(ImpedanceAngle))
    if x1 > 1500:
        x1 = xright
    elif x1 < -500:
        x1 = xcenter - xright
    ca.create_line(xcenter, ycenter, x1, y1, fill=COLORtrace2, width=3)
    #ca.create_line(x1, ycenter, xmag, ymag, fill=COLORtrace4, width=3)
    #ca.create_line(xcenter, y1, xmag, ymag, fill=COLORtrace4, width=3)
# display warning if input out of range
    if OverRangeFlagA == 1:
        x = X0L+GRW+10
        y = Y0T+GRH-40
        ca.create_rectangle(x-6, y-6, x+6, y+6, fill="#ff0000")
        ca.create_text (x+12, y, text="CHA Over Range", anchor=W, fill="#ff0000", font=("arial", 12 ))
    if OverRangeFlagB == 1:
        x = X0L+GRW+10
        y = Y0T+GRH-10
        ca.create_rectangle(x-6, y-6, x+6, y+6, fill="#ff0000")
        ca.create_text (x+12, y, text="CHB Over Range", anchor=W, fill="#ff0000", font=("arial", 12 ))
    # General information on top of the grid

    txt = "    Sample rate: " + str(SAMPLErate)
    txt = txt + "    FFT samples: " + str(SMPfft)

    if FFTwindow.get() == 0:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 1:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 2:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 3:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 4:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 5:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 6:
        txt = txt + "   " + FFTwindowname
        
    x = X0L
    y = 12
    idTXT = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)
    #
    x = X0L + GRW + 4
    y = 24
    txt = "Gain " + ' {0:.2f} '.format(PeakdbB-PeakdbA) + " dB" 
    TXT9  = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Phase " + ' {0:.2f} '.format(PeakRelPhase) + " Degrees"
    TXT10  = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Freq " + ' {0:.1f} '.format(PeakfreqA) + " Hertz"
    TXT11  = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Impedance Magnitude"
    TXT1 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = ' {0:.1f} '.format(ImpedanceMagnitude)
    TXT2 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Impedance Angle" 
    TXT3 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = ' {0:.1f} '.format(ImpedanceAngle)
    TXT4 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Impedance R series"
    TXT5 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = ' {0:.1f} '.format(ImpedanceRseries)
    TXT6 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = "Impedance X series"
    TXT7 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
    y = y + 24
    txt = ' {0:.1f} '.format(ImpedanceXseries)
    TXT8 = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 14 ))
#
    if ImpedanceXseries < 0: # calculate series capacitance
        y = y + 24
        txt = "Series Capacitance"
        ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
        Cseries = -1 / ( 2 * 3.14159 * PeakfreqA * ImpedanceXseries ) # in farads
        Cseries = Cseries * 1E6
        y = y + 20
        if Cseries < 1:
            Cseries = Cseries * 1E3
            txt = ' {0:.2f} '.format(Cseries) + "nF"
            ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
        else:
            txt = ' {0:.2f} '.format(Cseries) + "uF"
            ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
    elif ImpedanceXseries > 0: # calculate series inductance
        y = y + 24
        txt = "Series Inductance"
        ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
        Lseries = ImpedanceXseries / ( 2 * 3.14159 * PeakfreqA ) # in henry
        Lseries = Lseries * 1E3 # in millihenry
        y = y + 22
        if Lseries < 1:
            Lseries = Lseries * 1E3
            txt = ' {0:.2f} '.format(Lseries) + "uH"
            ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
        else:
            txt = ' {0:.2f} '.format(Lseries) + "mH"
            ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext, font=("arial", 12 ))
    # Start and stop frequency and trace mode
    txt = str(STARTfrequency.get()) + " to " + str(STOPfrequency.get()) + " Hz"
    txt = txt + "  FFT Bandwidth =" + ' {0:.2f} '.format(FFTbandwidth)
        
    x = X0L
    y = Y0T+GRH-13
    idTXT = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)
    
    if TRACEmode.get() == 1:
        txt ="Normal mode "

    if TRACEmode.get() == 2:
        txt = "Peak hold mode "
    
    if TRACEmode.get() == 3:
        txt = "Power average  mode (" + str(TRACEaverage.get()) + ") " 

    if ZEROstuffing.get() > 0:
        txt = txt + "Zero Stuffing = " + str(ZEROstuffing.get())
    # Runstatus and level information
    if (RUNstatus == 0):
        txt = txt + "  Stopped "
    else:
        txt = txt + "  Running "
    y = Y0T+GRH
    IDtxt  = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)

def INITIALIZEstart():
    global SMPfft
    global SMPfftpwrTwo
    global TRACEreset, TRACEmode

    # First some subroutines to set specific variables
    SMPfft = 2 ** int(SMPfftpwrTwo.get()) # Calculate the number of FFT samples from SMPfftpwrtwo

    CALCFFTwindowshape()
    if TRACEmode.get() == 1 and TRACEreset == False:
        TRACEreset = True               # Clear the memory for averaging or peak

def CALCFFTwindowshape():           # Make the FFTwindowshape for the windowing function
    global FFTbandwidth             # The FFT bandwidth
    global FFTwindow                # Which FFT window number is selected
    global FFTwindowname            # The name of the FFT window function
    global FFTwindowshape           # The window shape
    global SAMPLErate               # The sample rate
    global SMPfft                   # Number of FFT samples
    
    # FFTname and FFTbandwidth in milliHz
    FFTwindowname = "No such window"
    FFTbw = 0
    
    if FFTwindow.get() == 0:
        FFTwindowname = " Rectangular (no) window (B=1) "
        FFTbw = 1.0

    if FFTwindow.get() == 1:
        FFTwindowname = " Cosine window (B=1.24) "
        FFTbw = 1.24

    if FFTwindow.get() == 2:
        FFTwindowname = " Triangular window (B=1.33) "
        FFTbw = 1.33

    if FFTwindow.get() == 3:
        FFTwindowname = " Hann window (B=1.5) "
        FFTbw = 1.5

    if FFTwindow.get() == 4:
        FFTwindowname = " Blackman window (B=1.73) "
        FFTbw = 1.73

    if FFTwindow.get() == 5:
        FFTwindowname = " Nuttall window (B=2.02) "
        FFTbw = 2.02

    if FFTwindow.get() == 6:
        FFTwindowname = " Flat top window (B=3.77) "
        FFTbw = 3.77

    FFTbandwidth = int(FFTbw * SAMPLErate / float(SMPfft)) 

    # Calculate the shape
    FFTwindowshape = numpy.ones(SMPfft)         # Initialize with ones

    # m = 0                                       # For calculation of correction factor, furhter no function

    n = 0
    while n < SMPfft:

        # Cosine window function
        # medium-dynamic range B=1.24
        if FFTwindow.get() == 1:
            w = math.sin(math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.571

        # Triangular non-zero endpoints
        # medium-dynamic range B=1.33
        if FFTwindow.get() == 2:
            w = (2.0 / SMPfft) * ((SMPfft/ 2.0) - abs(n - (SMPfft - 1) / 2.0))
            FFTwindowshape[n] = w * 2.0

        # Hann window function
        # medium-dynamic range B=1.5
        if FFTwindow.get() == 3:
            w = 0.5 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.000

        # Blackman window, continuous first derivate function
        # medium-dynamic range B=1.73
        if FFTwindow.get() == 4:
            w = 0.42 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.08 * math.cos(4 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.381

        # Nuttall window, continuous first derivate function
        # high-dynamic range B=2.02
        if FFTwindow.get() == 5:
            w = 0.355768 - 0.487396 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.144232 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.012604 * math.cos(6 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.811

        # Flat top window, 
        # medium-dynamic range, extra wide bandwidth B=3.77
        if FFTwindow.get() == 6:
            w = 1.0 - 1.93 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 1.29 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.388 * math.cos(6 * math.pi * n / (SMPfft - 1)) + 0.032 * math.cos(8 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.000
        
        # m = m + w / SMPfft                          # For calculation of correction factor
        n = n + 1

    # if m > 0:                                     # For calculation of correction factor
    #     print "correction 1/m: ", 1/m             # For calculation of correction factor

def ConnectDevice():
    global devx, DevID, Both, CHA, CHB, bcon
    try:
        devx = Smu()
        brdnum = len(devx.devices)
        channum = len(devx.channels)
        CHA = devx.channels['A']
        CHA.set_mode('d')           # Put CHA in Hi Z mode
        CHB = devx.channels['B']
        CHB.set_mode('d')           # Put CHB in Hi Z mode
        DevID = devx.serials[0]
        print DevID
        ADsignal1 = []              # Ain signal array 
        Both = devx.devices[0]
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
        bcon.configure(text="Conn", bg="green")
        if channum == 4:
            CHC = devx.channels['C']
            CHC.set_mode('d')
            CHD = devx.channels['D']
            CHD.set_mode('d')
            DevTwo = devx.serials[1]
            print DevTwo
            BTwo = devx.devices[1]
    except:
        print 'No Device plugged IN!'
        DevID = "No Device"
        bcon.configure(text="Recon", bg="red")
    
# ================ Make Screen ==========================

frame2r = Frame(root, borderwidth=5, relief=RIDGE)
frame2r.pack(side=RIGHT, expand=1, fill=Y)

frame1 = Frame(root, borderwidth=5, relief=RIDGE)
frame1.pack(side=TOP, expand=1, fill=X)

frame2 = Frame(root, background="black", borderwidth=5, relief=RIDGE)
frame2.pack(side=TOP, expand=YES, fill=BOTH)

ca = Canvas(frame2, width=CANVASwidth, height=CANVASheight, background=COLORcanvas)
ca.pack(side=TOP)

# top menu buttons
FFTwindow = IntVar(0)   # FFT window function variable
FFTwindow.set(5)
FFTwindmenu = Menubutton(frame1, text="FFTwindow", relief=RAISED)
FFTwindmenu.menu = Menu(FFTwindmenu, tearoff = 0 )
FFTwindmenu["menu"]  = FFTwindmenu.menu
FFTwindmenu.menu.add_radiobutton(label='Rectangular window (B=1)', variable=FFTwindow, value=0)
FFTwindmenu.menu.add_radiobutton(label='Cosine window (B=1.24)', variable=FFTwindow, value=1)
FFTwindmenu.menu.add_radiobutton(label='Triangular window (B=1.33)', variable=FFTwindow, value=2)
FFTwindmenu.menu.add_radiobutton(label='Hann window (B=1.5)', variable=FFTwindow, value=3)
FFTwindmenu.menu.add_radiobutton(label='Blackman window (B=1.73)', variable=FFTwindow, value=4)
FFTwindmenu.menu.add_radiobutton(label='Nuttall window (B=2.02)', variable=FFTwindow, value=5)
FFTwindmenu.menu.add_radiobutton(label='Flat top window (B=3.77)', variable=FFTwindow, value=6)
FFTwindmenu.pack(side=LEFT)
#
b = Button(frame1, text="-Samples", width=Buttonwidth2, command=Bsamples1)
b.pack(side=LEFT)
b = Button(frame1, text="+Samples", width=Buttonwidth2, command=Bsamples2)
b.pack(side=LEFT)
#
b = Button(frame1, text="Exit", width=4, command=Bcloseexit)
b.pack(side=RIGHT)
b = Button(frame1, text="Stop", width=4, bg="red", command=BStop)
b.pack(side=RIGHT)
b = Button(frame1, text="Run", width=5, bg="green", command=BStart)
b.pack(side=RIGHT)
PwrBt = Button(frame1, text="PWR-ON", width=7, bg="green", command=BPower)
PwrBt.pack(side=RIGHT)
#
# right side drop down menu buttons
dropmenu = Frame( frame2r )
dropmenu.pack(side=TOP)
bcon = Button(dropmenu, text="Recon", width=5, command=ConnectDevice)
bcon.pack(side=LEFT, anchor=W)
# File menu 
Filemenu = Menubutton(dropmenu, text="File", relief=RAISED)
Filemenu.menu = Menu(Filemenu, tearoff = 0 )
Filemenu["menu"] = Filemenu.menu
Filemenu.menu.add_command(label="Save Config", command=BSaveConfig)
Filemenu.menu.add_command(label="Load Config", command=BLoadConfig)
Filemenu.menu.add_command(label="Save V Cal", command=BSaveCal)
Filemenu.menu.add_command(label="Load V Cal", command=BLoadCal)
Filemenu.menu.add_command(label="Save Screen", command=BSaveScreen)
Filemenu.menu.add_command(label="Help", command=BHelp)
Filemenu.pack(side=LEFT, anchor=W)
#
CutDC = IntVar(0)
CutDC.set(1)
Optionmenu = Menubutton(dropmenu, text="Options", relief=RAISED)
Optionmenu.menu = Menu(Optionmenu, tearoff = 0 )
Optionmenu["menu"]  = Optionmenu.menu
Optionmenu.menu.add_checkbutton(label='Cut-DC', variable=CutDC)
Optionmenu.menu.add_command(label="Zero stuffing", command=BSetup)
Optionmenu.pack(side=LEFT, anchor=W)
#rsystem = 1000 # Temp set source resistance to 1000
rsystem = Frame( frame2r )
rsystem.pack(side=TOP)
rsystemlab = Label(rsystem, text="Ext Res")
rsystemlab.pack(side=LEFT, anchor=W)
RsystemEntry = Entry(rsystem, width=5)
RsystemEntry.pack(side=LEFT, anchor=W)
RsystemEntry.delete(0,"end")
RsystemEntry.insert(4,1000)
# Res Sacle Spinbox
ressb = Frame( frame2r )
ressb.pack(side=TOP)
reslab = Label(ressb, text="Ohms/div ")
reslab.pack(side=LEFT)
ResScale = Spinbox(ressb, width=6, values=ResScalediv)
ResScale.pack(side=LEFT)
ResScale.delete(0,"end")
ResScale.insert(0,500)
# now AWG A
awg1lab = Label(frame2r, text="AWG CH A")
awg1lab.pack(side=TOP)
# AWG enable sub frame
AWGAMode = IntVar(0)   # AWG A mode variable
AWGAMode.set(0) # AWG A alway in SVMI mode
AWGATerm = IntVar(0)   # AWG A termination variable
AWGATerm.set(0) # AWG A termination alway open
AWGAShape = IntVar(0)  # AWG A Wave shape variable
AWGAShape.set(1) # AWG A Wave shape always sine
#
awg1ampl = Frame( frame2r )
awg1ampl.pack(side=TOP)
AWGAAmplEntry = Entry(awg1ampl, width=5)
AWGAAmplEntry.bind("<Return>", BAWGAAmpl)
AWGAAmplEntry.pack(side=LEFT, anchor=W)
AWGAAmplEntry.delete(0,"end")
AWGAAmplEntry.insert(4, 1.086)
amp1lab = Label(awg1ampl, text="Min")
amp1lab.pack(side=LEFT, anchor=W)
#
awg1off = Frame( frame2r )
awg1off.pack(side=TOP)
AWGAOffsetEntry = Entry(awg1off, width=5)
AWGAOffsetEntry.bind("<Return>", BAWGAOffset)
AWGAOffsetEntry.pack(side=LEFT, anchor=W)
AWGAOffsetEntry.delete(0,"end")
AWGAOffsetEntry.insert(4, 3.914)
off1lab = Label(awg1off, text="Max")
off1lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg1freq = Frame( frame2r )
awg1freq.pack(side=TOP)
AWGAFreqEntry = Entry(awg1freq, width=7)
AWGAFreqEntry.bind("<Return>", BAWGAFreq)
AWGAFreqEntry.pack(side=LEFT, anchor=W)
AWGAFreqEntry.delete(0,"end")
AWGAFreqEntry.insert(4,1000)
freq1lab = Label(awg1freq, text="Freq")
freq1lab.pack(side=LEFT, anchor=W)

# now AWG B
# AWG enable sub frame
AWGBMode = IntVar(0)   # AWG B mode variable
AWGBMode.set(2) # AWG B alway in Hi-Z mode
AWGBTerm = IntVar(0)   # AWG B termination variable
AWGBTerm.set(0) # AWG B termination alway open
AWGBShape = IntVar(0)  # AWG B Wave shape variable
AWGBShape.set(1) # AWG B Wave shape always sine
#
AWGSync = IntVar(0) # Sync start both AWG channels
AWGSync.set(1)
awgsync = Checkbutton(frame2r, text="Sync AWG", variable=AWGSync, command=BAWGSync)
awgsync.pack(side=TOP)
# sweep generator mode menu buttons
FSweepMode = IntVar(0)   # Sweep Generator mode
FSweepCont = IntVar(0)
#
# Digital Input / Output Widgets
digin = Frame( frame2r )
digin.pack(side=TOP)
DI0text = StringVar()
DI1text = StringVar()
DI2text = StringVar()
DI3text = StringVar()
DInLab = Label( digin, text = "D Inp ")
DInLab.pack(side=LEFT)
DI0 = Label( digin, textvariable=DI0text)
DI0text.set("x")
DI0.pack(side=LEFT,padx=0)
DI1 = Label( digin, textvariable=DI1text)
DI1text.set("x")
DI1.pack(side=LEFT,padx=0)
DI2 = Label( digin, textvariable=DI2text)
DI2text.set("x")
DI2.pack(side=LEFT,padx=0)
DI3 = Label( digin, textvariable=DI3text)
DI3text.set("x")
DI3.pack(side=LEFT,padx=0)
#
digout = Frame( frame2r )
digout.pack(side=TOP)
DOutLab = Label( digout, text = "D Out ")
DOutLab.pack(side=LEFT)
DO0 = Entry(digout, width=1)
DO0.bind("<Return>", SetDigOut)
DO0.pack(side=LEFT)
DO0.delete(0,"end")
DO0.insert(0,"x")
DO1 = Entry(digout, width=1)
DO1.bind("<Return>", SetDigOut)
DO1.pack(side=LEFT)
DO1.delete(0,"end")
DO1.insert(0,"x")
DO2 = Entry(digout, width=1)
DO2.bind("<Return>", SetDigOut)
DO2.pack(side=LEFT)
DO2.delete(0,"end")
DO2.insert(0,"x")
DO3 = Entry(digout, width=1)
DO3.bind("<Return>", SetDigOut)
DO3.pack(side=LEFT)
DO3.delete(0,"end")
DO3.insert(0,"x")
# input probe wigets
prlab = Label(frame2r, text="Input Gain Offset")
prlab.pack(side=TOP)
# Input Probes sub frame 
ProbeA = Frame( frame2r )
ProbeA.pack(side=TOP)
gain1lab = Label(ProbeA, text="CA")
gain1lab.pack(side=LEFT)
CHAVGainEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVGainEntry.pack(side=LEFT)
CHAVGainEntry.delete(0,"end")
CHAVGainEntry.insert(0,1.0)
CHAVOffsetEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVOffsetEntry.pack(side=LEFT)
CHAVOffsetEntry.delete(0,"end")
CHAVOffsetEntry.insert(0,0.0)
#
ProbeB = Frame( frame2r )
ProbeB.pack(side=TOP)
gain2lab = Label(ProbeB, text="CB")
gain2lab.pack(side=LEFT)
CHBVGainEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVGainEntry.pack(side=LEFT)
CHBVGainEntry.delete(0,"end")
CHBVGainEntry.insert(0,1.0)
CHBVOffsetEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVOffsetEntry.pack(side=LEFT)
CHBVOffsetEntry.delete(0,"end")
CHBVOffsetEntry.insert(0,0.0)
# add ADI logo Don't mess with this bit map data!
ADIlogo = """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"""
logo = PhotoImage(data=ADIlogo)
ADI1 = Label(frame2r, image=logo, anchor= "sw", height=49, width=116, compound="top")
ADI1.pack(side=TOP)

#===== Initalize device ======
ConnectDevice()
# ================ Call main routine ===============================
root.update()               # Activate updated screens
#
# Start sampling
Analog_In()
 


