#!/usr/bin/python
# ADALM1000 digitla and analog Strip chart recorder tool (1-29-2016)
# For Python version > = 2.7.8
# With external module libpysmu (.pyd / .so for ADALM1000 )
# Created by D Mercer ()
#
from Tkinter import *
import math, random, threading, time
import atexit
from collections import defaultdict
from operator import add
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
import libpysmu
from time import gmtime, strftime
# set up variables
InOffA = InOffB = 0.0
InGainA = InGainB = 1.0
CANVASwidth = 1000
CANVASheight = 450

# define ADALM1000 interface
class Smu(object):
    def __init__(self):
        atexit.register(libpysmu.cleanup)
        libpysmu.setup()
        self.load_channels()

    def load_channels(self):
        self.devices = libpysmu.get_dev_info()
        self.serials = {i: v[0] for i, v in enumerate(self.devices)}
        self.devices = [x[1] for x in self.devices]

        names = (chr(x) for x in xrange(65,91))
        channels = {names.next(): (i, v) for i, d in enumerate(self.devices)
                    for k, v in d.items()}
        self.channels = {k: Channel(k, self.serials[v[0]], v[1]) for k, v in channels.items()}

        device_channels = defaultdict(list)
        for k, v in channels.items():
            device_channels[v[0]].append(Channel(k, *v))
        self.devices = {i: Device(self.serials[i], device_channels[i]) for i, v
                        in enumerate(self.devices)}

    def ctrl_transfer(self, device, bm_request_type, b_request, wValue, wIndex,
                      data, wLength, timeout):
        data = str(data)
        if bm_request_type&0x80 == 0x80:
            if data == '0':
                data = '\x00'*wLength
        else:
            wLength = 0
        ret = libpysmu.ctrl_transfer(device, bm_request_type, b_request, wValue,
                                   wIndex, data, wLength, timeout)
        if bm_request_type&0x80 == 0x80:
            return map(ord, data)
        else:
            return ret

    def __repr__(self):
        return 'Devices: ' + str(self.devices)

class Device(object):
    def __init__(self, serial, channels):
        self.serial = serial
        self.channels = channels

    def get_samples(self, n_samples):
        """query device for a list of samples from all channels

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from all the device's channels
        """
        return libpysmu.get_all_inputs(self.serial, n_samples)

class Channel(object):
    def __init__(self, chan, dev_serial, signals):
        self.dev = dev_serial
        self.chan = ord(chan) - 65
        self.signals = {v: i for i, v in enumerate(signals)}

    def set_mode(self, mode):
        if mode == 'v' or mode == 'V':
            libpysmu.set_mode(self.dev, self.chan, 1)
            self.mode = 1
        elif mode == 'i' or mode == 'I':
            libpysmu.set_mode(self.dev, self.chan, 2)
            self.mode = 2
        elif mode == 'd' or mode == 'D':
            libpysmu.set_mode(self.dev, self.chan, 0)
            self.mode = 0
        else:
            raise ValueError('invalid mode')

    def arbitrary(self, *args, **kwargs):
        repeat = 0
        if 'repeat' in map(str.lower, kwargs.keys()):
            repeat = 1
        wave = map(float, reduce(add, [[s]*100*n for s, n in args]))
        return libpysmu.set_output_buffer(wave, self.dev, self.chan, self.mode, repeat)
    
    def get_samples(self, n_samples):
        """query channel for samples

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from the channel
        """
        return libpysmu.get_inputs(self.dev, self.chan, n_samples)

    def constant(self, val):
        """set output to a constant waveform"""
        return libpysmu.set_output_constant(self.dev, self.chan, self.mode, val)

    def square(self, midpoint, peak, period, phase, duty_cycle):
        """set output to a square waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 1, midpoint, peak, period, phase, duty_cycle)

    def sawtooth(self, midpoint, peak, period, phase):
        """set output to a sawtooth waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 2, midpoint, peak, period, phase, 42)

    def stairstep(self, midpoint, peak, period, phase):
        """set output to a stairstep waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 3, midpoint, peak, period, phase, 42)

    def sine(self, midpoint, peak, period, phase):
        """set output to a sinusoidal waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 4, midpoint, peak, period, phase, 42)

    def triangle(self, midpoint, peak, period, phase):
        """set output to a triangle waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 5, midpoint, peak, period, phase, 42)

    def __repr__(self):
        return 'Device: ' + str(self.dev) + '\nChannel: ' + str(self.chan) + '\nSignals: ' + str(self.signals)
#
class StripChart:
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global InOffA, InOffB, InGainA, InGainB
    global DCVA0, DCVB0, dlog, Dlog_open, Ztime
    
    def __init__(self, root):
        global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
        self.gf = self.makeGraph(root)
        self.cf = self.makeControls(root)
        self.gf.pack()
        self.cf.pack()
        self.Reset()

    def makeGraph(self, frame):
        global CANVASwidth, CANVASheight
        global gf
        self.sw = CANVASwidth # default 1000
        self.h = CANVASheight # default 450
        self.top = 2
        gf = Canvas(frame, width=self.sw, height=self.h+10,
                    bg="#000", bd=0, highlightthickness=0)
        gf.p = PhotoImage(width=2*self.sw, height=self.h)
        self.item = gf.create_image(0, self.top, image=gf.p, anchor=NW)
        return(gf)

    def makeControls(self, frame):
        global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
        global NumGrid, SelCHA, SelCHB, dlog, Dlog_open
        
        cf = Frame(frame, borderwidth=1, relief="raised")
        br = Button(cf, text="Run", command=self.Run)
        br.grid(column=2, row=2)
        bs = Button(cf, text="Stop", command=self.Stop)
        bs.grid(column=4, row=2)
        bt = Button(cf, text="Reset", command=self.Reset)
        bt.grid(column=6, row=2)
        bss = Button(cf, text="Save Screen", command=BSaveScreen)
        bss.grid(column=8, row=2, columnspan=2)
        dlog = IntVar()
        Dlog_open = IntVar()
        Dlog_open.set(0)
        dlog1 = Checkbutton(cf, text="Start Data Log",variable= dlog)
        dlog1.grid(column=10, row=2, columnspan=2)
        bexit = Button(cf, text="Exit", command=close_out)
        bexit.grid(column=12,row=2)
        # input probe wigets
        prlab = Label(cf, text="Channel Gain / Offset calibration")
        prlab.grid(row=2, column=0, sticky=W)
        # Input Probes sub frame 
        ProbeA = Frame( cf )
        ProbeA.grid(row=3, column=0, columnspan=8, sticky=W)
        gain1lab = Label(ProbeA, text="CA")
        gain1lab.pack(side=LEFT)
        CHAVGainEntry = Entry(ProbeA, width=6) #
        CHAVGainEntry.pack(side=LEFT)
        CHAVGainEntry.delete(0,"end")
        CHAVGainEntry.insert(0,1.0)
        CHAVOffsetEntry = Entry(ProbeA, width=6) # 
        CHAVOffsetEntry.pack(side=LEFT)
        CHAVOffsetEntry.delete(0,"end")
        CHAVOffsetEntry.insert(0,0.0)
        #
        gain2lab = Label(ProbeA, text="CB")
        gain2lab.pack(side=LEFT)
        CHBVGainEntry = Entry(ProbeA, width=6) # )
        CHBVGainEntry.pack(side=LEFT)
        CHBVGainEntry.delete(0,"end")
        CHBVGainEntry.insert(0,1.0)
        CHBVOffsetEntry = Entry(ProbeA, width=6) # 
        CHBVOffsetEntry.pack(side=LEFT)
        CHBVOffsetEntry.delete(0,"end")
        CHBVOffsetEntry.insert(0,0.0)
        b1 = Button(ProbeA, text='Save', command=BSaveCal)
        b1.pack(side=LEFT)
        b2 = Button(ProbeA, text='Load', command=BLoadCal)
        b2.pack(side=LEFT)
        #
        SelCHA = IntVar(0)
        selcha = Checkbutton(cf, text="CH A", variable=SelCHA)
        selcha.grid(row=3, column=8, sticky=W)
        SelCHB = IntVar(0)
        selchb = Checkbutton(cf, text="CH B", variable=SelCHB)
        selchb.grid(row=3, column=9, sticky=W)
        NumGrid = IntVar(0)
        rb1 = Radiobutton(cf, text="1 Grid", variable=NumGrid, value=0 )
        rb1.grid(row=3, column=10, sticky=W)
        rb2 = Radiobutton(cf, text="2 Grid", variable=NumGrid, value=1 )
        rb2.grid(row=3, column=11, sticky=W)
        #
        self.fps = Label(cf, text="0 Sps")
        self.fps.grid(row=3, column=12, columnspan=1)
        return(cf)

    def Run(self):
        global Ztime, Dlog_open, dlog
        self.go = 1
        for t in threading.enumerate():
            if t.name == "_gen_":
                print("already running")
                return
        threading.Thread(target=self.do_start, name="_gen_").start()
        if Dlog_open.get() == 1 and dlog > 0:
            Ztime = time.time()
        elif Dlog_open.get() == 0 and dlog > 0:
            open_out()
            Ztime = time.time()
        else:
            Dlog_open.set(0)

    def Stop(self):
        self.go = 0
        for t in threading.enumerate():
            if t.name == "_gen_":
                t.join()

    def Reset(self):
        self.Stop()
        self.clearstrip(self.gf.p, '#000')

    def do_start(self):
        global DCVA0, DCVB0
        global D0val, D1val, D2val, D3val
        global InOffA, InOffB, InGainA, InGainB
        global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
        global NumGrid, SelCHA, SelCHB
        
        t = 0
        y2 = 0
        tx = time.time()
        #
        try:
            InOffA = float(eval(CHAVOffsetEntry.get()))
        except:
            CHAVOffsetEntry.delete(0,END)
            CHAVOffsetEntry.insert(0, InOffA)
        try:
            InGainA = float(eval(CHAVGainEntry.get()))
        except:
            CHAVGainEntry.delete(0,END)
            CHAVGainEntry.insert(0, InGainA)
        try:
            InOffB = float(eval(CHBVOffsetEntry.get()))
        except:
            CHBVOffsetEntry.delete(0,END)
            CHBVOffsetEntry.insert(0, InOffB)
        try:
            InGainB = float(eval(CHBVGainEntry.get()))
        except:
            CHBVGainEntry.delete(0,END)
            CHBVGainEntry.insert(0, InGainB)
        while self.go:
            Analog_in()
            #
            if NumGrid.get() == 0: # both on one grid
                y1 = (DCVB0/6.25)-0.4 # scale / 0ffset 0 to 5 V to +/- 0.4
                y2 = (DCVA0/6.25)-0.4 # scale / 0ffset 0 to 5 V to +/- 0.4
                if SelCHA.get() and SelCHB.get():
                    self.scrollstrip(self.gf.p,
                       (0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,0.5+y1,0.5+y2),
                       ('#808080','#808080','#808080','#808080','#0000D0','#808080','#808080','#808000','#808080','#00ff00','#ff8000'),
                         "" if t % 65 else "#088")
                elif SelCHA.get():
                    self.scrollstrip(self.gf.p,
                       (0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,0.5+y2),
                       ('#808080','#808080','#808080','#808080','#0000D0','#808080','#808080','#808000','#808080','#ff8000'),
                         "" if t % 65 else "#088")
                elif SelCHB.get():
                    self.scrollstrip(self.gf.p,
                       (0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,0.5+y1),
                       ('#808080','#808080','#808080','#808080','#0000D0','#808080','#808080','#808000','#808080','#00ff00'),
                         "" if t % 65 else "#088")
            else: # two grids
                y1 = (DCVB0/25)-0.1 # scale / 0ffset 0 to 5 V to +/- 0.1
                y2 = (DCVA0/25)-0.1 # scale / 0ffset 0 to 5 V to +/- 0.1
                y3 = D0val[0] * 0.05
                y4 = D1val[0] * 0.05
                y5 = D2val[0] * 0.05
                y6 = D3val[0] * 0.05
                if SelCHA.get() and SelCHB.get():
                    self.scrollstrip(self.gf.p,
                       (0.05,0.15,0.25,0.35,0.45,0.55,0.65,0.75,0.85,0.95,0.65+y1,0.85+y2, 0.05+y3, 0.15+y4, 0.25+y5, 0.35+y6),
                       ('#008000','#008000','#008000','#008000','#008000','#804000','#804000','#804000','#804000','#804000','#00ff00','#ff8000','#00ff00','#00ff00','#00ff00','#00ff00'),
                         "" if t % 65 else "#088")
                elif SelCHA.get():
                    self.scrollstrip(self.gf.p,
                       (0.05,0.15,0.25,0.35,0.45,0.55,0.65,0.75,0.85,0.95,0.85+y2, 0.05+y3, 0.15+y4, 0.25+y5, 0.35+y6),
                       ('#008000','#008000','#008000','#008000','#008000','#804000','#804000','#804000','#804000','#804000','#ff8000','#00ff00','#00ff00','#00ff00','#00ff00'),
                         "" if t % 65 else "#088")
                elif SelCHB.get():
                    self.scrollstrip(self.gf.p,
                       (0.05,0.15,0.25,0.35,0.45,0.65+y1,0.55,0.65,0.75,0.85,0.95, 0.05+y3, 0.15+y4, 0.25+y5, 0.35+y6),
                       ('#008000','#008000','#008000','#008000','#008000','#00ff00','#804000','#804000','#804000','#804000','#804000','#00ff00','#00ff00','#00ff00','#00ff00'),
                         "" if t % 65 else "#088")
            t += 1
            if not t % 100:
                tx2 = time.time()
                self.fps.config(text='%d Sps' % int(100/(tx2 - tx)))
                tx = tx2
            #time.sleep(0.075)

    def clearstrip(self, p, color):  # Fill strip with background color
        self.bg = color              # save background color for scroll
        self.data = None             # clear previous data
        self.x = 0
        p.tk.call(p, 'put', color, '-to', 0, 0, p['width'], p['height'])

    def scrollstrip(self, p, data, colors, bar=""):   # Scroll the strip, add new data
        self.x = (self.x + 1) % self.sw               # x = double buffer position
        bg = bar if bar else self.bg
        p.tk.call(p, 'put', bg, '-to', self.x, 0,
                  self.x+1, self.h)
        p.tk.call(p, 'put', bg, '-to', self.x+self.sw, 0,
                  self.x+self.sw+1, self.h)
        self.gf.coords(self.item, -1-self.x, self.top)  # scroll to just-written column
        if not self.data:
            self.data = data
        for d in range(len(data)):
            y0 = int((self.h-1) * (1.0-self.data[d]))   # plot all the data points
            y1 = int((self.h-1) * (1.0-data[d]))
            ya, yb = sorted((y0, y1))
            for y in range(ya, yb+1):                   # connect the dots
                p.put(colors[d], (self.x,y))
                p.put(colors[d], (self.x+self.sw,y))
        self.data = data            # save for next call
#
def close_out():
    global DlogFile
# close opened files by this process
    try:
        DlogFile.close()
    except:
        print "no dlog file open"
    quit()
    
def open_out():
    global DlogFile
    global Dlog_open, dlog

    # open file for data logging
    tme =  strftime("%Y%b%d-%H%M%S", gmtime())      # The time
    filename = "StripChart-" + tme
    filename = filename + ".csv"
    try:
        DlogFile = open(filename, 'a')
    except:
        filename = asksaveasfilename(defaultextension = ".csv", filetypes=[("Comma Separated Values", "*.csv")])
        DlogFile = open(filename, 'a')
    DlogFile.write( 'Time, CA-V, CB-V \n' )
    Dlog_open.set(1)
    
def Analog_in():
    global devx, DevID, CHA, CHB, Both, dlog, DlogFile
    global DCVA0, DCVB0, Ztime
    global InOffA, InOffB, InGainA, InGainB
    global D0val, D1val, D2val, D3val
    #
    DCVA0 = DCVB0 = 0.0 # initalize measurment variable
    # Get A0 and B0 data
    ADsignal1 = Both.get_samples(60) # get samples for both channel A0 and B0
    D0val = devx.ctrl_transfer(DevID, 0xc0, 0x91, 0, 0, 0, 1, 100) # Read PIO 0
    D1val = devx.ctrl_transfer(DevID, 0xc0, 0x91, 1, 0, 0, 1, 100) # Read PIO 1
    D2val = devx.ctrl_transfer(DevID, 0xc0, 0x91, 2, 0, 0, 1, 100) # Read PIO 2
    D3val = devx.ctrl_transfer(DevID, 0xc0, 0x91, 3, 0, 0, 1, 100) # Read PIO 3
    # get_samples returns a list of values for voltage [0] and current [1]
    for index in range(50): # calculate average
        SPA = ADsignal1[0][index+10] # skip over first 10 readings
        SPB = ADsignal1[1][index+10]
        VAdata = (float(SPA[0]) - InOffA) * InGainA # 
        VBdata = (float(SPB[0]) - InOffB) * InGainB #
        DCVA0 += VAdata # Sum for average CA voltage 
        DCVB0 += VBdata # Sum for average CB voltage

    DCVA0 = DCVA0 / 50.0 # calculate average
    DCVB0 = DCVB0 / 50.0 # calculate average
    if dlog > 0:
        tstr1 = time.time()-Ztime
        DlogString = '{0:.3f}, '.format(tstr1) + '{0:.3f}, '.format(DCVA0) + '{0:.3f} '.format(DCVB0) + " \n"
        DlogFile.write( DlogString )
#
def BSaveCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global DevID

    devidstr = DevID[17:31]
    filename = devidstr + "_V.cal"
    CalFile = open(filename, "w")
    #
    CalFile.write('CHAVGainEntry.delete(0,END)\n')
    CalFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    CalFile.write('CHBVGainEntry.delete(0,END)\n')
    CalFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    CalFile.write('CHAVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    CalFile.write('CHBVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    CalFile.close()

def BLoadCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global DevID
    global InOffA, InOffB, InGainA, InGainB

    devidstr = DevID[17:31]
    filename = devidstr + "_V.cal"
    CalFile = open(filename)
    for line in CalFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skipping " + line.rstrip()
    CalFile.close()
    InOffA = float(eval(CHAVOffsetEntry.get()))
    InGainA = float(eval(CHAVGainEntry.get()))
    InOffB = float(eval(CHBVOffsetEntry.get()))
    InGainB = float(eval(CHBVGainEntry.get()))
#
def BSaveScreen():
    global CANVASwidth, CANVASheight
    global gf
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # next save postscript file
    gf.postscript(file=filename, height=CANVASheight, width=CANVASwidth, colormode='color')

    #
def main():
    global devx, DevID, CHA, CHB, Both
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    # setup main window
    TBicon = """
    R0lGODlhIAAgAHAAACH5BAEAAAIALAAAAAAgACAAgQAAAP///wAAAAAAAAJJhI+py+0PYwtBWkDp
    hTnv2XlfEobjUZZnmn4se72vJMtcbYN4ruz44uORgiodsfI4Im++2M5VW81OmBbVULxiRVrUsgsO
    i8fUAgA7
    """
    #
    root = Tk()
    root.title("ALM1000 Mixed Signal StripChart (1-29-2016)")
    img = PhotoImage(data=TBicon)
    root.call('wm', 'iconphoto', root._w, img)
    #
    # Setup ADAML1000
    devx = Smu()
    DevID = devx.serials[0]
    print DevID
    CHA = devx.channels['A']    # Open CHA
    CHA.set_mode('d')           # Put CHA in Hi Z mode
    CHB = devx.channels['B']    # Open CHB
    CHB.set_mode('d')           # Put CHB in Hi Z mode
    Both = devx.devices[0]
    devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
    ADsignal1 = []              # Ain signal array channel
    # start main loop
    app = StripChart(root)
    root.mainloop()

main()
