#!/usr/bin/python
# ADALM1000 alice-HS-1.0.py(w) (12-8-2015)
# For Python version > = 2.7.8
# With external module libpysmu (.pyd / .so for ADALM1000 )
# Created by D Mercer ()
import atexit
from collections import defaultdict
from operator import add
# from PIL import Image
# from PIL import ImageFile
import libpysmu

import math
import time
import tkFont
from Tkinter import *
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
import webbrowser
# from tkSimpleDialog import askstring
# from tkMessageBox import *

# Values that can be modified
GRW = 720                   # Width of the grid 720 default
GRH = 390                   # Height of the grid 390 default
X0L = 55                    # Left top X value of grid
Y0T = 25                    # Left top Y value of grid
# Colors that can be modified
COLORframes = "#000080"   # Color = "#rrggbb" rr=red gg=green bb=blue, Hexadecimal values 00 - ff
COLORcanvas = "#000000"   # 100% black
COLORgrid = "#808080"     # 50% Gray
COLORzeroline = "#0000ff" # 100% blue
COLORtrace1 = "#00ff00"   # 100% green
COLORtrace2 = "#ff8000"   # 100% orange
COLORtrace3 = "#00ffff"   # 100% cyan
COLORtrace4 = "#ffff00"   # 100% yellow
COLORtrace5 = "#ff00ff"   # 100% magenta
COLORtraceR1 = "#008000"   # 50% green
COLORtraceR2 = "#804000"   # 50% orange
COLORtraceR3 = "#008080"   # 50% cyan
COLORtraceR4 = "#808000"   # 50% yellow
COLORtraceR5 = "#800080"   # 50% magenta
COLORtext = "#ffffff"     # 100% white
COLORtrigger = "#ff0000"  # 100% red
# Set sample buffer size
# pysmu.setSampleRate(SAMPLErate)
# set initial trigger conditions
HoldOff = 0.01
SAMPLErate = 100000         # default Sample rate of the Scope channels
MinSamples = 1000
MaxSamples = 50000
Two28 = 268435456           # 2 raised to 28 power
mclk = 32000000             # dds master clock frequency
DevID = "m1k"
UPDATEspeed = 0.1           # Update speed, default 0.1, for slower PC's a higher value is perhaps required
TRIGGERlevel = 2.5          # Triggerlevel in volts

# Vertical Sensitivity list in v/div
CHvpdiv = (0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0)
# Vertical Sensitivity list in mA/div
CHipdiv = (1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0)
# Time list in ms/div
TMpdiv = (0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 200.0, 500.0, 1000.0)
# AWG variables
AWGAAmplvalue = 0.0
AWGAOffsetvalue = 0.0
AWGAFreqvalue = 0.0
AWGAPhasevalue = 0
AWGAdelayvalue = 0
AWGADutyCyclevalue = 50
AWGAWave = 'dc'
AWGBAmplvalue = 0.0
AWGBOffsetvalue = 0.0
AWGBFreqvalue = 0.0
AWGBPhasevalue = 0
AWGBdelayvalue = 0
AWGBDutyCyclevalue = 50
AWGBWave = 'dc'
#
DCV1 = DCV2 = MinV1 = MaxV1 = MinV2 = MaxV2 = MidV1 = PPV1 = MidV2 = PPV2 = SV1 = SI1 = 0
DCI1 = DCI2 = MinI1 = MaxI1 = MinI2 = MaxI2 = MidI1 = PPI1 = MidI2 = PPI2 = SV2 = SI2 = 0
CHAperiod = CHAfreq = CHBperiod = CHBfreq = 0
# Calibration coefficients
CHAVGain = CHBVGain = 1.0
CHAVOffset = CHBVOffset = 0.0
# Initialisation of general variables
CHAOffset = CHBOffset = 2.5
CHAIOffset = CHBIOffset = 0.0

# Other global variables required in various routines
CANVASwidth = GRW + 2 * X0L # The canvas width
CANVASheight = GRH + 80     # The canvas height

Ymin = Y0T                  # Minimum position of screen grid (top)
Ymax = Y0T + GRH            # Maximum position of screen grid (bottom)

LONGsweep = False           # True if sweeps longer than 1 second
LONGchunk = LONGsweep       # True if chunk for longsweep is used for the trace

ADsignal1 = []              # Ain signal array channel A and B
# ADsignal2 = []             AD signal array channel B

T1Vline = []                # Voltage Trace line channel A
T2Vline = []                # Voltage Trace line channel B
T1Iline = []                # Current Trace line channel A
T2Iline = []                # Current Trace line channel B
TXYline = []                # XY Trace line
TXYRline = []               # XY reference trace line
Tmathline = []              # Math trace line
T1VRline = []               # V reference Trace line channel A
T2VRline = []               # V reference Trace line channel B
T1IRline = []               # I reference Trace line channel A
T2IRline = []               # I reference Trace line channel B
TMRline = []                # Math reference Trace line
Triggerline = []            # Triggerline

SHOWsamples = 1000          # Number of samples on the screen   
SCstart = 0                 # Start sample of the trace

# TriggerPos = 0.0            # Starting trigger possition of the trace
TRACES = 1                  # Number of traces 1 or 2
TRACESread = 0              # Number of traces that have been read from ALM
XYMode = 0                  # XY Mode flag
RUNstatus = 0               # 0 stopped, 1 start, 2 running, 3 stop and restart, 4 stop
PowerStatus = 1
TRIGGERsample = 0           # AD sample trigger point

# ====== define libsmu funcions ======
class Smu(object):
    def __init__(self):
        atexit.register(libpysmu.cleanup)
        libpysmu.setup()
        self.load_channels()

    def load_channels(self):
        self.devices = libpysmu.get_dev_info()
        self.serials = {i: v[0] for i, v in enumerate(self.devices)}
        self.devices = [x[1] for x in self.devices]

        names = (chr(x) for x in xrange(65,91))
        channels = {names.next(): (i, v) for i, d in enumerate(self.devices)
                    for k, v in d.items()}
        self.channels = {k: Channel(k, self.serials[v[0]], v[1]) for k, v in channels.items()}

        device_channels = defaultdict(list)
        for k, v in channels.items():
            device_channels[v[0]].append(Channel(k, *v))
        self.devices = {i: Device(self.serials[i], device_channels[i]) for i, v
                        in enumerate(self.devices)}

    def ctrl_transfer(self, device, bm_request_type, b_request, wValue, wIndex,
                      data, wLength, timeout):
        data = str(data)
        if bm_request_type&0x80 == 0x80:
            if data == '0':
                data = '\x00'*wLength
        else:
            wLength = 0
        ret = libpysmu.ctrl_transfer(device, bm_request_type, b_request, wValue,
                                   wIndex, data, wLength, timeout)
        if bm_request_type&0x80 == 0x80:
            return map(ord, data)
        else:
            return ret

    def __repr__(self):
        return 'Devices: ' + str(self.devices)


class Device(object):
    def __init__(self, serial, channels):
        self.serial = serial
        self.channels = channels

    def get_samples(self, n_samples):
        """query device for a list of samples from all channels

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from all the device's channels
        """
        return libpysmu.get_all_inputs(self.serial, n_samples)

class Channel(object):
    def __init__(self, chan, dev_serial, signals):
        self.dev = dev_serial
        self.chan = ord(chan) - 65
        self.signals = {v: i for i, v in enumerate(signals)}

    def set_mode(self, mode):
        if mode == 'v' or mode == 'V':
            libpysmu.set_mode(self.dev, self.chan, 1)
            self.mode = 1
        elif mode == 'i' or mode == 'I':
            libpysmu.set_mode(self.dev, self.chan, 2)
            self.mode = 2
        elif mode == 'd' or mode == 'D':
            libpysmu.set_mode(self.dev, self.chan, 0)
            self.mode = 0
        else:
            raise ValueError('invalid mode')

    def arbitrary(self, *args, **kwargs):
        repeat = 0
        if 'repeat' in map(str.lower, kwargs.keys()):
            repeat = 1
        wave = map(float, reduce(add, [[s]*100*n for s, n in args]))
        return libpysmu.set_output_buffer(wave, self.dev, self.chan, self.mode, repeat)
    
    def get_samples(self, n_samples):
        """query channel for samples

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from the channel
        """
        return libpysmu.get_inputs(self.dev, self.chan, n_samples)

    def constant(self, val):
        """set output to a constant waveform"""
        return libpysmu.set_output_constant(self.dev, self.chan, self.mode, val)

    def square(self, midpoint, peak, period, phase, duty_cycle):
        """set output to a square waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 1, midpoint, peak, period, phase, duty_cycle)

    def sawtooth(self, midpoint, peak, period, phase):
        """set output to a sawtooth waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 2, midpoint, peak, period, phase, 42)

    def stairstep(self, midpoint, peak, period, phase):
        """set output to a stairstep waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 3, midpoint, peak, period, phase, 42)

    def sine(self, midpoint, peak, period, phase):
        """set output to a sinusoidal waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 4, midpoint, peak, period, phase, 42)

    def triangle(self, midpoint, peak, period, phase):
        """set output to a triangle waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 5, midpoint, peak, period, phase, 42)

    def __repr__(self):
        return 'Device: ' + str(self.dev) + '\nChannel: ' + str(self.chan) + '\nSignals: ' + str(self.signals)

# =========== Start widgets routines =============================
def BSaveConfig():
    global TgInput, TgEdge, SingleShot, AutoLevel
    global TRIGGERentry, TMsb, DisMode, Xsignal, Ysignal
    global CHAsb, CHAIsb, CHBsb, CHBIsb, FminEntry, MulXEntry
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, MathTrace
    global CHAOffsetEntry, CHAIOffsetEntry, CHBOffsetEntry, CHBIOffsetEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1, MeasDCI1, MeasMinI1
    global MeasMaxI1, MeasMidI1, MeasPPI1, MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2
    global MeasPPV2, MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSV2, MeasRMSI1, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
# open Config file for Write
    filename = asksaveasfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename, "w")
    ConfgFile.write('TRIGGERentry.delete(0,END)\n')
    ConfgFile.write('TRIGGERentry.insert(4, ' + TRIGGERentry.get() + ')\n')
    ConfgFile.write('TMsb.delete(0,END)\n')
    ConfgFile.write('TMsb.insert(0, ' + TMsb.get() + ')\n')
    ConfgFile.write('TgInput.set(' + str(TgInput.get()) + ')\n')
    ConfgFile.write('AutoLevel.set(' + str(AutoLevel.get()) + ')\n')
    ConfgFile.write('SingleShot.set(' + str(SingleShot.get()) + ')\n')
    ConfgFile.write('TgEdge.set(' + str(TgEdge.get()) + ')\n')
    ConfgFile.write('DisMode.set(' + str(DisMode.get()) + ')\n')
    ConfgFile.write('Xsignal.set(' + str(Xsignal.get()) + ')\n')
    ConfgFile.write('Ysignal.set(' + str(Ysignal.get()) + ')\n')
    #
    ConfgFile.write('FminEntry.delete(0,END)\n')
    ConfgFile.write('FminEntry.insert(0, ' + FminEntry.get() + ')\n')
    ConfgFile.write('MulXEntry.delete(0,END)\n')
    ConfgFile.write('MulXEntry.insert(0, ' + MulXEntry.get() + ')\n')
    #
    ConfgFile.write('ShowC1_V.set(' + str(ShowC1_V.get()) + ')\n')
    ConfgFile.write('ShowC1_I.set(' + str(ShowC1_I.get()) + ')\n')
    ConfgFile.write('ShowC2_V.set(' + str(ShowC2_V.get()) + ')\n')
    ConfgFile.write('ShowC2_I.set(' + str(ShowC2_I.get()) + ')\n')
    #
    ConfgFile.write('CHAOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAOffsetEntry.insert(4, ' + CHAOffsetEntry.get() + ')\n')
    ConfgFile.write('CHAIOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAIOffsetEntry.insert(4, ' + CHAIOffsetEntry.get() + ')\n')
    ConfgFile.write('CHAsb.delete(0,END)\n')
    ConfgFile.write('CHAsb.insert(0, ' + CHAsb.get() + ')\n')
    ConfgFile.write('CHAIsb.delete(0,END)\n')
    ConfgFile.write('CHAIsb.insert(0, ' + CHAIsb.get() + ')\n')
    #
    ConfgFile.write('CHBOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBOffsetEntry.insert(4, ' + CHBOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBIOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBIOffsetEntry.insert(4, ' + CHBIOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBsb.delete(0,END)\n')
    ConfgFile.write('CHBsb.insert(0, ' + CHBsb.get() + ')\n')
    ConfgFile.write('CHBIsb.delete(0,END)\n')
    ConfgFile.write('CHBIsb.insert(0, ' + CHBIsb.get() + ')\n')
    #
    ConfgFile.write('AWGAMode.set('+ str(AWGAMode.get()) + ')\n')
    ConfgFile.write('AWGATerm.set('+ str(AWGATerm.get()) + ')\n')
    ConfgFile.write('AWGAAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGAAmplEntry.insert(4, ' + AWGAAmplEntry.get() + ')\n')
    ConfgFile.write('AWGAOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGAOffsetEntry.insert(4, ' + AWGAOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGAFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGAFreqEntry.insert(4, ' + AWGAFreqEntry.get() + ')\n')
    ConfgFile.write('AWGAPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGAPhaseEntry.insert(4, ' + AWGAPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGADutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGADutyCycleEntry.insert(4, ' + AWGADutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGAShape.set(' + str(AWGAShape.get()) + ')\n')
    #
    ConfgFile.write('AWGBMode.set('+ str(AWGBMode.get()) + ')\n')
    ConfgFile.write('AWGBTerm.set('+ str(AWGBTerm.get()) + ')\n')
    ConfgFile.write('AWGBAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGBAmplEntry.insert(4, ' + AWGBAmplEntry.get() + ')\n')
    ConfgFile.write('AWGBOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGBOffsetEntry.insert(4, ' + AWGBOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGBFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGBFreqEntry.insert(4, ' + AWGBFreqEntry.get() + ')\n')
    ConfgFile.write('AWGBPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGBPhaseEntry.insert(4, ' + AWGBPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGBDutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGBDutyCycleEntry.insert(4, ' + AWGBDutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGBShape.set(' + str(AWGBShape.get()) + ')\n')
    #
    ConfgFile.write('AWGSync.set(' + str(AWGSync.get()) + ')\n')
    #
    ConfgFile.write('CHAVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    ConfgFile.write('CHBVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    ConfgFile.write('CHAVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    ConfgFile.write('MeasDCV1.set(' + str(MeasDCV1.get()) + ')\n')
    ConfgFile.write('MeasMinV1.set(' + str(MeasMinV1.get()) + ')\n')
    ConfgFile.write('MeasMaxV1.set(' + str(MeasMaxV1.get()) + ')\n')
    ConfgFile.write('MeasMidV1.set(' + str(MeasMidV1.get()) + ')\n')
    ConfgFile.write('MeasPPV1.set(' + str(MeasPPV1.get()) + ')\n')
    ConfgFile.write('MeasRMSV1.set(' + str(MeasRMSV1.get()) + ')\n')
    ConfgFile.write('MeasDCI1.set(' + str(MeasDCI1.get()) + ')\n')
    ConfgFile.write('MeasMinI1.set(' + str(MeasMinI1.get()) + ')\n')
    ConfgFile.write('MeasMaxI1.set(' + str(MeasMaxI1.get()) + ')\n')
    ConfgFile.write('MeasMidI1.set(' + str(MeasMidI1.get()) + ')\n')
    ConfgFile.write('MeasPPI1.set(' + str(MeasPPI1.get()) + ')\n')
    ConfgFile.write('MeasRMSI1.set(' + str(MeasRMSI1.get()) + ')\n')
    ConfgFile.write('MeasDCV2.set(' + str(MeasDCV2.get()) + ')\n')
    ConfgFile.write('MeasMinV2.set(' + str(MeasMinV2.get()) + ')\n')
    ConfgFile.write('MeasMaxV2.set(' + str(MeasMaxV2.get()) + ')\n')
    ConfgFile.write('MeasMidV2.set(' + str(MeasMidV2.get()) + ')\n')
    ConfgFile.write('MeasPPV2.set(' + str(MeasPPV2.get()) + ')\n')
    ConfgFile.write('MeasRMSV2.set(' + str(MeasRMSV2.get()) + ')\n')
    ConfgFile.write('MeasDCI2.set(' + str(MeasDCI2.get()) + ')\n')
    ConfgFile.write('MeasMinI2.set(' + str(MeasMinI2.get()) + ')\n')
    ConfgFile.write('MeasMaxI2.set(' + str(MeasMaxI2.get()) + ')\n')
    ConfgFile.write('MeasMidI2.set(' + str(MeasMidI2.get()) + ')\n')
    ConfgFile.write('MeasPPI2.set(' + str(MeasPPI2.get()) + ')\n')
    ConfgFile.write('MeasRMSI2.set(' + str(MeasRMSI2.get()) + ')\n')
    #
    ConfgFile.write('MeasAHW.set(' + str(MeasAHW.get()) + ')\n')
    ConfgFile.write('MeasALW.set(' + str(MeasALW.get()) + ')\n')
    ConfgFile.write('MeasADCy.set(' + str(MeasADCy.get()) + ')\n')
    ConfgFile.write('MeasAPER.set(' + str(MeasAPER.get()) + ')\n')
    ConfgFile.write('MeasAFREQ.set(' + str(MeasAFREQ.get()) + ')\n')
    ConfgFile.write('MeasBHW.set(' + str(MeasBHW.get()) + ')\n')
    ConfgFile.write('MeasBLW.set(' + str(MeasBLW.get()) + ')\n')
    ConfgFile.write('MeasBDCy.set(' + str(MeasBDCy.get()) + ')\n')
    ConfgFile.write('MeasBPER.set(' + str(MeasBPER.get()) + ')\n')
    ConfgFile.write('MeasBFREQ.set(' + str(MeasBFREQ.get()) + ')\n')
    ConfgFile.write('MeasPhase.set(' + str(MeasPhase.get()) + ')\n')
    #
    ConfgFile.write('MathTrace.set(' + str(MathTrace.get()) + ')\n')
    #
    ConfgFile.close()

def BLoadConfig():
    global TgInput, TgEdge, SingleShot, AutoLevel
    global TRIGGERentry, TMsb, DisMode, Xsignal, Ysignal
    global CHAsb, CHAIsb, CHBsb, CHBIsb, FminEntry, MulXEntry
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, MathTrace
    global CHAOffsetEntry, CHAIOffsetEntry, CHBOffsetEntry, CHBIOffsetEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1, MeasDCI1, MeasMinI1
    global MeasMaxI1, MeasMidI1, MeasPPI1, MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2
    global MeasPPV2, MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSV2, MeasRMSI1, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
    # Read configuration values from file
    filename = askopenfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename)
    for line in ConfgFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skipping " + line.rstrip()
    ConfgFile.close()
    
def BSaveScreen():
    global CANVASwidth, CANVASheight
    global COLORtext
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # temp chnage text corlor to black
    COLORtext = "#000000"
    UpdateScreen()
    # first save postscript file
    ca.postscript(file=filename, height=CANVASheight, width=CANVASwidth, colormode='color')
    # now convert to bit map
    # img = Image.open("screen_shot.eps")
    # img.save("screen_shot.gif", "gif")
    COLORtext = "#ffffff"
    UpdateScreen()

def BSaveData():
    global ADsignal1 # , ADsignal2
    global SAMPLErate, SHOWsamples
    # open file to save data
    filename = asksaveasfilename(defaultextension = ".csv", filetypes=[("Comma Separated Values", "*.csv")])
    DataFile = open(filename, 'a')
    DataFile.write( 'Sample-#, CA-V, CA-I, CB-V, CB-I \n' )
    for index in range(int(SHOWsamples)):
        SPA = ADsignal1[0][index]
        SPB = ADsignal1[1][index]
        DataFile.write( str(index) + ', ' + str(SPA[0]) + ', ' + str(SPA[1]) + ', '
                        + str(SPB[0]) + ', ' + str(SPB[1]) + '\n')
    DataFile.close()

def BHelp():
    # open a URL, in this case, the ALICE user's guide
    url = "https://wiki.analog.com/university/tools/alice-1.0-users-guide"
    webbrowser.open(url,new=2)
    
def BSnapShot():
    global T1Vline, T2Vline, T1Iline, T2Iline
    global TXYline, Tmathline, TMRline, TXYRline
    global T1VRline, T2VRline, T1IRline, T2IRline
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, ShowMath, MathTrace, DisMode

    if ShowC1_V.get() == 1:
        T1VRline = T1Vline               # V reference Trace line channel A
    if ShowC2_V.get() == 1:
        T2VRline = T2Vline               # V reference Trace line channel B
    if ShowC1_I.get() == 1:
        T1IRline = T1Iline               # I reference Trace line channel A
    if ShowC2_I.get() == 1:
        T2IRline = T2Iline               # I reference Trace line channel B
    if MathTrace.get() > 0:
        TMRline = Tmathline              # Math reference Trace line
    if DisMode.get() == 1:
        TXYRline = TXYline               # XY reference trace line
    
def donothing():
    global XYMode

def BShowCurvesAll():
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    ShowC1_V.set(1)
    ShowC1_I.set(1)
    ShowC2_V.set(1)
    ShowC2_I.set(1)

def BShowCurvesNone():
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    ShowC1_V.set(0)
    ShowC1_I.set(0)
    ShowC2_V.set(0)
    ShowC2_I.set(0)
    
def BTriggerEdge():
    global TgEdge
                                       
# TRIGCOND trigcondRisingPositive = 0
# TRIGCOND trigcondFallingNegative = 1

def BTrigger50p():
    global TgInput, TRIGGERlevel, TRIGGERentry
    global DCV1, DCV2, DCI1, DCI2

    if (TgInput.get() == 0):
        DCString = "0.0"
    elif (TgInput.get() == 1 ):
        DCString = ' {0:.2f} '.format(DCV1)
    elif (TgInput.get() == 2 ):
        DCString = ' {0:.2f} '.format(DCI1)
    elif (TgInput.get() == 3 ):
        DCString = ' {0:.2f} '.format(DCV2)
    elif (TgInput.get() == 4 ):
        DCString = ' {0:.2f} '.format(DCI2)

    TRIGGERlevel = eval(DCString)
    TRIGGERentry.delete(0,END)
    TRIGGERentry.insert(4, DCString)
    # set new trigger level
    
    UpdateTrace()           # Always Update
    
def BTriggerMode(): # place holder for future hardware triggering if implemented
    global TgInput

#    if (TgInput.get() == 0):
        # no trigger
         
#    elif (TgInput.get() == 1):
        # trigger source set to detector of analog in channels
         
        # auto trigger timeout value
        
#    elif (TgInput.get() == 2):
        # trigger source set to detector of analog in channels
         
        # 0 disables auto trigger
        
    
def BTriglevel(event):
    global TRIGGERlevel, TRIGGERentry

    # evalute entry string to a numerical value
    try:
        TRIGGERlevel = float(eval(TRIGGERentry.get()))
    except:
        TRIGGERentry.delete(0,END)
        TRIGGERentry.insert(0, TRIGGERlevel)
    # set new trigger level
    
    UpdateTrace()           # Always Update

def BHoldOff(event):
    global HoldOff, HoldOffentry

    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
        
def Bcloseexit():
    global RUNstatus
    
    RUNstatus = 0
    # pysmu.pause()
    # close ALM and exit
    root.destroy()
    exit()

def BStart():
    global RUNstatus, PowerStatus, devx, PwrBt

    if PowerStatus == 0:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power
    if (RUNstatus == 0):
        RUNstatus = 1
        # pysmu.start(stopOnClose=True)
    SetAD9833
    # UpdateScreen()          # Always Update

def BStop():
    global RUNstatus
    
    if (RUNstatus == 1):
        RUNstatus = 0
        # pysmu.pause()
    elif (RUNstatus == 2):
        RUNstatus = 3
    elif (RUNstatus == 3):
        RUNstatus = 3
    elif (RUNstatus == 4):
        RUNstatus = 3
    UpdateScreen()          # Always Update

def BPower():
    global RUNstatus, PowerStatus, devx, PwrBt

    if (RUNstatus == 1):
        BStop()
    if PowerStatus == 1:
        PowerStatus = 0
        PwrBt.config(bg="red",text="PWR-Off")
        devx.ctrl_transfer(DevID, 0x40, 0x50, 49, 0, 0, 0, 100) # turn off analog power
    else:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power
    
def BTime():
    global TIMEdiv, TMsb
    global SAMPLErate, RUNstatus

    try:
        TIMEdiv = float(eval(TMsb.get()))
    except:
        TIMEdiv = 1.0
        # change sample rate based on time scale
        # Currently 100000
    
    # SAMPLErate = 100000
        # pysmu.pause()
        # pysmu.setSampleRate(SAMPLErate)
        # pysmu.start(stopOnClose=True)
    BAWGEnab()
        
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4
        # pysmu.start(stopOnClose=True)

    UpdateTrace()           # Always Update

def BCHAlevel():
    global CHAsb
    
    try:
        CH1vpdvLevel = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvLevel)
    UpdateTrace()           # Always Update

def BCHAIlevel():
    global CHAIsb
    
    try:
        CH1ipdvLevel = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1ipdvLevel)
    UpdateTrace()           # Always Update

def BCHBlevel():
    global CHBsb
    
    try:
        CH2vpdvLevel = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvLevel)
    UpdateTrace()           # Always Update    

def BCHBIlevel():
    global CHBIsb
    
    try:
        CH2ipdvLevel = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2ipdvLevel)
    UpdateTrace()           # Always Update    

def BOffsetA(event):
    global CHAOffset, CHAOffsetEntry

    # evalute entry string to a numerical value
    try:
        CHAOffset = float(eval(CHAOffsetEntry.get()))
    except:
        CHAOffsetEntry.delete(0,END)
        CHAOffsetEntry.insert(0, CHAOffset)
    # set new offset level
    UpdateTrace()           # Always Update

def BIOffsetA(event):
    global CHAIOffset, CHAIOffsetEntry

    # evalute entry string to a numerical value
    try:
        CHAIOffset = float(eval(CHAIOffsetEntry.get()))
    except:
        CHAIOffsetEntry.delete(0,END)
        CHAIOffsetEntry.insert(0, CHAIOffset)
    # set new offset level
    UpdateTrace()           # Always Update

def BOffsetB(event):
    global CHBOffset, CHBOffsetEntry

    # evalute entry string to a numerical value
    try:
        CHBOffset = float(eval(CHBOffsetEntry.get()))
    except:
        CHBOffsetEntry.delete(0,END)
        CHBOffsetEntry.insert(0, CHBOffset)
    # set new offset level
    UpdateTrace()           # Always Update

def BIOffsetB(event):
    global CHBIOffset, CHBIOffsetEntry

    # evalute entry string to a numerical value
    try:
        CHBIOffset = float(eval(CHBIOffsetEntry.get()))
    except:
        CHBIOffsetEntry.delete(0,END)
        CHBIOffsetEntry.insert(0, CHBIOffset)
    # set new offset level
    UpdateTrace()           # Always Update

def BDisMode():
    global DisMode, XYMode

    if DisMode.get() == 0:
        XYMode = 0
    else:
        XYMode = 1
    UpdateTrace()           # Always Update
        
# Awg functions
def BAWGAAmpl(temp):
    global AWGAAmplEntry, AWGAAmplvalue

    try:
        AWGAAmplvalue = float(eval(AWGAAmplEntry.get()))
    except:
        AWGAAmplEntry.delete(0,"end")
        AWGAAmplEntry.insert(0, AWGAAmplvalue)
    #UpdateAWGA()
    
def BAWGAOffset(temp):
    global AWGAOffsetEntry, AWGAOffsetvalue

    try:
        AWGAOffsetvalue = float(eval(AWGAOffsetEntry.get()))
    except:
        AWGAOffsetEntry.delete(0,"end")
        AWGAOffsetEntry.insert(0, AWGAOffsetvalue)
    #UpdateAWGA()
    
def BAWGAFreq(temp):
    global AWGAFreqEntry, AWGAFreqvalue

    try:
        AWGAFreqvalue = float(eval(AWGAFreqEntry.get()))
    except:
        AWGAFreqEntry.delete(0,"end")
        AWGAFreqEntry.insert(0, AWGAFreqvalue)

    if AWGAFreqvalue > 20000: # max freq is 20KHz
        AWGAFreqvalue = 20000
    #UpdateAWGA()

def BAWGAPhase(temp):
    global AWGAPhaseEntry, AWGAPhasevalue

    try:
        AWGAPhasevalue = float(eval(AWGAPhaseEntry.get()))
    except:
        AWGAPhaseEntry.delete(0,"end")
        AWGAPhaseEntry.insert(0, AWGAPhasevalue)

    if AWGAPhasevalue > 360: # max phase is 360 degrees
        AWGAPhasevalue = 360
    if AWGAPhasevalue < 0: # min phase is 0 degrees
        AWGAPhasevalue = 0

def BAWGADutyCycle(temp):
    global AWGADutyCycleEntry, AWGADutyCyclevalue

    try:
        AWGADutyCyclevalue = float(eval(AWGADutyCycleEntry.get()))/100
    except:
        AWGADutyCycleEntry.delete(0,"end")
        AWGADutyCycleEntry.insert(0, AWGADutyCyclevalue)

    if AWGADutyCyclevalue > 1: # max duty cycle is 100%
        AWGADutyCyclevalue = 1
    if AWGADutyCyclevalue < 0: # min duty cycle is 0%
        AWGADutyCyclevalue = 0
    #UpdateAWGA()

def BAWGAShape():
    global AWGAShape, AWGAWave

    if AWGAShape.get() == 0:
        AWGAWave = 'dc'
    if AWGAShape.get() == 1:
        AWGAWave = 'sine' 
    if AWGAShape.get() == 2:
        AWGAWave = 'triangle'
    if AWGAShape.get() == 3:
        AWGAWave = 'sawtooth'
    if AWGAShape.get() == 4:
        AWGAWave = 'square'
    if AWGAShape.get() == 5:
        AWGAWave = 'stairstep'
    if AWGAShape.get() == 6:
        AWGAWave = 'arbitrary'
    # UpdateAWGA() 

def UpdateAWGA():
    global AWGAAmplvalue, AWGAOffsetvalue
    global AWGAFreqvalue, AWGAPhasevalue
    global AWGADutyCyclevalue
    global AWGAWave, AWGAMode, AWGATerm
    global CHA, SAMPLErate, DevID, devx

    temp = 0
    BAWGAAmpl(temp)
    BAWGAOffset(temp)
    BAWGAFreq(temp)
    BAWGAPhase(temp)
    BAWGADutyCycle(temp)
    BAWGAShape()
    
    if AWGAFreqvalue > 0.0:
        AWGAperiodvalue = SAMPLErate/AWGAFreqvalue
    else:
        AWGAperiodvalue = 0.0

    if AWGAPhasevalue > 0:
        AWGAdelayvalue = AWGAperiodvalue * AWGAPhasevalue / 360
    else:
        AWGAdelayvalue = 0

    if AWGATerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
    elif AWGATerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 33, 0, 0, 0, 100) # set GND switch to closed
    elif AWGATerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 32, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
        
    if AWGAWave == 'dc':
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue)
            # 
        elif AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)
            # 
        elif AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)
            #
        elif AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)

    else:
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
     
        if AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)          
        else:
            if AWGAWave == 'sine':
                CHA.sine(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'triangle':
                CHA.triangle(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'sawtooth':
                CHA.sawtooth(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'square':
                CHA.square(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue, AWGADutyCyclevalue)
            elif AWGAWave == 'stairstep':
                CHA.stairstep(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'arbitrary':
                CHA.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGBAmpl(temp):
    global AWGBAmplEntry, AWGBAmplvalue

    try:
        AWGBAmplvalue = float(eval(AWGBAmplEntry.get()))
    except:
        AWGBAmplEntry.delete(0,"end")
        AWGBAmplEntry.insert(0, AWGBAmplvalue)
    # UpdateAWGB()
    
def BAWGBOffset(temp):
    global AWGBOffsetEntry, AWGBOffsetvalue

    try:
        AWGBOffsetvalue = float(eval(AWGBOffsetEntry.get()))
    except:
        AWGBOffsetEntry.delete(0,"end")
        AWGBOffsetEntry.insert(0, AWGBOffsetvalue)
    # UpdateAWGB()
    
def BAWGBFreq(temp):
    global AWGBFreqEntry, AWGBFreqvalue

    try:
        AWGBFreqvalue = float(eval(AWGBFreqEntry.get()))
    except:
        AWGBFreqEntry.delete(0,"end")
        AWGBFreqEntry.insert(0, AWGBFreqvalue)

    if AWGBFreqvalue > 20000: # max freq is 20KHz
        AWGBFreqvalue = 20000
    # UpdateAWGB()
    
def BAWGBPhase(temp):
    global AWGBPhaseEntry, AWGBPhasevalue

    try:
        AWGBPhasevalue = float(eval(AWGBPhaseEntry.get()))
    except:
        AWGBPhaseEntry.delete(0,"end")
        AWGBPhaseEntry.insert(0, AWGBPhasevalue)

    if AWGBPhasevalue > 360: # max phase is 360 degrees
        AWGBPhasevalue = 360
    if AWGBPhasevalue < 0: # min phase is 0 degrees
        AWGBPhasevalue = 0
    
def BAWGBDutyCycle(temp):
    global AWGBDutyCycleEntry, AWGBDutyCyclevalue

    try:
        AWGBDutyCyclevalue = float(eval(AWGBDutyCycleEntry.get()))/100
    except:
        AWGBDutyCycleEntry.delete(0,"end")
        AWGBDutyCycleEntry.insert(0, AWGBDutyCyclevalue)

    if AWGBDutyCyclevalue > 1: # max duty cycle is 100%
        AWGBDutyCyclevalue = 1
    if AWGBDutyCyclevalue < 0: # min duty cycle is 0%
        AWGBDutyCyclevalue = 0
    # UpdateAWGB()
    
def BAWGBShape():
    global AWGBShape, AWGBWave
    
    if AWGBShape.get() == 0:
        AWGBWave = 'dc'
    if AWGBShape.get() == 1:
        AWGBWave = 'sine'
    if AWGBShape.get() == 2:
        AWGBWave = 'triangle'
    if AWGBShape.get() == 3:
        AWGBWave = 'sawtooth'
    if AWGBShape.get() == 4:
        AWGBWave = 'square'
    if AWGBShape.get() == 5:
        AWGBWave = 'stairstep'
    if AWGBShape.get() == 6:
        AWGBWave = 'arbitrary'        
    # UpdateAWGB()
    
def UpdateAWGB():
    global AWGBAmplvalue, AWGBOffsetvalue
    global AWGBFreqvalue, AWGBPhasevalue
    global AWGBDutyCyclevalue
    global AWGBWave, AWGBMode, AWGBTerm
    global CHB, SAMPLErate, DevID, devx

    temp = 0
    BAWGBAmpl(temp)
    BAWGBOffset(temp)
    BAWGBFreq(temp)
    BAWGBPhase(temp)
    BAWGBDutyCycle(temp)
    BAWGBShape()
    
    if AWGBFreqvalue > 0.0:
        AWGBperiodvalue = SAMPLErate/AWGBFreqvalue
    else:
        AWGBperiodvalue = 0.0
    
    if AWGBPhasevalue > 0:
        AWGBdelayvalue = AWGBperiodvalue * AWGBPhasevalue / 360
    else:
        AWGBdelayvalue = 0
        
    if AWGBTerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
    elif AWGBTerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 38, 0, 0, 0, 100) # set GND switch to closed
    elif AWGBTerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 37, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
        
    if AWGBWave == 'dc':
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            CHB.constant(AWGBOffsetvalue)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            CHB.constant(AWGBOffsetvalue/1000)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            CHB.constant(0.0) #
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            CHB.constant(AWGBOffsetvalue/1000)
    else:
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        if AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            CHB.constant(0.0)
        else:
            if AWGBWave == 'sine':
                CHB.sine(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'triangle':
                CHB.triangle(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'sawtooth':
                CHB.sawtooth(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'square':
                CHB.square(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue, AWGBDutyCyclevalue)
            elif AWGBWave == 'stairstep':
                CHB.stairstep(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'arbitrary':
                CHB.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGEnab():
    global AWGAMode, AWGBMode, AWGSync
    global CHA, CHB

    if  AWGAMode.get() == 2:
        CHA.set_mode('d')
        CHA.constant(0.0) # 
    else:
        UpdateAWGA()

    if  AWGBMode.get() == 2:
        CHB.set_mode('d')
        CHB.constant(0.0) # 
    else:
        UpdateAWGB()
              
#    if AWGSync.get() > 0:
#    else:
#        if AWG1Enab.get() > 0:     
#        else:     
#        if AWG2Enab.get() > 0:         
#        else:
            
def BAWGSync():
    global AWGSync

#    if AWGSync.get() > 0:     
#    else:

# ========================= Main routine ====================================
    
def Analog_In():   # Read the analog data and store the data into the arrays
    global ADsignal1 # , ADsignal2
    global AWGSync, AWGAMode, AWGBMode, TMsb, HoldOff, HoldOffentry
    global CHA, CHB, Both, devx
    global TRACES, TRACESread
    global RUNstatus, SingleShot
    global TIMEdiv1x, TIMEdiv
    global SAMPLErate, SHOWsamples, MinSamples, MaxSamples
    global LONGsweep, LONGchunk
    global UPDATEspeed, TRACErefresh
    global SCREENrefresh, DCrefresh
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2
    global SV1, SI1, SV2, SI2
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAOffsetEntry, CHAIOffsetEntry, CHBOffsetEntry, CHBIOffsetEntry
 
    # get time scale
    TIMEdiv = eval(TMsb.get())
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    #
    while (True):       # Main loop
        # RUNstatus = 1 : Open Acquisition
        # RUNstatus = 2
        if (RUNstatus == 1) or (RUNstatus == 2):
        # Do input probe Calibration CH1VGain, CH2VGain, CH1VOffset, CH2VOffset
            try:
                InOffA = float(eval(CHAVOffsetEntry.get()))
            except:
                CHAVOffsetEntry.delete(0,END)
                CHAVOffsetEntry.insert(0, InOffA)
            try:
                InGainA = float(eval(CHAVGainEntry.get()))
            except:
                CHAVGainEntry.delete(0,END)
                CHAVGainEntry.insert(0, InGainA)
            try:
                InOffB = float(eval(CHBVOffsetEntry.get()))
            except:
                CHBVOffsetEntry.delete(0,END)
                CHBVOffsetEntry.insert(0, InOffB)
            try:
                InGainB = float(eval(CHBVGainEntry.get()))
            except:
                CHBVGainEntry.delete(0,END)
                CHBVGainEntry.insert(0, InGainB)
            try:
                HoldOff = float(eval(HoldOffentry.get()))
            except:
                HoldOffentry.delete(0,END)
                HoldOffentry.insert(0, HoldOff)
#
            hldn = int(HoldOff * 100 )
            SHOWsamples = int(SAMPLErate * 20.0 * TIMEdiv / 1000.0) + hldn # number of samples to acquire, 2 screen widths
            if SHOWsamples > MaxSamples: # or a Max of 50,000 samples
                SHOWsamples = MaxSamples
            if SHOWsamples < MinSamples: # or a Min of 1000 samples
                SHOWsamples = MinSamples
        # Starting acquisition This is a HACK to get around non-continous AWG mode!
            # restat AWGs if indicated
            if AWGSync.get() > 0:
                BAWGEnab()
            # 
            time.sleep(0.01)
            # GetDigIn() # Update digital inputs display after AWGs have started
            ADsignal1 = Both.get_samples(SHOWsamples) # get samples for both channel A and B
            # waite to finish then return to open termination
            devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set CHA 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set CHA GND switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
            if AWGAMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            if AWGBMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            DCV1 = 0.0 # initalize measurment variables
            DCI1 = 0.0
            DCV2 = 0.0
            DCI2 = 0.0
            SV1 = 0.0
            SV2 = 0.0
            SI1 = 0.0
            SI2 = 0.0
            MinV1 = 50.0
            MaxV1 = -50.0
            MinV2 = 50.0
            MaxV2 = -50.0
            MinI1 = 200.0
            MaxI1 = -200.0
            MinI2 = 200.0
            MaxI2 = -200.0
            for index in range(SHOWsamples): # calculate measurements
                if index > hldn:
                    SPA = ADsignal1[0][index]
                    SPB = ADsignal1[1][index]
                    VAdata = float(SPA[0])
                    IAdata = float(SPA[1])
                    VBdata = float(SPB[0])
                    IBdata = float(SPB[1])
                    DCV1 += VAdata # Sum for average voltage 
                    DCI1 += IAdata # Sum for average current
                    VA = (VAdata - InOffA) * InGainA
                    VA2 = VA * VA
                    SV1 += VA2 # Sum for RMS voltage 
                    IA2 = IAdata * IAdata
                    SI1 += IA2 # Sum for RMS current 
                    if VAdata > MaxV1:
                        MaxV1 = VAdata
                    if VAdata < MinV1:
                        MinV1 = VAdata
                    if IAdata > MaxI1:
                        MaxI1 = IAdata
                    if IAdata < MinI1:
                        MinI1 = IAdata
                    DCV2 += VBdata # Sum for average voltage
                    DCI2 += IBdata # Sum for average current
                    VB = (VBdata - InOffB) * InGainB
                    VB2 = VB * VB
                    SV2 += VB2 # Sum for RMS voltage 
                    IB2 = IBdata * IBdata
                    SI2 += IB2 # Sum for RMS current 
                    if VBdata > MaxV2:
                        MaxV2 = VBdata
                    if VBdata < MinV2:
                        MinV2 = VBdata
                    if IBdata > MaxI2:
                        MaxI2 = IBdata
                    if IBdata < MinI2:
                        MinI2 = IBdata
        # DC value = average of the data record
            numsamples = SHOWsamples - hldn
            DCV1 = DCV1 / numsamples
            DCI1 = DCI1 / numsamples
            DCV2 = DCV2 / numsamples
            DCI2 = DCI2 / numsamples
        # RMS value = square root of average of the data record squared
            SV1 = SV1 / numsamples
            SV1 = math.sqrt(SV1)
            SI1 = SI1 / numsamples
            SI1 = math.sqrt(SI1)
            SV2 = SV2 / numsamples
            SV2 = math.sqrt(SV2)
            SI2 = SI2 / numsamples
            SI2 = math.sqrt(SI2)
            #
            DCV1 = (DCV1 - InOffA) * InGainA
            DCV2 = (DCV2 - InOffB) * InGainB
            MinV1 = (MinV1 - InOffA) * InGainA
            MaxV1 = (MaxV1 - InOffA) * InGainA
            MinV2 = (MinV2 - InOffB) * InGainB
            MaxV2 = (MaxV2 - InOffB) * InGainB
            # convert to mA
            DCI1 = DCI1 * 1000
            DCI2 = DCI2 * 1000
            MinI1 = MinI1 * 1000
            MinI2 = MinI2 * 1000
            MaxI1 = MaxI1 * 1000
            MaxI2 = MaxI2 * 1000
            SI1 = SI1 * 1000
            SI2 = SI2 * 1000
            #
            TRACESread = 2
            UpdateAll()         # Update Data, trace and screen
            if SingleShot.get() == 1: # Single shot manual trigger is on
                RUNstatus = 0

        # RUNstatus = 3: Stop
        # RUNstatus = 4: Stop and restart
        if (RUNstatus == 3) or (RUNstatus == 4):
            if RUNstatus == 3:
                RUNstatus = 0
                # pysmu.pause()      # Status is stopped 
            if RUNstatus == 4:          
                RUNstatus = 1
                # pysmu.start(stopOnClose=True)# Status is (re)start

            UpdateScreen()
        # Update tasks and screens by TKinter
        root.update_idletasks()
        root.update()                   # update screens
        #try:
            #subwindow.update()
        #except:
            #donothing()

def FindRisingEdge():
    global ADsignal1, MinV1, MaxV1, MinV2, MaxV2, HoldOff
    global InOffA, InGainA, InOffB, InGainB, CHABphase
    global SHOWsamples, SAMPLErate, CHAperiod, CHAfreq, CHBperiod, CHBfreq
    global CHAHW, CHALW, CHADCy, CHBHW, CHBLW, CHBDCy, ShowC1_V, ShowC2_V
    
# find first rising edge
    Nmax = SHOWsamples - 8
    anr1 = 0
    bnr1 = 0
    anf1 = 0
    bnf1 = 0
    anr2 = 0
    bnr2 = 0
    if ShowC1_V.get() == 1:
        n = int(HoldOff * 100)
        MidV1 = (MaxV1+MinV1)/2
        Mid1V2 = 0.99 * MidV1 # Hysteresis to avoid triggering on noise
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while ( ChInput >= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
#
        anr1 = n
        n = n + 1
# skip over falling edge
        Mid1V2 = 1.01 * MidV1
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput >= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
#
        anf1 = n
        n = n + 1
# find second rising edge
        Mid1V2 = 0.99 * MidV1
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while ( ChInput >= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
            SPA = ADsignal1[0][int(n-1)]
        anr2 = n
        CHAperiod = float(((anr2 - anr1) * 1000.0) / SAMPLErate) # time in mSec
        CHAHW = float(((anf1 - anr1) * 1000.0) / SAMPLErate)
        CHALW = float(((anr2 - anf1) * 1000.0) / SAMPLErate)
        CHADCy = float(anf1 - anr1) / float(anr2 - anr1) * 100.0 # in percent
        CHAfreq = float(SAMPLErate / (anr2 - anr1))
# search channel B
    if ShowC2_V.get() == 1:
        n = int(HoldOff * 100)
        MidV2 = (MaxV2+MinV2)/2
        Mid2V2 = 0.99 * MidV2 # Hysteresis to avoid triggering on noise
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while ( ChInput >= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
#
        bnr1 = n
        n = n + 1
# skip over falling edge
        Mid2V2 = 1.01 * MidV2
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput >= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
#
        bnf1 = n
        n = n + 1
# find second rising edge
        Mid2V2 = 0.99 * MidV2
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while ( ChInput >= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
            SPB = ADsignal1[1][int(n-1)]
        bnr2 = n
        CHBperiod = float(((bnr2 - bnr1) * 1000.0) / SAMPLErate) # time in mSec
        CHBHW = float(((bnf1 - bnr1) * 1000.0) / SAMPLErate)
        CHBLW = float(((bnr2 - bnf1) * 1000.0) / SAMPLErate)
        CHBDCy = float(bnf1 - bnr1) / float(bnr2 - bnr1) * 100.0 # in percent
        CHBfreq = float(SAMPLErate / (bnr2 - bnr1))
        #
    try:
        CHABphase = 360.0*(float((bnr1 - anr1) * 1000.0 / SAMPLErate))/CHAperiod
    except:
        CHABphase = 0.0
    if CHABphase < 0.0:
        CHABphase = CHABphase + 360.0



def UpdateAll():        # Update Data, trace and screen
    
    MakeTrace()         # Update the traces
    # GetDigIn()          # Update digital inputs display
    UpdateScreen()      # Update the screen 

def UpdateTrace():      # Update trace and screen
    
    MakeTrace()         # Update traces
    UpdateScreen()      # Update the screen

def UpdateScreen():     # Update screen with trace and text
    
    MakeScreen()        # Update the screen
    root.update()       # Activate updated screens
    #try:
        #subwindow.update()
    #except:
        #donothing()

# define ad9833 actions
def ShiftOut(DValue,GValue):
    global devx, DevID
    # do AD9833
    binstr = bin(DValue)

    binlen = len(binstr)
    datastr = binstr[2:binlen]
    datalen = len(datastr)

    if datalen < 16:
       datastr = str.rjust(datastr , 16 , '0')
       datalen = len(datastr)
    # do MiniGen
    genstr = bin(GValue)

    genlen = len(genstr)
    genstr = genstr[2:genlen]
    genlen = len(genstr)

    if genlen < 16:
       genstr = str.rjust(genstr , 16 , '0')
       genlen = len(genstr)
    i = 1
    devx.ctrl_transfer(DevID, 0x40, 0x50, 2, 0, 0, 0, 100) # fsync to 0
    # sending 0x50 = set to 0, 0x51 = set to 1
    # PIO 0 is clock
    # PIO 1 is AD9833 data
    # PIO 2 is FSYNC
    # PIO 3 is minigen data
    while i < datalen+1:
        D1code = 0x50 + int(datastr[i-1])
        D2code = 0x50 + int(genstr[i-1])
        # print D1code
        devx.ctrl_transfer(DevID, 0x40, D1code, 1, 0, 0, 0, 100) # ad9833 data bit
        devx.ctrl_transfer(DevID, 0x40, D2code, 3, 0, 0, 0, 100) # minigen data bit
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100) # sclk to 1
        devx.ctrl_transfer(DevID, 0x40, 0x50, 0, 0, 0, 0, 100) # sclk to 0
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100) # sclk to 1
        i = i + 1
    devx.ctrl_transfer(DevID, 0x40, 0x51, 2, 0, 0, 0, 100) # fsync to 1
    #
def BsetAD9833(Temp):
    
    SetAD9833()
    
def SetAD9833():
    global FminEntry, MulXEntry, MiniGenEntry # , Fclk, Fout
    global Two28, mclk, Omode

    DValue = 8192 + 40 # set mode for square wave output
    GValue = 8192 + Omode.get() # set MiniGen mode
    ShiftOut(DValue,GValue)
    #
    try:
        MulX = float(eval(MulXEntry.get()))
        if MulX < 1:
            MulX = 1
    except:
        MulXEntry.delete(0,END)
        MulXEntry.insert(0, MulX)
    try:
        FminE = float(eval(FminEntry.get()))*1000
    except:
        FminEntry.delete(0,END)
        FminEntry.insert(0, FminE/1000)
    if MulX == 1:
        Fmin = FminE
    else:
        Fmin = FminE - (FminE/MulX)
    #
    try:
        Fmingen = float(eval(MiniGenEntry.get()))*1000
    except:
        MiniGenEntry.delete(0,END)
        MiniGenEntry.insert(0, Fmingen/1000)
    # do AD9833
    Freg = int((Fmin*Two28)/mclk)
    Foutstr = bin(Freg)
    Foutlen = len(Foutstr)
    datastr = Foutstr[2:Foutlen]
    datalen = len(datastr)
    if datalen < 28:
       datastr = str.rjust(datastr , 28 , '0')
       datalen = len(datastr)
    Fmsb = '0b01' + datastr[0:14]
    Flsb = '0b01' + datastr[14:]
    # do MiniGen
    Fgen = int((Fmingen*Two28)/16000000)
    Fgenstr = bin(Fgen)
    Fgenlen = len(Fgenstr)
    Genstr = Fgenstr[2:Fgenlen]
    Genlen = len(Genstr)
    if Genlen < 28:
       Genstr = str.rjust(Genstr , 28 , '0')
       Genlen = len(Genstr)
    Gmsb = '0b01' + Genstr[0:14]
    Glsb = '0b01' + Genstr[14:]

    FValue = int(eval(Flsb))
    GValue = int(eval(Glsb))
    ShiftOut(FValue,GValue)
    FValue = int(eval(Fmsb))
    GValue = int(eval(Gmsb))
    ShiftOut(FValue,GValue)

def MakeTrace():    # Make the traces
    global ADsignal1 # , ADsignal2
    global T1Vline, T2Vline, T1Iline, T2Iline
    global TXYline, Tmathline
    global Triggerline, TgInput, TgEdge, HoldOff, HoldOffentry
    global X0L, Y0T
    global GRW, GRH
    global Ymin, Ymax
    global SHOWsamples, ZOHold, AWGBMode
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    global TRACES
    global TRACESread
    global RUNstatus
    global XYMode, DisMode
    global Xsignal, Ysignal
    global CHAsb, CHBsb
    global CHAOffset, CHBOffset
    global CHAIsb, CHBIsb
    global CHAIOffset, CHBIOffset
    global TMpdiv       # Array with time / div values in ms
    global TMsb         # Time per div spin box variable
    global TIMEdiv      # current spin box value
    global SAMPLErate
    global SCstart
    global TRIGGERsample
    global TRIGGERlevel, TRIGGERentry, AutoLevel
    global InOffA, InGainA, InOffB, InGainB
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAOffsetEntry, CHAIOffsetEntry, CHAOffsetEntry, CHBIOffsetEntry
 
    # Set the TRACEsize variable
    TRACEsize = SHOWsamples               # Set the trace length
    # get time scale
    try:
        TIMEdiv = float(eval(TMsb.get()))
    except:
        TIMEdiv = 0.1
    # prevent divide by zero error
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    # get the vertical ranges
    try:
        CH1pdvRange = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvRange)
    try:
        CH2pdvRange = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvRange)
    try:
        CH1IpdvRange = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1IpdvRange)
    try:
        CH2IpdvRange = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2IpdvRange)
    # get the vertical offsets
    try:
        CHAOffset = float(eval(CHAOffsetEntry.get()))
    except:
        CHAOffsetEntry.delete(0,END)
        CHAOffsetEntry.insert(0, CHAOffset)
    try:
        CHAIOffset = float(eval(CHAIOffsetEntry.get()))
    except:
        CHAIOffsetEntry.delete(0,END)
        CHAIOffsetEntry.insert(0, CHAIOffset)
    try:
        CHBOffset = float(eval(CHBOffsetEntry.get()))
    except:
        CHBOffsetEntry.delete(0,END)
        CHBOffsetEntry.insert(0, CHBOffset)
    try:
        CHBIOffset = float(eval(CHBIOffsetEntry.get()))
    except:
        CHBIOffsetEntry.delete(0,END)
        CHBIOffsetEntry.insert(0, CHBIOffset)
    # prevent divide by zero error
    if CH1pdvRange < 0.001:
        CH1pdvRange = 0.001
    if CH2pdvRange < 0.001:
        CH2pdvRange = 0.001
    if CH1IpdvRange < 1.0:
        CH1IpdvRange = 1.0
    if CH2IpdvRange < 1.0:
        CH2IpdvRange = 1.0
        
    # Continue with finding the trigger sample and drawing the traces 
    if TRACEsize == 0:                          # If no trace, skip rest of this routine
        T1Vline = []                            # Trace line channel A V
        T2Vline = []                            # Trace line channel B V
        T1Iline = []
        T2Iline = []
        TXYline = []
        Tmathline = []
        return() 

    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
# Find trigger sample
    try:
        if AutoLevel.get() == 1:
            if TgInput.get() == 1:
                TRIGGERlevel = (MinV1 + MaxV1)/2
            if TgInput.get() == 2:
                TRIGGERlevel = (MinI1 + MaxI1)/2
            if TgInput.get() == 3:
                TRIGGERlevel = (MinV2 + MaxV2)/2
            if TgInput.get() == 4:
                TRIGGERlevel = (MinI2 + MaxI2)/2
            TRIGGERentry.delete(0,"end")
            TRIGGERentry.insert(0,TRIGGERlevel)
        else:
            TRIGGERlevel = eval(TRIGGERentry.get())
    except:
        TRIGGERentry.delete(0,END)
        TRIGGERentry.insert(0, TRIGGERlevel)
#
    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
    # Start from first sample after HoldOff
    hldn = int(HoldOff * 100)
    TRIGGERsample = hldn # + (TriggerPos * SAMPLErate)
#
    if LONGchunk == False:          # For long sweeps no trigger        
        Nmax = int(TRACEsize / 1.5) # first 2/3 of data set
        DX = 0
        n = TRIGGERsample
        if (TgInput.get() == 1 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinV1:
                TRIGGERlevel2 = MinV1
            if TRIGGERlevel2 > MaxV1:
                TRIGGERlevel2 = MaxV1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
            Prev = ChInput
            while ( ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                Prev = ChInput
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
            DY = ChInput - Prev
            try:
                DX = (TRIGGERlevel - Prev)/DY # calculate interpolated trigger point
            except:
                DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinV1:
                    TRIGGERlevel2 = MinV1
                if TRIGGERlevel2 > MaxV1:
                    TRIGGERlevel2 = MaxV1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
                Prev = ChInput
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = (float(SPA[0]) - InOffA) * InGainA
                while (ChInput >= TRIGGERlevel) and n < Nmax:
                    Prev = ChInput
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = (float(SPA[0]) - InOffA) * InGainA
                DY = Prev - ChInput
                try:
                    DX = (Prev - TRIGGERlevel)/DY # calculate interpolated trigger point
                except:
                    DX = 0
        elif (TgInput.get() == 2 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinI1:
                TRIGGERlevel2 = MinI1
            if TRIGGERlevel2 > MaxI1:
                TRIGGERlevel2 = MaxI1
            SPA = ADsignal1[0][int(n)]
            ChInput = float(SPA[1])*1000
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
            DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel1 = 1 * TRIGGERlevel
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinI1:
                    TRIGGERlevel2 = MinI1
                if TRIGGERlevel2 > MaxI1:
                    TRIGGERlevel2 = MaxI1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = float(SPA[1])*1000
                while (ChInput >= TRIGGERlevel1) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = float(SPA[1])*1000
                DX = 0
        elif (TgInput.get() == 3 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinV2:
                TRIGGERlevel2 = MinV2
            if TRIGGERlevel2 > MaxV2:
                TRIGGERlevel2 = MaxV2
            SPB = ADsignal1[1][int(n)]
            ChInput = ( float(SPB[0]) - InOffB) * InGainB
            Prev = ChInput
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                Prev = ChInput
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
            DY = ChInput - Prev
            try:
                DX = (TRIGGERlevel - Prev)/DY # calculate interpolated trigger point
            except:
                DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinV2:
                    TRIGGERlevel2 = MinV2
                if TRIGGERlevel2 > MaxV2:
                    TRIGGERlevel2 = MaxV2
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
                Prev = ChInput
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = ( float(SPB[0]) - InOffB) * InGainB
                while (ChInput >= TRIGGERlevel) and n < Nmax:
                    Prev = ChInput
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = ( float(SPB[0]) - InOffB) * InGainB
                DY = Prev - ChInput
                try:
                    DX = (Prev - TRIGGERlevel)/DY # calculate interpolated trigger point
                except:
                    DX = 0    
        elif (TgInput.get() == 4 ):
            TRIGGERlevel1 = 1 * TRIGGERlevel
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinI2:
                TRIGGERlevel2 = MinI2
            if TRIGGERlevel2 > MaxI2:
                TRIGGERlevel2 = MaxI2
            SPB = ADsignal1[1][int(n)]
            ChInput = float(SPB[1])*1000
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
            while (ChInput <= TRIGGERlevel1) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
            DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel1 = 1 * TRIGGERlevel
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinI2:
                    TRIGGERlevel2 = MinI2
                if TRIGGERlevel2 > MaxI2:
                    TRIGGERlevel2 = MaxI2
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = float(SPB[1])*1000
                while (ChInput >= TRIGGERlevel1) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = float(SPB[1])*1000
                DX = 0
# check to insure trigger point is in bounds                
        if n < Nmax:
            TRIGGERsample = n - 1
        elif n > Nmax: # Didn't find edge in first 2/3 of data set
            TRIGGERsample = 1 + hldn # (TriggerPos * SAMPLErate) # reset to begining
        if DX > 1:
            DX = 1 # never more than 100% of a sample period
    
    # set and/or corrected for in range
    SCmin = int(-1 * TRIGGERsample)
    SCmax = int(TRACEsize - TRIGGERsample - 20)
    
    if SCstart < SCmin:             # No reading before start of array
        SCstart = SCmin
    if SCstart  > SCmax:            # No reading after end of array
        SCstart = SCmax
    
    # Make Trace lines etc.

    Yconv1 = float(GRH/10) / CH1pdvRange    # Conversion factors from samples to screen points
    Yconv2 = float(GRH/10) / CH2pdvRange
    YIconv1 = float(GRH/10) / CH1IpdvRange
    YIconv2 = float(GRH/10) / CH2IpdvRange

    c1 = GRH / 2 + Y0T    # fixed correction channel A
    c2 = GRH / 2 + Y0T    # fixed correction channel B
 
    DISsamples = SAMPLErate * 10.0 * TIMEdiv / 1000 # number of samples to display
    T1Vline = []                    # V Trace line channel A
    T2Vline = []                    # V Trace line channel B
    T1Iline = []                    # I Trace line channel A
    T2Iline = []                    # I Trace line channel B
    TXYline = []                    # XY Trace line
    Tmathline = []                  # math trce line
    t = SCstart + TRIGGERsample # - (TriggerPos * SAMPLErate) # t = Start sample in trace
    if t < 0:
        t = 0
    x = 0                           # Horizontal screen pixel

    if DisMode.get() == 0: # 0 for time plots
        if (DISsamples <= GRW):
            Xstep = GRW / DISsamples
            if AWGBMode.get() == 2:
                xa = int(Xstep/-2) - int(Xstep*DX)
            else:
                xa = 0 - int(Xstep*DX)       # adjust start pixel for interpolated trigger point
            x = 0 - int(Xstep*DX)
            Tstep = 1
            x1 = 0                      # x position of trace line
            xa1 = 0
            y1 = 0.0                    # y position of trace line
            SPA = ADsignal1[0][t]
            CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
            ypv1 = int(c1 - Yconv1 * (CH1Vcal - CHAOffset))
            ytemp = (float(SPA[1]) * 1000)
            ypi1 = int(c1 - YIconv1 * (ytemp - CHAIOffset))
            SPB = ADsignal1[1][int(t)]
            CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
            ypv2 = int(c2 - Yconv2 * (CH2Vcal - CHBOffset))
            ytemp = (float(SPB[1]) * 1000)
            ypi2 = int(c1 - YIconv2 * (ytemp - CHBIOffset))
            ypm = GRH / 2 + Y0T
            if TgInput.get() == 0:
                Xlimit = GRW
            else:
                Xlimit = GRW+Xstep
            while x <= Xlimit:
                if t < TRACEsize:
                    xa1 = xa + X0L
                    x1 = x + X0L
                    SPA = ADsignal1[0][int(t)]
                    CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                    y1 = int(c1 - Yconv1 * (CH1Vcal - CHAOffset))
                    ytemp = (float(SPA[1]) * 1000)
                    yi1 = int(c1 - YIconv1 * (ytemp - CHAIOffset))
                    
                    if y1 < Ymin: # clip waveform if going off grid
                        y1 = Ymin
                    if y1 > Ymax:
                        y1 = Ymax
                    if yi1 < Ymin:
                        yi1 = Ymin
                    if yi1 > Ymax:
                        yi1 = Ymax
                    if ShowC1_V.get() == 1 :
                        if ZOHold.get() == 1:
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(ypv1))
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(y1))
                        else:    
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(y1))
                    if ShowC1_I.get() == 1:
                        if ZOHold.get() == 1:
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(ypi1))
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(yi1))
                        else:
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(yi1))

                    ypv1 = y1
                    ypi1 = yi1
                    SPB = ADsignal1[1][int(t)]
                    CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                    y1 = int(c2 - Yconv2 * (CH2Vcal - CHBOffset))
                    ytemp = (float(SPB[1]) * 1000)
                    yi1 = int(c1 - YIconv2 * (ytemp - CHBIOffset))

                    if y1 < Ymin: # clip waveform if going off grid
                        y1 = Ymin
                    if y1 > Ymax:
                        y1 = Ymax
                    if yi1 < Ymin:
                        yi1 = Ymin
                    if yi1 > Ymax:
                        yi1 = Ymax
                    if ShowC2_V.get() == 1:
                        if ZOHold.get() == 1:
                            T2Vline.append(int(x1))
                            T2Vline.append(int(ypv2))
                            T2Vline.append(int(x1))
                            T2Vline.append(int(y1))
                        else:
                            T2Vline.append(int(x1))
                            T2Vline.append(int(y1))
                    if ShowC2_I.get() == 1:
                        if (ZOHold.get() == 1):
                            T2Iline.append(int(x1))
                            T2Iline.append(int(ypi2))
                            T2Iline.append(int(x1))
                            T2Iline.append(int(yi1))
                        else:
                            T2Iline.append(int(x1))
                            T2Iline.append(int(yi1))
                    ypv2 = y1
                    ypi2 = yi1
                    if MathTrace.get() == 1: # plot sum of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal + CH2Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 2: # plot difference of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal - CH2Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 3: # plot difference of CB-V and CA-V 
                        y1 = int(c1 - Yconv1 * (CH2Vcal - CH1Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 4: # plot product of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Ypower = CH1Vcal * (float(SPA[1]) * 1000) # mAmps * Volts = mWatts
                        ytemp = YIconv1 * (Ypower - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if yi1 < Ymin: # clip waveform if going off grid
                            yi1 = Ymin
                        if yi1 > Ymax:
                            yi1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 5): # plot product of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Ypower = CH2Vcal * (float(SPB[1]) * 1000) # mAmps * Volts = mWatts
                        ytemp = YIconv2 * (Ypower - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 6): # plot ratio of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Yohms = CH1Vcal / (float(SPA[1])) #  Volts / Amps = ohms
                        ytemp = YIconv1 * (Yohms - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 7): # plot ratio of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Yohms = CH2Vcal / (float(SPB[1])) #  Volts / Amps = ohms
                        ytemp = YIconv2 * (Yohms - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 8): # plot difference of CA-I and CB-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = (float(SPA[1]) - float(SPB[1])) * 1000 #  convert to mA
                        ytemp = YIconv1 * (Ydif - CHAIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 9): # plot difference of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = (float(SPB[1]) - float(SPA[1])) * 1000 #  convert to mA
                        ytemp = YIconv2 * (Ydif - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 10): # plot ratio of CB-V and CA-V
                        y1 = int(c1 - Yconv2 * ((CH2Vcal / CH1Vcal) - CHBOffset)) #  voltage gain A to B
                        if (y1 < Ymin): # clip waveform if going off grid
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 11): # plot ratio of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Y1 = float(SPB[1]) / float(SPA[1]) # current gain A to B
                        ytemp = YIconv2 * (Y1 - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    
                t = t + Tstep
                x = x + Xstep
                xa = xa + Xstep
                
        if (DISsamples > GRW): # if the number of samples is larger than the grid width need to ship over samples
            Xstep = 1
            Tstep = DISsamples / GRW       # number of samples to skip per grid pixel
            x1 = 0                          # x position of trace line
            ylo = 0.0                       # ymin position of trace 1 line
            yhi = 0.0                       # ymax position of trace 1 line
            
            t = SCstart + TRIGGERsample # - (TriggerPos * SAMPLErate) # t = Start sample in trace
            if t < 0:
                t = 0
            x = 0                           # Horizontal screen pixel
    
            while (x <= GRW):
                if (t < TRACEsize):
                    x1 = x + X0L
                    SPA = ADsignal1[0][int(t)]
                    CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                    ylo = CH1Vcal - CHAOffset
                    ilo = ((float(SPA[1]) * 1000) - CHAIOffset)
                    yhi = ylo
                    ihi = ilo
                    n = t
                    while n < (t + Tstep) and n < TRACEsize:
                        SPA = ADsignal1[0][int(t)]
                        CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                        v = CH1Vcal - CHAOffset
                        i = ((float(SPA[1]) * 1000) - CHAIOffset)
                        if v < ylo:
                            ylo = v
                        if v > yhi:
                            yhi = v
                        if i < ilo:
                            ilo = i
                        if i > ihi:
                            ihi = i

                        n = n + 1   
                
                    ylo = int(c1 - Yconv1 * ylo)
                    yhi = int(c1 - Yconv1 * yhi)
                    ilo = int(c1 - YIconv1 * ilo)
                    ihi = int(c1 - YIconv1 * ihi)
                    if (ylo < Ymin):
                        ylo = Ymin
                    if (ylo > Ymax):
                        ylo = Ymax

                    if (yhi < Ymin):
                        yhi = Ymin
                    if (yhi > Ymax):
                        yhi = Ymax

                    if (ilo < Ymin):
                        ilo = Ymin
                    if (ilo > Ymax):
                        ilo = Ymax

                    if (ihi < Ymin):
                        ihi = Ymin
                    if (ihi > Ymax):
                        ihi = Ymax
                    
                    if ( ShowC1_V.get() == 1 ):
                        T1Vline.append(int(x1))
                        T1Vline.append(int(ylo))        
                        T1Vline.append(int(x1))
                        T1Vline.append(int(yhi))
                    if ( ShowC1_I.get() == 1 ):
                        T1Iline.append(int(x1))
                        T1Iline.append(int(ilo))        
                        T1Iline.append(int(x1))
                        T1Iline.append(int(ihi)) 
                    SPB = ADsignal1[1][int(t)]
                    CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                    ylo = CH2Vcal - CHBOffset
                    ilo = ((float(SPB[1]) * 1000) - CHBIOffset)
                    yhi = ylo
                    ihi = ilo
                    n = t
                    while n < (t + Tstep) and n < TRACEsize:
                        SPB = ADsignal1[1][int(t)]
                        CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                        v = CH2Vcal - CHBOffset
                        i = ((float(SPB[1]) * 1000) - CHBIOffset)
                        if v < ylo:
                            ylo = v
                        if v > yhi:
                            yhi = v
                        if i < ilo:
                            ilo = i
                        if i > ihi:
                            ihi = i

                        n = n + 1   
                
                    ylo = int(c2 - Yconv2 * ylo)
                    yhi = int(c2 - Yconv2 * yhi)
                    ilo = int(c2 - YIconv2 * ilo)
                    ihi = int(c2 - YIconv2 * ihi)
                    if (ylo < Ymin):
                        ylo = Ymin
                    if (ylo > Ymax):
                        ylo = Ymax

                    if (yhi < Ymin):
                         yhi = Ymin
                    if (yhi > Ymax):
                        yhi = Ymax

                    if (ilo < Ymin):
                        ilo = Ymin
                    if (ilo > Ymax):
                        ilo = Ymax

                    if (ihi < Ymin):
                        ihi = Ymin
                    if (ihi > Ymax):
                        ihi = Ymax

                    if ( ShowC2_V.get() == 1 ):
                        T2Vline.append(int(x1))
                        T2Vline.append(int(ylo))        
                        T2Vline.append(int(x1))
                        T2Vline.append(int(yhi))
                    if ( ShowC2_I.get() == 1 ):
                        T2Iline.append(int(x1))
                        T2Iline.append(int(ilo))        
                        T2Iline.append(int(x1))
                        T2Iline.append(int(ihi))
                    if ( MathTrace.get() == 1): # plot sum of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal + CH2Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 2): # plot difference of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal - CH2Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 3): # plot difference of CB-V and CA-V 
                        y1 = int(c1 - Yconv1 * (CH2Vcal - CH1Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 4): # plot product of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Ypower = CH1Vcal * (float(SPA[1]) * 1000) # mAmps * Volts = mWatts
                        ytemp = YIconv1 * (Ypower - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 5): # plot product of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Ypower = CH2Vcal * (float(SPB[1]) * 1000) # mAmps * Volts = mWatts
                        ytemp = YIconv2 * (Ypower - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 6): # plot ratio of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Yohms = CH1Vcal / (float(SPA[1])) #  Volts / Amps = ohms
                        ytemp = YIconv1 * (Yohms- CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 7): # plot ratio of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Yohms = CH2Vcal / (float(SPB[1])) #  Volts / Amps = ohms
                        ytemp = YIconv2 * (Yohms - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 8): # plot difference of CA-I and CB-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = (float(SPA[1]) - float(SPB[1])) * 1000 #  convert to mA
                        ytemp = YIconv1 * (Ydif - CHAIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 9): # plot difference of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = (float(SPB[1]) - float(SPA[1])) * 1000 #  convert to mA
                        ytemp = YIconv2 * (Ydif - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 10): # plot ratio of CB-V and CA-V
                        y1 = int(c1 - Yconv2 * ((CH2Vcal / CH1Vcal) - CHBOffset)) #  voltage gain A to B
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 11): # plot ratio of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Y1 = float(SPB[1]) / float(SPA[1]) # current gain A to B
                        ytemp = YIconv2 * (Y1 - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                        
                t = t + Tstep
                x = x + Xstep
    else:   # draw an X/Y plot 
        t = TRIGGERsample # skip over sampled before hold off time and trigger point
        c2 = GRW / 2 + X0L   # Hor correction factor
        
        if (Xsignal.get() == 3 and Ysignal.get() == 1): # mode CAV/CBV
            Xconv1 = float(GRW/10) / CH2pdvRange
            Yconv1 = float(GRH/10) / CH1pdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                ylo = CH1Vcal - CHAOffset
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 1 and Ysignal.get() == 3): # mode CBV/CAV
            Xconv1 = float(GRW/10) / CH1pdvRange
            Yconv1 = float(GRH/10) / CH2pdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainA
                ylo = CH2Vcal - CHBOffset
                SPA = ADsignal1[0][int(t)]
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainB
                xlo = CH1Vcal - CHAOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 1 and Ysignal.get() == 2): # mode CAI/CAV
            Xconv1 = float(GRW/10) / CH1pdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                ylo = ((float(SPA[1]) * 1000) - CHAIOffset)
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                xlo = CH1Vcal - CHAOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 3 and Ysignal.get() == 2): # mode CAI/CBV
            Xconv1 = float(GRW/10) / CH2pdvRange
            YIconv1 = float(GRH/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                ylo = ((float(SPA[1]) * 1000) - CHAIOffset)
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 1): # mode CAV/CAI
            YIconv1 = float(GRW/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                xlo = ((float(SPA[1]) * 1000) - CHAIOffset)
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                ylo = CH1Vcal - CHAOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + YIconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 3): # mode CBV/CAI
            YIconv1 = float(GRW/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                xlo = ((float(SPA[1]) * 1000) - CHAIOffset)
                SPB = ADsignal1[1][int(t)]
                CH1Vcal = (float(SPB[0]) - InOffB) * InGainB
                ylo = CH1Vcal - CHBOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + YIconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 3 and Ysignal.get() == 4): # mode CBI/CBV
            Yconv2 = float(GRW/10) / CH2pdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                ylo = ((float(SPB[1]) * 1000) - CHBIOffset)
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - YIconv2 * ylo)
                xlo = int(c2 + Yconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 4 and Ysignal.get() == 3): # mode CBV/CBI
            YIconv2 = float(GRW/10) / CH2IpdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                xlo = ((float(SPB[1]) * 1000) - CHBIOffset)
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                ylo = CH2Vcal - CHBOffset
                ylo = int(c1 - Yconv2 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 4 and Ysignal.get() == 2): # mode CAI/CBI
            YIconv2 = float(GRW/10) / CH2IpdvRange
            YIconv1 = float(GRH/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                SPB = ADsignal1[1][int(t)]
                xlo = ((float(SPB[1]) * 1000) - CHBIOffset)
                ylo = ((float(SPA[1]) * 1000) - CHAIOffset)
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 4): # mode CBI/CAI
            YIconv2 = float(GRW/10) / CH1IpdvRange
            YIconv1 = float(GRH/10) / CH2IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                SPB = ADsignal1[1][int(t)]
                xlo = ((float(SPA[1]) * 1000) - CHAIOffset)
                ylo = ((float(SPB[1]) * 1000) - CHBIOffset)
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
            
    # Make trigger triangle pointer
    Triggerline = []                # Triggerline
    if ( TgInput.get() == 0 ):
        x1 = X0L
        y1 = Ymin
    elif ( TgInput.get() == 1 ): # triggering on CA-V
        x1 = X0L
        ytemp = Yconv1 * (float(TRIGGERlevel)-CHAOffset) # / InGainA
        y1 = int(c1 - ytemp)
    elif ( TgInput.get() == 2 ):  # triggering on CA-I
        x1 = X0L+GRW
        y1 = int(c1 - YIconv1 * (float(TRIGGERlevel) - CHAIOffset))
    elif ( TgInput.get() == 3 ):  # triggering on CB-V
        x1 = X0L
        ytemp = Yconv2 * (float(TRIGGERlevel)-CHBOffset) # / InGainB         
        y1 = int(c2 - ytemp)
    elif ( TgInput.get() == 4 ): # triggering on CB-I
        x1 = X0L+GRW
        y1 = int(c2 - YIconv2 * (float(TRIGGERlevel) - CHBIOffset))
        
    if (y1 < Ymin):
        y1 = Ymin
    if (y1 > Ymax):
        y1 = Ymax
    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1+5))
    
    Triggerline.append(int(x1+5))
    Triggerline.append(int(y1))
    
    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1-5))        

    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1+5))

def MakeScreen():     # Update the screen with traces and text
    global ADsignal1 #, ADsignal2
    global T1Vline, T2Vline, T1Iline, T2Iline, TXYline # active trave lines
    global T1VRline, T2VRline, T1IRline, T2IRline # reference trace lines
    global Triggerline, Tmathline, TMRline, TXYRline
    global X0L          # Left top X value
    global Y0T          # Left top Y value
    global GRW          # Screenwidth
    global GRH          # Screenheight
    global Ymin, Ymax
    global SHOWsamples  # Number of samples in data record
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, ShowRXY
    global ShowRA_V, ShowRA_I, ShowRB_V, ShowRB_I, ShowMath
    global Xsignal, Ysignal, MathTrace
    global RUNstatus, SingleShot    # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
    global CHAsb        # spinbox Index for channel 1 V
    global CHBsb        # spinbox Index for channel 2 V
    global CHAOffset    # Offset value for channel 1 V
    global CHBOffset    # Offset value for channel 2 V
    global CHAIsb       # spinbox Index for channel 1 I
    global CHBIsb       # spinbox Index for channel 2 I
    global CHAIOffset   # Offset value for channel 1 I
    global CHBIOffset   # Offset value for channel 2 I
    global XYMode     
    global DisMode      # 0 = Time axis 1 = X Y mode
    global TMpdiv       # Array with time / div values in ms
    global TMsb         # Time per div spin box variable
    global TIMEdiv      # current spin box value
    global SAMPLErate, MulXEntry
    global TRIGGERsample, TRIGGERlevel, HoldOff, HoldOffentry
    global LONGchunk    # If longchunk is used for longsweep
    global COLORgrid    # The colors
    global COLORzeroline
    global COLORtrace1, COLORtrace2, COLORtrace3, COLORtrace4, COLORtrace5
    global COLORtraceR1, COLORtraceR2, COLORtraceR3, COLORtraceR4, COLORtraceR5
    global COLORtext
    global COLORtrigger
    global CANVASwidth, CANVASheight
    global TRACErefresh
    global SCREENrefresh, SmoothCurves
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2, CHAHW, CHALW, CHADCy, CHAperiod, CHAfreq
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2, CHBHW, CHBLW, CHBDCy, CHBperiod, CHBfreq
    global SV1, SI1, SV2, SI2, CHABphase
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1
    global MeasDCI1, MeasMinI1, MeasMaxI1, MeasMidI1, MeasPPI1
    global MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2, MeasPPV2
    global MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSI1, MeasRMSV2, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
    global AWGAShape, AWGBShape
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAOffsetEntry, CHAIOffsetEntry, CHAOffsetEntry, CHBIOffsetEntry
    global DevID
    # m1k dev info
    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
    # Delete all items on the screen
    de = ca.find_enclosed ( -1000, -1000, CANVASwidth+1000, CANVASheight+1000)    
    for n in de: 
        ca.delete(n)
    # get time scale
    TIMEdiv = eval(TMsb.get())
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    # get the vertical ranges
    try:
        CH1pdvRange = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvRange)
    try:
        CH2pdvRange = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvRange)
    try:
        CH1IpdvRange = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1IpdvRange)
    try:
        CH2IpdvRange = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2IpdvRange)
    # get the vertical offsets
    try:
        CHAOffset = float(eval(CHAOffsetEntry.get()))
    except:
        CHAOffsetEntry.delete(0,END)
        CHAOffsetEntry.insert(0, CHAOffset)
    try:
        CHAIOffset = float(eval(CHAIOffsetEntry.get()))
    except:
        CHAIOffsetEntry.delete(0,END)
        CHAIOffsetEntry.insert(0, CHAIOffset)
    try:
        CHBOffset = float(eval(CHBOffsetEntry.get()))
    except:
        CHBOffsetEntry.delete(0,END)
        CHBOffsetEntry.insert(0, CHBOffset)
    try:
        CHBIOffset = float(eval(CHBIOffsetEntry.get()))
    except:
        CHBIOffsetEntry.delete(0,END)
        CHBIOffsetEntry.insert(0, CHBIOffset)
    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
    try:
        Mulx = float(eval(MulXEntry.get()))
        if Mulx < 1:
            Mulx = 1
    except:
        MulXEntry.delete(0,END)
        MulXEntry.insert(0, 1)
    # prevent divide by zero error
    if CH1pdvRange < 0.01:
        CH1pdvRange = 0.01
    if CH2pdvRange < 0.01:
        CH2pdvRange = 0.01
    if CH1IpdvRange < 1.0:
        CH1IpdvRange = 1.0
    if CH2IpdvRange < 1.0:
        CH2IpdvRange = 1.0
    
    # Draw horizontal grid lines
    i = 0
    x1 = X0L
    x2 = X0L + GRW
    mg_siz = GRW/10
    mg_inc = mg_siz/5
    while (i < 11):
        y = Y0T + i * GRH/10
        Dline = [x1,y,x2,y]
        if i == 5:
            ca.create_line(Dline, fill=COLORzeroline)   # Blue line at center of grid
            k = 0
            while (k < 10):
                l = 1
                while (l < 5):
                    Dline = [x1+k*mg_siz+l*mg_inc,y-5,x1+k*mg_siz+l*mg_inc,y+5]
                    ca.create_line(Dline, fill=COLORgrid)
                    l = l + 1
                k = k + 1
        else:
            ca.create_line(Dline, fill=COLORgrid)

        if (ShowC1_V.get() == 1 or MathTrace.get() == 1 or MathTrace.get() == 2):
            Vaxis_value = (((5-i) * CH1pdvRange ) + CHAOffset)
            # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
            Vaxis_label = str(Vaxis_value)
            ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
            
        if (ShowC1_I.get() == 1 or MathTrace.get() == 4 or MathTrace.get() == 6 or MathTrace.get() == 8):
            Iaxis_value = 1.0 * (((5-i) * CH1IpdvRange ) + CHAIOffset)
            Iaxis_label = str(Iaxis_value)
            ca.create_text(x2+2, y, text=Iaxis_label, fill=COLORtrace3, anchor="w", font=("arial", 8 ))
            
        if (ShowC2_V.get() == 1 or MathTrace.get() == 3 or MathTrace.get() == 10):
            Vaxis_value = (((5-i) * CH2pdvRange ) + CHBOffset)
            # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
            Vaxis_label = str(Vaxis_value)
            ca.create_text(x1-26, y, text=Vaxis_label, fill=COLORtrace2, anchor="e", font=("arial", 8 ))
            
        if (ShowC2_I.get() == 1 or MathTrace.get() == 5 or MathTrace.get() == 7 or MathTrace.get() == 9 or MathTrace.get() == 11):
            Iaxis_value = 1.0 * (((5-i) * CH2IpdvRange ) + CHBIOffset)
            Iaxis_label = str(Iaxis_value)
            ca.create_text(x2+28, y, text=Iaxis_label, fill=COLORtrace4, anchor="w", font=("arial", 8 ))
        else:
            if Ysignal.get() == 1:
                Vaxis_value = (((5-i) * CH1pdvRange ) + CHAOffset)
                # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
                Vaxis_label = str(Vaxis_value)
                ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 2:
                Iaxis_value = 1.0 * (((5-i) * CH1IpdvRange ) + CHAIOffset)
                Iaxis_label = str(Iaxis_value)
                ca.create_text(x1-3, y, text=Iaxis_label, fill=COLORtrace3, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 3:
                Vaxis_value = (((5-i) * CH2pdvRange ) + CHBOffset)
                # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
                Vaxis_label = str(Vaxis_value)
                ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace2, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 4:
                Iaxis_value = 1.0 * (((5-i) * CH2IpdvRange ) + CHBIOffset)
                Iaxis_label = str(Iaxis_value)
                ca.create_text(x1-3, y, text=Iaxis_label, fill=COLORtrace4, anchor="e", font=("arial", 8 ))

        i = i + 1

    # Draw vertical grid lines
    i = 0
    y1 = Y0T
    y2 = Y0T + GRH
    mg_siz = GRH/10
    mg_inc = mg_siz/5
    vx = TIMEdiv/Mulx
    vt = HoldOff/Mulx # invert sign and scale to mSec
    while (i < 11):
        x = X0L + i * GRW/10
        Dline = [x,y1,x,y2]
        if (i == 5):
            ca.create_line(Dline, fill=COLORzeroline)   # Blue vertical line at center of grid
            k = 0
            while (k < 10):
                l = 1
                while (l < 5):
                    Dline = [x-5,y1+k*mg_siz+l*mg_inc,x+5,y1+k*mg_siz+l*mg_inc]
                    ca.create_line(Dline, fill=COLORgrid)
                    l = l + 1
                k = k + 1
            if DisMode.get() == 0:
                if vx >= 1000:
                    axis_value = ((i * vx)+ vt) / 1000
                    axis_label = str(int(axis_value)) + " S"
                if vx < 1000 and vx >= 1:
                    axis_value = (i * vx) + vt
                    axis_label = str(int(axis_value)) + " mS"
                if vx < 1 and vx >= 0.001:
                    axis_value = ((i * vx) + vt) * 1000
                    axis_label = str(int(axis_value)) + " uS"
                if vx < 0.001:
                    axis_value = ((i * vx) + vt) * 1000000
                    axis_label = str(int(axis_value)) + " nS"
                ca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
            else:
                if Xsignal.get() == 1: # modes "Time", "C1/C2", "C1 I/V", "C2 I/V"
                    Vaxis_value = (((i-5) * CH1pdvRange ) + CHAOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace1, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 2:
                    Iaxis_value = 1.0 * (((i-5) * CH1IpdvRange ) + CHAIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace3, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 3:
                    Vaxis_value = (((i-5) * CH2pdvRange ) + CHBOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace2, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 4:
                    Iaxis_value = 1.0 * (((i-5) * CH2IpdvRange ) + CHBIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace4, anchor="n", font=("arial", 8 ))
        else:
            ca.create_line(Dline, fill=COLORgrid)
            if DisMode.get() == 0:
                if vx >= 1000:
                    axis_value = ((i * vx)+ vt) / 1000
                    axis_label = str(int(axis_value)) + " S"
                if vx < 1000 and vx >= 1:
                    axis_value = (i * vx) + vt
                    axis_label = str(int(axis_value)) + " mS"
                if vx < 1 and vx >= 0.001:
                    axis_value = ((i * vx) + vt) * 1000
                    axis_label = str(int(axis_value)) + " uS"
                if vx < 0.001:
                    axis_value = ((i * vx) + vt) * 1000000
                    axis_label = str(int(axis_value)) + " nS"
                ca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
            else:
                if Xsignal.get() == 1: # modes "Time", "C1/C2", "C1 I/V", "C2 I/V"
                    Vaxis_value = (((i-5) * CH1pdvRange ) + CHAOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace1, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 2:
                    Iaxis_value = 1.0 * (((i-5) * CH1IpdvRange ) + CHAIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace3, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 3:
                    Vaxis_value = (((i-5) * CH2pdvRange ) + CHBOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace2, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 4:
                    Iaxis_value = 1.0 * (((i-5) * CH2IpdvRange ) + CHBIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace4, anchor="n", font=("arial", 8 ))
                
        i = i + 1

    # Write the trigger line if available
    if len(Triggerline) > 2:                    # Avoid writing lines with 1 coordinate
        ca.create_polygon(Triggerline, outline=COLORtrigger, fill=COLORtrigger, width=1)

    SmoothBool = SmoothCurves.get()
    # Write the traces if available
    if DisMode.get() == 0: 
        if len(T1Vline) > 4: # Avoid writing lines with 1 coordinate    
            ca.create_line(T1Vline, fill=COLORtrace1, smooth=SmoothBool, splinestep=5)   # Write the voltage trace 1
        if len(T1Iline) > 4: # Avoid writing lines with 1 coordinate
            ca.create_line(T1Iline, fill=COLORtrace3, smooth=SmoothBool, splinestep=5)   # Write the current trace 1
        if len(T2Vline) > 4: # Write the trace 2 if active
            ca.create_line(T2Vline, fill=COLORtrace2, smooth=SmoothBool, splinestep=5)
        if len(T2Iline) > 4:
            ca.create_line(T2Iline, fill=COLORtrace4, smooth=SmoothBool, splinestep=5)
        if len(Tmathline) > 4 and MathTrace.get() > 0: # Write Math tace if active
            ca.create_line(Tmathline, fill=COLORtrace5, smooth=SmoothBool, splinestep=5)
        if ShowRA_V.get() == 1 and len(T1VRline) > 4:
            ca.create_line(T1VRline, fill=COLORtraceR1, smooth=SmoothBool, splinestep=5)
        if ShowRA_I.get() == 1 and len(T1IRline) > 4:
            ca.create_line(T1IRline, fill=COLORtraceR3, smooth=SmoothBool, splinestep=5)
        if ShowRB_V.get() == 1 and len(T2VRline) > 4:
            ca.create_line(T2VRline, fill=COLORtraceR2, smooth=SmoothBool, splinestep=5)
        if ShowRB_I.get() == 1 and len(T2IRline) > 4:
            ca.create_line(T2IRline, fill=COLORtraceR4, smooth=SmoothBool, splinestep=5)
        if ShowMath.get() == 1 and len(TMRline) > 4:
            ca.create_line(TMRline, fill=COLORtraceR5, smooth=SmoothBool, splinestep=5)
    else:
        if len(TXYline) > 4:                    # Avoid writing lines with 1 coordinate
            if  Xsignal.get() == 1:
                ca.create_line(TXYline, fill=COLORtrace1)
            elif  Xsignal.get() == 2:
                ca.create_line(TXYline, fill=COLORtrace3)
            elif  Xsignal.get() == 3:
                ca.create_line(TXYline, fill=COLORtrace2)
            elif  Xsignal.get() == 4:
                ca.create_line(TXYline, fill=COLORtrace4)
        if len(TXYRline) > 4 and ShowRXY.get() == 1:
            ca.create_line(TXYRline, fill=COLORtraceR1)

    # General information on top of the grid
    # Sweep information
    if LONGsweep == False and LONGchunk == False:
        sttxt = "Running"
        if SingleShot.get() == 1:
            sttxt = "Single Shot"
        if (RUNstatus == 0) or (RUNstatus == 3):
            sttxt = "Stopped"

    if LONGsweep == True or LONGchunk == True:
        sttxt = "Running long sweep, wait"
        if (RUNstatus == 0) or (RUNstatus == 3):
            sttxt = "Stopped long sweep, press Start"

    txt = "Device ID " + DevID + " Sample rate: " + str(SAMPLErate) + " " + sttxt

    x = X0L
    y = 12
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)

    # Time sweep information and view at information
    # vx = TIMEdiv
    if vx >= 1000:
        txt = str(int(vx/1000)) + " S/div"
    if vx < 1000 and vx >= 1:
        txt = str(int(vx)) + " mS/div"
    if vx < 1 and vx >= 0.001:
        txt = str(int(vx * 1000)) + " uS/div"
    if vx < 0.001:
        txt = str(int(vx * 1000000)) + " nS/div"
    txt = txt + "  "
    #
    txt = txt + "View at "
    if abs(vt) >= 1000:
        txt = txt + str(vt / 1000) + " S "
    if abs(vt) < 1000 and abs(vt) >= 1:
        txt = txt + str(vt) + " mS "
    if abs(vt) < 1 and vx >= 0.001:
        txt = txt + str(vt * 1000) + " uS "
    if abs(vt) < 0.001:
        txt = txt + str(int(vt * 1000000)) + " nS"
    # print period and frequency of displayed channels
    if ShowC1_V.get() == 1 or ShowC2_V.get() == 1:
        FindRisingEdge()
        if ShowC1_V.get() == 1:
            if MeasAHW.get() == 1:
                txt = txt + " CA Hi Width = "
                V1String = ' {0:.2f} '.format(CHAHW/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasALW.get() == 1:
                txt = txt + " CA Lo Width = "
                V1String = ' {0:.2f} '.format(CHALW/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasADCy.get() == 1:
                txt = txt + " CA DutyCycle = "
                V1String = ' {0:.1f} '.format(CHADCy)
                txt = txt + str(V1String) + " % "
            if MeasAPER.get() == 1:
                txt = txt + " CA Period = "
                V1String = ' {0:.2f} '.format(CHAperiod/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasAFREQ.get() == 1:
                txt = txt + " CA Freq = "
                ChaF = CHAfreq*Mulx
                if ChaF < 1000:
                    V1String = ' {0:.1f} '.format(ChaF)
                    txt = txt + str(V1String) + " Hz "
                if ChaF > 1000 and ChaF < 1000000:
                    V1String = ' {0:.1f} '.format(ChaF/1000)
                    txt = txt + str(V1String) + " KHz "
                if ChaF > 1000000:
                    V1String = ' {0:.1f} '.format(ChaF/1000000)
                    txt = txt + str(V1String) + " MHz "
        if ShowC2_V.get() == 1:
            if MeasBHW.get() == 1:
                txt = txt + " CB Hi Width = "
                V1String = ' {0:.2f} '.format(CHBHW/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasBLW.get() == 1:
                txt = txt + " CB Lo Width = "
                V1String = ' {0:.2f} '.format(CHBLW/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasBDCy.get() == 1:
                txt = txt + " CB DutyCycle = "
                V1String = ' {0:.1f} '.format(CHBDCy)
                txt = txt + str(V1String) + " % "
            if MeasBPER.get() == 1:
                txt = txt + " CB Period = "
                V1String = ' {0:.2f} '.format(CHBperiod/Mulx)
                txt = txt + str(V1String) + " mS "
            if MeasBFREQ.get() == 1:
                txt = txt + " CB Freq = "
                ChaF = CHBfreq*Mulx
                if ChaF < 1000:
                    V1String = ' {0:.1f} '.format(ChaF)
                    txt = txt + str(V1String) + " Hz "
                if ChaF > 1000 and ChaF < 1000000:
                    V1String = ' {0:.1f} '.format(ChaF/1000)
                    txt = txt + str(V1String) + " KHz "
                if ChaF > 1000000:
                    V1String = ' {0:.1f} '.format(ChaF/1000000)
                    txt = txt + str(V1String) + " MHz "
        if MeasPhase.get() == 1:
            txt = txt + " CA-B Phase = "
            V1String = ' {0:.1f} '.format(CHABphase)
            txt = txt + str(V1String) + " deg "
            
    x = X0L
    y = Y0T+GRH+20
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)
    txt = " "
    if ShowC1_V.get() == 1 or Xsignal.get() == 1 or Ysignal.get() == 1:
    # Channel A information
        txt = "CHA: "
        txt = txt + str(CH1pdvRange) + " V/div"
        if MeasDCV1.get() == 1:
            V1String = ' {0:.4f} '.format(DCV1)
            txt = txt + " AvgV = " + V1String
        if MeasMaxV1.get() == 1:
            V1String = ' {0:.4f} '.format(MaxV1)
            txt = txt +  " MaxV = " + V1String
        if MeasMinV1.get() == 1:
            V1String = ' {0:.4f} '.format(MinV1)
            txt = txt +  " MinV = " + V1String
        if MeasMidV1.get() == 1:
            MidV1 = (MaxV1+MinV1)/2
            V1String = ' {0:.4f} '.format(MidV1)
            txt = txt +  " MidV = " + V1String
        if MeasPPV1.get() == 1:
            PPV1 = MaxV1-MinV1
            V1String = ' {0:.4f} '.format(PPV1)
            txt = txt +  " P-PV = " + V1String
        if MeasRMSV1.get() == 1:
            V1String = ' {0:.4f} '.format(SV1)
            txt = txt +  " RMS = " + V1String
    if (ShowC1_I.get() == 1 and ShowC1_V.get() == 0):
        txt = "CHA: "
        txt = txt + str(CH1IpdvRange) + " mA/div"
    elif (ShowC1_I.get() == 1 and ShowC1_V.get() == 1):
        txt = txt + "CHA: "
        txt = txt + str(CH1IpdvRange) + " mA/div"
    if ShowC1_I.get() == 1 or Xsignal.get() == 2 or Ysignal.get() == 2:
        if MeasDCI1.get() == 1:
            V1String = ' {0:.2f} '.format(DCI1)
            txt = txt + " AvgI = " + V1String
            if AWGAShape.get() == 0: # if this is a DC measurement calc resistance
                try:
                    Resvalue = (DCV1/DCI1)*1000
                    R1String = ' {0:.1f} '.format(Resvalue)
                    txt = txt + " Res = " + R1String
                except:
                    txt = txt + " Res = OverRange" 
        if MeasMaxI1.get() == 1:
            V1String = ' {0:.2f} '.format(MaxI1)
            txt = txt +  " MaxI = " + V1String
        if MeasMinI1.get() == 1:
            V1String = ' {0:.2f} '.format(MinI1)
            txt = txt +  " MinI = " + V1String
        if MeasMidI1.get() == 1:
            MidI1 = (MaxI1+MinI1)/2
            V1String = ' {0:.2f} '.format(MidI1)
            txt = txt +  " MidV = " + V1String
        if MeasPPI1.get() == 1:
            PPI1 = MaxI1-MinI1
            V1String = ' {0:.2f} '.format(PPI1)
            txt = txt +  " P-PI = " + V1String
        if MeasRMSI1.get() == 1:
            V1String = ' {0:.4f} '.format(SI1)
            txt = txt +  " RMS = " + V1String
        
    x = X0L
    y = Y0T+GRH+32
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)
    txt= " "
    # Channel B information
    if ShowC2_V.get() == 1 or Xsignal.get() == 3 or Ysignal.get() == 3:
        txt = "CHB: "
        txt = txt + str(CH2pdvRange) + " V/div"
        if MeasDCV2.get() == 1:
            V1String = ' {0:.4f} '.format(DCV2)
            txt = txt + " AvgV = " + V1String
        if MeasMaxV2.get() == 1:
            V1String = ' {0:.4f} '.format(MaxV2)
            txt = txt +  " MaxV = " + V1String
        if MeasMinV2.get() == 1:
            V1String = ' {0:.4f} '.format(MinV2)
            txt = txt +  " MinV = " + V1String
        if MeasMidV2.get() == 1:
            MidV2 = (MaxV2+MinV2)/2
            V1String = ' {0:.4f} '.format(MidV2)
            txt = txt +  " MidV = " + V1String
        if MeasPPV2.get() == 1:
            PPV2 = MaxV2-MinV2
            V1String = ' {0:.4f} '.format(PPV2)
            txt = txt +  " P-PV = " + V1String
        if MeasRMSV2.get() == 1:
            V1String = ' {0:.4f} '.format(SV2)
            txt = txt +  " RMS = " + V1String
    if (ShowC2_I.get() == 1 and ShowC2_V.get() == 0):
        txt = "CHB: "
        txt = txt + str(CH2IpdvRange) + " mA/div"
    elif (ShowC2_I.get() == 1 and ShowC2_V.get() == 1):
        txt = txt + "CHB: "
        txt = txt + str(CH2IpdvRange) + " mA/div"
    if ShowC2_I.get() == 1 or Xsignal.get() == 4 or Ysignal.get() == 4:
        if MeasDCI2.get() == 1:
            V1String = ' {0:.2f} '.format(DCI2)
            txt = txt + " AvgI = " + V1String
            if AWGBShape.get() == 0: # if this is a DC measurement calc resistance
                try:
                    Resvalue = (DCV2/DCI2)*1000
                    R1String = ' {0:.1f} '.format(Resvalue)
                    txt = txt + " Res = " + R1String
                except:
                    txt = txt + " Res = OverRange" 
        if MeasMaxI2.get() == 1:
            V1String = ' {0:.2f} '.format(MaxI2)
            txt = txt +  " MaxI = " + V1String
        if MeasMinI2.get() == 1:
            V1String = ' {0:.2f} '.format(MinI2)
            txt = txt +  " MinI = " + V1String
        if MeasMidI2.get() == 1:
            MidI2 = (MaxI2+MinI2)/2
            V1String = ' {0:.2f} '.format(MidI2)
            txt = txt +  " MidV = " + V1String
        if MeasPPI2.get() == 1:
            PPI2 = MaxI2-MinI2
            V1String = ' {0:.2f} '.format(PPI2)
            txt = txt +  " P-PI = " + V1String
        if MeasRMSI2.get() == 1:
            V1String = ' {0:.4f} '.format(SI2)
            txt = txt +  " RMS = " + V1String
            
    x = X0L
    y = Y0T+GRH+44
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)

def ConnectDevice():
    global devx, DevID, Both, CHA, CHB, bcon
    try:
        devx = Smu()
        brdnum = len(devx.devices)
        channum = len(devx.channels)
        CHA = devx.channels['A']
        CHB = devx.channels['B']
        DevID = devx.serials[0]
        print DevID
        Both = devx.devices[0]
        bcon.configure(text="Conn", bg="green")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100) # sclk to 1
        devx.ctrl_transfer(DevID, 0x40, 0x51, 2, 0, 0, 0, 100) # fsync to 1
        if channum == 4:
            CHC = devx.channels['C']
            CHC.set_mode('d')
            CHD = devx.channels['D']
            CHD.set_mode('d')
            DevTwo = devx.serials[1]
            print DevTwo
            BTwo = devx.devices[1]
    except:
        print 'No Device plugged IN!'
        DevID = "No Device"
        bcon.configure(text="Recon", bg="red")
        
# ================ Make Screen ==========================
TBicon = """
R0lGODlhIAAgAHAAACH5BAEAAAIALAAAAAAgACAAgQAAAP///wAAAAAAAAJJhI+py+0PYwtBWkDp
hTnv2XlfEobjUZZnmn4se72vJMtcbYN4ruz44uORgiodsfI4Im++2M5VW81OmBbVULxiRVrUsgsO
i8fUAgA7
"""

root=Tk()
root.title("ALICE-HS 1.0 (12-8-2015): ALM1000 Sampling Oscilloscope")
img = PhotoImage(data=TBicon)
root.call('wm', 'iconphoto', root._w, img)

root.minsize(100, 100)

frame2r = Frame(root, borderwidth=5, relief=RIDGE)
frame2r.pack(side=RIGHT, expand=1, fill=Y)

frame1 = Frame(root, borderwidth=5, relief=RIDGE)
frame1.pack(side=TOP, expand=1, fill=X)

frame2 = Frame(root, background="black", borderwidth=5, relief=RIDGE)
frame2.pack(side=TOP, fill=BOTH, expand=YES)

frame3 = Frame(root, borderwidth=5, relief=RIDGE)
frame3.pack(side=TOP, expand=1, fill=X)
#
# create a pulldown menu
# Trigger signals
TgInput = IntVar(0)   # Trigger Input variable
SingleShot = IntVar(0) # variable for Single Shot manual trigger
AutoLevel = IntVar(0) # variable for Auto Level trigger at mid point
Triggermenu = Menubutton(frame1, text="Trigger", relief=RAISED)
Triggermenu.menu = Menu(Triggermenu, tearoff = 0 )
Triggermenu["menu"]  = Triggermenu.menu
Triggermenu.menu.add_radiobutton(label='None', variable=TgInput, value=0)
Triggermenu.menu.add_radiobutton(label='CA-V', variable=TgInput, value=1)
Triggermenu.menu.add_radiobutton(label='CA-I', variable=TgInput, value=2)
Triggermenu.menu.add_radiobutton(label='CB-V', variable=TgInput, value=3)
Triggermenu.menu.add_radiobutton(label='CB-I', variable=TgInput, value=4)
Triggermenu.menu.add_checkbutton(label='Auto Level', variable=AutoLevel)
Triggermenu.menu.add_checkbutton(label='SingleShot', variable=SingleShot)
Triggermenu.pack(side=LEFT)
#
TgEdge = IntVar(0)   # Trigger edge variable
Edgemenu = Menubutton(frame1, text="Edge", relief=RAISED)
Edgemenu.menu = Menu(Edgemenu, tearoff = 0 )
Edgemenu["menu"]  = Edgemenu.menu
Edgemenu.menu.add_radiobutton(label='Rising', variable=TgEdge, value=0)
Edgemenu.menu.add_radiobutton(label='Falling', variable=TgEdge, value=1)
Edgemenu.pack(side=LEFT)
#
tlab = Label(frame1, text="Trig Level")
tlab.pack(side=LEFT)
TRIGGERentry = Entry(frame1, width=5)
TRIGGERentry.bind("<Return>", BTriglevel)
TRIGGERentry.pack(side=LEFT)
TRIGGERentry.delete(0,"end")
TRIGGERentry.insert(0,0.0)
#
tgb = Button(frame1, text="50%", width=4, command=BTrigger50p)
tgb.pack(side=LEFT)
#
hldlab = Label(frame1, text="Hold Off")
hldlab.pack(side=LEFT)
HoldOffentry = Entry(frame1, width=4)
HoldOffentry.bind("<Return>", BHoldOff)
HoldOffentry.pack(side=LEFT)
HoldOffentry.delete(0,"end")
HoldOffentry.insert(0,0.01)
#
bcon = Button(frame1, text="Recon", width=5, command=ConnectDevice)
bcon.pack(side=RIGHT)
PwrBt = Button(frame1, text="PWR-ON", width=7, bg="green", command=BPower)
PwrBt.pack(side=RIGHT)
# Show channels menu
ShowC1_V = IntVar(0)   # curves to display variables
ShowC1_I = IntVar(0)
ShowC2_V = IntVar(0)
ShowC2_I = IntVar(0)
ShowAV_I = IntVar(0)
ShowBV_I = IntVar(0)
ShowRA_V = IntVar(0)
ShowRA_I = IntVar(0)
ShowRB_V = IntVar(0)
ShowRB_I = IntVar(0)
ShowMath = IntVar(0)
#
Showmenu = Menubutton(frame1, text="Curves", relief=RAISED)
Showmenu.menu = Menu(Showmenu, tearoff = 0 )
Showmenu["menu"] = Showmenu.menu
Showmenu.menu.add_command(label="-Show-", command=donothing)
Showmenu.menu.add_command(label="All", command=BShowCurvesAll)
Showmenu.menu.add_command(label="None", command=BShowCurvesNone)
Showmenu.menu.add_checkbutton(label='CA-V', variable=ShowC1_V)
Showmenu.menu.add_checkbutton(label='CA-I', variable=ShowC1_I)
Showmenu.menu.add_checkbutton(label='CB-V', variable=ShowC2_V)
Showmenu.menu.add_checkbutton(label='CB-I', variable=ShowC2_I)
Showmenu.menu.add_separator()
Showmenu.menu.add_checkbutton(label='RA-V', variable=ShowRA_V)
Showmenu.menu.add_checkbutton(label='RA-I', variable=ShowRA_I)
Showmenu.menu.add_checkbutton(label='RB-V', variable=ShowRB_V)
Showmenu.menu.add_checkbutton(label='RB-I', variable=ShowRB_I)
Showmenu.menu.add_checkbutton(label='RMath', variable=ShowMath)
Showmenu.pack(side=RIGHT)
# Disply mode menu
DisMode = IntVar(0)   # Display mode variable
Xsignal = IntVar(0)   # Signal for X axis variable
Ysignal = IntVar(0)   # Signal for X axis variable
ShowRXY = IntVar(0)   # show reference XY trace
Displaymenu = Menubutton(frame1, text="Display", relief=RAISED)
Displaymenu.menu = Menu(Displaymenu, tearoff = 0 )
Displaymenu["menu"]  = Displaymenu.menu
Displaymenu.menu.add_radiobutton(label='Time', variable=DisMode, value=0)
Displaymenu.menu.add_radiobutton(label='X-Y', variable=DisMode, value=1)
Displaymenu.menu.add_separator()
# X - Y mode signal select
Displaymenu.menu.add_command(label="-X Axis-", command=donothing)
Displaymenu.menu.add_radiobutton(label='none', variable=Xsignal, value=0)
Displaymenu.menu.add_radiobutton(label='CA-V', variable=Xsignal, value=1)
Displaymenu.menu.add_radiobutton(label='CA-I', variable=Xsignal, value=2)
Displaymenu.menu.add_radiobutton(label='CB-V', variable=Xsignal, value=3)
Displaymenu.menu.add_radiobutton(label='CB-I', variable=Xsignal, value=4)
#
Displaymenu.menu.add_separator()
Displaymenu.menu.add_command(label="-Y Axis-", command=donothing)
Displaymenu.menu.add_radiobutton(label='none', variable=Ysignal, value=0)
Displaymenu.menu.add_radiobutton(label='CA-V', variable=Ysignal, value=1)
Displaymenu.menu.add_radiobutton(label='CA-I', variable=Ysignal, value=2)
Displaymenu.menu.add_radiobutton(label='CB-V', variable=Ysignal, value=3)
Displaymenu.menu.add_radiobutton(label='CB-I', variable=Ysignal, value=4)
Displaymenu.menu.add_separator()
Displaymenu.menu.add_checkbutton(label='RX-Y', variable=ShowRXY)
Displaymenu.pack(side=RIGHT)
# Time per Div
TMsb = Spinbox(frame1, width=5, values= TMpdiv, command=BTime)
TMsb.pack(side=RIGHT)
TMsb.delete(0,"end")
TMsb.insert(0,0.5)
TMlab = Label(frame1, text="Time mS/Div")
TMlab.pack(side=RIGHT)
#
# Sampling controls Widgets
fminlab2 = Label(frame1, text="KHz")
fminlab2.pack(side=RIGHT)
FminEntry = Entry(frame1, width=4)
FminEntry.bind("<Return>", BsetAD9833)
FminEntry.pack(side=RIGHT)
FminEntry.delete(0,"end")
FminEntry.insert(0,25)
fminlab = Label(frame1, text="Fmin")
fminlab.pack(side=RIGHT)
#
MulXEntry = Entry(frame1, width=4)
MulXEntry.bind("<Return>", BsetAD9833)
MulXEntry.pack(side=RIGHT)
MulXEntry.delete(0,"end")
MulXEntry.insert(0,1000)
mulxlab = Label( frame1, text = "Mul X")
mulxlab.pack(side=RIGHT)
#
ca = Canvas(frame2, width=CANVASwidth, height=CANVASheight, background=COLORcanvas)
ca.pack(side=TOP, fill=BOTH, expand=YES)
# right side menu buttons
rsemenu = Frame( frame2r )
rsemenu.pack(side=TOP)
rb = Button(rsemenu, text="Run", width=3, bg="green", command=BStart)
rb.pack(side=LEFT, anchor=W)
sb = Button(rsemenu, text="Stop", width=4, bg="red", command=BStop)
sb.pack(side=LEFT, anchor=W)
eb = Button(rsemenu, text="Exit", width=4, command=Bcloseexit)
eb.pack(side=LEFT, anchor=W)
#
dropmenu = Frame( frame2r )
dropmenu.pack(side=TOP)
# File menu
Filemenu = Menubutton(dropmenu, text="File", relief=RAISED)
Filemenu.menu = Menu(Filemenu, tearoff = 0 )
Filemenu["menu"] = Filemenu.menu
Filemenu.menu.add_command(label="Save Config", command=BSaveConfig)
Filemenu.menu.add_command(label="Load Config", command=BLoadConfig)
Filemenu.menu.add_command(label="Save Screen", command=BSaveScreen)
Filemenu.menu.add_command(label="Save Data", command=BSaveData)
Filemenu.menu.add_command(label="Help", command=BHelp)
Filemenu.pack(side=LEFT, anchor=W)
#
SmoothCurves = IntVar(0)
ZOHold = IntVar(0)
Optionmenu = Menubutton(dropmenu, text="Options", relief=RAISED)
Optionmenu.menu = Menu(Optionmenu, tearoff = 0 )
Optionmenu["menu"]  = Optionmenu.menu
Optionmenu.menu.add_checkbutton(label='Smooth', variable=SmoothCurves)
Optionmenu.menu.add_checkbutton(label='Z-O-Hold', variable=ZOHold)
Optionmenu.menu.add_command(label="SnapShot", command=BSnapShot)
Optionmenu.pack(side=LEFT, anchor=W)
#
dropmenu2 = Frame( frame2r )
dropmenu2.pack(side=TOP)
# Math trace menu
MathTrace = IntVar(0)
MathMenu = Menubutton(dropmenu2, text="Math", width=3, relief=RAISED)
MathMenu.menu = Menu(MathMenu, tearoff = 0 )
MathMenu["menu"]  = MathMenu.menu
MathMenu.menu.add_radiobutton(label='none', variable=MathTrace, value=0)
MathMenu.menu.add_radiobutton(label='CAV+CBV', variable=MathTrace, value=1)
MathMenu.menu.add_radiobutton(label='CAV-CBV', variable=MathTrace, value=2)
MathMenu.menu.add_radiobutton(label='CBV-CAV', variable=MathTrace, value=3)
MathMenu.menu.add_radiobutton(label='CAI-CBI', variable=MathTrace, value=8)
MathMenu.menu.add_radiobutton(label='CBI-CAI', variable=MathTrace, value=9)
MathMenu.menu.add_radiobutton(label='CAV*CAI', variable=MathTrace, value=4)
MathMenu.menu.add_radiobutton(label='CBV*CBI', variable=MathTrace, value=5)
MathMenu.menu.add_radiobutton(label='CAV/CAI', variable=MathTrace, value=6)
MathMenu.menu.add_radiobutton(label='CBV/CBI', variable=MathTrace, value=7)
MathMenu.menu.add_radiobutton(label='CBV/CAV', variable=MathTrace, value=10)
MathMenu.menu.add_radiobutton(label='CBI/CAI', variable=MathTrace, value=11)
MathMenu.pack(side=RIGHT, anchor=W)
# Measurments menu
MeasDCV1 = IntVar(0) # define vertical measurment variables
MeasMinV1 = IntVar(0)
MeasMaxV1 = IntVar(0)
MeasMidV1 = IntVar(0)
MeasPPV1 = IntVar(0)
MeasRMSV1 = IntVar(0)
MeasDCI1 = IntVar(0)
MeasMinI1 = IntVar(0)
MeasMaxI1 = IntVar(0)
MeasMidI1 = IntVar(0)
MeasPPI1 = IntVar(0)
MeasRMSI1 = IntVar(0)
MeasDCV2 = IntVar(0)
MeasMinV2 = IntVar(0)
MeasMaxV2 = IntVar(0)
MeasMidV2 = IntVar(0)
MeasPPV2 = IntVar(0)
MeasRMSV2 = IntVar(0)
MeasDCI2 = IntVar(0)
MeasMinI2 = IntVar(0)
MeasMaxI2 = IntVar(0)
MeasMidI2 = IntVar(0)
MeasPPI2 = IntVar(0)
MeasRMSI2 = IntVar(0)
MeasAHW = IntVar(0)
MeasALW = IntVar(0)
MeasADCy = IntVar(0)
MeasAPER = IntVar(0)
MeasAFREQ = IntVar(0)
MeasBHW = IntVar(0)
MeasBLW = IntVar(0)
MeasBDCy = IntVar(0)
MeasBPER = IntVar(0)
MeasBFREQ = IntVar(0)
MeasPhase = IntVar(0)
#
measlab = Label(dropmenu2, text="Meas")
measlab.pack(side=LEFT, anchor=W)
MeasmenuA = Menubutton(dropmenu2, text="CA", width=2, relief=RAISED)
MeasmenuA.menu = Menu(MeasmenuA, tearoff = 0 )
MeasmenuA["menu"]  = MeasmenuA.menu
MeasmenuA.menu.add_command(label="-CA-V-", command=donothing)
MeasmenuA.menu.add_checkbutton(label='Avg', variable=MeasDCV1)
MeasmenuA.menu.add_checkbutton(label='Min', variable=MeasMinV1)
MeasmenuA.menu.add_checkbutton(label='Max', variable=MeasMaxV1)
MeasmenuA.menu.add_checkbutton(label='Mid', variable=MeasMidV1)
MeasmenuA.menu.add_checkbutton(label='P-P', variable=MeasPPV1)
MeasmenuA.menu.add_checkbutton(label='RMS', variable=MeasRMSV1)
MeasmenuA.menu.add_separator()
#
MeasmenuA.menu.add_command(label="-CA-I-", command=donothing)
MeasmenuA.menu.add_checkbutton(label='Avg', variable=MeasDCI1)
MeasmenuA.menu.add_checkbutton(label='Min', variable=MeasMinI1)
MeasmenuA.menu.add_checkbutton(label='Max', variable=MeasMaxI1)
MeasmenuA.menu.add_checkbutton(label='Mid', variable=MeasMidI1)
MeasmenuA.menu.add_checkbutton(label='P-P', variable=MeasPPI1)
MeasmenuA.menu.add_checkbutton(label='RMS', variable=MeasRMSI1)
MeasmenuA.menu.add_separator()
#
MeasmenuA.menu.add_command(label="CA-Time", command=donothing)
MeasmenuA.menu.add_checkbutton(label='H-Width', variable=MeasAHW)
MeasmenuA.menu.add_checkbutton(label='L-Width', variable=MeasALW)
MeasmenuA.menu.add_checkbutton(label='DutyCyle', variable=MeasADCy)
MeasmenuA.menu.add_checkbutton(label='Period', variable=MeasAPER)
MeasmenuA.menu.add_checkbutton(label='Freq', variable=MeasAFREQ)
MeasmenuA.menu.add_checkbutton(label='A-B Phase', variable=MeasPhase)
#
MeasmenuA.pack(side=LEFT)
#
MeasmenuB = Menubutton(dropmenu2, text="CB", width=2, relief=RAISED)
MeasmenuB.menu = Menu(MeasmenuB, tearoff = 0 )
MeasmenuB["menu"]  = MeasmenuB.menu
MeasmenuB.menu.add_command(label="-CB-V-", command=donothing)
MeasmenuB.menu.add_checkbutton(label='Avg', variable=MeasDCV2)
MeasmenuB.menu.add_checkbutton(label='Min', variable=MeasMinV2)
MeasmenuB.menu.add_checkbutton(label='Max', variable=MeasMaxV2)
MeasmenuB.menu.add_checkbutton(label='Mid', variable=MeasMidV2)
MeasmenuB.menu.add_checkbutton(label='P-P', variable=MeasPPV2)
MeasmenuB.menu.add_checkbutton(label='RMS', variable=MeasRMSV2)
MeasmenuB.menu.add_separator()
#
MeasmenuB.menu.add_command(label="-CB-I-", command=donothing)
MeasmenuB.menu.add_checkbutton(label='Avg', variable=MeasDCI2)
MeasmenuB.menu.add_checkbutton(label='Min', variable=MeasMinI2)
MeasmenuB.menu.add_checkbutton(label='Max', variable=MeasMaxI2)
MeasmenuB.menu.add_checkbutton(label='Mid', variable=MeasMidI2)
MeasmenuB.menu.add_checkbutton(label='P-P', variable=MeasPPI2)
MeasmenuB.menu.add_checkbutton(label='RMS', variable=MeasRMSI2)
MeasmenuB.menu.add_separator()
#
MeasmenuB.menu.add_command(label="CB-Time", command=donothing)
MeasmenuB.menu.add_checkbutton(label='H-Width', variable=MeasBHW)
MeasmenuB.menu.add_checkbutton(label='L-Width', variable=MeasBLW)
MeasmenuB.menu.add_checkbutton(label='DutyCyle', variable=MeasBDCy)
MeasmenuB.menu.add_checkbutton(label='Period', variable=MeasBPER)
MeasmenuB.menu.add_checkbutton(label='Freq', variable=MeasBFREQ)
MeasmenuB.pack(side=LEFT)
# now AWG A
awg1lab = Label(frame2r, text="AWG CH A")
awg1lab.pack(side=TOP)
# AWG enable sub frame
AWGAMode = IntVar(0)   # AWG A mode variable
AWGATerm = IntVar(0)   # AWG A termination variable
AWGAShape = IntVar(0)  # AWG A Wave shape variable
awg1eb = Frame( frame2r )
awg1eb.pack(side=TOP)
ModeAMenu = Menubutton(awg1eb, text="Mode", relief=RAISED)
ModeAMenu.menu = Menu(ModeAMenu, tearoff = 0 )
ModeAMenu["menu"] = ModeAMenu.menu
ModeAMenu.menu.add_command(label="-Mode-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="SVMI", variable=AWGAMode, value=0)
ModeAMenu.menu.add_radiobutton(label="SIMV", variable=AWGAMode, value=1)
ModeAMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGAMode, value=2)
ModeAMenu.menu.add_radiobutton(label="Split I/O", variable=AWGAMode, value=3)
ModeAMenu.menu.add_separator()
ModeAMenu.menu.add_command(label="-Term-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="Open", variable=AWGATerm, value=0)
ModeAMenu.menu.add_radiobutton(label="To GND", variable=AWGATerm, value=1)
ModeAMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGATerm, value=2)
ModeAMenu.pack(side=LEFT, anchor=W)
ShapeAMenu = Menubutton(awg1eb, text="Shape", relief=RAISED)
ShapeAMenu.menu = Menu(ShapeAMenu, tearoff = 0 )
ShapeAMenu["menu"] = ShapeAMenu.menu
ShapeAMenu.menu.add_radiobutton(label="DC", variable=AWGAShape, value=0)
ShapeAMenu.menu.add_radiobutton(label="Sine", variable=AWGAShape, value=1)
ShapeAMenu.menu.add_radiobutton(label="Triangle", variable=AWGAShape, value=2)
ShapeAMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGAShape, value=3)
ShapeAMenu.menu.add_radiobutton(label="Square", variable=AWGAShape, value=4)
ShapeAMenu.menu.add_radiobutton(label="StairStep", variable=AWGAShape, value=5)
ShapeAMenu.menu.add_radiobutton(label="Custom", variable=AWGAShape, value=6)
ShapeAMenu.pack(side=LEFT, anchor=W)
#
awg1ampl = Frame( frame2r )
awg1ampl.pack(side=TOP)
AWGAAmplEntry = Entry(awg1ampl, width=5)
AWGAAmplEntry.bind("<Return>", BAWGAAmpl)
AWGAAmplEntry.pack(side=LEFT, anchor=W)
AWGAAmplEntry.delete(0,"end")
AWGAAmplEntry.insert(0,0.0)
amp1lab = Label(awg1ampl, text="Min")
amp1lab.pack(side=LEFT, anchor=W)
#
awg1off = Frame( frame2r )
awg1off.pack(side=TOP)
AWGAOffsetEntry = Entry(awg1off, width=5)
AWGAOffsetEntry.bind("<Return>", BAWGAOffset)
AWGAOffsetEntry.pack(side=LEFT, anchor=W)
AWGAOffsetEntry.delete(0,"end")
AWGAOffsetEntry.insert(0,0.0)
off1lab = Label(awg1off, text="Max")
off1lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg1freq = Frame( frame2r )
awg1freq.pack(side=TOP)
AWGAFreqEntry = Entry(awg1freq, width=7)
AWGAFreqEntry.bind("<Return>", BAWGAFreq)
AWGAFreqEntry.pack(side=LEFT, anchor=W)
AWGAFreqEntry.delete(0,"end")
AWGAFreqEntry.insert(0,0.0)
freq1lab = Label(awg1freq, text="Freq")
freq1lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg1phase = Frame( frame2r )
awg1phase.pack(side=TOP)
AWGAPhaseEntry = Entry(awg1phase, width=5)
AWGAPhaseEntry.bind("<Return>", BAWGAPhase)
AWGAPhaseEntry.pack(side=LEFT, anchor=W)
AWGAPhaseEntry.delete(0,"end")
AWGAPhaseEntry.insert(0,0)
phase1lab = Label(awg1phase, text="Phase")
phase1lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg1dc = Frame( frame2r )
awg1dc.pack(side=TOP)
AWGADutyCycleEntry = Entry(awg1dc, width=5)
AWGADutyCycleEntry.bind("<Return>", BAWGADutyCycle)
AWGADutyCycleEntry.pack(side=LEFT, anchor=W)
AWGADutyCycleEntry.delete(0,"end")
AWGADutyCycleEntry.insert(0,50)
duty1lab = Label(awg1dc, text="%")
duty1lab.pack(side=LEFT, anchor=W)

# now AWG B
awg2lab = Label(frame2r, text="AWG CH B")
awg2lab.pack(side=TOP)
# AWG enable sub frame
AWGBMode = IntVar(0)   # AWG B mode variable
AWGBTerm = IntVar(0)   # AWG B termination variable
AWGBShape = IntVar(0)  # AWG B Wave shape variable
awg2eb = Frame( frame2r )
awg2eb.pack(side=TOP)
ModeBMenu = Menubutton(awg2eb, text="Mode", relief=RAISED)
ModeBMenu.menu = Menu(ModeBMenu, tearoff = 0 )
ModeBMenu["menu"] = ModeBMenu.menu
ModeBMenu.menu.add_command(label="-Mode-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="SVMI", variable=AWGBMode, value=0)
ModeBMenu.menu.add_radiobutton(label="SIMV", variable=AWGBMode, value=1)
ModeBMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGBMode, value=2)
ModeBMenu.menu.add_radiobutton(label="Split I/O", variable=AWGBMode, value=3)
ModeBMenu.menu.add_separator()
ModeBMenu.menu.add_command(label="-Term-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="Open", variable=AWGBTerm, value=0)
ModeBMenu.menu.add_radiobutton(label="To GND", variable=AWGBTerm, value=1)
ModeBMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGBTerm, value=2)
ModeBMenu.pack(side=LEFT, anchor=W)
ShapeBMenu = Menubutton(awg2eb, text="Shape", relief=RAISED)
ShapeBMenu.menu = Menu(ShapeBMenu, tearoff = 0 )
ShapeBMenu["menu"] = ShapeBMenu.menu
ShapeBMenu.menu.add_radiobutton(label="DC", variable=AWGBShape, value=0)
ShapeBMenu.menu.add_radiobutton(label="Sine", variable=AWGBShape, value=1)
ShapeBMenu.menu.add_radiobutton(label="Triangle", variable=AWGBShape, value=2)
ShapeBMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGBShape, value=3)
ShapeBMenu.menu.add_radiobutton(label="Square", variable=AWGBShape, value=4)
ShapeBMenu.menu.add_radiobutton(label="StairStep", variable=AWGBShape, value=5)
ShapeBMenu.menu.add_radiobutton(label="Custom", variable=AWGBShape, value=6)
ShapeBMenu.pack(side=LEFT, anchor=W)
#
awg2ampl = Frame( frame2r )
awg2ampl.pack(side=TOP)
AWGBAmplEntry = Entry(awg2ampl, width=5)
AWGBAmplEntry.bind("<Return>", BAWGBAmpl)
AWGBAmplEntry.pack(side=LEFT, anchor=W)
AWGBAmplEntry.delete(0,"end")
AWGBAmplEntry.insert(0,0.0)
amp2lab = Label(awg2ampl, text="Min")
amp2lab.pack(side=LEFT, anchor=W)
#
awg2off = Frame( frame2r )
awg2off.pack(side=TOP)
AWGBOffsetEntry = Entry(awg2off, width=5)
AWGBOffsetEntry.bind("<Return>", BAWGBOffset)
AWGBOffsetEntry.pack(side=LEFT, anchor=W)
AWGBOffsetEntry.delete(0,"end")
AWGBOffsetEntry.insert(0,0.0)
off2lab = Label(awg2off, text="Max")
off2lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg2freq = Frame( frame2r )
awg2freq.pack(side=TOP)
AWGBFreqEntry = Entry(awg2freq, width=7)
AWGBFreqEntry.bind("<Return>", BAWGBFreq)
AWGBFreqEntry.pack(side=LEFT, anchor=W)
AWGBFreqEntry.delete(0,"end")
AWGBFreqEntry.insert(0,0.0)
freq2lab = Label(awg2freq, text="Freq")
freq2lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg2phase = Frame( frame2r )
awg2phase.pack(side=TOP)
AWGBPhaseEntry = Entry(awg2phase, width=5)
AWGBPhaseEntry.bind("<Return>", BAWGBPhase)
AWGBPhaseEntry.pack(side=LEFT, anchor=W)
AWGBPhaseEntry.delete(0,"end")
AWGBPhaseEntry.insert(0,0)
phase2lab = Label(awg2phase, text="Phase")
phase2lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg2dc = Frame( frame2r )
awg2dc.pack(side=TOP)
AWGBDutyCycleEntry = Entry(awg2dc, width=5)
AWGBDutyCycleEntry.bind("<Return>", BAWGBDutyCycle)
AWGBDutyCycleEntry.pack(side=LEFT, anchor=W)
AWGBDutyCycleEntry.delete(0,"end")
AWGBDutyCycleEntry.insert(0,50)
duty2lab = Label(awg2dc, text="%")
duty2lab.pack(side=LEFT, anchor=W)

AWGSync = IntVar(0) # Sync start both AWG channels
awgsync = Checkbutton(frame2r, text="Sync AWG", variable=AWGSync, command=BAWGSync)
awgsync.pack(side=TOP)
# MiniGen wigets
Omode = IntVar(0)
minigenfr = Frame( frame2r )
minigenfr.pack(side=TOP)
MiniGenMenu = Menubutton(minigenfr, text="MiniGen", relief=RAISED)
MiniGenMenu.menu = Menu(MiniGenMenu, tearoff = 0 )
MiniGenMenu["menu"] = MiniGenMenu.menu
MiniGenMenu.menu.add_radiobutton(label="Sine", variable=Omode, value=0, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Triangle", variable=Omode, value=2, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Square", variable=Omode, value=40, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Square/2", variable=Omode, value=32, command=SetAD9833)
MiniGenMenu.pack(side=LEFT, anchor=W)
MiniGenEntry = Entry(minigenfr, width=5)
MiniGenEntry.bind("<Return>", BsetAD9833)
MiniGenEntry.pack(side=LEFT, anchor=W)
MiniGenEntry.delete(0,"end")
MiniGenEntry.insert(0,10.0)
# input probe wigets
prlab = Label(frame2r, text="Input Gain Offset")
prlab.pack(side=TOP)
# Input Probes sub frame 
ProbeA = Frame( frame2r )
ProbeA.pack(side=TOP)
gain1lab = Label(ProbeA, text="CA")
gain1lab.pack(side=LEFT)
CHAVGainEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVGainEntry.pack(side=LEFT)
CHAVGainEntry.delete(0,"end")
CHAVGainEntry.insert(0,1.0)
CHAVOffsetEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVOffsetEntry.pack(side=LEFT)
CHAVOffsetEntry.delete(0,"end")
CHAVOffsetEntry.insert(0,0.0)
#
ProbeB = Frame( frame2r )
ProbeB.pack(side=TOP)
gain2lab = Label(ProbeB, text="CB")
gain2lab.pack(side=LEFT)
CHBVGainEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVGainEntry.pack(side=LEFT)
CHBVGainEntry.delete(0,"end")
CHBVGainEntry.insert(0,1.0)
CHBVOffsetEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVOffsetEntry.pack(side=LEFT)
CHBVOffsetEntry.delete(0,"end")
CHBVOffsetEntry.insert(0,0.0)
# add ADI logo Don't mess with this bit map data!
ADIlogo = """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"""
logo = PhotoImage(data=ADIlogo)
ADI1 = Label(frame2r, image=logo, anchor= "sw", height=49, width=116, compound="top")
ADI1.pack(side=TOP)

# Bottom Buttons
# Voltage channel A
CHAsb = Spinbox(frame3, width=4, values=CHvpdiv, command=BCHAlevel)
CHAsb.pack(side=LEFT)
CHAsb.delete(0,"end")
CHAsb.insert(0,0.5)
CHAlab = Label(frame3, text="CA V/Div", bg=COLORtrace1)
CHAlab.pack(side=LEFT)

CHAOffsetEntry = Entry(frame3, width=5)
CHAOffsetEntry.bind("<Return>", BOffsetA)
CHAOffsetEntry.pack(side=LEFT)
CHAOffsetEntry.delete(0,"end")
CHAOffsetEntry.insert(0,2.5)
CHAofflab = Label(frame3, text="CA V Pos", bg=COLORtrace1)
CHAofflab.pack(side=LEFT)
# Current channel A
CHAIsb = Spinbox(frame3, width=4, values=CHipdiv, command=BCHAIlevel)
CHAIsb.pack(side=LEFT)
CHAIsb.delete(0,"end")
CHAIsb.insert(0,50.0)
CHAIlab = Label(frame3, text="CA mA/Div", bg=COLORtrace3)
CHAIlab.pack(side=LEFT)

CHAIOffsetEntry = Entry(frame3, width=5)
CHAIOffsetEntry.bind("<Return>", BIOffsetA)
CHAIOffsetEntry.pack(side=LEFT)
CHAIOffsetEntry.delete(0,"end")
CHAIOffsetEntry.insert(0,0.0)
CHAIofflab = Label(frame3, text="CA I Pos", bg=COLORtrace3)
CHAIofflab.pack(side=LEFT)
# Voltage channel B
CHBsb = Spinbox(frame3, width=4, values=CHvpdiv, command=BCHBlevel)
CHBsb.pack(side=LEFT)
CHBsb.delete(0,"end")
CHBsb.insert(0,0.5)
CHBlab = Label(frame3, text="CB V/Div", bg=COLORtrace2)
CHBlab.pack(side=LEFT)

CHBOffsetEntry = Entry(frame3, width=5)
CHBOffsetEntry.bind("<Return>", BOffsetB)
CHBOffsetEntry.pack(side=LEFT)
CHBOffsetEntry.delete(0,"end")
CHBOffsetEntry.insert(0,2.5)
CHBofflab = Label(frame3, text="CB V Pos", bg=COLORtrace2)
CHBofflab.pack(side=LEFT)
# Current channel B
CHBIsb = Spinbox(frame3, width=4, values=CHipdiv, command=BCHBIlevel)
CHBIsb.pack(side=LEFT)
CHBIsb.delete(0,"end")
CHBIsb.insert(0,50.0)
CHBIlab = Label(frame3, text="CB mA/Div", bg=COLORtrace4)
CHBIlab.pack(side=LEFT)

CHBIOffsetEntry = Entry(frame3, width=5)
CHBIOffsetEntry.bind("<Return>", BIOffsetB)
CHBIOffsetEntry.pack(side=LEFT)
CHBIOffsetEntry.delete(0,"end")
CHBIOffsetEntry.insert(0,0.0)
CHBIofflab = Label(frame3, text="CB I Pos", bg=COLORtrace4)
CHBIofflab.pack(side=LEFT)

#===== Initalize device ======
#
AWGSync.set(1)
AWGAMode.set(2)
AWGBMode.set(2)
ConnectDevice()

# ================ Call main routine ===============================
#
#
root.update()               # Activate updated screens
# subwindow.update()
#
# Start sampling
Analog_In()


