#!/usr/bin/python
# ADALM1000 alice-desktop 1.0.py(w) (2-5-2016)
# For Python version > = 2.7.8
# With external module libpysmu (.pyd / .so for ADALM1000 )
# Created by D Mercer ()
#
import atexit
from collections import defaultdict
from operator import add

import libpysmu

import math
import time
import struct
import numpy.fft
from time import gmtime, strftime
import tkFont
from Tkinter import *
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
from tkSimpleDialog import askstring
from tkMessageBox import *
import webbrowser
#
TBicon = """
R0lGODlhIAAgAHAAACH5BAEAAAIALAAAAAAgACAAgQAAAP///wAAAAAAAAJJhI+py+0PYwtBWkDp
hTnv2XlfEobjUZZnmn4se72vJMtcbYN4ruz44uORgiodsfI4Im++2M5VW81OmBbVULxiRVrUsgsO
i8fUAgA7
"""

root=Tk()
root.title("ALICE DeskTop 1.0 (2-5-2016): ALM1000 Oscilloscope")
img = PhotoImage(data=TBicon)
root.call('wm', 'iconphoto', root._w, img)

root.minsize(100, 100)

# Values that can be modified
GRW = 720                   # Width of the grid 720 default
GRH = 390                   # Height of the grid 390 default
X0L = 55                    # Left top X value of grid
Y0T = 25                    # Left top Y value of grid
# Colors that can be modified
COLORframes = "#000080"   # Color = "#rrggbb" rr=red gg=green bb=blue, Hexadecimal values 00 - ff
COLORcanvas = "#000000"   # 100% black
COLORgrid = "#808080"     # 50% Gray
COLORzeroline = "#0000ff" # 100% blue
COLORtrace1 = "#00ff00"   # 100% green
COLORtrace2 = "#ff8000"   # 100% orange
COLORtrace3 = "#00ffff"   # 100% cyan
COLORtrace4 = "#ffff00"   # 100% yellow
COLORtrace5 = "#ff00ff"   # 100% magenta
COLORtraceR1 = "#008000"   # 50% green
COLORtraceR2 = "#804000"   # 50% orange
COLORtraceR3 = "#008080"   # 50% cyan
COLORtraceR4 = "#808000"   # 50% yellow
COLORtraceR5 = "#800080"   # 50% magenta
COLORtext = "#ffffff"     # 100% white
COLORtrigger = "#ff0000"  # 100% red
COLORsignalband = "#ff0000" # 100% red
# Button sizes that can be modified
Buttonwidth1 = 12
Buttonwidth2 = 8
# Set sample buffer size
# pysmu.setSampleRate(SAMPLErate)
# set initial trigger conditions
HoldOff = 0.01
SAMPLErate = 100000         # default Sample rate of the Scope channels
MinSamples = 1000
MaxSamples = 50000
DevID = "m1k"
UPDATEspeed = 0.1           # Update speed, default 0.1, for slower PC's a higher value is perhaps required
TRIGGERlevel = 2.5          # Triggerlevel in volts
# Vertical Sensitivity list in v/div
CHvpdiv = (0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0)
# Vertical Sensitivity list in mA/div
CHipdiv = (0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0)
# Time list in ms/div
TMpdiv = (0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 200.0, 500.0, 1000.0)
# AWG variables
AWGAAmplvalue = 0.0
AWGAOffsetvalue = 0.0
AWGAFreqvalue = 0.0
AWGAPhasevalue = 0
AWGAdelayvalue = 0
AWGADutyCyclevalue = 50
AWGAWave = 'dc'
AWGBAmplvalue = 0.0
AWGBOffsetvalue = 0.0
AWGBFreqvalue = 0.0
AWGBPhasevalue = 0
AWGBdelayvalue = 0
AWGBDutyCyclevalue = 50
AWGBWave = 'dc'
#
DCV1 = DCV2 = MinV1 = MaxV1 = MinV2 = MaxV2 = MidV1 = PPV1 = MidV2 = PPV2 = SV1 = SI1 = 0
DCI1 = DCI2 = MinI1 = MaxI1 = MinI2 = MaxI2 = MidI1 = PPI1 = MidI2 = PPI2 = SV2 = SI2 = 0
CHAperiod = CHAfreq = CHBperiod = CHBfreq = 0
# Calibration coefficients
CHAVGain = CHBVGain = 1.0
CHAVOffset = CHBVOffset = 0.0

# Initialisation of general variables
CHAOffset = CHBOffset = 2.5
CHAIOffset = CHBIOffset = 0.0

# Other global variables required in various routines
CANVASwidth = GRW + 2 * X0L # The canvas width
CANVASheight = GRH + 80     # The canvas height

Ymin = Y0T                  # Minimum position of screen grid (top)
Ymax = Y0T + GRH            # Maximum position of screen grid (bottom)

LONGsweep = False           # True if sweeps longer than 1 second
LONGchunk = LONGsweep       # True if chunk for longsweep is used for the trace

ADsignal1 = []              # Ain signal array channel A and B
# ADsignal2 = []             AD signal array channel B

T1Vline = []                # Voltage Trace line channel A
T2Vline = []                # Voltage Trace line channel B
T1Iline = []                # Current Trace line channel A
T2Iline = []                # Current Trace line channel B
TXYline = []                # XY Trace line
TXYRline = []               # XY reference trace line
Tmathline = []              # Math trace line
T1VRline = []               # V reference Trace line channel A
T2VRline = []               # V reference Trace line channel B
T1IRline = []               # I reference Trace line channel A
T2IRline = []               # I reference Trace line channel B
TMRline = []                # Math reference Trace line
Triggerline = []            # Triggerline

SHOWsamples = 1000          # Number of samples on the screen   
SCstart = 0                 # Start sample of the trace

# TriggerPos = 0.0            # Starting trigger possition of the trace
TRACES = 1                  # Number of traces 1 or 2
TRACESread = 0              # Number of traces that have been read from ALM
XYMode = 0                  # XY Mode flag
#
STARTfrequency = IntVar(0)  # Startfrequency
STARTfrequency.set(0) 
STOPfrequency = IntVar(0)   # Stopfrequency
STOPfrequency.set(20000)
ZEROstuffing = IntVar(0) # The zero stuffing value is 2 ** ZERO stuffing, calculated on initialize
ZEROstuffing.set(1)
NSteps = IntVar(0)  # number of frequency sweep steps
NSteps.set(0)
LoopNum = IntVar(0)
LoopNum.set(1)
FSweepMode = IntVar(0)
FSweepCont = IntVar(0)
#
RUNstatus = IntVar(0)       # 0 stopped, 1 start, 2 running, 3 stop and restart, 4 stop
PowerStatus = 1
TRIGGERsample = 0           # AD sample trigger point
# Spectrum Values that can be modified
DBdivlist = [1, 2, 3, 5, 10, 15, 20]    # dB per division
DBdivindex = IntVar(0)      # 10 dB/div as initial value
DBdivindex.set(4)
DBlevel = IntVar(0)     # Reference level
DBlevel.set(0)

GRWN = 720                  # Width of the grid 720 default
GRHN = 390                  # Height of the grid 390 default
X0LF = 37                   # Left top X value of grid
Y0TF = 25                   # Left top Y value of grid
# Other global variables required in various routines
GRWF = GRWN                      # Initialize GRW
GRHF = GRHN                      # Initialize GRH

CANVASwidthF = GRWF + 2 * X0LF     # The canvas width
CANVASheightF = GRHF + 80         # The canvas height

ADsignal1 = []                  # Audio trace channel 1

FFTbandwidth = 0                # The FFT bandwidth
FFTresultA = []                 # FFT result CHA
PhaseA = []
FFTresultB = []                 # FFT result CHB
PhaseB = []
# FFTwindow = 5             # FFTwindow 0=None (rectangular B=1), 1=Cosine (B=1.24), 2=Triangular non-zero endpoints (B=1.33),
                                # 3=Hann (B=1.5), 4=Blackman (B=1.73), 5=Nuttall (B=2.02), 6=Flat top (B=3.77)
FFTwindowname = "--"            # The FFT window name
FFTmemoryA = numpy.ones(1)       # The memory for averaging
PhaseMemoryA = numpy.ones(1)
FFTmemoryB = numpy.ones(1)       # The memory for averaging
PhaseMemoryB = numpy.ones(1)
SMPfftpwrTwo = IntVar(0)        # The power of two of SMPfft
SMPfftpwrTwo.set(11)
SMPfft = 2 ** SMPfftpwrTwo.get()    # Initialize

STOREtrace = False              # Store and display trace

FFTwindowshape = numpy.ones(SMPfft) # The FFT window curve
T1Fline = []                # Frequency Trace line channel A
T2Fline = []                # Frequency Trace line channel B
T1Pline = []                # Phase angle Trace line channel A - B
T2Pline = []                # Phase angle Trace line channel B - A
T1FRline = []               # F reference Trace line channel A
T1FRline = []               # F reference Trace line channel B
T1PRline = []               # Phase reference Trace line channel A - B
T2PRline = []               # Phase reference Trace line channel B - A
TFMline = []                # Frequency Math Trace
TFRMline = []               # Frequency reference Math Trace
TRACEmode = IntVar(0)       # 1 normal mode, 2 max hold mode, 3 average mode
TRACEmode.set(1)
TRACEaverage = IntVar(0)           # Number of average sweeps for average mode
TRACEaverage.set(8)
TRACEreset = True           # True for first new trace, reset max hold and averageing 
MinSamplesSA = 64
MaxSamplesSA = 65536
Vdiv = IntVar(0)
Vdiv.set(10)                # Number of vertical divisions
# Calibration coefficients
CHAVGain = CHBVGain = 1.0
CHAVOffset = CHBVOffset = 0.0
# ====== define libsmu funcions ======
class Smu(object):
    def __init__(self):
        atexit.register(libpysmu.cleanup)
        libpysmu.setup()
        self.load_channels()

    def load_channels(self):
        self.devices = libpysmu.get_dev_info()
        self.serials = {i: v[0] for i, v in enumerate(self.devices)}
        self.devices = [x[1] for x in self.devices]

        names = (chr(x) for x in xrange(65,91))
        channels = {names.next(): (i, v) for i, d in enumerate(self.devices)
                    for k, v in d.items()}
        self.channels = {k: Channel(k, self.serials[v[0]], v[1]) for k, v in channels.items()}

        device_channels = defaultdict(list)
        for k, v in channels.items():
            device_channels[v[0]].append(Channel(k, *v))
        self.devices = {i: Device(self.serials[i], device_channels[i]) for i, v
                        in enumerate(self.devices)}

    def ctrl_transfer(self, device, bm_request_type, b_request, wValue, wIndex,
                      data, wLength, timeout):
        data = str(data)
        if bm_request_type&0x80 == 0x80:
            if data == '0':
                data = '\x00'*wLength
        else:
            wLength = 0
        ret = libpysmu.ctrl_transfer(device, bm_request_type, b_request, wValue,
                                   wIndex, data, wLength, timeout)
        if bm_request_type&0x80 == 0x80:
            return map(ord, data)
        else:
            return ret

    def __repr__(self):
        return 'Devices: ' + str(self.devices)


class Device(object):
    def __init__(self, serial, channels):
        self.serial = serial
        self.channels = channels

    def get_samples(self, n_samples):
        """query device for a list of samples from all channels

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from all the device's channels
        """
        return libpysmu.get_all_inputs(self.serial, n_samples)

class Channel(object):
    def __init__(self, chan, dev_serial, signals):
        self.dev = dev_serial
        self.chan = ord(chan) - 65
        self.signals = {v: i for i, v in enumerate(signals)}

    def set_mode(self, mode):
        if mode == 'v' or mode == 'V':
            libpysmu.set_mode(self.dev, self.chan, 1)
            self.mode = 1
        elif mode == 'i' or mode == 'I':
            libpysmu.set_mode(self.dev, self.chan, 2)
            self.mode = 2
        elif mode == 'd' or mode == 'D':
            libpysmu.set_mode(self.dev, self.chan, 0)
            self.mode = 0
        else:
            raise ValueError('invalid mode')

    def arbitrary(self, *args, **kwargs):
        repeat = 0
        if 'repeat' in map(str.lower, kwargs.keys()):
            repeat = 1
        wave = map(float, reduce(add, [[s]*100*n for s, n in args]))
        return libpysmu.set_output_buffer(wave, self.dev, self.chan, self.mode, repeat)
    
    def get_samples(self, n_samples):
        """query channel for samples

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from the channel
        """
        return libpysmu.get_inputs(self.dev, self.chan, n_samples)

    def constant(self, val):
        """set output to a constant waveform"""
        return libpysmu.set_output_constant(self.dev, self.chan, self.mode, val)

    def square(self, midpoint, peak, period, phase, duty_cycle):
        """set output to a square waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 1, midpoint, peak, period, phase, duty_cycle)

    def sawtooth(self, midpoint, peak, period, phase):
        """set output to a sawtooth waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 2, midpoint, peak, period, phase, 42)

    def stairstep(self, midpoint, peak, period, phase):
        """set output to a stairstep waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 3, midpoint, peak, period, phase, 42)

    def sine(self, midpoint, peak, period, phase):
        """set output to a sinusoidal waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 4, midpoint, peak, period, phase, 42)

    def triangle(self, midpoint, peak, period, phase):
        """set output to a triangle waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 5, midpoint, peak, period, phase, 42)

    def __repr__(self):
        return 'Device: ' + str(self.dev) + '\nChannel: ' + str(self.chan) + '\nSignals: ' + str(self.signals)

# =========== Start widgets routines =============================
def BSaveConfig():
    global TgInput, TgEdge, SingleShot, AutoLevel
    global TRIGGERentry, TMsb, DisMode, Xsignal, Ysignal
    global CHAsb, CHAIsb, CHBsb, CHBIsb
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, MathTrace
    global CHAVPosEntry, CHAIPosEntry, CHBVPosEntry, CHBIPosEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1, MeasDCI1, MeasMinI1
    global MeasMaxI1, MeasMidI1, MeasPPI1, MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2
    global MeasPPV2, MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSV2, MeasRMSI1, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntry
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P, CutDC
    global FFTwindow, DBdivindex, DBlevel, TRACEmode, TRACEaverage, Vdiv
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
# open Config file for Write
    filename = asksaveasfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename, "w")
    ConfgFile.write('TRIGGERentry.delete(0,END)\n')
    ConfgFile.write('TRIGGERentry.insert(4, ' + TRIGGERentry.get() + ')\n')
    ConfgFile.write('TMsb.delete(0,END)\n')
    ConfgFile.write('TMsb.insert(0, ' + TMsb.get() + ')\n')
    ConfgFile.write('TgInput.set(' + str(TgInput.get()) + ')\n')
    ConfgFile.write('AutoLevel.set(' + str(AutoLevel.get()) + ')\n')
    ConfgFile.write('SingleShot.set(' + str(SingleShot.get()) + ')\n')
    ConfgFile.write('TgEdge.set(' + str(TgEdge.get()) + ')\n')
    ConfgFile.write('DisMode.set(' + str(DisMode.get()) + ')\n')
    ConfgFile.write('Xsignal.set(' + str(Xsignal.get()) + ')\n')
    ConfgFile.write('Ysignal.set(' + str(Ysignal.get()) + ')\n')
    #
    ConfgFile.write('ShowC1_V.set(' + str(ShowC1_V.get()) + ')\n')
    ConfgFile.write('ShowC1_I.set(' + str(ShowC1_I.get()) + ')\n')
    ConfgFile.write('ShowC2_V.set(' + str(ShowC2_V.get()) + ')\n')
    ConfgFile.write('ShowC2_I.set(' + str(ShowC2_I.get()) + ')\n')
    #
    ConfgFile.write('CHAVPosEntry.delete(0,END)\n')
    ConfgFile.write('CHAVPosEntry.insert(4, ' + CHAVPosEntry.get() + ')\n')
    ConfgFile.write('CHAIPosEntry.delete(0,END)\n')
    ConfgFile.write('CHAIPosEntry.insert(4, ' + CHAIPosEntry.get() + ')\n')
    ConfgFile.write('CHAsb.delete(0,END)\n')
    ConfgFile.write('CHAsb.insert(0, ' + CHAsb.get() + ')\n')
    ConfgFile.write('CHAIsb.delete(0,END)\n')
    ConfgFile.write('CHAIsb.insert(0, ' + CHAIsb.get() + ')\n')
    #
    ConfgFile.write('CHBVPosEntry.delete(0,END)\n')
    ConfgFile.write('CHBVPosEntry.insert(4, ' + CHBVPosEntry.get() + ')\n')
    ConfgFile.write('CHBIPosEntry.delete(0,END)\n')
    ConfgFile.write('CHBIPosEntry.insert(4, ' + CHBIPosEntry.get() + ')\n')
    ConfgFile.write('CHBsb.delete(0,END)\n')
    ConfgFile.write('CHBsb.insert(0, ' + CHBsb.get() + ')\n')
    ConfgFile.write('CHBIsb.delete(0,END)\n')
    ConfgFile.write('CHBIsb.insert(0, ' + CHBIsb.get() + ')\n')
    # AWG stuff
    ConfgFile.write('AWGAMode.set('+ str(AWGAMode.get()) + ')\n')
    ConfgFile.write('AWGATerm.set('+ str(AWGATerm.get()) + ')\n')
    ConfgFile.write('AWGAAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGAAmplEntry.insert(4, ' + AWGAAmplEntry.get() + ')\n')
    ConfgFile.write('AWGAOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGAOffsetEntry.insert(4, ' + AWGAOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGAFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGAFreqEntry.insert(4, ' + AWGAFreqEntry.get() + ')\n')
    ConfgFile.write('AWGAPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGAPhaseEntry.insert(4, ' + AWGAPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGADutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGADutyCycleEntry.insert(4, ' + AWGADutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGAShape.set(' + str(AWGAShape.get()) + ')\n')
    #
    ConfgFile.write('AWGBMode.set('+ str(AWGBMode.get()) + ')\n')
    ConfgFile.write('AWGBTerm.set('+ str(AWGBTerm.get()) + ')\n')
    ConfgFile.write('AWGBAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGBAmplEntry.insert(4, ' + AWGBAmplEntry.get() + ')\n')
    ConfgFile.write('AWGBOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGBOffsetEntry.insert(4, ' + AWGBOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGBFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGBFreqEntry.insert(4, ' + AWGBFreqEntry.get() + ')\n')
    ConfgFile.write('AWGBPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGBPhaseEntry.insert(4, ' + AWGBPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGBDutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGBDutyCycleEntry.insert(4, ' + AWGBDutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGBShape.set(' + str(AWGBShape.get()) + ')\n')
    #
    ConfgFile.write('AWGSync.set(' + str(AWGSync.get()) + ')\n')
    #
    ConfgFile.write('CHAVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    ConfgFile.write('CHBVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    ConfgFile.write('CHAVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    ConfgFile.write('MeasDCV1.set(' + str(MeasDCV1.get()) + ')\n')
    ConfgFile.write('MeasMinV1.set(' + str(MeasMinV1.get()) + ')\n')
    ConfgFile.write('MeasMaxV1.set(' + str(MeasMaxV1.get()) + ')\n')
    ConfgFile.write('MeasMidV1.set(' + str(MeasMidV1.get()) + ')\n')
    ConfgFile.write('MeasPPV1.set(' + str(MeasPPV1.get()) + ')\n')
    ConfgFile.write('MeasRMSV1.set(' + str(MeasRMSV1.get()) + ')\n')
    ConfgFile.write('MeasDCI1.set(' + str(MeasDCI1.get()) + ')\n')
    ConfgFile.write('MeasMinI1.set(' + str(MeasMinI1.get()) + ')\n')
    ConfgFile.write('MeasMaxI1.set(' + str(MeasMaxI1.get()) + ')\n')
    ConfgFile.write('MeasMidI1.set(' + str(MeasMidI1.get()) + ')\n')
    ConfgFile.write('MeasPPI1.set(' + str(MeasPPI1.get()) + ')\n')
    ConfgFile.write('MeasRMSI1.set(' + str(MeasRMSI1.get()) + ')\n')
    ConfgFile.write('MeasDCV2.set(' + str(MeasDCV2.get()) + ')\n')
    ConfgFile.write('MeasMinV2.set(' + str(MeasMinV2.get()) + ')\n')
    ConfgFile.write('MeasMaxV2.set(' + str(MeasMaxV2.get()) + ')\n')
    ConfgFile.write('MeasMidV2.set(' + str(MeasMidV2.get()) + ')\n')
    ConfgFile.write('MeasPPV2.set(' + str(MeasPPV2.get()) + ')\n')
    ConfgFile.write('MeasRMSV2.set(' + str(MeasRMSV2.get()) + ')\n')
    ConfgFile.write('MeasDCI2.set(' + str(MeasDCI2.get()) + ')\n')
    ConfgFile.write('MeasMinI2.set(' + str(MeasMinI2.get()) + ')\n')
    ConfgFile.write('MeasMaxI2.set(' + str(MeasMaxI2.get()) + ')\n')
    ConfgFile.write('MeasMidI2.set(' + str(MeasMidI2.get()) + ')\n')
    ConfgFile.write('MeasPPI2.set(' + str(MeasPPI2.get()) + ')\n')
    ConfgFile.write('MeasRMSI2.set(' + str(MeasRMSI2.get()) + ')\n')
    #
    ConfgFile.write('MeasAHW.set(' + str(MeasAHW.get()) + ')\n')
    ConfgFile.write('MeasALW.set(' + str(MeasALW.get()) + ')\n')
    ConfgFile.write('MeasADCy.set(' + str(MeasADCy.get()) + ')\n')
    ConfgFile.write('MeasAPER.set(' + str(MeasAPER.get()) + ')\n')
    ConfgFile.write('MeasAFREQ.set(' + str(MeasAFREQ.get()) + ')\n')
    ConfgFile.write('MeasBHW.set(' + str(MeasBHW.get()) + ')\n')
    ConfgFile.write('MeasBLW.set(' + str(MeasBLW.get()) + ')\n')
    ConfgFile.write('MeasBDCy.set(' + str(MeasBDCy.get()) + ')\n')
    ConfgFile.write('MeasBPER.set(' + str(MeasBPER.get()) + ')\n')
    ConfgFile.write('MeasBFREQ.set(' + str(MeasBFREQ.get()) + ')\n')
    ConfgFile.write('MeasPhase.set(' + str(MeasPhase.get()) + ')\n')
    #
    ConfgFile.write('MathTrace.set(' + str(MathTrace.get()) + ')\n')
    #
    ConfgFile.write('CHAIGainEntry.delete(0,END)\n')
    ConfgFile.write('CHAIGainEntry.insert(4, ' + CHAIGainEntry.get() + ')\n')
    ConfgFile.write('CHBIGainEntry.delete(0,END)\n')
    ConfgFile.write('CHBIGainEntry.insert(4, ' + CHBIGainEntry.get() + ')\n')
    ConfgFile.write('CHAIOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAIOffsetEntry.insert(4, ' + CHAIOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBIOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBIOffsetEntry.insert(4, ' + CHBIOffsetEntry.get() + ')\n')
    # Spectrum stuff
    ConfgFile.write('ShowC1_VdB.set(' + str(ShowC1_VdB.get()) + ')\n')
    ConfgFile.write('ShowC1_P.set(' + str(ShowC1_P.get()) + ')\n')
    ConfgFile.write('ShowC2_VdB.set(' + str(ShowC2_VdB.get()) + ')\n')
    ConfgFile.write('ShowC2_P.set(' + str(ShowC2_P.get()) + ')\n')
    #
    ConfgFile.write('FFTwindow.set(' + str(FFTwindow.get()) + ')\n')
    ConfgFile.write('SMPfftpwrTwo.set(' + str(SMPfftpwrTwo.get()) + ')\n')
    ConfgFile.write('STARTfrequency.set(' + str(STARTfrequency.get()) + ')\n')
    ConfgFile.write('STOPfrequency.set(' + str(STOPfrequency.get()) + ')\n')
    ConfgFile.write('ZEROstuffing.set(' + str(ZEROstuffing.get()) + ')\n')
    ConfgFile.write('Vdiv.set(' + str(Vdiv.get()) + ')\n')
    ConfgFile.write('NSteps.set(' + str(NSteps.get()) + ')\n')
    ConfgFile.write('FSweepMode.set(' + str(FSweepMode.get()) + ')\n')
    #
    ConfgFile.write('DBdivindex.set(' + str(DBdivindex.get()) + ')\n')
    ConfgFile.write('DBlevel.set(' + str(DBlevel.get()) + ')\n')
    ConfgFile.write('TRACEmode.set(' + str(TRACEmode.get()) + ')\n')
    ConfgFile.write('TRACEaverage.set(' + str(TRACEaverage.get()) + ')\n')
    ConfgFile.write('CutDC.set(' + str(CutDC.get()) + ')\n')
    #
    ConfgFile.close()

def BLoadConfig():
    global TgInput, TgEdge, SingleShot, AutoLevel
    global TRIGGERentry, TMsb, DisMode, Xsignal, Ysignal
    global CHAsb, CHAIsb, CHBsb, CHBIsb
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, MathTrace
    global CHAVPosEntry, CHAIPosEntry, CHBVPosEntry, CHBIPosEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1, MeasDCI1, MeasMinI1
    global MeasMaxI1, MeasMidI1, MeasPPI1, MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2
    global MeasPPV2, MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSV2, MeasRMSI1, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntry
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P, CutDC
    global FFTwindow, DBdivindex, DBlevel, TRACEmode, TRACEaverage, Vdiv
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
    # Read configuration values from file
    filename = askopenfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename)
    for line in ConfgFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skiping " + line.rstrip()
    ConfgFile.close()
    
def BSaveScreen():
    global CANVASwidth, CANVASheight
    global COLORtext
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # temp chnage text corlor to black
    COLORtext = "#000000"
    UpdateTimeScreen()
    # first save postscript file
    ca.postscript(file=filename, height=CANVASheight, width=CANVASwidth, colormode='color')
    # now convert to bit map
    # img = Image.open("screen_shot.eps")
    # img.save("screen_shot.gif", "gif")
    COLORtext = "#ffffff"
    UpdateTimeScreen()

def BSaveData():
    global ADsignal1 # , ADsignal2
    global SAMPLErate, SHOWsamples
    # open file to save data
    filename = asksaveasfilename(defaultextension = ".csv", filetypes=[("Comma Separated Values", "*.csv")])
    DataFile = open(filename, 'a')
    DataFile.write( 'Sample-#, CA-V, CA-I, CB-V, CB-I \n' )
    for index in range(int(SHOWsamples)):
        SPA = ADsignal1[0][index]
        SPB = ADsignal1[1][index]
        DataFile.write( str(index) + ', ' + str(SPA[0]) + ', ' + str(SPA[1]) + ', '
                        + str(SPB[0]) + ', ' + str(SPB[1]) + '\n')
    DataFile.close()

def BHelp():
    # open a URL, in this case, the ALICE user's guide
    url = "https://wiki.analog.com/university/tools/alice-1.0-users-guide"
    webbrowser.open(url,new=2)
    
def BSnapShot():
    global T1Vline, T2Vline, T1Iline, T2Iline
    global TXYline, Tmathline, TMRline, TXYRline
    global T1VRline, T2VRline, T1IRline, T2IRline
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, ShowMath, MathTrace, DisMode

    if ShowC1_V.get() == 1:
        T1VRline = T1Vline               # V reference Trace line channel A
    if ShowC2_V.get() == 1:
        T2VRline = T2Vline               # V reference Trace line channel B
    if ShowC1_I.get() == 1:
        T1IRline = T1Iline               # I reference Trace line channel A
    if ShowC2_I.get() == 1:
        T2IRline = T2Iline               # I reference Trace line channel B
    if MathTrace.get() > 0:
        TMRline = Tmathline              # Math reference Trace line
    if DisMode.get() == 1:
        TXYRline = TXYline               # XY reference trace line    
#
def BSaveCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntry
    global DevID

    devidstr = DevID[17:31]
    filename = devidstr + "_O.cal"
    CalFile = open(filename, "w")
    #
    CalFile.write('CHAVGainEntry.delete(0,END)\n')
    CalFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    CalFile.write('CHBVGainEntry.delete(0,END)\n')
    CalFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    CalFile.write('CHAVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    CalFile.write('CHBVOffsetEntry.delete(0,END)\n')
    CalFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    CalFile.write('CHAIGainEntry.delete(0,END)\n')
    CalFile.write('CHAIGainEntry.insert(4, ' + CHAIGainEntry.get() + ')\n')
    CalFile.write('CHBIGainEntry.delete(0,END)\n')
    CalFile.write('CHBIGainEntry.insert(4, ' + CHBIGainEntry.get() + ')\n')
    CalFile.write('CHAIOffsetEntry.delete(0,END)\n')
    CalFile.write('CHAIOffsetEntry.insert(4, ' + CHAIOffsetEntry.get() + ')\n')
    CalFile.write('CHBIOffsetEntry.delete(0,END)\n')
    CalFile.write('CHBIOffsetEntry.insert(4, ' + CHBIOffsetEntry.get() + ')\n')
    #
    CalFile.close()

def BLoadCal():
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntry
    global DevID

    devidstr = DevID[17:31]
    filename = devidstr + "_O.cal"
    try:
        CalFile = open(filename)
        for line in CalFile:
            exec( line.rstrip() )
        CalFile.close()
    except:
        print "Cal file for this device not found"
#
def donothing():
    global XYMode

def BShowCurvesAll():
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    ShowC1_V.set(1)
    ShowC1_I.set(1)
    ShowC2_V.set(1)
    ShowC2_I.set(1)

def BShowCurvesNone():
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    ShowC1_V.set(0)
    ShowC1_I.set(0)
    ShowC2_V.set(0)
    ShowC2_I.set(0)
    
def BTriggerEdge():
    global TgEdge
                                       
# TRIGCOND trigcondRisingPositive = 0
# TRIGCOND trigcondFallingNegative = 1

def BTrigger50p():
    global TgInput, TRIGGERlevel, TRIGGERentry
    global DCV1, DCV2, DCI1, DCI2

    if (TgInput.get() == 0):
        DCString = "0.0"
    elif (TgInput.get() == 1 ):
        DCString = ' {0:.2f} '.format(DCV1)
    elif (TgInput.get() == 2 ):
        DCString = ' {0:.2f} '.format(DCI1)
    elif (TgInput.get() == 3 ):
        DCString = ' {0:.2f} '.format(DCV2)
    elif (TgInput.get() == 4 ):
        DCString = ' {0:.2f} '.format(DCI2)

    TRIGGERlevel = eval(DCString)
    TRIGGERentry.delete(0,END)
    TRIGGERentry.insert(4, DCString)
    # set new trigger level
    
    UpdateTimeTrace()           # Always Update
    
def BTriggerMode(): # place holder for future hardware triggering if implemented
    global TgInput

#    if (TgInput.get() == 0):
        # no trigger
         
#    elif (TgInput.get() == 1):
        # trigger source set to detector of analog in channels
         
        # auto trigger timeout value
        
#    elif (TgInput.get() == 2):
        # trigger source set to detector of analog in channels
         
        # 0 disables auto trigger
        
    
def BTriglevel(event):
    global TRIGGERlevel, TRIGGERentry

    # evalute entry string to a numerical value
    try:
        TRIGGERlevel = float(eval(TRIGGERentry.get()))
    except:
        TRIGGERentry.delete(0,END)
        TRIGGERentry.insert(0, TRIGGERlevel)
    # set new trigger level
    
    UpdateTimeTrace()           # Always Update

def BHoldOff(event):
    global HoldOff, HoldOffentry

    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
        
def Bcloseexit():
    global RUNstatus
    
    RUNstatus.set(0)
    # pysmu.pause()
    # close ALM and exit
    root.destroy()
    exit()

def BStart():
    global RUNstatus, PowerStatus, devx, PwrBt, DevID
    if DevID == "No Device":
        showwarning("WARNING","No Device Plugged In!")
    else:
        if PowerStatus == 0:
            PowerStatus = 1
            PwrBt.config(bg="green",text="PWR-On")
            devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power
        if (RUNstatus.get() == 0):
            RUNstatus.set(1)
        # pysmu.start(stopOnClose=True)
    # UpdateTimeScreen()          # Always Update

def BStop():
    global RUNstatus
    
    if (RUNstatus.get() == 1):
        RUNstatus.set(0)
        # pysmu.pause()
    elif (RUNstatus.get() == 2):
        RUNstatus.set(3)
    elif (RUNstatus.get() == 3):
        RUNstatus.set(3)
    elif (RUNstatus.get() == 4):
        RUNstatus.set(3)
    UpdateTimeScreen()          # Always Update

def BPower():
    global RUNstatus, PowerStatus, devx, PwrBt

    if (RUNstatus.get() == 1):
        BStop()
    if PowerStatus == 1:
        PowerStatus = 0
        PwrBt.config(bg="red",text="PWR-Off")
        devx.ctrl_transfer(DevID, 0x40, 0x50, 49, 0, 0, 0, 100) # turn off analog power
    else:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power
    
def BTime():
    global TIMEdiv, TMsb
    global SAMPLErate, RUNstatus

    try:
        TIMEdiv = float(eval(TMsb.get()))
    except:
        TIMEdiv = 1.0
        # change sample rate based on time scale
        # Currently 100000
    
    # SAMPLErate = 100000
        # pysmu.pause()
        # pysmu.setSampleRate(SAMPLErate)
        # pysmu.start(stopOnClose=True)
    BAWGEnab()
        
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)
        # pysmu.start(stopOnClose=True)

    UpdateTimeTrace()           # Always Update

def BCHAlevel():
    global CHAsb
    
    try:
        CH1vpdvLevel = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvLevel)
    UpdateTimeTrace()           # Always Update

def BCHAIlevel():
    global CHAIsb
    
    try:
        CH1ipdvLevel = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1ipdvLevel)
    UpdateTimeTrace()           # Always Update

def BCHBlevel():
    global CHBsb
    
    try:
        CH2vpdvLevel = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvLevel)
    UpdateTimeTrace()           # Always Update    

def BCHBIlevel():
    global CHBIsb
    
    try:
        CH2ipdvLevel = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2ipdvLevel)
    UpdateTimeTrace()           # Always Update    

def BOffsetA(event):
    global CHAOffset, CHAVPosEntry

    # evalute entry string to a numerical value
    try:
        CHAOffset = float(eval(CHAVPosEntry.get()))
    except:
        CHAVPosEntry.delete(0,END)
        CHAVPosEntry.insert(0, CHAOffset)
    # set new offset level
    UpdateTimeTrace()           # Always Update

def BIOffsetA(event):
    global CHAIOffset, CHAIPosEntry

    # evalute entry string to a numerical value
    try:
        CHAIOffset = float(eval(CHAIPosEntry.get()))
    except:
        CHAIPosEntry.delete(0,END)
        CHAIPosEntry.insert(0, CHAIOffset)
    # set new offset level
    UpdateTimeTrace()           # Always Update

def BOffsetB(event):
    global CHBOffset, CHBVPosEntry

    # evalute entry string to a numerical value
    try:
        CHBOffset = float(eval(CHBVPosEntry.get()))
    except:
        CHBVPosEntry.delete(0,END)
        CHBVPosEntry.insert(0, CHBOffset)
    # set new offset level
    UpdateTimeTrace()           # Always Update

def BIOffsetB(event):
    global CHBIOffset, CHBIPosEntry

    # evalute entry string to a numerical value
    try:
        CHBIOffset = float(eval(CHBIPosEntry.get()))
    except:
        CHBIPosEntry.delete(0,END)
        CHBIPosEntry.insert(0, CHBIOffset)
    # set new offset level
    UpdateTimeTrace()           # Always Update

def BDisMode():
    global DisMode, XYMode

    if DisMode.get() == 0:
        XYMode = 0
    else:
        XYMode = 1
    UpdateTimeTrace()           # Always Update
    
# ========================= Main routine ====================================
def Analog_In():
    global RUNstatus, SingleShot, RUNmode
    while (True):       # Main loop
        # RUNstatus = 1 : Open Acquisition
        # RUNstatus = 2
        if (RUNstatus.get() == 1) or (RUNstatus.get() == 2):
            if RUNmode.get() == 0:
                Analog_Time_In()
            if RUNmode.get() == 1:
                Analog_Freq_In()
            if RUNmode.get() == 2:
                Analog_Time_In()
                Analog_Freq_In()
        root.update_idletasks()
        root.update()
        
def Analog_Time_In():   # Read the analog data and store the data into the arrays
    global ADsignal1 # , ADsignal2
    global AWGSync, AWGAMode, AWGBMode, TMsb, HoldOff, HoldOffentry
    global CHA, CHB, Both, devx
    global TRACES, TRACESread
    global RUNstatus, SingleShot
    global TIMEdiv1x, TIMEdiv
    global SAMPLErate, SHOWsamples, MinSamples, MaxSamples
    global LONGsweep, LONGchunk
    global UPDATEspeed, TRACErefresh
    global SCREENrefresh, DCrefresh
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2
    global SV1, SI1, SV2, SI2
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAVPosEntry, CHAIPosEntry, CHBVPosEntry, CHBIPosEntry
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntr
 
    # get time scale
    TIMEdiv = eval(TMsb.get())
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    #
    # Do input probe Calibration CH1VGain, CH2VGain, CH1VOffset, CH2VOffset
    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
#
    try:
        CurOffA = float(CHAIOffsetEntry.get())/1000 # convert to Amps
    except:
        CurOffA = 0.0
    try:
        CurOffB = float(CHBIOffsetEntry.get())/1000 # convert to Amps
    except:
        CurOffB = 0.0
    try:
        CurGainA = float(CHAIGainEntry.get())
    except:
        CurGainA = 1.0
    try:
        CurGainB = float(CHBIGainEntry.get())
    except:
        CurGainB = 1.0
#
    hldn = int(HoldOff * 100 )
    twoscreens = int(SAMPLErate * 20.0 * TIMEdiv / 1000.0) # number of samples to acquire, 2 screen widths
    if hldn > MaxSamples-twoscreens:
        hldn = MaxSamples-twoscreens
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, hldn/100)
    SHOWsamples = twoscreens + hldn    
    if SHOWsamples > MaxSamples: # or a Max of 50,000 samples
        SHOWsamples = MaxSamples
    if SHOWsamples < MinSamples: # or a Min of 1000 samples
        SHOWsamples = MinSamples
# Starting acquisition This is a HACK to get around non-continous AWG mode!
    # restat AWGs if indicated
    if AWGSync.get() > 0:
        BAWGEnab()
    # 
    time.sleep(0.001)
    ADsignal1 = Both.get_samples(SHOWsamples) # get samples for both channel A and B
    # waite to finish then return to open termination
    devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set CHA 2.5 V switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set CHA GND switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
    if AWGAMode.get() == 3: # Split Input / Output mode
        devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
    if AWGBMode.get() == 3: # Split Input / Output mode
        devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
    DCV1 = 0.0 # initalize measurment variables
    DCI1 = 0.0
    DCV2 = 0.0
    DCI2 = 0.0
    SV1 = 0.0
    SV2 = 0.0
    SI1 = 0.0
    SI2 = 0.0
    MinV1 = 50.0
    MaxV1 = -50.0
    MinV2 = 50.0
    MaxV2 = -50.0
    MinI1 = 200.0
    MaxI1 = -200.0
    MinI2 = 200.0
    MaxI2 = -200.0
    for index in range(SHOWsamples): # calculate measurements
        if index > hldn:
            SPA = ADsignal1[0][index]
            SPB = ADsignal1[1][index]
            VAdata = float(SPA[0])
            IAdata = float(SPA[1])
            VBdata = float(SPB[0])
            IBdata = float(SPB[1])
            DCV1 += VAdata # Sum for average voltage 
            DCI1 += IAdata # Sum for average current
            VA = (VAdata - InOffA) * InGainA
            VA2 = VA * VA
            SV1 += VA2 # Sum for RMS voltage
            IA = (IAdata-CurOffA) * CurGainA
            IA2 = IA * IA
            SI1 += IA2 # Sum for RMS current 
            if VAdata > MaxV1:
                MaxV1 = VAdata
            if VAdata < MinV1:
                MinV1 = VAdata
            if IAdata > MaxI1:
                MaxI1 = IAdata
            if IAdata < MinI1:
                MinI1 = IAdata
            DCV2 += VBdata # Sum for average voltage
            DCI2 += IBdata # Sum for average current
            VB = (VBdata - InOffB) * InGainB
            VB2 = VB * VB
            SV2 += VB2 # Sum for RMS voltage 
            IB = (IBdata-CurOffB) * CurGainB
            IB2 = IB * IB
            SI2 += IB2 # Sum for RMS current 
            if VBdata > MaxV2:
                MaxV2 = VBdata
            if VBdata < MinV2:
                MinV2 = VBdata
            if IBdata > MaxI2:
                MaxI2 = IBdata
            if IBdata < MinI2:
                MinI2 = IBdata
# DC value = average of the data record
    numsamples = SHOWsamples - hldn
    DCV1 = DCV1 / numsamples
    DCI1 = DCI1 / numsamples
    DCV2 = DCV2 / numsamples
    DCI2 = DCI2 / numsamples
# RMS value = square root of average of the data record squared
    SV1 = SV1 / numsamples
    SV1 = math.sqrt(SV1)
    SI1 = SI1 / numsamples
    SI1 = math.sqrt(SI1)
    SV2 = SV2 / numsamples
    SV2 = math.sqrt(SV2)
    SI2 = SI2 / numsamples
    SI2 = math.sqrt(SI2)
    #
    DCV1 = (DCV1 - InOffA) * InGainA
    DCV2 = (DCV2 - InOffB) * InGainB
    MinV1 = (MinV1 - InOffA) * InGainA
    MaxV1 = (MaxV1 - InOffA) * InGainA
    MinV2 = (MinV2 - InOffB) * InGainB
    MaxV2 = (MaxV2 - InOffB) * InGainB
    # convert to mA
    DCI1 = (DCI1-CurOffA) * CurGainA * 1000
    DCI2 = (DCI2-CurOffB) * CurGainB * 1000
    MinI1 = (MinI1-CurOffA) * CurGainA * 1000
    MinI2 = (MinI2-CurOffB) * CurGainB * 1000
    MaxI1 = (MaxI1-CurOffA) * CurGainA * 1000
    MaxI2 = (MaxI2-CurOffB) * CurGainB * 1000
    SI1 = SI1 * 1000
    SI2 = SI2 * 1000
    #
    TRACESread = 2
    #
    UpdateTimeAll()         # Update Data, trace and screen
    if SingleShot.get() == 1: # Single shot manual trigger is on
        RUNstatus.set(0)

    # RUNstatus = 3: Stop
    # RUNstatus = 4: Stop and restart
    if (RUNstatus.get() == 3) or (RUNstatus.get() == 4):
        if RUNstatus.get() == 3:
            RUNstatus.set(0)
            # pysmu.pause()      # Status is stopped 
        if RUNstatus.get() == 4:          
            RUNstatus.set(1)
            # pysmu.start(stopOnClose=True)# Status is (re)start

        UpdateTimeScreen()
        # Update tasks and screens by TKinter
        # update screens

def FindRisingEdge():
    global ADsignal1, MinV1, MaxV1, MinV2, MaxV2, HoldOff
    global InOffA, InGainA, InOffB, InGainB, CHABphase
    global SHOWsamples, SAMPLErate, CHAperiod, CHAfreq, CHBperiod, CHBfreq
    global CHAHW, CHALW, CHADCy, CHBHW, CHBLW, CHBDCy, ShowC1_V, ShowC2_V
    
# find first rising edge
    Nmax = SHOWsamples - 8
    anr1 = 0
    bnr1 = 0
    anf1 = 0
    bnf1 = 0
    anr2 = 0
    bnr2 = 0
    if ShowC1_V.get() == 1:
        n = int(HoldOff * 100)
        MidV1 = (MaxV1+MinV1)/2
        Mid1V2 = 0.99 * MidV1 # Hysteresis to avoid triggering on noise
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while ( ChInput >= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
#
        anr1 = n
        n = n + 1
# skip over falling edge
        Mid1V2 = 1.01 * MidV1
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput >= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
#
        anf1 = n
        n = n + 1
# find second rising edge
        Mid1V2 = 0.99 * MidV1
        SPA = ADsignal1[0][int(n)]
        ChInput = (float(SPA[0]) - InOffA) * InGainA
        while ( ChInput >= Mid1V2) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
        while (ChInput <= MidV1) and n < Nmax:
            n = n + 1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
            SPA = ADsignal1[0][int(n-1)]
        anr2 = n
        CHAperiod = float(((anr2 - anr1) * 1000.0) / SAMPLErate) # time in mSec
        CHAHW = float(((anf1 - anr1) * 1000.0) / SAMPLErate)
        CHALW = float(((anr2 - anf1) * 1000.0) / SAMPLErate)
        CHADCy = float(anf1 - anr1) / float(anr2 - anr1) * 100.0 # in percent
        CHAfreq = float(SAMPLErate / (anr2 - anr1))
# search channel B
    if ShowC2_V.get() == 1:
        n = int(HoldOff * 100)
        MidV2 = (MaxV2+MinV2)/2
        Mid2V2 = 0.99 * MidV2 # Hysteresis to avoid triggering on noise
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while ( ChInput >= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
#
        bnr1 = n
        n = n + 1
# skip over falling edge
        Mid2V2 = 1.01 * MidV2
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput >= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
#
        bnf1 = n
        n = n + 1
# find second rising edge
        Mid2V2 = 0.99 * MidV2
        SPB = ADsignal1[1][int(n)]
        ChInput = (float(SPB[0]) - InOffB) * InGainB
        while ( ChInput >= Mid2V2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
        while (ChInput <= MidV2) and n < Nmax:
            n = n + 1
            SPB = ADsignal1[1][int(n)]
            ChInput = (float(SPB[0]) - InOffB) * InGainB
            SPB = ADsignal1[1][int(n-1)]
        bnr2 = n
        CHBperiod = float(((bnr2 - bnr1) * 1000.0) / SAMPLErate) # time in mSec
        CHBHW = float(((bnf1 - bnr1) * 1000.0) / SAMPLErate)
        CHBLW = float(((bnr2 - bnf1) * 1000.0) / SAMPLErate)
        CHBDCy = float(bnf1 - bnr1) / float(bnr2 - bnr1) * 100.0 # in percent
        CHBfreq = float(SAMPLErate / (bnr2 - bnr1))
        #
    try:
        CHABphase = 360.0*(float((bnr1 - anr1) * 1000.0 / SAMPLErate))/CHAperiod
    except:
        CHABphase = 0.0
    if CHABphase < 0.0:
        CHABphase = CHABphase + 360.0

def SetDigOut(): # set digital outputs
    global DI0, DI1, DI2, DI3, DI4, DI5, DI6, DI7
    global DevID, devx    
# sending 0x50 = set to 0, 0x51 = set to 1
    if DI0.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI0.get(), 0, 0, 0, 0, 100) # set PIO 0
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 0, 0, 0, 1, 100)
    if DI1.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI1.get(), 1, 0, 0, 0, 100) # set PIO 1
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 1, 0, 0, 1, 100)
    if DI2.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI2.get(), 2, 0, 0, 0, 100) # set PIO 2
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 2, 0, 0, 1, 100)
    if DI3.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI3.get(), 3, 0, 0, 0, 100) # set PIO 3
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 3, 0, 0, 1, 100)
    if DI4.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI4.get(), 4, 0, 0, 0, 100) # set PIO 4
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 4, 0, 0, 1, 100)
    if DI5.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI5.get(), 5, 0, 0, 0, 100) # set PIO 5
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 5, 0, 0, 1, 100)
    if DI6.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI6.get(), 6, 0, 0, 0, 100) # set PIO 6
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 6, 0, 0, 1, 100)
    if DI7.get() > 0:
        devx.ctrl_transfer(DevID, 0x40, DI7.get(), 7, 0, 0, 0, 100) # set PIO 7
    else:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 7, 0, 0, 1, 100)
        
def UpdateTimeAll():        # Update Data, trace and screen
    
    MakeTimeTrace()         # Update the traces
    # GetDigIn()          # Update digital inputs display
    UpdateTimeScreen()      # Update the screen 

def UpdateTimeTrace():      # Update trace and screen
    
    MakeTimeTrace()         # Update traces
    UpdateTimeScreen()      # Update the screen

def UpdateTimeScreen():     # Update screen with trace and text
    
    MakeTimeScreen()        # Update the screen
    root.update()       # Activate updated screens    

def MakeTimeTrace():    # Make the traces
    global ADsignal1 # , ADsignal2
    global T1Vline, T2Vline, T1Iline, T2Iline
    global TXYline, Tmathline
    global Triggerline, TgInput, TgEdge, HoldOff, HoldOffentry
    global X0L, Y0T
    global GRW, GRH
    global Ymin, Ymax
    global SHOWsamples, ZOHold, AWGBMode
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I
    global TRACES
    global TRACESread
    global RUNstatus
    global XYMode, DisMode
    global Xsignal, Ysignal
    global CHAsb, CHBsb
    global CHAOffset, CHBOffset
    global CHAIsb, CHBIsb
    global CHAIOffset, CHBIOffset
    global TMpdiv       # Array with time / div values in ms
    global TMsb         # Time per div spin box variable
    global TIMEdiv      # current spin box value
    global SAMPLErate
    global SCstart
    global TRIGGERsample
    global TRIGGERlevel, TRIGGERentry, AutoLevel
    global InOffA, InGainA, InOffB, InGainB
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAVPosEntry, CHAIPosEntry, CHAVPosEntry, CHBIPosEntry
    global CHAIGainEntry, CHBIGainEntry, CHAIOffsetEntry, CHBIOffsetEntry
 
    # Set the TRACEsize variable
    TRACEsize = SHOWsamples               # Set the trace length
    # get time scale
    try:
        TIMEdiv = float(eval(TMsb.get()))
    except:
        TIMEdiv = 0.1
    # prevent divide by zero error
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    # get the vertical ranges
    try:
        CH1pdvRange = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvRange)
    try:
        CH2pdvRange = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvRange)
    try:
        CH1IpdvRange = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1IpdvRange)
    try:
        CH2IpdvRange = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2IpdvRange)
    # get the vertical offsets
    try:
        CHAOffset = float(eval(CHAVPosEntry.get()))
    except:
        CHAVPosEntry.delete(0,END)
        CHAVPosEntry.insert(0, CHAOffset)
    try:
        CHAIOffset = float(eval(CHAIPosEntry.get()))
    except:
        CHAIPosEntry.delete(0,END)
        CHAIPosEntry.insert(0, CHAIOffset)
    try:
        CHBOffset = float(eval(CHBVPosEntry.get()))
    except:
        CHBVPosEntry.delete(0,END)
        CHBVPosEntry.insert(0, CHBOffset)
    try:
        CHBIOffset = float(eval(CHBIPosEntry.get()))
    except:
        CHBIPosEntry.delete(0,END)
        CHBIPosEntry.insert(0, CHBIOffset)
    # prevent divide by zero error
    if CH1pdvRange < 0.001:
        CH1pdvRange = 0.001
    if CH2pdvRange < 0.001:
        CH2pdvRange = 0.001
    if CH1IpdvRange < 0.1:
        CH1IpdvRange = 0.1
    if CH2IpdvRange < 0.1:
        CH2IpdvRange = 0.1
        
    # Continue with finding the trigger sample and drawing the traces 
    if TRACEsize == 0:                          # If no trace, skip rest of this routine
        T1Vline = []                            # Trace line channel A V
        T2Vline = []                            # Trace line channel B V
        T1Iline = []
        T2Iline = []
        TXYline = []
        Tmathline = []
        return() 

    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
# 
    try:
        CurOffA = float(CHAIOffsetEntry.get())/1000 # convert to Amps
    except:
        CurOffA = 0.0
    try:
        CurOffB = float(CHBIOffsetEntry.get())/1000 # convert to Amps
    except:
        CurOffB = 0.0
    try:
        CurGainA = float(CHAIGainEntry.get())
    except:
        CurGainA = 1.0
    try:
        CurGainB = float(CHBIGainEntry.get())
    except:
        CurGainB = 1.0
# Find trigger sample
    try:
        if AutoLevel.get() == 1:
            if TgInput.get() == 1:
                TRIGGERlevel = (MinV1 + MaxV1)/2
            if TgInput.get() == 2:
                TRIGGERlevel = (MinI1 + MaxI1)/2
            if TgInput.get() == 3:
                TRIGGERlevel = (MinV2 + MaxV2)/2
            if TgInput.get() == 4:
                TRIGGERlevel = (MinI2 + MaxI2)/2
            TRIGGERentry.delete(0,"end")
            TRIGGERentry.insert(0,TRIGGERlevel)
        else:
            TRIGGERlevel = eval(TRIGGERentry.get())
    except:
        TRIGGERentry.delete(0,END)
        TRIGGERentry.insert(0, TRIGGERlevel)
#
    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
    # Start from first sample after HoldOff
    hldn = int(HoldOff * 100)
    TRIGGERsample = hldn # + (TriggerPos * SAMPLErate)
#
    if LONGchunk == False:          # For long sweeps no trigger        
        Nmax = int(TRACEsize / 1.5) # first 2/3 of data set
        DX = 0
        n = TRIGGERsample
        if (TgInput.get() == 1 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinV1:
                TRIGGERlevel2 = MinV1
            if TRIGGERlevel2 > MaxV1:
                TRIGGERlevel2 = MaxV1
            SPA = ADsignal1[0][int(n)]
            ChInput = (float(SPA[0]) - InOffA) * InGainA
            Prev = ChInput
            while ( ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                Prev = ChInput
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
            DY = ChInput - Prev
            try:
                DX = (TRIGGERlevel - Prev)/DY # calculate interpolated trigger point
            except:
                DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinV1:
                    TRIGGERlevel2 = MinV1
                if TRIGGERlevel2 > MaxV1:
                    TRIGGERlevel2 = MaxV1
                SPA = ADsignal1[0][int(n)]
                ChInput = (float(SPA[0]) - InOffA) * InGainA
                Prev = ChInput
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = (float(SPA[0]) - InOffA) * InGainA
                while (ChInput >= TRIGGERlevel) and n < Nmax:
                    Prev = ChInput
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = (float(SPA[0]) - InOffA) * InGainA
                DY = Prev - ChInput
                try:
                    DX = (Prev - TRIGGERlevel)/DY # calculate interpolated trigger point
                except:
                    DX = 0
        elif (TgInput.get() == 2 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinI1:
                TRIGGERlevel2 = MinI1
            if TRIGGERlevel2 > MaxI1:
                TRIGGERlevel2 = MaxI1
            SPA = ADsignal1[0][int(n)]
            ChInput = float(SPA[1])*1000
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                n = n + 1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
            DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel1 = 1 * TRIGGERlevel
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinI1:
                    TRIGGERlevel2 = MinI1
                if TRIGGERlevel2 > MaxI1:
                    TRIGGERlevel2 = MaxI1
                SPA = ADsignal1[0][int(n)]
                ChInput = float(SPA[1])*1000
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = float(SPA[1])*1000
                while (ChInput >= TRIGGERlevel1) and n < Nmax:
                    n = n + 1
                    SPA = ADsignal1[0][int(n)]
                    ChInput = float(SPA[1])*1000
                DX = 0
        elif (TgInput.get() == 3 ):
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinV2:
                TRIGGERlevel2 = MinV2
            if TRIGGERlevel2 > MaxV2:
                TRIGGERlevel2 = MaxV2
            SPB = ADsignal1[1][int(n)]
            ChInput = ( float(SPB[0]) - InOffB) * InGainB
            Prev = ChInput
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
            while (ChInput <= TRIGGERlevel) and n < Nmax:
                Prev = ChInput
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
            DY = ChInput - Prev
            try:
                DX = (TRIGGERlevel - Prev)/DY # calculate interpolated trigger point
            except:
                DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinV2:
                    TRIGGERlevel2 = MinV2
                if TRIGGERlevel2 > MaxV2:
                    TRIGGERlevel2 = MaxV2
                SPB = ADsignal1[1][int(n)]
                ChInput = ( float(SPB[0]) - InOffB) * InGainB
                Prev = ChInput
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = ( float(SPB[0]) - InOffB) * InGainB
                while (ChInput >= TRIGGERlevel) and n < Nmax:
                    Prev = ChInput
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = ( float(SPB[0]) - InOffB) * InGainB
                DY = Prev - ChInput
                try:
                    DX = (Prev - TRIGGERlevel)/DY # calculate interpolated trigger point
                except:
                    DX = 0    
        elif (TgInput.get() == 4 ):
            TRIGGERlevel1 = 1 * TRIGGERlevel
            TRIGGERlevel2 = 0.99 * TRIGGERlevel # Hysteresis to avoid triggering on noise
            if TRIGGERlevel2 < MinI2:
                TRIGGERlevel2 = MinI2
            if TRIGGERlevel2 > MaxI2:
                TRIGGERlevel2 = MaxI2
            SPB = ADsignal1[1][int(n)]
            ChInput = float(SPB[1])*1000
            while (ChInput >= TRIGGERlevel2) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
            while (ChInput <= TRIGGERlevel1) and n < Nmax:
                n = n + 1
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
            DX = 0
            if TgEdge.get() == 1:
                TRIGGERlevel1 = 1 * TRIGGERlevel
                TRIGGERlevel2 = 1.01 * TRIGGERlevel
                if TRIGGERlevel2 < MinI2:
                    TRIGGERlevel2 = MinI2
                if TRIGGERlevel2 > MaxI2:
                    TRIGGERlevel2 = MaxI2
                SPB = ADsignal1[1][int(n)]
                ChInput = float(SPB[1])*1000
                while (ChInput <= TRIGGERlevel2) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = float(SPB[1])*1000
                while (ChInput >= TRIGGERlevel1) and n < Nmax:
                    n = n + 1
                    SPB = ADsignal1[1][int(n)]
                    ChInput = float(SPB[1])*1000
                DX = 0
# check to insure trigger point is in bounds                
        if n < Nmax:
            TRIGGERsample = n - 1
        elif n > Nmax: # Didn't find edge in first 2/3 of data set
            TRIGGERsample = 1 + hldn # (TriggerPos * SAMPLErate) # reset to begining
        if DX > 1:
            DX = 1 # never more than 100% of a sample period
    
    # set and/or corrected for in range
    SCmin = int(-1 * TRIGGERsample)
    SCmax = int(TRACEsize - TRIGGERsample - 20)
    
    if SCstart < SCmin:             # No reading before start of array
        SCstart = SCmin
    if SCstart  > SCmax:            # No reading after end of array
        SCstart = SCmax
    
    # Make Trace lines etc.

    Yconv1 = float(GRH/10) / CH1pdvRange    # Conversion factors from samples to screen points
    Yconv2 = float(GRH/10) / CH2pdvRange
    YIconv1 = float(GRH/10) / CH1IpdvRange
    YIconv2 = float(GRH/10) / CH2IpdvRange

    c1 = GRH / 2 + Y0T    # fixed correction channel A
    c2 = GRH / 2 + Y0T    # fixed correction channel B
 
    DISsamples = SAMPLErate * 10.0 * TIMEdiv / 1000 # number of samples to display
    T1Vline = []                    # V Trace line channel A
    T2Vline = []                    # V Trace line channel B
    T1Iline = []                    # I Trace line channel A
    T2Iline = []                    # I Trace line channel B
    TXYline = []                    # XY Trace line
    Tmathline = []                  # math trce line
    t = SCstart + TRIGGERsample # - (TriggerPos * SAMPLErate) # t = Start sample in trace
    if t < 0:
        t = 0
    x = 0                           # Horizontal screen pixel

    if DisMode.get() == 0: # 0 for time plots
        if (DISsamples <= GRW):
            Xstep = GRW / DISsamples
            if AWGBMode.get() == 2:
                xa = int(Xstep/-2) - int(Xstep*DX)
            else:
                xa = 0 - int(Xstep*DX)       # adjust start pixel for interpolated trigger point
            x = 0 - int(Xstep*DX)
            Tstep = 1
            x1 = 0                      # x position of trace line
            xa1 = 0
            y1 = 0.0                    # y position of trace line
            SPA = ADsignal1[0][t]
            CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
            ypv1 = int(c1 - Yconv1 * (CH1Vcal - CHAOffset))
            ytemp = ((float(SPA[1])-CurOffA)) * CurGainA * 1000
            ypi1 = int(c1 - YIconv1 * (ytemp - CHAIOffset))
            SPB = ADsignal1[1][int(t)]
            CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
            ypv2 = int(c2 - Yconv2 * (CH2Vcal - CHBOffset))
            ytemp = ((float(SPB[1])-CurOffB)) * CurGainB * 1000
            ypi2 = int(c1 - YIconv2 * (ytemp - CHBIOffset))
            ypm = GRH / 2 + Y0T
            if TgInput.get() == 0:
                Xlimit = GRW
            else:
                Xlimit = GRW+Xstep
            while x <= Xlimit:
                if t < TRACEsize:
                    xa1 = xa + X0L
                    x1 = x + X0L
                    SPA = ADsignal1[0][int(t)]
                    CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                    y1 = int(c1 - Yconv1 * (CH1Vcal - CHAOffset))
                    ytemp = ((float(SPA[1])-CurOffA)) * CurGainA * 1000
                    yi1 = int(c1 - YIconv1 * (ytemp - CHAIOffset))
                    
                    if y1 < Ymin: # clip waveform if going off grid
                        y1 = Ymin
                    if y1 > Ymax:
                        y1 = Ymax
                    if yi1 < Ymin:
                        yi1 = Ymin
                    if yi1 > Ymax:
                        yi1 = Ymax
                    if ShowC1_V.get() == 1 :
                        if ZOHold.get() == 1:
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(ypv1))
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(y1))
                        else:    
                            T1Vline.append(int(xa1))
                            T1Vline.append(int(y1))
                    if ShowC1_I.get() == 1:
                        if ZOHold.get() == 1:
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(ypi1))
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(yi1))
                        else:
                            T1Iline.append(int(xa1))
                            T1Iline.append(int(yi1))

                    ypv1 = y1
                    ypi1 = yi1
                    SPB = ADsignal1[1][int(t)]
                    CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                    y1 = int(c2 - Yconv2 * (CH2Vcal - CHBOffset))
                    ytemp = ((float(SPB[1])-CurOffB)) * CurGainB * 1000
                    yi1 = int(c1 - YIconv2 * (ytemp - CHBIOffset))

                    if y1 < Ymin: # clip waveform if going off grid
                        y1 = Ymin
                    if y1 > Ymax:
                        y1 = Ymax
                    if yi1 < Ymin:
                        yi1 = Ymin
                    if yi1 > Ymax:
                        yi1 = Ymax
                    if ShowC2_V.get() == 1:
                        if ZOHold.get() == 1:
                            T2Vline.append(int(x1))
                            T2Vline.append(int(ypv2))
                            T2Vline.append(int(x1))
                            T2Vline.append(int(y1))
                        else:
                            T2Vline.append(int(x1))
                            T2Vline.append(int(y1))
                    if ShowC2_I.get() == 1:
                        if (ZOHold.get() == 1):
                            T2Iline.append(int(x1))
                            T2Iline.append(int(ypi2))
                            T2Iline.append(int(x1))
                            T2Iline.append(int(yi1))
                        else:
                            T2Iline.append(int(x1))
                            T2Iline.append(int(yi1))
                    ypv2 = y1
                    ypi2 = yi1
                    if MathTrace.get() == 1: # plot sum of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal + CH2Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 2: # plot difference of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal - CH2Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 3: # plot difference of CB-V and CA-V 
                        y1 = int(c1 - Yconv1 * (CH2Vcal - CH1Vcal - CHAOffset))
                        if y1 < Ymin: # clip waveform if going off grid
                            y1 = Ymin
                        if y1 > Ymax:
                            y1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if MathTrace.get() == 4: # plot product of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Ypower = CH1Vcal * ((float(SPA[1])-CurOffA)) * CurGainA * 1000 # mAmps * Volts = mWatts
                        ytemp = YIconv1 * (Ypower - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if yi1 < Ymin: # clip waveform if going off grid
                            yi1 = Ymin
                        if yi1 > Ymax:
                            yi1 = Ymax
                        if ZOHold.get() == 1:
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 5): # plot product of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Ypower = CH2Vcal * ((float(SPB[1])-CurOffB)) * CurGainB * 1000 # mAmps * Volts = mWatts
                        ytemp = YIconv2 * (Ypower - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 6): # plot ratio of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Yohms = CH1Vcal / (((float(SPA[1])-CurOffA)) * CurGainA ) #  Volts / Amps = ohms
                        ytemp = YIconv1 * (Yohms - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 7): # plot ratio of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Yohms = CH2Vcal / (((float(SPB[1])-CurOffB)) * CurGainB ) #  Volts / Amps = ohms
                        ytemp = YIconv2 * (Yohms - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 8): # plot difference of CA-I and CB-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = ((((float(SPA[1])-CurOffA)) * CurGainA ) - (((float(SPB[1])-CurOffB)) * CurGainB )) * 1000 #  convert to mA
                        ytemp = YIconv1 * (Ydif - CHAIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 9): # plot difference of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = ((((float(SPB[1])-CurOffB)) * CurGainB ) - (((float(SPA[1])-CurOffA)) * CurGainA )) * 1000 #  convert to mA
                        ytemp = YIconv2 * (Ydif - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 10): # plot ratio of CB-V and CA-V
                        y1 = int(c1 - Yconv2 * ((CH2Vcal / CH1Vcal) - CHBOffset)) #  voltage gain A to B
                        if (y1 < Ymin): # clip waveform if going off grid
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 11): # plot ratio of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Y1 = (((float(SPB[1])-CurOffB)) * CurGainB ) / (((float(SPA[1])-CurOffA)) * CurGainA ) # current gain A to B
                        ytemp = YIconv2 * (Y1 - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin): # clip waveform if going off grid
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    
                t = t + Tstep
                x = x + Xstep
                xa = xa + Xstep
                
        if (DISsamples > GRW): # if the number of samples is larger than the grid width need to ship over samples
            Xstep = 1
            Tstep = DISsamples / GRW       # number of samples to skip per grid pixel
            x1 = 0                          # x position of trace line
            ylo = 0.0                       # ymin position of trace 1 line
            yhi = 0.0                       # ymax position of trace 1 line

            t = SCstart + TRIGGERsample # - (TriggerPos * SAMPLErate) # t = Start sample in trace
            if t < 0:
                t = 0
            x = 0                           # Horizontal screen pixel
    
            while (x <= GRW):
                if (t < TRACEsize):
                    x1 = x + X0L
                    SPA = ADsignal1[0][int(t)]
                    CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                    ylo = CH1Vcal - CHAOffset
                    ilo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                    yhi = ylo
                    ihi = ilo
                    n = t
                    while n < (t + Tstep) and n < TRACEsize:
                        SPA = ADsignal1[0][int(t)]
                        CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                        v = CH1Vcal - CHAOffset
                        i = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                        if v < ylo:
                            ylo = v
                        if v > yhi:
                            yhi = v
                        if i < ilo:
                            ilo = i
                        if i > ihi:
                            ihi = i

                        n = n + 1   
                
                    ylo = int(c1 - Yconv1 * ylo)
                    yhi = int(c1 - Yconv1 * yhi)
                    ilo = int(c1 - YIconv1 * ilo)
                    ihi = int(c1 - YIconv1 * ihi)
                    if (ylo < Ymin):
                        ylo = Ymin
                    if (ylo > Ymax):
                        ylo = Ymax

                    if (yhi < Ymin):
                        yhi = Ymin
                    if (yhi > Ymax):
                        yhi = Ymax

                    if (ilo < Ymin):
                        ilo = Ymin
                    if (ilo > Ymax):
                        ilo = Ymax

                    if (ihi < Ymin):
                        ihi = Ymin
                    if (ihi > Ymax):
                        ihi = Ymax
                    
                    if ( ShowC1_V.get() == 1 ):
                        T1Vline.append(int(x1))
                        T1Vline.append(int(ylo))        
                        T1Vline.append(int(x1))
                        T1Vline.append(int(yhi))
                    if ( ShowC1_I.get() == 1 ):
                        T1Iline.append(int(x1))
                        T1Iline.append(int(ilo))        
                        T1Iline.append(int(x1))
                        T1Iline.append(int(ihi)) 
                    SPB = ADsignal1[1][int(t)]
                    CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                    ylo = CH2Vcal - CHBOffset
                    ilo = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                    yhi = ylo
                    ihi = ilo
                    n = t
                    while n < (t + Tstep) and n < TRACEsize:
                        SPB = ADsignal1[1][int(t)]
                        CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                        v = CH2Vcal - CHBOffset
                        i = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                        if v < ylo:
                            ylo = v
                        if v > yhi:
                            yhi = v
                        if i < ilo:
                            ilo = i
                        if i > ihi:
                            ihi = i

                        n = n + 1   
                
                    ylo = int(c2 - Yconv2 * ylo)
                    yhi = int(c2 - Yconv2 * yhi)
                    ilo = int(c2 - YIconv2 * ilo)
                    ihi = int(c2 - YIconv2 * ihi)
                    if (ylo < Ymin):
                        ylo = Ymin
                    if (ylo > Ymax):
                        ylo = Ymax

                    if (yhi < Ymin):
                         yhi = Ymin
                    if (yhi > Ymax):
                        yhi = Ymax

                    if (ilo < Ymin):
                        ilo = Ymin
                    if (ilo > Ymax):
                        ilo = Ymax

                    if (ihi < Ymin):
                        ihi = Ymin
                    if (ihi > Ymax):
                        ihi = Ymax

                    if ( ShowC2_V.get() == 1 ):
                        T2Vline.append(int(x1))
                        T2Vline.append(int(ylo))        
                        T2Vline.append(int(x1))
                        T2Vline.append(int(yhi))
                    if ( ShowC2_I.get() == 1 ):
                        T2Iline.append(int(x1))
                        T2Iline.append(int(ilo))        
                        T2Iline.append(int(x1))
                        T2Iline.append(int(ihi))
                    if ( MathTrace.get() == 1): # plot sum of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal + CH2Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 2): # plot difference of CA-V and CB-V 
                        y1 = int(c1 - Yconv1 * (CH1Vcal - CH2Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 3): # plot difference of CB-V and CA-V 
                        y1 = int(c1 - Yconv1 * (CH2Vcal - CH1Vcal - CHAOffset))
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 4): # plot product of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Ypower = CH1Vcal * (((float(SPA[1])-CurOffA)) * CurGainA ) * 1000 # mAmps * Volts = mWatts
                        ytemp = YIconv1 * (Ypower - CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 5): # plot product of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Ypower = CH2Vcal * (((float(SPB[1])-CurOffB)) * CurGainB ) * 1000 # mAmps * Volts = mWatts
                        ytemp = YIconv2 * (Ypower - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 6): # plot ratio of CA-V and CA-I
                        SPA = ADsignal1[0][int(t)]
                        Yohms = CH1Vcal / (((float(SPA[1])-CurOffA)) * CurGainA ) #  Volts / Amps = ohms
                        ytemp = YIconv1 * (Yohms- CHAIOffset)
                        yi1 = int(c1 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 7): # plot ratio of CB-V and CB-I
                        SPB = ADsignal1[1][int(t)]
                        Yohms = CH2Vcal / (((float(SPB[1])-CurOffB)) * CurGainB ) #  Volts / Amps = ohms
                        ytemp = YIconv2 * (Yohms - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 8): # plot difference of CA-I and CB-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = ((((float(SPA[1])-CurOffA)) * CurGainA ) - (((float(SPB[1])-CurOffB)) * CurGainB )) * 1000 #  convert to mA
                        ytemp = YIconv1 * (Ydif - CHAIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 9): # plot difference of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Ydif = ((((float(SPB[1])-CurOffB)) * CurGainB ) - (((float(SPA[1])-CurOffA)) * CurGainA )) * 1000 #  convert to mA
                        ytemp = YIconv2 * (Ydif - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                    if ( MathTrace.get() == 10): # plot ratio of CB-V and CA-V
                        y1 = int(c1 - Yconv2 * ((CH2Vcal / CH1Vcal) - CHBOffset)) #  voltage gain A to B
                        if (y1 < Ymin):
                            y1 = Ymin
                        if (y1 > Ymax):
                            y1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(y1))
                        ypm = y1
                    if ( MathTrace.get() == 11): # plot ratio of CB-I and CA-I
                        SPA = ADsignal1[0][int(t)]
                        SPB = ADsignal1[1][int(t)]
                        Y1 = (((float(SPB[1])-CurOffB)) * CurGainB ) / (((float(SPA[1])-CurOffA)) * CurGainA ) # current gain A to B
                        ytemp = YIconv2 * (Y1 - CHBIOffset)
                        yi1 = int(c2 - ytemp)
                        if (yi1 < Ymin):
                            yi1 = Ymin
                        if (yi1 > Ymax):
                            yi1 = Ymax
                        if (ZOHold.get() == 1):
                            Tmathline.append(int(x1))
                            Tmathline.append(int(ypm))
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        else:    
                            Tmathline.append(int(x1))
                            Tmathline.append(int(yi1))
                        ypm = yi1
                        
                t = t + Tstep
                x = x + Xstep
    else:   # draw an X/Y plot 
        t = TRIGGERsample # skip over sampled before hold off time and trigger point
        c2 = GRW / 2 + X0L   # Hor correction factor
        
        if (Xsignal.get() == 3 and Ysignal.get() == 1): # mode CAV/CBV
            Xconv1 = float(GRW/10) / CH2pdvRange
            Yconv1 = float(GRH/10) / CH1pdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                ylo = CH1Vcal - CHAOffset
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 1 and Ysignal.get() == 3): # mode CBV/CAV
            Xconv1 = float(GRW/10) / CH1pdvRange
            Yconv1 = float(GRH/10) / CH2pdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainA
                ylo = CH2Vcal - CHBOffset
                SPA = ADsignal1[0][int(t)]
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainB
                xlo = CH1Vcal - CHAOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 1 and Ysignal.get() == 2): # mode CAI/CAV
            Xconv1 = float(GRW/10) / CH1pdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                ylo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                xlo = CH1Vcal - CHAOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 3 and Ysignal.get() == 2): # mode CAI/CBV
            Xconv1 = float(GRW/10) / CH2pdvRange
            YIconv1 = float(GRH/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                ylo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                SPB = ADsignal1[1][int(t)]
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + Xconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 1): # mode CAV/CAI
            YIconv1 = float(GRW/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                xlo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                CH1Vcal = (float(SPA[0]) - InOffA) * InGainA
                ylo = CH1Vcal - CHAOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + YIconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 3): # mode CBV/CAI
            YIconv1 = float(GRW/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                xlo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                SPB = ADsignal1[1][int(t)]
                CH1Vcal = (float(SPB[0]) - InOffB) * InGainB
                ylo = CH1Vcal - CHBOffset
                ylo = int(c1 - Yconv1 * ylo)
                xlo = int(c2 + YIconv1 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 3 and Ysignal.get() == 4): # mode CBI/CBV
            Yconv2 = float(GRW/10) / CH2pdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                ylo = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                xlo = CH2Vcal - CHBOffset
                ylo = int(c1 - YIconv2 * ylo)
                xlo = int(c2 + Yconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 4 and Ysignal.get() == 3): # mode CBV/CBI
            YIconv2 = float(GRW/10) / CH2IpdvRange
            while (t < TRACEsize):
                SPB = ADsignal1[1][int(t)]
                xlo = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                CH2Vcal = (float(SPB[0]) - InOffB) * InGainB
                ylo = CH2Vcal - CHBOffset
                ylo = int(c1 - Yconv2 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 4 and Ysignal.get() == 2): # mode CAI/CBI
            YIconv2 = float(GRW/10) / CH2IpdvRange
            YIconv1 = float(GRH/10) / CH1IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                SPB = ADsignal1[1][int(t)]
                xlo = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                ylo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
        elif (Xsignal.get() == 2 and Ysignal.get() == 4): # mode CBI/CAI
            YIconv2 = float(GRW/10) / CH1IpdvRange
            YIconv1 = float(GRH/10) / CH2IpdvRange
            while (t < TRACEsize):
                SPA = ADsignal1[0][int(t)]
                SPB = ADsignal1[1][int(t)]
                xlo = ((((float(SPA[1])-CurOffA)) * CurGainA ) * 1000) - CHAIOffset
                ylo = ((((float(SPB[1])-CurOffB)) * CurGainB ) * 1000) - CHBIOffset
                ylo = int(c1 - YIconv1 * ylo)
                xlo = int(c2 + YIconv2 * xlo)
                TXYline.append(int(xlo))
                TXYline.append(int(ylo))
                t = t + 1
            
    # Make trigger triangle pointer
    Triggerline = []                # Triggerline
    if ( TgInput.get() == 0 ):
        x1 = X0L
        y1 = Ymin
    elif ( TgInput.get() == 1 ): # triggering on CA-V
        x1 = X0L
        ytemp = Yconv1 * (float(TRIGGERlevel)-CHAOffset) # / InGainA
        y1 = int(c1 - ytemp)
    elif ( TgInput.get() == 2 ):  # triggering on CA-I
        x1 = X0L+GRW
        y1 = int(c1 - YIconv1 * (float(TRIGGERlevel) - CHAIOffset))
    elif ( TgInput.get() == 3 ):  # triggering on CB-V
        x1 = X0L
        ytemp = Yconv2 * (float(TRIGGERlevel)-CHBOffset) # / InGainB         
        y1 = int(c2 - ytemp)
    elif ( TgInput.get() == 4 ): # triggering on CB-I
        x1 = X0L+GRW
        y1 = int(c2 - YIconv2 * (float(TRIGGERlevel) - CHBIOffset))
        
    if (y1 < Ymin):
        y1 = Ymin
    if (y1 > Ymax):
        y1 = Ymax
    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1+5))
    
    Triggerline.append(int(x1+5))
    Triggerline.append(int(y1))
    
    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1-5))        

    Triggerline.append(int(x1-5))
    Triggerline.append(int(y1+5))
    
def MakeTimeScreen():     # Update the screen with traces and text
    global ADsignal1 #, ADsignal2
    global T1Vline, T2Vline, T1Iline, T2Iline, TXYline # active trave lines
    global T1VRline, T2VRline, T1IRline, T2IRline # reference trace lines
    global Triggerline, Tmathline, TMRline, TXYRline
    global X0L          # Left top X value
    global Y0T          # Left top Y value
    global GRW          # Screenwidth
    global GRH          # Screenheight
    global Ymin, Ymax
    global SHOWsamples  # Number of samples in data record
    global ShowC1_V, ShowC1_I, ShowC2_V, ShowC2_I, ShowRXY
    global ShowRA_V, ShowRA_I, ShowRB_V, ShowRB_I, ShowMath
    global Xsignal, Ysignal, MathTrace
    global RUNstatus, SingleShot    # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
    global CHAsb        # spinbox Index for channel 1 V
    global CHBsb        # spinbox Index for channel 2 V
    global CHAOffset    # Offset value for channel 1 V
    global CHBOffset    # Offset value for channel 2 V
    global CHAIsb       # spinbox Index for channel 1 I
    global CHBIsb       # spinbox Index for channel 2 I
    global CHAIOffset   # Offset value for channel 1 I
    global CHBIOffset   # Offset value for channel 2 I
    global XYMode     
    global DisMode      # 0 = Time axis 1 = X Y mode
    global TMpdiv       # Array with time / div values in ms
    global TMsb         # Time per div spin box variable
    global TIMEdiv      # current spin box value
    global SAMPLErate
    global TRIGGERsample, TRIGGERlevel, HoldOff, HoldOffentry
    global LONGchunk    # If longchunk is used for longsweep
    global COLORgrid    # The colors
    global COLORzeroline
    global COLORtrace1, COLORtrace2, COLORtrace3, COLORtrace4, COLORtrace5
    global COLORtraceR1, COLORtraceR2, COLORtraceR3, COLORtraceR4, COLORtraceR5
    global COLORtext
    global COLORtrigger
    global CANVASwidth, CANVASheight
    global TRACErefresh
    global SCREENrefresh, SmoothCurves
    global DCV1, DCV2, MinV1, MaxV1, MinV2, MaxV2, CHAHW, CHALW, CHADCy, CHAperiod, CHAfreq
    global DCI1, DCI2, MinI1, MaxI1, MinI2, MaxI2, CHBHW, CHBLW, CHBDCy, CHBperiod, CHBfreq
    global SV1, SI1, SV2, SI2, CHABphase
    global MeasDCV1, MeasMinV1, MeasMaxV1, MeasMidV1, MeasPPV1
    global MeasDCI1, MeasMinI1, MeasMaxI1, MeasMidI1, MeasPPI1
    global MeasDCV2, MeasMinV2, MeasMaxV2, MeasMidV2, MeasPPV2
    global MeasDCI2, MeasMinI2, MeasMaxI2, MeasMidI2, MeasPPI2
    global MeasRMSV1, MeasRMSI1, MeasRMSV2, MeasRMSI2, MeasPhase
    global MeasAHW, MeasALW, MeasADCy, MeasAPER, MeasAFREQ
    global MeasBHW, MeasBLW, MeasBDCy, MeasBPER, MeasBFREQ
    global AWGAShape, AWGBShape
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global CHAVPosEntry, CHAIPosEntry, CHAVPosEntry, CHBIPosEntry
    global DI0, DI1, DI2, DI3, DI4, DI5, DI6, DI7
    global DevID, devx
    # m1k dev info
    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
    # Delete all items on the screen
    de = ca.find_enclosed ( -1000, -1000, CANVASwidth+1000, CANVASheight+1000)    
    for n in de: 
        ca.delete(n)
    # get time scale
    TIMEdiv = eval(TMsb.get())
    if TIMEdiv < 0.0002:
        TIMEdiv = 0.1
    # get the vertical ranges
    try:
        CH1pdvRange = float(eval(CHAsb.get()))
    except:
        CHAsb.delete(0,END)
        CHAsb.insert(0, CH1vpdvRange)
    try:
        CH2pdvRange = float(eval(CHBsb.get()))
    except:
        CHBsb.delete(0,END)
        CHBsb.insert(0, CH2vpdvRange)
    try:
        CH1IpdvRange = float(eval(CHAIsb.get()))
    except:
        CHAIsb.delete(0,END)
        CHAIsb.insert(0, CH1IpdvRange)
    try:
        CH2IpdvRange = float(eval(CHBIsb.get()))
    except:
        CHBIsb.delete(0,END)
        CHBIsb.insert(0, CH2IpdvRange)
    # get the vertical offsets
    try:
        CHAOffset = float(eval(CHAVPosEntry.get()))
    except:
        CHAVPosEntry.delete(0,END)
        CHAVPosEntry.insert(0, CHAOffset)
    try:
        CHAIOffset = float(eval(CHAIPosEntry.get()))
    except:
        CHAIPosEntry.delete(0,END)
        CHAIPosEntry.insert(0, CHAIOffset)
    try:
        CHBOffset = float(eval(CHBVPosEntry.get()))
    except:
        CHBVPosEntry.delete(0,END)
        CHBVPosEntry.insert(0, CHBOffset)
    try:
        CHBIOffset = float(eval(CHBIPosEntry.get()))
    except:
        CHBIPosEntry.delete(0,END)
        CHBIPosEntry.insert(0, CHBIOffset)
    try:
        HoldOff = float(eval(HoldOffentry.get()))
    except:
        HoldOffentry.delete(0,END)
        HoldOffentry.insert(0, HoldOff)
    # prevent divide by zero error
    if CH1pdvRange < 0.01:
        CH1pdvRange = 0.01
    if CH2pdvRange < 0.01:
        CH2pdvRange = 0.01
    if CH1IpdvRange < 1.0:
        CH1IpdvRange = 1.0
    if CH2IpdvRange < 1.0:
        CH2IpdvRange = 1.0
    
    # Draw horizontal grid lines
    i = 0
    x1 = X0L
    x2 = X0L + GRW
    mg_siz = GRW/10
    mg_inc = mg_siz/5
    while (i < 11):
        y = Y0T + i * GRH/10
        Dline = [x1,y,x2,y]
        if i == 5:
            ca.create_line(Dline, fill=COLORzeroline)   # Blue line at center of grid
            k = 0
            while (k < 10):
                l = 1
                while (l < 5):
                    Dline = [x1+k*mg_siz+l*mg_inc,y-5,x1+k*mg_siz+l*mg_inc,y+5]
                    ca.create_line(Dline, fill=COLORgrid)
                    l = l + 1
                k = k + 1
        else:
            ca.create_line(Dline, fill=COLORgrid)

        if (ShowC1_V.get() == 1 or MathTrace.get() == 1 or MathTrace.get() == 2):
            Vaxis_value = (((5-i) * CH1pdvRange ) + CHAOffset)
            # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
            Vaxis_label = str(Vaxis_value)
            ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
            
        if (ShowC1_I.get() == 1 or MathTrace.get() == 4 or MathTrace.get() == 6 or MathTrace.get() == 8):
            Iaxis_value = 1.0 * (((5-i) * CH1IpdvRange ) + CHAIOffset)
            Iaxis_label = str(Iaxis_value)
            ca.create_text(x2+2, y, text=Iaxis_label, fill=COLORtrace3, anchor="w", font=("arial", 8 ))
            
        if (ShowC2_V.get() == 1 or MathTrace.get() == 3 or MathTrace.get() == 10):
            Vaxis_value = (((5-i) * CH2pdvRange ) + CHBOffset)
            # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
            Vaxis_label = str(Vaxis_value)
            ca.create_text(x1-26, y, text=Vaxis_label, fill=COLORtrace2, anchor="e", font=("arial", 8 ))
            
        if (ShowC2_I.get() == 1 or MathTrace.get() == 5 or MathTrace.get() == 7 or MathTrace.get() == 9 or MathTrace.get() == 11):
            Iaxis_value = 1.0 * (((5-i) * CH2IpdvRange ) + CHBIOffset)
            Iaxis_label = str(Iaxis_value)
            ca.create_text(x2+28, y, text=Iaxis_label, fill=COLORtrace4, anchor="w", font=("arial", 8 ))
        else:
            if Ysignal.get() == 1:
                Vaxis_value = (((5-i) * CH1pdvRange ) + CHAOffset)
                # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
                Vaxis_label = str(Vaxis_value)
                ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 2:
                Iaxis_value = 1.0 * (((5-i) * CH1IpdvRange ) + CHAIOffset)
                Iaxis_label = str(Iaxis_value)
                ca.create_text(x1-3, y, text=Iaxis_label, fill=COLORtrace3, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 3:
                Vaxis_value = (((5-i) * CH2pdvRange ) + CHBOffset)
                # Vaxis_label = ' {0:.2f} '.format(Vaxis_value)
                Vaxis_label = str(Vaxis_value)
                ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace2, anchor="e", font=("arial", 8 ))
            elif Ysignal.get() == 4:
                Iaxis_value = 1.0 * (((5-i) * CH2IpdvRange ) + CHBIOffset)
                Iaxis_label = str(Iaxis_value)
                ca.create_text(x1-3, y, text=Iaxis_label, fill=COLORtrace4, anchor="e", font=("arial", 8 ))

        i = i + 1

    # Draw vertical grid lines
    i = 0
    y1 = Y0T
    y2 = Y0T + GRH
    mg_siz = GRH/10
    mg_inc = mg_siz/5
    vx = TIMEdiv
    vt = HoldOff # invert sign and scale to mSec
    while (i < 11):
        x = X0L + i * GRW/10
        Dline = [x,y1,x,y2]
        if (i == 5):
            ca.create_line(Dline, fill=COLORzeroline)   # Blue vertical line at center of grid
            k = 0
            while (k < 10):
                l = 1
                while (l < 5):
                    Dline = [x-5,y1+k*mg_siz+l*mg_inc,x+5,y1+k*mg_siz+l*mg_inc]
                    ca.create_line(Dline, fill=COLORgrid)
                    l = l + 1
                k = k + 1
            if DisMode.get() == 0:
                if vx >= 1000:
                    axis_value = ((i * vx)+ vt) / 1000
                    axis_label = str(int(axis_value)) + " S"
                if vx < 1000 and vx >= 1:
                    axis_value = (i * vx) + vt
                    axis_label = str(int(axis_value)) + " mS"
                if vx < 1:
                    axis_value = ((i * vx) + vt) * 1000
                    axis_label = str(int(axis_value)) + " uS"
                ca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
            else:
                if Xsignal.get() == 1: # modes "Time", "C1/C2", "C1 I/V", "C2 I/V"
                    Vaxis_value = (((i-5) * CH1pdvRange ) + CHAOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace1, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 2:
                    Iaxis_value = 1.0 * (((i-5) * CH1IpdvRange ) + CHAIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace3, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 3:
                    Vaxis_value = (((i-5) * CH2pdvRange ) + CHBOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace2, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 4:
                    Iaxis_value = 1.0 * (((i-5) * CH2IpdvRange ) + CHBIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace4, anchor="n", font=("arial", 8 ))
        else:
            ca.create_line(Dline, fill=COLORgrid)
            if DisMode.get() == 0:
                if vx >= 1000:
                    axis_value = ((i * vx)+ vt) / 1000
                    axis_label = str(int(axis_value)) + " S"
                if vx < 1000 and vx >= 1:
                    axis_value = (i * vx) + vt
                    axis_label = str(int(axis_value)) + " mS"
                if vx < 1:
                    axis_value = ((i * vx) + vt) * 1000
                    axis_label = str(int(axis_value)) + " uS"
                ca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
            else:
                if Xsignal.get() == 1: # modes "Time", "C1/C2", "C1 I/V", "C2 I/V"
                    Vaxis_value = (((i-5) * CH1pdvRange ) + CHAOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace1, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 2:
                    Iaxis_value = 1.0 * (((i-5) * CH1IpdvRange ) + CHAIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace3, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 3:
                    Vaxis_value = (((i-5) * CH2pdvRange ) + CHBOffset)
                    Vaxis_label = str(Vaxis_value)
                    ca.create_text(x, y2+3, text=Vaxis_label, fill=COLORtrace2, anchor="n", font=("arial", 8 ))
                elif Xsignal.get() == 4:
                    Iaxis_value = 1.0 * (((i-5) * CH2IpdvRange ) + CHBIOffset)
                    Iaxis_label = str(Iaxis_value)
                    ca.create_text(x, y2+3, text=Iaxis_label, fill=COLORtrace4, anchor="n", font=("arial", 8 ))
                
        i = i + 1

    # Write the trigger line if available
    if len(Triggerline) > 2:                    # Avoid writing lines with 1 coordinate
        ca.create_polygon(Triggerline, outline=COLORtrigger, fill=COLORtrigger, width=1)

    SmoothBool = SmoothCurves.get()
    # Write the traces if available
    if DisMode.get() == 0: 
        if len(T1Vline) > 4: # Avoid writing lines with 1 coordinate    
            ca.create_line(T1Vline, fill=COLORtrace1, smooth=SmoothBool, splinestep=5)   # Write the voltage trace 1
        if len(T1Iline) > 4: # Avoid writing lines with 1 coordinate
            ca.create_line(T1Iline, fill=COLORtrace3, smooth=SmoothBool, splinestep=5)   # Write the current trace 1
        if len(T2Vline) > 4: # Write the trace 2 if active
            ca.create_line(T2Vline, fill=COLORtrace2, smooth=SmoothBool, splinestep=5)
        if len(T2Iline) > 4:
            ca.create_line(T2Iline, fill=COLORtrace4, smooth=SmoothBool, splinestep=5)
        if len(Tmathline) > 4 and MathTrace.get() > 0: # Write Math tace if active
            ca.create_line(Tmathline, fill=COLORtrace5, smooth=SmoothBool, splinestep=5)
        if ShowRA_V.get() == 1 and len(T1VRline) > 4:
            ca.create_line(T1VRline, fill=COLORtraceR1, smooth=SmoothBool, splinestep=5)
        if ShowRA_I.get() == 1 and len(T1IRline) > 4:
            ca.create_line(T1IRline, fill=COLORtraceR3, smooth=SmoothBool, splinestep=5)
        if ShowRB_V.get() == 1 and len(T2VRline) > 4:
            ca.create_line(T2VRline, fill=COLORtraceR2, smooth=SmoothBool, splinestep=5)
        if ShowRB_I.get() == 1 and len(T2IRline) > 4:
            ca.create_line(T2IRline, fill=COLORtraceR4, smooth=SmoothBool, splinestep=5)
        if ShowMath.get() == 1 and len(TMRline) > 4:
            ca.create_line(TMRline, fill=COLORtraceR5, smooth=SmoothBool, splinestep=5)
    else:
        if len(TXYline) > 4:                    # Avoid writing lines with 1 coordinate
            if  Xsignal.get() == 1:
                ca.create_line(TXYline, fill=COLORtrace1)
            elif  Xsignal.get() == 2:
                ca.create_line(TXYline, fill=COLORtrace3)
            elif  Xsignal.get() == 3:
                ca.create_line(TXYline, fill=COLORtrace2)
            elif  Xsignal.get() == 4:
                ca.create_line(TXYline, fill=COLORtrace4)
        if len(TXYRline) > 4 and ShowRXY.get() == 1:
            ca.create_line(TXYRline, fill=COLORtraceR1)

    # General information on top of the grid
    # Sweep information
    if LONGsweep == False and LONGchunk == False:
        sttxt = "Running"
        if SingleShot.get() == 1:
            sttxt = "Single Shot"
        if (RUNstatus.get() == 0) or (RUNstatus.get() == 3):
            sttxt = "Stopped"

    if LONGsweep == True or LONGchunk == True:
        sttxt = "Running long sweep, wait"
        if (RUNstatus.get() == 0) or (RUNstatus.get() == 3):
            sttxt = "Stopped long sweep, press Start"
     
    txt = "Device ID " + DevID[17:31] + " Sample rate: " + str(SAMPLErate) + " " + sttxt

    x = X0L
    y = 12
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)
    # digital I/O indicators
    x2 = X0L + GRW
    BoxColor = "#808080"   # gray
    if DI0.get() == 0 and DI4.get() == 0:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 0, 0, 0, 1, 100)
        if Dval[0] == 1:
            BoxColor = "#00ff00"   # 100% green
        elif Dval[0] == 0:
            BoxColor = "#ff0000"   # 100% red
        ca.create_rectangle(x2-12, 6, x2, 18, fill=BoxColor)
    else:
        ca.create_rectangle(x2-12, 6, x2, 18, fill="yellow")
    if DI1.get() == 0 and DI5.get() == 0:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 1, 0, 0, 1, 100)
        if Dval[0] == 1:
            BoxColor = "#00ff00"   # 100% green
        elif Dval[0] == 0:
            BoxColor = "#ff0000"   # 100% red
        ca.create_rectangle(x2-26, 6, x2-14, 18, fill=BoxColor)
    else:
        ca.create_rectangle(x2-26, 6, x2-14, 18, fill="yellow")
    if DI2.get() == 0 and DI6.get() == 0:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 2, 0, 0, 1, 100)
        if Dval[0] == 1:
            BoxColor = "#00ff00"   # 100% green
        elif Dval[0] == 0:
            BoxColor = "#ff0000"   # 100% red
        ca.create_rectangle(x2-40, 6, x2-28, 18, fill=BoxColor)
    else:
        ca.create_rectangle(x2-40, 6, x2-28, 18, fill="yellow")
    if DI3.get() == 0 and DI7.get() == 0:
        Dval = devx.ctrl_transfer(DevID, 0xc0, 0x91, 3, 0, 0, 1, 100)
        if Dval[0] == 1:
            BoxColor = "#00ff00"   # 100% green
        elif Dval[0] == 0:
            BoxColor = "#ff0000"   # 100% red
        ca.create_rectangle(x2-54, 6, x2-42, 18, fill=BoxColor)
    else:
        ca.create_rectangle(x2-54, 6, x2-42, 18, fill="yellow")
    ca.create_text(x2-56, 12, text="Digital Inputs", anchor=E, fill=COLORtext)
    # Time sweep information and view at information
    # vx = TIMEdiv
    if vx >= 1000:
        txt = str(int(vx/1000)) + " S/div"
    if vx < 1000 and vx >= 1:
        txt = str(int(vx)) + " mS/div"
    if vx < 1:
        txt = str(int(vx * 1000)) + " uS/div"

    txt = txt + "  "
    # 
    txt = txt + "View at "
    if abs(vt) >= 1000:
        txt = txt + str(vt / 1000) + " S "
    if abs(vt) < 1000 and abs(vt) >= 1:
        txt = txt + str(vt) + " mS "
    if abs(vt) < 1:
        txt = txt + str(vt * 1000) + " uS "
    # print period and frequency of displayed channels
    if ShowC1_V.get() == 1 or ShowC2_V.get() == 1:
        FindRisingEdge()
        if ShowC1_V.get() == 1:
            if MeasAHW.get() == 1:
                txt = txt + " CA Hi Width = "
                V1String = ' {0:.2f} '.format(CHAHW)
                txt = txt + str(V1String) + " mS "
            if MeasALW.get() == 1:
                txt = txt + " CA Lo Width = "
                V1String = ' {0:.2f} '.format(CHALW)
                txt = txt + str(V1String) + " mS "
            if MeasADCy.get() == 1:
                txt = txt + " CA DutyCycle = "
                V1String = ' {0:.1f} '.format(CHADCy)
                txt = txt + str(V1String) + " % "
            if MeasAPER.get() == 1:
                txt = txt + " CA Period = "
                V1String = ' {0:.2f} '.format(CHAperiod)
                txt = txt + str(V1String) + " mS "
            if MeasAFREQ.get() == 1:
                txt = txt + " CA Freq = "
                V1String = ' {0:.1f} '.format(CHAfreq)
                txt = txt + str(V1String) + " Hz "
        if ShowC2_V.get() == 1:
            if MeasBHW.get() == 1:
                txt = txt + " CB Hi Width = "
                V1String = ' {0:.2f} '.format(CHBHW)
                txt = txt + str(V1String) + " mS "
            if MeasBLW.get() == 1:
                txt = txt + " CB Lo Width = "
                V1String = ' {0:.2f} '.format(CHBLW)
                txt = txt + str(V1String) + " mS "
            if MeasBDCy.get() == 1:
                txt = txt + " CB DutyCycle = "
                V1String = ' {0:.1f} '.format(CHBDCy)
                txt = txt + str(V1String) + " % "
            if MeasBPER.get() == 1:
                txt = txt + " CB Period = "
                V1String = ' {0:.2f} '.format(CHBperiod)
                txt = txt + str(V1String) + " mS "
            if MeasBFREQ.get() == 1:
                txt = txt + " CB Freq = "
                V1String = ' {0:.1f} '.format(CHBfreq)
                txt = txt + str(V1String) + " Hz "
        if MeasPhase.get() == 1:
            txt = txt + " CA-B Phase = "
            V1String = ' {0:.1f} '.format(CHABphase)
            txt = txt + str(V1String) + " deg "
            
    x = X0L
    y = Y0T+GRH+20
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)
    txt = " "
    if ShowC1_V.get() == 1 or Xsignal.get() == 1 or Ysignal.get() == 1:
    # Channel A information
        txt = "CHA: "
        txt = txt + str(CH1pdvRange) + " V/div"
        if MeasDCV1.get() == 1:
            V1String = ' {0:.4f} '.format(DCV1)
            txt = txt + " AvgV = " + V1String
        if MeasMaxV1.get() == 1:
            V1String = ' {0:.4f} '.format(MaxV1)
            txt = txt +  " MaxV = " + V1String
        if MeasMinV1.get() == 1:
            V1String = ' {0:.4f} '.format(MinV1)
            txt = txt +  " MinV = " + V1String
        if MeasMidV1.get() == 1:
            MidV1 = (MaxV1+MinV1)/2
            V1String = ' {0:.4f} '.format(MidV1)
            txt = txt +  " MidV = " + V1String
        if MeasPPV1.get() == 1:
            PPV1 = MaxV1-MinV1
            V1String = ' {0:.4f} '.format(PPV1)
            txt = txt +  " P-PV = " + V1String
        if MeasRMSV1.get() == 1:
            V1String = ' {0:.4f} '.format(SV1)
            txt = txt +  " RMS = " + V1String
    if (ShowC1_I.get() == 1 and ShowC1_V.get() == 0):
        txt = "CHA: "
        txt = txt + str(CH1IpdvRange) + " mA/div"
    elif (ShowC1_I.get() == 1 and ShowC1_V.get() == 1):
        txt = txt + "CHA: "
        txt = txt + str(CH1IpdvRange) + " mA/div"
    if ShowC1_I.get() == 1 or Xsignal.get() == 2 or Ysignal.get() == 2:
        if MeasDCI1.get() == 1:
            V1String = ' {0:.2f} '.format(DCI1)
            txt = txt + " AvgI = " + V1String
            if AWGAShape.get() == 0: # if this is a DC measurement calc resistance
                try:
                    Resvalue = (DCV1/DCI1)*1000
                    R1String = ' {0:.1f} '.format(Resvalue)
                    txt = txt + " Res = " + R1String
                except:
                    txt = txt + " Res = OverRange" 
        if MeasMaxI1.get() == 1:
            V1String = ' {0:.2f} '.format(MaxI1)
            txt = txt +  " MaxI = " + V1String
        if MeasMinI1.get() == 1:
            V1String = ' {0:.2f} '.format(MinI1)
            txt = txt +  " MinI = " + V1String
        if MeasMidI1.get() == 1:
            MidI1 = (MaxI1+MinI1)/2
            V1String = ' {0:.2f} '.format(MidI1)
            txt = txt +  " MidV = " + V1String
        if MeasPPI1.get() == 1:
            PPI1 = MaxI1-MinI1
            V1String = ' {0:.2f} '.format(PPI1)
            txt = txt +  " P-PI = " + V1String
        if MeasRMSI1.get() == 1:
            V1String = ' {0:.4f} '.format(SI1)
            txt = txt +  " RMS = " + V1String
        
    x = X0L
    y = Y0T+GRH+32
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)
    txt= " "
    # Channel B information
    if ShowC2_V.get() == 1 or Xsignal.get() == 3 or Ysignal.get() == 3:
        txt = "CHB: "
        txt = txt + str(CH2pdvRange) + " V/div"
        if MeasDCV2.get() == 1:
            V1String = ' {0:.4f} '.format(DCV2)
            txt = txt + " AvgV = " + V1String
        if MeasMaxV2.get() == 1:
            V1String = ' {0:.4f} '.format(MaxV2)
            txt = txt +  " MaxV = " + V1String
        if MeasMinV2.get() == 1:
            V1String = ' {0:.4f} '.format(MinV2)
            txt = txt +  " MinV = " + V1String
        if MeasMidV2.get() == 1:
            MidV2 = (MaxV2+MinV2)/2
            V1String = ' {0:.4f} '.format(MidV2)
            txt = txt +  " MidV = " + V1String
        if MeasPPV2.get() == 1:
            PPV2 = MaxV2-MinV2
            V1String = ' {0:.4f} '.format(PPV2)
            txt = txt +  " P-PV = " + V1String
        if MeasRMSV2.get() == 1:
            V1String = ' {0:.4f} '.format(SV2)
            txt = txt +  " RMS = " + V1String
    if (ShowC2_I.get() == 1 and ShowC2_V.get() == 0):
        txt = "CHB: "
        txt = txt + str(CH2IpdvRange) + " mA/div"
    elif (ShowC2_I.get() == 1 and ShowC2_V.get() == 1):
        txt = txt + "CHB: "
        txt = txt + str(CH2IpdvRange) + " mA/div"
    if ShowC2_I.get() == 1 or Xsignal.get() == 4 or Ysignal.get() == 4:
        if MeasDCI2.get() == 1:
            V1String = ' {0:.2f} '.format(DCI2)
            txt = txt + " AvgI = " + V1String
            if AWGBShape.get() == 0: # if this is a DC measurement calc resistance
                try:
                    Resvalue = (DCV2/DCI2)*1000
                    R1String = ' {0:.1f} '.format(Resvalue)
                    txt = txt + " Res = " + R1String
                except:
                    txt = txt + " Res = OverRange" 
        if MeasMaxI2.get() == 1:
            V1String = ' {0:.2f} '.format(MaxI2)
            txt = txt +  " MaxI = " + V1String
        if MeasMinI2.get() == 1:
            V1String = ' {0:.2f} '.format(MinI2)
            txt = txt +  " MinI = " + V1String
        if MeasMidI2.get() == 1:
            MidI2 = (MaxI2+MinI2)/2
            V1String = ' {0:.2f} '.format(MidI2)
            txt = txt +  " MidV = " + V1String
        if MeasPPI2.get() == 1:
            PPI2 = MaxI2-MinI2
            V1String = ' {0:.2f} '.format(PPI2)
            txt = txt +  " P-PI = " + V1String
        if MeasRMSI2.get() == 1:
            V1String = ' {0:.4f} '.format(SI2)
            txt = txt +  " RMS = " + V1String
            
    x = X0L
    y = Y0T+GRH+44
    ca.create_text(x, y, text=txt, anchor=W, fill=COLORtext)

def ConnectDevice():
    global devx, DevID, Both, CHA, CHB, bcon
    try:
        devx = Smu()
        brdnum = len(devx.devices)
        channum = len(devx.channels)
        CHA = devx.channels['A']
        CHB = devx.channels['B']
        DevID = devx.serials[0]
        print DevID
        Both = devx.devices[0]
        bcon.configure(text="Conn", bg="green")
        if channum == 4:
            CHC = devx.channels['C']
            CHC.set_mode('d')
            CHD = devx.channels['D']
            CHD.set_mode('d')
            DevTwo = devx.serials[1]
            print DevTwo
            BTwo = devx.devices[1]

    except:
        print 'No Device plugged IN!'
        DevID = "No Device"
        bcon.configure(text="Recon", bg="red")
#
# =========== Awg functions ==================
def BAWGAAmpl(temp):
    global AWGAAmplEntry, AWGAAmplvalue

    try:
        AWGAAmplvalue = float(eval(AWGAAmplEntry.get()))
    except:
        AWGAAmplEntry.delete(0,"end")
        AWGAAmplEntry.insert(0, AWGAAmplvalue)
    #UpdateAWGA()
    
def BAWGAOffset(temp):
    global AWGAOffsetEntry, AWGAOffsetvalue

    try:
        AWGAOffsetvalue = float(eval(AWGAOffsetEntry.get()))
    except:
        AWGAOffsetEntry.delete(0,"end")
        AWGAOffsetEntry.insert(0, AWGAOffsetvalue)
    #UpdateAWGA()
    
def BAWGAFreq(temp):
    global AWGAFreqEntry, AWGAFreqvalue

    try:
        AWGAFreqvalue = float(eval(AWGAFreqEntry.get()))
    except:
        AWGAFreqEntry.delete(0,"end")
        AWGAFreqEntry.insert(0, AWGAFreqvalue)

    if AWGAFreqvalue > 20000: # max freq is 20KHz
        AWGAFreqvalue = 20000
    #UpdateAWGA()

def BAWGAPhase(temp):
    global AWGAPhaseEntry, AWGAPhasevalue

    try:
        AWGAPhasevalue = float(eval(AWGAPhaseEntry.get()))
    except:
        AWGAPhaseEntry.delete(0,"end")
        AWGAPhaseEntry.insert(0, AWGAPhasevalue)

    if AWGAPhasevalue > 360: # max phase is 360 degrees
        AWGAPhasevalue = 360
    if AWGAPhasevalue < 0: # min phase is 0 degrees
        AWGAPhasevalue = 0

def BAWGADutyCycle(temp):
    global AWGADutyCycleEntry, AWGADutyCyclevalue

    try:
        AWGADutyCyclevalue = float(eval(AWGADutyCycleEntry.get()))/100
    except:
        AWGADutyCycleEntry.delete(0,"end")
        AWGADutyCycleEntry.insert(0, AWGADutyCyclevalue)

    if AWGADutyCyclevalue > 1: # max duty cycle is 100%
        AWGADutyCyclevalue = 1
    if AWGADutyCyclevalue < 0: # min duty cycle is 0%
        AWGADutyCyclevalue = 0
    #UpdateAWGA()

def BAWGAShape():
    global AWGAShape, AWGAWave

    if AWGAShape.get() == 0:
        AWGAWave = 'dc'
    if AWGAShape.get() == 1:
        AWGAWave = 'sine' 
    if AWGAShape.get() == 2:
        AWGAWave = 'triangle'
    if AWGAShape.get() == 3:
        AWGAWave = 'sawtooth'
    if AWGAShape.get() == 4:
        AWGAWave = 'square'
    if AWGAShape.get() == 5:
        AWGAWave = 'stairstep'
    if AWGAShape.get() == 6:
        AWGAWave = 'arbitrary'
    # UpdateAWGA() 

def UpdateAWGA():
    global AWGAAmplvalue, AWGAOffsetvalue
    global AWGAFreqvalue, AWGAPhasevalue
    global AWGADutyCyclevalue, FSweepMode
    global AWGAWave, AWGAMode, AWGATerm
    global CHA, SAMPLErate, DevID, devx

    temp = 0
    BAWGAAmpl(temp)
    BAWGAOffset(temp)
    BAWGAFreq(temp)
    BAWGAPhase(temp)
    BAWGADutyCycle(temp)
    BAWGAShape()
    
    if AWGAFreqvalue > 0.0:
        AWGAperiodvalue = SAMPLErate/AWGAFreqvalue
    else:
        AWGAperiodvalue = 0.0

    if AWGAPhasevalue > 0:
        AWGAdelayvalue = AWGAperiodvalue * AWGAPhasevalue / 360
    else:
        AWGAdelayvalue = 0

    if AWGATerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
    elif AWGATerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 33, 0, 0, 0, 100) # set GND switch to closed
    elif AWGATerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 32, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
        
    if AWGAWave == 'dc':
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue)
            # 
        elif AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)
            # 
        elif AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)
            #
        elif AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)

    else:
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
     
        if AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)          
        else:
            if AWGAWave == 'sine':
                CHA.sine(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'triangle':
                CHA.triangle(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'sawtooth':
                CHA.sawtooth(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'square':
                CHA.square(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue, AWGADutyCyclevalue)
            elif AWGAWave == 'stairstep':
                CHA.stairstep(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'arbitrary':
                CHA.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGBAmpl(temp):
    global AWGBAmplEntry, AWGBAmplvalue

    try:
        AWGBAmplvalue = float(eval(AWGBAmplEntry.get()))
    except:
        AWGBAmplEntry.delete(0,"end")
        AWGBAmplEntry.insert(0, AWGBAmplvalue)
    # UpdateAWGB()
    
def BAWGBOffset(temp):
    global AWGBOffsetEntry, AWGBOffsetvalue

    try:
        AWGBOffsetvalue = float(eval(AWGBOffsetEntry.get()))
    except:
        AWGBOffsetEntry.delete(0,"end")
        AWGBOffsetEntry.insert(0, AWGBOffsetvalue)
    # UpdateAWGB()
    
def BAWGBFreq(temp):
    global AWGBFreqEntry, AWGBFreqvalue

    try:
        AWGBFreqvalue = float(eval(AWGBFreqEntry.get()))
    except:
        AWGBFreqEntry.delete(0,"end")
        AWGBFreqEntry.insert(0, AWGBFreqvalue)

    if AWGBFreqvalue > 20000: # max freq is 20KHz
        AWGBFreqvalue = 20000
    # UpdateAWGB()
    
def BAWGBPhase(temp):
    global AWGBPhaseEntry, AWGBPhasevalue

    try:
        AWGBPhasevalue = float(eval(AWGBPhaseEntry.get()))
    except:
        AWGBPhaseEntry.delete(0,"end")
        AWGBPhaseEntry.insert(0, AWGBPhasevalue)

    if AWGBPhasevalue > 360: # max phase is 360 degrees
        AWGBPhasevalue = 360
    if AWGBPhasevalue < 0: # min phase is 0 degrees
        AWGBPhasevalue = 0
    
def BAWGBDutyCycle(temp):
    global AWGBDutyCycleEntry, AWGBDutyCyclevalue

    try:
        AWGBDutyCyclevalue = float(eval(AWGBDutyCycleEntry.get()))/100
    except:
        AWGBDutyCycleEntry.delete(0,"end")
        AWGBDutyCycleEntry.insert(0, AWGBDutyCyclevalue)

    if AWGBDutyCyclevalue > 1: # max duty cycle is 100%
        AWGBDutyCyclevalue = 1
    if AWGBDutyCyclevalue < 0: # min duty cycle is 0%
        AWGBDutyCyclevalue = 0
    # UpdateAWGB()
    
def BAWGBShape():
    global AWGBShape, AWGBWave
    
    if AWGBShape.get() == 0:
        AWGBWave = 'dc'
    if AWGBShape.get() == 1:
        AWGBWave = 'sine'
    if AWGBShape.get() == 2:
        AWGBWave = 'triangle'
    if AWGBShape.get() == 3:
        AWGBWave = 'sawtooth'
    if AWGBShape.get() == 4:
        AWGBWave = 'square'
    if AWGBShape.get() == 5:
        AWGBWave = 'stairstep'
    if AWGBShape.get() == 6:
        AWGBWave = 'arbitrary'        
    # UpdateAWGB()
    
def UpdateAWGB():
    global AWGBAmplvalue, AWGBOffsetvalue
    global AWGBFreqvalue, AWGBPhasevalue
    global AWGBDutyCyclevalue, FSweepMode
    global AWGBWave, AWGBMode, AWGBTerm
    global CHB, SAMPLErate, DevID, devx

    temp = 0
    BAWGBAmpl(temp)
    BAWGBOffset(temp)
    BAWGBFreq(temp)
    BAWGBPhase(temp)
    BAWGBDutyCycle(temp)
    BAWGBShape()
    
    if AWGBFreqvalue > 0.0:
        AWGBperiodvalue = SAMPLErate/AWGBFreqvalue
    else:
        AWGBperiodvalue = 0.0
    
    if AWGBPhasevalue > 0:
        AWGBdelayvalue = AWGBperiodvalue * AWGBPhasevalue / 360
    else:
        AWGBdelayvalue = 0
        
    if AWGBTerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
    elif AWGBTerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 38, 0, 0, 0, 100) # set GND switch to closed
    elif AWGBTerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 37, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
        
    if AWGBWave == 'dc':
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            CHB.constant(AWGBOffsetvalue)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            CHB.constant(AWGBOffsetvalue/1000)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            CHB.constant(0.0) #
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            CHB.constant(AWGBOffsetvalue/1000)
    else:
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        if AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            CHB.constant(0.0)
        else:
            if AWGBWave == 'sine':
                CHB.sine(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'triangle':
                CHB.triangle(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'sawtooth':
                CHB.sawtooth(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'square':
                CHB.square(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue, AWGBDutyCyclevalue)
            elif AWGBWave == 'stairstep':
                CHB.stairstep(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'arbitrary':
                CHB.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGEnab():
    global AWGAMode, AWGBMode, AWGSync
    global CHA, CHB

    if  AWGAMode.get() == 2:
        CHA.set_mode('d')
        CHA.constant(0.0) # 
    else:
        UpdateAWGA()

    if  AWGBMode.get() == 2:
        CHB.set_mode('d')
        CHB.constant(0.0) # 
    else:
        UpdateAWGB()
              
#    if AWGSync.get() > 0:
#    else:
#        if AWG1Enab.get() > 0:     
#        else:     
#        if AWG2Enab.get() > 0:         
#        else:
            
def BAWGSync():
    global AWGSync

#    if AWGSync.get() > 0:     
#    else:

#
# ======= Spectrum Analyzer functions ===========
#
def BSaveScreenSA():
    global CANVASwidthF, CANVASheightF
    global COLORtext
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # temp change text color to black
    COLORtext = "#000000"
    UpdateFreqScreen()
    # save postscript file
    Freqca.postscript(file=filename, height=CANVASheightF, width=CANVASwidthF, colormode='color')
    # 
    COLORtext = "#ffffff"
    UpdateFreqScreen()
#
def Bnot():
    print "Routine not made yet"

def BShowCurvesAllSA():
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P
    ShowC1_VdB.set(1)
    ShowC1_P.set(1)
    ShowC2_VdB.set(1)
    ShowC2_P.set(1)

def BShowCurvesNoneSA():
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P
    ShowC1_VdB.set(0)
    ShowC1_P.set(0)
    ShowC2_VdB.set(0)
    ShowC2_P.set(0)
    
def BcloseexitSA():
    global RUNstatus
    
    RUNstatus.set(0)
    # close ALM and exit
    subwindow.destroy()
    exit()
    
def BNormalmode():
    global RUNstatus
    global TRACEmode

    TRACEmode.set(1)
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqScreen()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)
    
def BPeakholdmode():
    global RUNstatus
    global TRACEmode

    TRACEmode.set(2)
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqScreen()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)
    
def BAveragemode():
    global RUNstatus
    global TRACEaverage
    global TRACEmode

    TRACEmode.set(3)

    s = askstring("Power averaging", "Value: " + str(TRACEaverage.get()) + "x\n\nNew value:\n(1-n)")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        TRACEaverage.set(v)

    if TRACEaverage.get() < 1:
        TRACEaverage.set(1)

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqScreen()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)

def BSTOREtraceSA():
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P, STOREtrace, ShowMathSA
    global T1Fline, T2Fline, T1FRline, T1FRline, TFRMline, TFMline
    global T1Pline, T2Pline, T1PRline, T2PRline
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB, PeakfreqRA, PeakfreqRB
    global PeakxRA, PeakyRA, PeakxRB, PeakyRB, PeakdbRA, PeakdbRB
    global PeakxRM, PeakyRM, PeakRMdb, PeakfreqRM
    
    if ShowC1_VdB.get() == 1:
        T1FRline = T1Fline
        PeakxRA = PeakxA
        PeakyRA = PeakyA
        PeakdbRA = PeakdbA
        PeakfreqRA = PeakfreqA
        STOREtrace = True
    if ShowC2_VdB.get() == 1:
        T1FRline = T2Fline
        PeakxRB = PeakxB
        PeakyRB = PeakyB
        PeakdbRB = PeakdbB
        PeakfreqRB = PeakfreqB
        STOREtrace = True
    if ShowC1_P.get() == 1:
        T1PRline = T1Pline
        STOREtrace = True
    if ShowC2_P.get() == 1:
        T2PRline = T2Pline
        STOREtrace = True
    if ShowMathSA.get() > 0:
        TFRMline = TFMline
        PeakxRM = PeakxM
        PeakyRM = PeakyM
        PeakRMdb = PeakMdb
        PeakfreqRM = PeakfreqM
        STOREtrace = True

    UpdateFreqTrace()           # Always Update

def BCSVfile():
    STOREcsvfile()          # Store the trace as CSV file

def BScreensetup():
    global GRWN
    global GRW
    global GRHN
    global GRH
    global STOREtrace
    global Vdiv
    
    if (STOREtrace == True):
        showwarning("WARNING","Clearing stored trace first")
        STOREtrace = False
        return()

    s = askstring("Screensize", "Grid size scale factor")

    try:    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = float(s)
    except:
        s = "error"

    if s != "error":
        GRWF = int(GRWN*v)
        GRHF = int(GRHN*v)

    s = askstring("Divisions", "Value: " + str(Vdiv.get()) + "\n\nNew value:\n(4-20)")

    if (s == None): # If Cancel pressed, then None
        return()

    try:        # Error if for example no numeric characters or OK pressed without input (s = "")
        vs = int(s)
    except:
        s = "error"

    if s != "error":
        Vdiv.set(vs)

    if Vdiv.get() < 4:
        Vdiv.set(4)

    if Vdiv.get() > 20:
        Vdiv.set(20)
    UpdateFreqTrace()

def BStartSA():
    global RUNstatus, LoopNum, PowerStatus, devx, PwrBt
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P, STOREtrace, ShowMathSA

    if PowerStatus == 0:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power

    if ShowC1_VdB.get() == 0 and ShowC2_VdB.get() == 0 and ShowMathSA.get() == 0:
        showwarning("WARNING","Select at least one trace first")
        return()    
    if RUNstatus.get() == 0:
        RUNstatus.set(1)
        LoopNum.set(1)
    # Analog_In()
    UpdateFreqScreen()          # Always Update

def BStopSA():
    global RUNstatus
    
    if (RUNstatus.get() == 1):
        RUNstatus.set(0)
    elif (RUNstatus.get() == 2):
        RUNstatus.set(3)
    elif (RUNstatus.get() == 3):
        RUNstatus.set(3)
    elif (RUNstatus.get() == 4):
        RUNstatus.set(3)
    UpdateFreqScreen()          # Always Update
    
def Blevel1():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() - 1)
    
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def Blevel2():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() + 1)
    
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def Blevel3():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() - 10)
    
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def Blevel4():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() + 10)
    
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def BSetup():
    global RUNstatus
    global SAMPLErate
    global T1line
    global ZEROstuffing
    
    if (RUNstatus.get() != 0):
        showwarning("WARNING","Stop sweep first")
        return()
    
    s = askstring("Zero stuffing","For better interpolation of levels between frequency samples.\nBut increases processing time!\n\nValue: " + str(ZEROstuffing.get()) + "\n\nNew value:\n(0-5, 0 is no zero stuffing)")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        if v < 0:
            v = 0
        if v > 5:
            v = 5
        ZEROstuffing.set(v)

    UpdateFreqScreen()          # Always Update    

def BStartfrequency():
    global RUNstatus
    global STARTfrequency
    global STOPfrequency
    
    s = askstring("Startfrequency: ","Value: " + str(STARTfrequency.get()) + " Hz\n\nNew value:\n")
    
    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        STARTfrequency.set(v)

    if STARTfrequency.get() < 0:
        STARTfrequency.set(0)

    if STOPfrequency.get() <= STARTfrequency.get():
        STOPfrequency.set( STARTfrequency.get() + 1 )

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)

def BStopfrequency():
    global RUNstatus
    global STARTfrequency
    global STOPfrequency
    
    # if (RUNstatus != 0):
    #    showwarning("WARNING","Stop sweep first")
    #    return()

    s = askstring("Stopfrequency: ","Value: " + str(STOPfrequency.get()) + " Hz\n\nNew value:\n")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        STOPfrequency.set(abs(v))

    if STOPfrequency.get() < 10:  # Minimum stopfrequency 10 Hz
        STOPfrequency.set(10)
        
    if STARTfrequency.get() >= STOPfrequency.get():
        STARTfrequency.set(STOPfrequency.get() - 1)

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)

def Bsamples1():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
    
    if (SMPfftpwrTwo.get() > 6):  # Min 64
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() - 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqScreen()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)

def Bsamples2():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
      
    if (SMPfftpwrTwo.get() < 16): # Max 65536
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() + 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqScreen()
    if RUNstatus.get() == 2:      # Restart if running
        RUNstatus.set(4)

def BDBdiv1():
    global DBdivindex
    global RUNstatus
    
    if (DBdivindex.get() >= 1):
        DBdivindex.set(DBdivindex.get() - 1)

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def BDBdiv2():
    global DBdivindex
    global DBdivlist
    global RUNstatus
    
    if (DBdivindex.get() < len(DBdivlist) - 1):
        DBdivindex.set(DBdivindex.get() + 1)

    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()

def BSweepsteps():
    global RUNstatus
    global NSteps, LoopNum
    
    s = askstring("Number of frequency steps: ","Value: " + str(NSteps.get()) + " \n\nNew value:\n")
    
    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        NSteps.set(v)

    if NSteps.get() <= 0:
        NSteps.set(1)
    if NSteps.get() > 1000:
        NSteps.set(1000)
    if RUNstatus.get() == 0:      # Update if stopped
        UpdateFreqTrace()
    if RUNstatus.get() == 2:      # Restart loop count if running
        LoopNum.set(1)
        
# ============================================ Main routine ====================================================

def Analog_Freq_In():   # Read the audio from the stream and store the data into the arrays
    global ADsignal1, FFTBuffA, FFTBuffB
    global AWGSync, AWGAMode, AWGBMode
    global AWGAFreqvalue, AWGBFreqvalue
    global NSteps, LoopNum, FSweepMode
    global STARTfrequency, STOPfrequency
    global CHA, CHB, Both, devx
    global RUNstatus, SingleShot, FSweepCont
    global SAMPLErate
    global SMPfft, OverRangeFlagA, OverRangeFlagB
    global DCA, DCB
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    
    # Do input probe Calibration CH1VGain, CH2VGain, CH1VOffset, CH2VOffset
    try:
        InOffA = float(eval(CHAVOffsetEntry.get()))
    except:
        CHAVOffsetEntry.delete(0,END)
        CHAVOffsetEntry.insert(0, InOffA)
    try:
        InGainA = float(eval(CHAVGainEntry.get()))
    except:
        CHAVGainEntry.delete(0,END)
        CHAVGainEntry.insert(0, InGainA)
    try:
        InOffB = float(eval(CHBVOffsetEntry.get()))
    except:
        CHBVOffsetEntry.delete(0,END)
        CHBVOffsetEntry.insert(0, InOffB)
    try:
        InGainB = float(eval(CHBVGainEntry.get()))
    except:
        CHBVGainEntry.delete(0,END)
        CHBVGainEntry.insert(0, InGainB)
    #
    INITIALIZEstart()
    # Starting acquisition This is a HACK to get around non-continous AWG mode!
    # restat AWGs if indicated
    if FSweepMode.get() > 0:
        FStep = float(STOPfrequency.get() - STARTfrequency.get()) / float(NSteps.get())
        FregPoint = (LoopNum.get() * FStep) + float(STARTfrequency.get())
        if FSweepMode.get() == 1:
            AWGAFreqEntry.delete(0,END)
            AWGAFreqEntry.insert(4, FregPoint)
        if FSweepMode.get() == 2:
            AWGBFreqEntry.delete(0,END)
            AWGBFreqEntry.insert(4, FregPoint)
    if AWGSync.get() > 0:
        BAWGEnab()
    # 
    SHOWsamples = SMPfft + 128 # get 128 extra samples
    time.sleep(0.005)
    ADsignal1 = Both.get_samples(SHOWsamples) # get samples for both channel A and B
    # waite to finish then return to open termination
    devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set CHA 2.5 V switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set CHA GND switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
    devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
    if AWGAMode.get() == 3: # Split Input / Output mode
        devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
    if AWGBMode.get() == 3: # Split Input / Output mode
        devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
    
    FFTBuffA = [] # Clear the FFTBuff array for trace A
    FFTBuffB = [] # Clear the FFTBuff array for trace B
    OverRangeFlagA = OverRangeFlagB = 0 # Clear over range flags
    index = 127 # skip first 128 samples
    while index < SHOWsamples:
        SPA = ADsignal1[0][index]
        VAdata = float(SPA[0])
        VA = (VAdata - InOffA) * InGainA
        FFTBuffA.append(VA)
        SPB = ADsignal1[1][index]
        VBdata = float(SPB[0])
        VB = (VBdata - InOffB) * InGainB
        FFTBuffB.append(VB)
        if VAdata >= 5.0 or VAdata <= 0.0:
            OverRangeFlagA = 1

        if VBdata >= 5.0 or VBdata <= 0.0:
            OverRangeFlagB = 1
        index = index + 1

    DCA = numpy.average(FFTBuffA)
    DCB = numpy.average(FFTBuffB)
    if CutDC.get() == 1:
        FFTBuffA = FFTBuffA - DCA
        FFTBuffB = FFTBuffB - DCB
    UpdateFreqAll()                         # Update Data, trace and screen
    if SingleShot.get() == 1: # Single shot sweep is on
        RUNstatus.set(0)
# RUNstatus = 3: Stop
# RUNstatus = 4: Stop and restart
    if (RUNstatus.get() == 3) or (RUNstatus.get() == 4):
        if RUNstatus.get() == 3:
            RUNstatus.set(0)                # Status is stopped 
        if RUNstatus.get() == 4:          
            RUNstatus.set(1)                # Status is (re)start
        UpdateFreqScreen()                  # UpdateScreen() call
    if FSweepMode.get() > 0:
        LoopNum.set(LoopNum.get() + 1)
        if LoopNum.get() > NSteps.get():
            LoopNum.set(1)
            if FSweepCont.get() == 0:
                RUNstatus.set(0)

def UpdateFreqAll():        # Update Data, trace and screen
    global FFTBuffA, FFTBuffB
    global SMPfft

    if len(FFTBuffA) < SMPfft and len(FFTBuffB) < SMPfft:
        return
    
    DoFFT()             # Fast Fourier transformation
    MakeFreqTrace()         # Update the traces
    UpdateFreqScreen()      # Update the screen 

def UpdateFreqTrace():      # Update trace and screen
    MakeFreqTrace()         # Update traces
    UpdateFreqScreen()      # Update the screen

def UpdateFreqScreen():     # Update screen with trace and text
    MakeFreqScreen()        # Update the screen
#    subwindow.update()       # Activate updated screens    

def DoFFT():            # Fast Fourier transformation
    global FFTBuffA, FFTBuffB
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global FFTwindowshape, FFTbandwidth
    global SAMPLErate, STARTfrequency
    global SMPfft, LoopNum
    # global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample
    global TRACEaverage, TRACEmode, FSweepMode
    global TRACEreset
    global ZEROstuffing
    
    # T1 = time.time()              # For time measurement of FFT routine
    
    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffA[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Analog level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryA = FFTresultA
    if TRACEmode.get() == 3:
        PhaseMemoryA = PhaseA

    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseA = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (V*V) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseA = PhaseA[:le]
    PhaseA = numpy.degrees(PhaseA)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultA = Totalcorr * ALL

    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffB[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Audio level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryB = FFTresultB
    if TRACEmode.get() == 3:
        PhaseMemoryB = PhaseB
    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseB = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (U*U) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseB = PhaseB[:le]
    PhaseB = numpy.degrees(PhaseB)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultB = Totalcorr * ALL
    TRACEsize = len(FFTresultB)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum.get() == 1:
        PhaseMemoryB = PhaseB
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
#
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultB = FFTmemoryB + (FFTresultB - FFTmemoryB) / TRACEaverage.get()
        PhaseB = PhaseMemoryB +(PhaseB - PhaseMemoryB) / TRACEaverage.get()
# 
    TRACEsize = len(FFTresultA)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum.get() == 1:
        PhaseMemoryA = PhaseA
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
#
        FFTresultA = numpy.maximum(FFTresultA, FFTmemoryA)
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultA = FFTmemoryA + (FFTresultA - FFTmemoryA) / TRACEaverage.get()
        PhaseA = PhaseMemoryA +(PhaseA - PhaseMemoryA) / TRACEaverage.get()
#
    if TRACEmode.get() == 2 and TRACEreset == False:
        FFTresultB = numpy.maximum(FFTresultB, FFTmemoryB)

    TRACEreset = False          # Trace reset done

    # T2 = time.time()
    # print (T2 - T1)           # For time measurement of FFT routine

def MakeFreqTrace():        # Update the grid and trace
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global ShowC1_VdB, ShowC1_P, ShowC2_VdB, ShowC2_P, ShowMathSA
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB
    global DBdivindex   # Index value
    global DBdivlist    # dB per division list
    global DBlevel      # Reference level
    global GRHF          # Screenheight
    global GRWF          # Screenwidth
    global SAMPLErate
    global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample, LoopNum, FSweepMode
    global STOREtrace, TRACEmode
    global T1Fline, T2Fline, TFMline, T1Pline, T2Pline
    global Vdiv         # Number of vertical divisions
    global X0LF          # Left top X value
    global Y0TF          # Left top Y value

    # Set the TRACEsize variable
    TRACEsize = 0
    if ShowC1_VdB.get() == 1 or ShowMathSA.get() > 0:
        TRACEsize = len(FFTresultA)     # Set the trace length
    elif ShowC2_VdB.get() == 1 or ShowMathSA.get() > 0:
        TRACEsize = len(FFTresultB)
    if TRACEsize == 0:                  # If no trace, skip rest of this routine
        return()
    if FSweepMode.get() > 0 and LoopNum.get() == NSteps.get():
        PhaseA = PhaseMemoryA
        PhaseB = PhaseMemoryB
    # Vertical conversion factors (level dBs) and border limits
    Yconv = float(GRH) / (Vdiv.get() * DBdivlist[DBdivindex.get()])     # Conversion factors, Yconv is the number of screenpoints per dB
    Yc = float(Y0TF) + Yconv * (DBlevel.get())  # Yc is the 0 dBm position, can be outside the screen!
    Ymin = Y0TF                  # Minimum position of screen grid (top)
    Ymax = Y0TF + GRHF            # Maximum position of screen grid (bottom)
    Yphconv = float(GRH) / 360
    Yp = float(Y0TF) + Yphconv + 180
    # Horizontal conversion factors (frequency Hz) and border limits
    Fpixel = float(STOPfrequency.get() - STARTfrequency.get()) / GRWF    # Frequency step per screen pixel
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)       # Frequency step per sample   
    STARTsample = float(STARTfrequency.get()) / Fsample     # First sample in FFTresult[] that is used
    STARTsample = int(math.ceil(STARTsample))               # First within screen range

    STOPsample = float(STOPfrequency.get()) / Fsample       # Last sample in FFTresult[] that is used
    STOPsample = int(math.floor(STOPsample))                # Last within screen range, math.floor actually not necessary, part of int

    MAXsample = TRACEsize                                   # Just an out of range check
    if STARTsample > (MAXsample - 1):
        STARTsample = MAXsample - 1

    if STOPsample > MAXsample:
        STOPsample = MAXsample

    T1Fline = []
    T2Fline = []
    T1Pline = []
    T2Pline = []
    TFMline = []
    n = STARTsample
    PeakfreqA = PeakfreqB = PeakfreqM = F = n * Fsample
    if ShowC1_VdB.get() == 1:
        PeakdbA = (10 * math.log10(float(FFTresultA[n])) + 17)
        PeakyA = Yc - Yconv * PeakdbA
    if ShowC2_VdB.get() == 1:
        PeakdbB = (10 * math.log10(float(FFTresultB[n])) + 17)
        PeakyB = Yc - Yconv * PeakdbB
    if ShowMathSA.get() > 0:
        if ShowMathSA.get() == 1:
            PeakMdb = PeakdbA - PeakdbB
        if ShowMathSA.get() == 2:
            PeakMdb = PeakdbB - PeakdbA
        PeakyM = Yc - Yconv * PeakMdb
    PeakxA = PeakxB = PeakxM = x = X0LF + (F - STARTfrequency.get())  / Fpixel
    while n <= STOPsample:
        F = n * Fsample
        x = X0LF + (F - STARTfrequency.get())  / Fpixel
        if ShowC1_VdB.get() == 1: 
            T1Fline.append(int(x + 0.5))
            try:
                dbA = (10 * math.log10(float(FFTresultA[n])) + 17)   # Convert power to DBs, except for log(0) error
                ya = Yc - Yconv * dbA  # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
            except:
                ya = Ymax
            if (ya < Ymin):
                ya = Ymin
            if (ya > Ymax):
                ya = Ymax
            if dbA > PeakdbA:
                PeakdbA = dbA
                PeakyA = int(ya + 0.5)
                PeakxA = int(x + 0.5)
                PeakfreqA = F
            T1Fline.append(int(ya + 0.5))
        if ShowC2_VdB.get() == 1:
            T2Fline.append(int(x + 0.5))
            try:
                dbB = (10 * math.log10(float(FFTresultB[n])) + 17) # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
                yb = Yc - Yconv * dbB 
            except:
                yb = Ymax
            if (yb < Ymin):
                yb = Ymin
            if (yb > Ymax):
                yb = Ymax
            if dbB > PeakdbB:
                PeakdbB = dbB
                PeakyB = int(yb + 0.5)
                PeakxB = int(x + 0.5)
                PeakfreqB = F
            T2Fline.append(int(yb + 0.5))
        if ShowC1_P.get() == 1:
            T1Pline.append(int(x + 0.5))
            if FSweepMode.get() > 0:
                RelPhase = PhaseMemoryA[n]-PhaseMemoryB[n]
            else:
                RelPhase = PhaseA[n]-PhaseB[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            PhErr = 0.0018 * n * Fsample # calculate pahse error due half sample period offset
            RelPhase = RelPhase + PhErr - 12
            ya = Yp - Yphconv * RelPhase
            T1Pline.append(int(ya + 0.5))
        if ShowC2_P.get() == 1:
            T2Pline.append(int(x + 0.5))
            if FSweepMode.get() > 0:
                RelPhase = PhaseMemoryB[n]-PhaseMemoryA[n]
            else:
                RelPhase = PhaseB[n]-PhaseA[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            PhErr = 0.0018 * n * Fsample # calculate pahse error due half sample period offset
            RelPhase = RelPhase - PhErr - 12
            ya = Yp - Yphconv * RelPhase
            T2Pline.append(int(ya + 0.5))
        if ShowMathSA.get() > 0:
            TFMline.append(int(x + 0.5))
            dbA = (10 * math.log10(float(FFTresultA[n])) + 17) # Convert power to DBs, except for log(0) error
            dbB = (10 * math.log10(float(FFTresultB[n])) + 17) # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
            if ShowMathSA.get() == 1:
                MdB = dbA - dbB
            elif ShowMathSA.get() == 2:
                MdB = dbB - dbA
            yb = Yc - Yconv * MdB
            if (yb < Ymin):
                yb = Ymin
            if (yb > Ymax):
                yb = Ymax
            if MdB > PeakMdb:
                PeakMdb = MdB
                PeakyM = int(yb + 0.5)
                PeakxM = int(x + 0.5)
                PeakfreqM = F
            TFMline.append(int(yb + 0.5))
        n = n + 1               

def STOREcsvfile():     # Store the trace as CSV file [frequency, magnitude or dB value]
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB
    global SAMPLErate, ShowC1_VdB, ShowC2_VdB
    
    # Set the TRACEsize variable
    if ShowC1_VdB.get() == 1:
        TRACEsize = len(FFTresultA)     # Set the trace length
    elif ShowC2_VdB.get() == 1:
        TRACEsize = len(FFTresultB)
    if TRACEsize == 0:                  # If no trace, skip rest of this routine
        return()
# ask if save as magnitude or dB
    s = askstring("Mag or dB: ","Save amplidude data as Mag (0) or dB (1):\n")

    if (s == None):         # If Cancel pressed, then None
        return()
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        dB = int(s)
    except:
        dB = 1 # default to dB

    # Make the file name and open it
    tme =  strftime("%Y%b%d-%H%M%S", gmtime())      # The time
    filename = "Spectrum-" + tme
    filename = filename + ".csv"
    # open file to save data
    filename = asksaveasfilename(initialfile = filename, defaultextension = ".csv", filetypes=[("Comma Separated Values", "*.csv")])
    DataFile = open(filename,'a')  # Open output file
    HeaderString = 'Frequency-#, '
    if ShowC1_VdB.get() == 1:
        if dB == 1:
            HeaderString = HeaderString + 'CA-dB, '
        if dB == 0:
            HeaderString = HeaderString + 'CA-Mag, '
    if ShowC2_VdB.get() == 1:
        if dB == 1:
            HeaderString = HeaderString + 'CB-dB, '
        if dB == 0:
            HeaderString = HeaderString + 'CB-Mag, '
    if ShowC1_P.get() == 1:
        HeaderString = HeaderString + 'Phase A-B, '
    if ShowC2_P.get() == 1:
        HeaderString = HeaderString + 'Phase B-A, '
    HeaderString = HeaderString + '\n'
    DataFile.write( HeaderString )

    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)   # Frequency step per sample   

    n = 0
    while n < TRACEsize:
        F = n * Fsample
        txt = str(F)
        if ShowC1_VdB.get() == 1:
            if dB == 1:
                V = 10 * math.log10(float(FFTresultA[n])) + 17  # Add 17 dB for max value of +10 dB
            else:
                V = 50.12 * float(FFTresultA[n])# scale to Vrms
            txt = txt + "," + str(V) 
        if ShowC2_VdB.get() == 1:
            if dB == 1:
                V = 10 * math.log10(float(FFTresultB[n])) + 17  # Add 17 dB for max value of +10 dB
            else:
                V = 50.12 * float(FFTresultB[n]) # scale to Vrms
            txt = txt  + "," + str(V)
        if ShowC1_P.get() == 1:
            RelPhase = PhaseA[n]-PhaseB[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            txt = txt + "," + str(RelPhase)
        if ShowC2_P.get() == 1:
            RelPhase = PhaseB[n]-PhaseA[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            txt = txt + "," + str(RelPhase)
        txt = txt + "\n"
        DataFile.write(txt)
        n = n + 1    

    DataFile.close()                           # Close the file

def MakeFreqScreen():       # Update the screen with traces and text
    global CANVASheightF, CANVASwidthF, SmoothCurves
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB, PeakfreqRA, PeakfreqRB
    global PeakxRA, PeakyRA, PeakxRB, PeakyRB, PeakdbRA, PeakdbRB
    global PeakxRM, PeakyRM, PeakRMdb, PeakfreqRM
    global COLORgrid    # The colors
    global COLORsignalband, COLORtext
    global COLORtrace1, COLORtrace2
    global FSweepMode, LoopNum
    global DBdivindex   # Index value
    global DBdivlist    # dB per division list
    global DBlevel      # Reference level
    global FFTwindow, FFTbandwidth, ZEROstuffing, FFTwindowname
    global X0LF          # Left top X value
    global Y0TF          # Left top Y value
    global GRWF          # Screenwidth
    global GRHF          # Screenheight
    global RUNstatus    # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
    global SAMPLErate, SingleShot
    global SMPfft       # number of FFT samples
    global STARTfrequency, STOPfrequency
    global STOREtrace, ShowC1_P, ShowC2_P, ShowRA_VdB, ShowRB_VdB, ShowMarker
    global ShowRA_P, ShowRB_P, ShowMathSA
    global T1Fline, T2Fline, T1Pline, T1FRline, T1FRline, TFMline, TFRMline
    global T1PRline, T2PRline
    global TRACEaverage # Number of traces for averageing
    global TRACEmode    # 1 normal 2 max 3 average
    global Vdiv         # Number of vertical divisions

    # Delete all items on the screen
    de = Freqca.find_enclosed ( -1000, -1000, CANVASwidthF+1000, CANVASheightF+1000 )    
    for n in de: 
        Freqca.delete(n)
 
    # Draw horizontal grid lines
    i = 0
    x1 = X0LF
    x2 = X0LF + GRW
    while (i <= Vdiv.get()):
        y = Y0TF + i * GRH/Vdiv.get()
        Dline = [x1,y,x2,y]
        if i == 0 or i == Vdiv.get():
            Freqca.create_line(Dline, fill=COLORgrid)
        else:
            Freqca.create_line(Dline, dash=(4,3), fill=COLORgrid)
        Vaxis_value = (DBlevel.get() - (i * DBdivlist[DBdivindex.get()]))
        Vaxis_label = str(Vaxis_value)
        Freqca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
        if ShowC1_P.get() == 1 or ShowC2_P.get() == 1:
            Vaxis_value = ( 180 - ( i * (360 / Vdiv.get())))
            Vaxis_label = str(Vaxis_value)
            Freqca.create_text(x2+3, y, text=Vaxis_label, fill=COLORtrace3, anchor="w", font=("arial", 8 ))
        i = i + 1

    # Draw vertical grid lines
    i = 0
    y1 = Y0TF
    y2 = Y0TF + GRH
    Freqdiv = (STOPfrequency.get() - STARTfrequency.get()) / 10
    while (i < 11):
        x = X0LF + i * GRW/10
        Dline = [x,y1,x,y2]
        if i == 0 or i == 10:
            Freqca.create_line(Dline, fill=COLORgrid)
        else:
            Freqca.create_line(Dline, dash=(4,3), fill=COLORgrid)
        axis_value = (STARTfrequency.get() + (i * Freqdiv))
        axis_label = str(axis_value)
        Freqca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
        i = i + 1
    SmoothBool = SmoothCurves.get()
    # Draw traces
    if len(T1Fline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the trace CHA
        if OverRangeFlagA == 1:
            Freqca.create_line(T1Fline, fill=COLORsignalband, smooth=SmoothBool, splinestep=5)
        else:
            Freqca.create_line(T1Fline, fill=COLORtrace1, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbA) + ',' + ' {0:.1f} '.format(PeakfreqA)
            Freqca.create_text(PeakxA, PeakyA, text=Peak_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
    if len(T2Fline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the trace CHB
        if OverRangeFlagB == 1:
            Freqca.create_line(T2Fline, fill=COLORsignalband, smooth=SmoothBool, splinestep=5)
        else:
            Freqca.create_line(T2Fline, fill=COLORtrace2, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbB) + ',' + ' {0:.1f} '.format(PeakfreqB)
            Freqca.create_text(PeakxB, PeakyB, text=Peak_label, fill=COLORtrace2, anchor="w", font=("arial", 8 ))
    if len(T1Pline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the phase trace A-B 
        Freqca.create_line(T1Pline, fill=COLORtrace3, smooth=SmoothBool, splinestep=5)
    if len(T2Pline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the phase trace A-B 
        Freqca.create_line(T2Pline, fill=COLORtrace4, smooth=SmoothBool, splinestep=5)
    if ShowRA_VdB.get() == 1 and len(T1FRline) > 4:   # Write the ref trace A if active
        Freqca.create_line(T1FRline, fill=COLORtraceR1, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbRA) + ',' + ' {0:.1f} '.format(PeakfreqRA)
            Freqca.create_text(PeakxRA, PeakyRA, text=Peak_label, fill=COLORtraceR1, anchor="e", font=("arial", 8 ))
    if ShowRB_VdB.get() == 1 and len(T1FRline) > 4:   # Write the ref trace B if active
        Freqca.create_line(T1FRline, fill=COLORtraceR2, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbRB) + ',' + ' {0:.1f} '.format(PeakfreqRB)
            Freqca.create_text(PeakxRB, PeakyRB, text=Peak_label, fill=COLORtraceR2, anchor="w", font=("arial", 8 ))
    if ShowRA_P.get() == 1 and len(T1PRline) > 4:   # Write the ref trace A if active
        Freqca.create_line(T1PRline, fill=COLORtraceR3, smooth=SmoothBool, splinestep=5)
    if ShowRB_P.get() == 1 and len(T2PRline) > 4:   # Write the ref trace A if active
        Freqca.create_line(T2PRline, fill=COLORtraceR4, smooth=SmoothBool, splinestep=5)
    if ShowMathSA.get() > 0 and len(TFMline) > 4:   # Write the Math trace if active
        Freqca.create_line(TFMline, fill=COLORtrace5, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() ==1:
            Peak_label = ' {0:.2f} '.format(PeakMdb) + ',' + ' {0:.1f} '.format(PeakfreqM)
            Freqca.create_text(PeakxM, PeakyM, text=Peak_label, fill=COLORtrace5, anchor="w", font=("arial", 8 ))
    if ShowRMath.get() == 1 and len(TFRMline) > 4:   # Write the ref math trace if active
        Freqca.create_line(TFRMline, fill=COLORtraceR5, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() ==1:
            Peak_label = ' {0:.2f} '.format(PeakRMdb) + ',' + ' {0:.1f} '.format(PeakfreqRM)
            Freqca.create_text(PeakxRM, PeakyRM, text=Peak_label, fill=COLORtraceR5, anchor="w", font=("arial", 8 ))
    # General information on top of the grid

    txt = "    Sample rate: " + str(SAMPLErate)
    txt = txt + "    FFT samples: " + str(SMPfft)

    if FFTwindow.get() == 0:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 1:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 2:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 3:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 4:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 5:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 6:
        txt = txt + "   " + FFTwindowname
        
    x = X0LF
    y = 12
    idTXT = Freqca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)

    # Start and stop frequency and dB/div and trace mode
    txt = str(STARTfrequency.get()) + " to " + str(STOPfrequency.get()) + " Hz"
    txt = txt +  "  " + str(DBdivlist[DBdivindex.get()]) + " dB/div"
    txt = txt + "  Level: " + str(DBlevel.get()) + " dB "
    txt = txt + "  FFT Bandwidth =" + ' {0:.2f} '.format(FFTbandwidth)
    
    x = X0LF
    y = Y0TF+GRH+23
    idTXT = Freqca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)
    
    if TRACEmode.get() == 1:
        txt ="Normal mode "

    if TRACEmode.get() == 2:
        txt = "Peak hold mode "
    
    if TRACEmode.get() == 3:
        txt = "Power average  mode (" + str(TRACEaverage.get()) + ") " 

    if ZEROstuffing.get() > 0:
        txt = txt + "Zero Stuffing = " + str(ZEROstuffing.get())
    # Runstatus and level information
    if (RUNstatus.get() == 0) and (SingleShot.get() == 0):
        txt = txt + "  Stopped "
    elif SingleShot.get() == 1:
        txt = txt + "  Single Shot Trace "
    else:
        txt = txt + "  Running "
    if FSweepMode.get() > 0:
        txt = txt + "  Freq Step = " + str(LoopNum.get())
    x = X0LF
    y = Y0TF+GRH+34
    IDtxt  = Freqca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)

def INITIALIZEstart():
    global SMPfft
    global SMPfftpwrTwo
    global TRACEreset, TRACEmode, LoopNum, FSweepMode, FSweepCont

    # First some subroutines to set specific variables
    SMPfft = 2 ** int(SMPfftpwrTwo.get()) # Calculate the number of FFT samples from SMPfftpwrtwo

    CALCFFTwindowshape()
    if TRACEmode.get() == 1 and TRACEreset == False:
        TRACEreset = True               # Clear the memory for averaging or peak
    if TRACEmode.get() == 2 and LoopNum.get() == 1 and FSweepMode.get() > 0 and FSweepCont.get() == 0:
        TRACEreset = True               # Clear the memory for peak hold when using sweep generator

def CALCFFTwindowshape():           # Make the FFTwindowshape for the windowing function
    global FFTbandwidth             # The FFT bandwidth
    global FFTwindow                # Which FFT window number is selected
    global FFTwindowname            # The name of the FFT window function
    global FFTwindowshape           # The window shape
    global SAMPLErate               # The sample rate
    global SMPfft                   # Number of FFT samples
    
    # FFTname and FFTbandwidth in milliHz
    FFTwindowname = "No such window"
    FFTbw = 0
    
    if FFTwindow.get() == 0:
        FFTwindowname = " Rectangular (no) window (B=1) "
        FFTbw = 1.0

    if FFTwindow.get() == 1:
        FFTwindowname = " Cosine window (B=1.24) "
        FFTbw = 1.24

    if FFTwindow.get() == 2:
        FFTwindowname = " Triangular window (B=1.33) "
        FFTbw = 1.33

    if FFTwindow.get() == 3:
        FFTwindowname = " Hann window (B=1.5) "
        FFTbw = 1.5

    if FFTwindow.get() == 4:
        FFTwindowname = " Blackman window (B=1.73) "
        FFTbw = 1.73

    if FFTwindow.get() == 5:
        FFTwindowname = " Nuttall window (B=2.02) "
        FFTbw = 2.02

    if FFTwindow.get() == 6:
        FFTwindowname = " Flat top window (B=3.77) "
        FFTbw = 3.77

    FFTbandwidth = int(FFTbw * SAMPLErate / float(SMPfft)) 

    # Calculate the shape
    FFTwindowshape = numpy.ones(SMPfft)         # Initialize with ones

    # m = 0                                       # For calculation of correction factor, furhter no function

    n = 0
    while n < SMPfft:

        # Cosine window function
        # medium-dynamic range B=1.24
        if FFTwindow.get() == 1:
            w = math.sin(math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.571

        # Triangular non-zero endpoints
        # medium-dynamic range B=1.33
        if FFTwindow.get() == 2:
            w = (2.0 / SMPfft) * ((SMPfft/ 2.0) - abs(n - (SMPfft - 1) / 2.0))
            FFTwindowshape[n] = w * 2.0

        # Hann window function
        # medium-dynamic range B=1.5
        if FFTwindow.get() == 3:
            w = 0.5 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.000

        # Blackman window, continuous first derivate function
        # medium-dynamic range B=1.73
        if FFTwindow.get() == 4:
            w = 0.42 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.08 * math.cos(4 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.381

        # Nuttall window, continuous first derivate function
        # high-dynamic range B=2.02
        if FFTwindow.get() == 5:
            w = 0.355768 - 0.487396 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.144232 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.012604 * math.cos(6 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.811

        # Flat top window, 
        # medium-dynamic range, extra wide bandwidth B=3.77
        if FFTwindow.get() == 6:
            w = 1.0 - 1.93 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 1.29 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.388 * math.cos(6 * math.pi * n / (SMPfft - 1)) + 0.032 * math.cos(8 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.000
        
        # m = m + w / SMPfft                          # For calculation of correction factor
        n = n + 1

    # if m > 0:                                     # For calculation of correction factor
    #     print "correction 1/m: ", 1/m             # For calculation of correction factor


# ================ Make Screen ==========================

frame2r = Frame(root, borderwidth=5, relief=RIDGE)
frame2r.pack(side=RIGHT, expand=1, fill=Y)

frame1 = Frame(root, borderwidth=5, relief=RIDGE)
frame1.pack(side=TOP, expand=1, fill=X)

frame2 = Frame(root, background="black", borderwidth=5, relief=RIDGE)
frame2.pack(side=TOP, fill=BOTH, expand=YES)

frame3 = Frame(root, borderwidth=5, relief=RIDGE)
frame3.pack(side=TOP, expand=1, fill=X)
#
# create a pulldown menu
# Trigger signals
TgInput = IntVar(0)   # Trigger Input variable
SingleShot = IntVar(0) # variable for Single Shot manual trigger
AutoLevel = IntVar(0) # variable for Auto Level trigger at mid point
Triggermenu = Menubutton(frame1, text="Trigger", relief=RAISED)
Triggermenu.menu = Menu(Triggermenu, tearoff = 0 )
Triggermenu["menu"]  = Triggermenu.menu
Triggermenu.menu.add_radiobutton(label='None', variable=TgInput, value=0)
Triggermenu.menu.add_radiobutton(label='CA-V', variable=TgInput, value=1)
Triggermenu.menu.add_radiobutton(label='CA-I', variable=TgInput, value=2)
Triggermenu.menu.add_radiobutton(label='CB-V', variable=TgInput, value=3)
Triggermenu.menu.add_radiobutton(label='CB-I', variable=TgInput, value=4)
Triggermenu.menu.add_checkbutton(label='Auto Level', variable=AutoLevel)
Triggermenu.menu.add_checkbutton(label='SingleShot', variable=SingleShot)
Triggermenu.pack(side=LEFT)
#
TgEdge = IntVar(0)   # Trigger edge variable
Edgemenu = Menubutton(frame1, text="Edge", relief=RAISED)
Edgemenu.menu = Menu(Edgemenu, tearoff = 0 )
Edgemenu["menu"]  = Edgemenu.menu
Edgemenu.menu.add_radiobutton(label='Rising', variable=TgEdge, value=0)
Edgemenu.menu.add_radiobutton(label='Falling', variable=TgEdge, value=1)
Edgemenu.pack(side=LEFT)
#
tlab = Label(frame1, text="Trig Level")
tlab.pack(side=LEFT)
TRIGGERentry = Entry(frame1, width=5)
TRIGGERentry.bind("<Return>", BTriglevel)
TRIGGERentry.pack(side=LEFT)
TRIGGERentry.delete(0,"end")
TRIGGERentry.insert(0,0.0)
#
tgb = Button(frame1, text="50%", width=4, command=BTrigger50p)
tgb.pack(side=LEFT)
#
hldlab = Label(frame1, text="Hold Off")
hldlab.pack(side=LEFT)
HoldOffentry = Entry(frame1, width=4)
HoldOffentry.bind("<Return>", BHoldOff)
HoldOffentry.pack(side=LEFT)
HoldOffentry.delete(0,"end")
HoldOffentry.insert(0,0.01)
#
b = Button(frame1, text="Exit", width=4, command=Bcloseexit)
b.pack(side=RIGHT)
b = Button(frame1, text="Stop", width=4, bg="red", command=BStop)
b.pack(side=RIGHT)
b = Button(frame1, text="Run", width=3, bg="green", command=BStart)
b.pack(side=RIGHT)
PwrBt = Button(frame1, text="PWR-ON", width=7, bg="green", command=BPower)
PwrBt.pack(side=RIGHT)
# Show channels menu
ShowC1_V = IntVar(0)   # curves to display variables
ShowC1_I = IntVar(0)
ShowC2_V = IntVar(0)
ShowC2_I = IntVar(0)
ShowAV_I = IntVar(0)
ShowBV_I = IntVar(0)
ShowRA_V = IntVar(0)
ShowRA_I = IntVar(0)
ShowRB_V = IntVar(0)
ShowRB_I = IntVar(0)
ShowMath = IntVar(0)
#
Showmenu = Menubutton(frame1, text="Curves", relief=RAISED)
Showmenu.menu = Menu(Showmenu, tearoff = 0 )
Showmenu["menu"] = Showmenu.menu
Showmenu.menu.add_command(label="-Show-", command=donothing)
Showmenu.menu.add_command(label="All", command=BShowCurvesAll)
Showmenu.menu.add_command(label="None", command=BShowCurvesNone)
Showmenu.menu.add_checkbutton(label='CA-V', variable=ShowC1_V)
Showmenu.menu.add_checkbutton(label='CA-I', variable=ShowC1_I)
Showmenu.menu.add_checkbutton(label='CB-V', variable=ShowC2_V)
Showmenu.menu.add_checkbutton(label='CB-I', variable=ShowC2_I)
Showmenu.menu.add_separator()
Showmenu.menu.add_checkbutton(label='RA-V', variable=ShowRA_V)
Showmenu.menu.add_checkbutton(label='RA-I', variable=ShowRA_I)
Showmenu.menu.add_checkbutton(label='RB-V', variable=ShowRB_V)
Showmenu.menu.add_checkbutton(label='RB-I', variable=ShowRB_I)
Showmenu.menu.add_checkbutton(label='RMath', variable=ShowMath)
Showmenu.pack(side=RIGHT)
# Disply mode menu
DisMode = IntVar(0)   # Display mode variable
Xsignal = IntVar(0)   # Signal for X axis variable
Ysignal = IntVar(0)   # Signal for X axis variable
ShowRXY = IntVar(0)   # show reference XY trace
Displaymenu = Menubutton(frame1, text="Display", relief=RAISED)
Displaymenu.menu = Menu(Displaymenu, tearoff = 0 )
Displaymenu["menu"]  = Displaymenu.menu
Displaymenu.menu.add_radiobutton(label='Time', variable=DisMode, value=0)
Displaymenu.menu.add_radiobutton(label='X-Y', variable=DisMode, value=1)
Displaymenu.menu.add_separator()
# X - Y mode signal select
Displaymenu.menu.add_command(label="-X Axis-", command=donothing)
Displaymenu.menu.add_radiobutton(label='none', variable=Xsignal, value=0)
Displaymenu.menu.add_radiobutton(label='CA-V', variable=Xsignal, value=1)
Displaymenu.menu.add_radiobutton(label='CA-I', variable=Xsignal, value=2)
Displaymenu.menu.add_radiobutton(label='CB-V', variable=Xsignal, value=3)
Displaymenu.menu.add_radiobutton(label='CB-I', variable=Xsignal, value=4)
#
Displaymenu.menu.add_separator()
Displaymenu.menu.add_command(label="-Y Axis-", command=donothing)
Displaymenu.menu.add_radiobutton(label='none', variable=Ysignal, value=0)
Displaymenu.menu.add_radiobutton(label='CA-V', variable=Ysignal, value=1)
Displaymenu.menu.add_radiobutton(label='CA-I', variable=Ysignal, value=2)
Displaymenu.menu.add_radiobutton(label='CB-V', variable=Ysignal, value=3)
Displaymenu.menu.add_radiobutton(label='CB-I', variable=Ysignal, value=4)
Displaymenu.menu.add_separator()
Displaymenu.menu.add_checkbutton(label='RX-Y', variable=ShowRXY)
Displaymenu.pack(side=RIGHT)
# Time per Div
TMsb = Spinbox(frame1, width=5, values= TMpdiv, command=BTime)
TMsb.pack(side=RIGHT)
TMsb.delete(0,"end")
TMsb.insert(0,0.5)
TMlab = Label(frame1, text="Time mS/Div")
TMlab.pack(side=RIGHT)
#
ca = Canvas(frame2, width=CANVASwidth, height=CANVASheight, background=COLORcanvas)
ca.pack(side=TOP, fill=BOTH, expand=YES)
# right side menu buttons
dropmenu = Frame( frame2r )
dropmenu.pack(side=TOP)
bcon = Button(dropmenu, text="Recon", width=5, command=ConnectDevice)
bcon.pack(side=LEFT, anchor=W)
# File menu
Filemenu = Menubutton(dropmenu, text="File", relief=RAISED)
Filemenu.menu = Menu(Filemenu, tearoff = 0 )
Filemenu["menu"] = Filemenu.menu
Filemenu.menu.add_command(label="Save Config", command=BSaveConfig)
Filemenu.menu.add_command(label="Load Config", command=BLoadConfig)
Filemenu.menu.add_command(label="Save Screen", command=BSaveScreen)
Filemenu.menu.add_command(label="Save Data", command=BSaveData)
Filemenu.menu.add_command(label="Help", command=BHelp)
Filemenu.pack(side=LEFT, anchor=W)
#
SmoothCurves = IntVar(0)
ZOHold = IntVar(0)
Optionmenu = Menubutton(dropmenu, text="Options", relief=RAISED)
Optionmenu.menu = Menu(Optionmenu, tearoff = 0 )
Optionmenu["menu"]  = Optionmenu.menu
Optionmenu.menu.add_checkbutton(label='Smooth', variable=SmoothCurves)
Optionmenu.menu.add_checkbutton(label='Z-O-Hold', variable=ZOHold)
Optionmenu.menu.add_command(label="SnapShot", command=BSnapShot)
Optionmenu.menu.add_command(label="Save Cal", command=BSaveCal)
Optionmenu.menu.add_command(label="Load Cal", command=BLoadCal)
Optionmenu.pack(side=LEFT, anchor=W)
#
dropmenu2 = Frame( frame2r )
dropmenu2.pack(side=TOP)
# Math trace menu
MathTrace = IntVar(0)
MathMenu = Menubutton(dropmenu2, text="Math", width=3, relief=RAISED)
MathMenu.menu = Menu(MathMenu, tearoff = 0 )
MathMenu["menu"]  = MathMenu.menu
MathMenu.menu.add_radiobutton(label='none', variable=MathTrace, value=0)
MathMenu.menu.add_radiobutton(label='CAV+CBV', variable=MathTrace, value=1)
MathMenu.menu.add_radiobutton(label='CAV-CBV', variable=MathTrace, value=2)
MathMenu.menu.add_radiobutton(label='CBV-CAV', variable=MathTrace, value=3)
MathMenu.menu.add_radiobutton(label='CAI-CBI', variable=MathTrace, value=8)
MathMenu.menu.add_radiobutton(label='CBI-CAI', variable=MathTrace, value=9)
MathMenu.menu.add_radiobutton(label='CAV*CAI', variable=MathTrace, value=4)
MathMenu.menu.add_radiobutton(label='CBV*CBI', variable=MathTrace, value=5)
MathMenu.menu.add_radiobutton(label='CAV/CAI', variable=MathTrace, value=6)
MathMenu.menu.add_radiobutton(label='CBV/CBI', variable=MathTrace, value=7)
MathMenu.menu.add_radiobutton(label='CBV/CAV', variable=MathTrace, value=10)
MathMenu.menu.add_radiobutton(label='CBI/CAI', variable=MathTrace, value=11)
MathMenu.pack(side=RIGHT, anchor=W)
# Measurments menu
MeasDCV1 = IntVar(0) # define vertical measurment variables
MeasMinV1 = IntVar(0)
MeasMaxV1 = IntVar(0)
MeasMidV1 = IntVar(0)
MeasPPV1 = IntVar(0)
MeasRMSV1 = IntVar(0)
MeasDCI1 = IntVar(0)
MeasMinI1 = IntVar(0)
MeasMaxI1 = IntVar(0)
MeasMidI1 = IntVar(0)
MeasPPI1 = IntVar(0)
MeasRMSI1 = IntVar(0)
MeasDCV2 = IntVar(0)
MeasMinV2 = IntVar(0)
MeasMaxV2 = IntVar(0)
MeasMidV2 = IntVar(0)
MeasPPV2 = IntVar(0)
MeasRMSV2 = IntVar(0)
MeasDCI2 = IntVar(0)
MeasMinI2 = IntVar(0)
MeasMaxI2 = IntVar(0)
MeasMidI2 = IntVar(0)
MeasPPI2 = IntVar(0)
MeasRMSI2 = IntVar(0)
MeasAHW = IntVar(0)
MeasALW = IntVar(0)
MeasADCy = IntVar(0)
MeasAPER = IntVar(0)
MeasAFREQ = IntVar(0)
MeasBHW = IntVar(0)
MeasBLW = IntVar(0)
MeasBDCy = IntVar(0)
MeasBPER = IntVar(0)
MeasBFREQ = IntVar(0)
MeasPhase = IntVar(0)
#
measlab = Label(dropmenu2, text="Meas")
measlab.pack(side=LEFT, anchor=W)
MeasmenuA = Menubutton(dropmenu2, text="CA", width=2, relief=RAISED)
MeasmenuA.menu = Menu(MeasmenuA, tearoff = 0 )
MeasmenuA["menu"]  = MeasmenuA.menu
MeasmenuA.menu.add_command(label="-CA-V-", command=donothing)
MeasmenuA.menu.add_checkbutton(label='Avg', variable=MeasDCV1)
MeasmenuA.menu.add_checkbutton(label='Min', variable=MeasMinV1)
MeasmenuA.menu.add_checkbutton(label='Max', variable=MeasMaxV1)
MeasmenuA.menu.add_checkbutton(label='Mid', variable=MeasMidV1)
MeasmenuA.menu.add_checkbutton(label='P-P', variable=MeasPPV1)
MeasmenuA.menu.add_checkbutton(label='RMS', variable=MeasRMSV1)
MeasmenuA.menu.add_separator()
#
MeasmenuA.menu.add_command(label="-CA-I-", command=donothing)
MeasmenuA.menu.add_checkbutton(label='Avg', variable=MeasDCI1)
MeasmenuA.menu.add_checkbutton(label='Min', variable=MeasMinI1)
MeasmenuA.menu.add_checkbutton(label='Max', variable=MeasMaxI1)
MeasmenuA.menu.add_checkbutton(label='Mid', variable=MeasMidI1)
MeasmenuA.menu.add_checkbutton(label='P-P', variable=MeasPPI1)
MeasmenuA.menu.add_checkbutton(label='RMS', variable=MeasRMSI1)
MeasmenuA.menu.add_separator()
#
MeasmenuA.menu.add_command(label="CA-Time", command=donothing)
MeasmenuA.menu.add_checkbutton(label='H-Width', variable=MeasAHW)
MeasmenuA.menu.add_checkbutton(label='L-Width', variable=MeasALW)
MeasmenuA.menu.add_checkbutton(label='DutyCyle', variable=MeasADCy)
MeasmenuA.menu.add_checkbutton(label='Period', variable=MeasAPER)
MeasmenuA.menu.add_checkbutton(label='Freq', variable=MeasAFREQ)
MeasmenuA.menu.add_checkbutton(label='A-B Phase', variable=MeasPhase)
#
MeasmenuA.pack(side=LEFT)
#
MeasmenuB = Menubutton(dropmenu2, text="CB", width=2, relief=RAISED)
MeasmenuB.menu = Menu(MeasmenuB, tearoff = 0 )
MeasmenuB["menu"]  = MeasmenuB.menu
MeasmenuB.menu.add_command(label="-CB-V-", command=donothing)
MeasmenuB.menu.add_checkbutton(label='Avg', variable=MeasDCV2)
MeasmenuB.menu.add_checkbutton(label='Min', variable=MeasMinV2)
MeasmenuB.menu.add_checkbutton(label='Max', variable=MeasMaxV2)
MeasmenuB.menu.add_checkbutton(label='Mid', variable=MeasMidV2)
MeasmenuB.menu.add_checkbutton(label='P-P', variable=MeasPPV2)
MeasmenuB.menu.add_checkbutton(label='RMS', variable=MeasRMSV2)
MeasmenuB.menu.add_separator()
#
MeasmenuB.menu.add_command(label="-CB-I-", command=donothing)
MeasmenuB.menu.add_checkbutton(label='Avg', variable=MeasDCI2)
MeasmenuB.menu.add_checkbutton(label='Min', variable=MeasMinI2)
MeasmenuB.menu.add_checkbutton(label='Max', variable=MeasMaxI2)
MeasmenuB.menu.add_checkbutton(label='Mid', variable=MeasMidI2)
MeasmenuB.menu.add_checkbutton(label='P-P', variable=MeasPPI2)
MeasmenuB.menu.add_checkbutton(label='RMS', variable=MeasRMSI2)
MeasmenuB.menu.add_separator()
#
MeasmenuB.menu.add_command(label="CB-Time", command=donothing)
MeasmenuB.menu.add_checkbutton(label='H-Width', variable=MeasBHW)
MeasmenuB.menu.add_checkbutton(label='L-Width', variable=MeasBLW)
MeasmenuB.menu.add_checkbutton(label='DutyCyle', variable=MeasBDCy)
MeasmenuB.menu.add_checkbutton(label='Period', variable=MeasBPER)
MeasmenuB.menu.add_checkbutton(label='Freq', variable=MeasBFREQ)
MeasmenuB.pack(side=LEFT)
# Mode selector
RUNmode = IntVar(0)
rb1 = Radiobutton(frame2r, text="Time", variable=RUNmode, value=0 )
rb1.pack(side=TOP)
rb2 = Radiobutton(frame2r, text="Freq", variable=RUNmode, value=1 )
rb2.pack(side=TOP)
rb3 = Radiobutton(frame2r, text="Both", variable=RUNmode, value=2 )
rb3.pack(side=TOP)
# Digital Input / Output Widgets
DI0 = IntVar(0)
DI1 = IntVar(0)
DI2 = IntVar(0)
DI3 = IntVar(0)
DI4 = IntVar(0)
DI5 = IntVar(0)
DI6 = IntVar(0)
DI7 = IntVar(0)
dig1 = Frame( frame2r )
dig1.pack(side=TOP)
Dig1Menu = Menubutton(dig1, text="Digital 0-3", relief=RAISED)
Dig1Menu.menu = Menu(Dig1Menu, tearoff = 0 )
Dig1Menu["menu"] = Dig1Menu.menu
Dig1Menu.menu.add_radiobutton(label="PA0-0", variable=DI0, value=0x50, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA0-Z", variable=DI0, value=0, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA0-1", variable=DI0, value=0x51, command=SetDigOut)
Dig1Menu.menu.add_separator()
Dig1Menu.menu.add_radiobutton(label="PA1-0", variable=DI1, value=0x50, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA1-Z", variable=DI1, value=0, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA1-1", variable=DI1, value=0x51, command=SetDigOut)
Dig1Menu.menu.add_separator()
Dig1Menu.menu.add_radiobutton(label="PA2-0", variable=DI2, value=0x50, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA2-Z", variable=DI2, value=0, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA2-1", variable=DI2, value=0x51, command=SetDigOut)
Dig1Menu.menu.add_separator()
Dig1Menu.menu.add_radiobutton(label="PA3-0", variable=DI3, value=0x50, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA3-Z", variable=DI3, value=0, command=SetDigOut)
Dig1Menu.menu.add_radiobutton(label="PA3-1", variable=DI3, value=0x51, command=SetDigOut)
Dig1Menu.pack(side=LEFT, anchor=W)
#
dig2 = Frame( frame2r )
dig2.pack(side=TOP)
Dig2Menu = Menubutton(dig2, text="Digital 4-7", relief=RAISED)
Dig2Menu.menu = Menu(Dig2Menu, tearoff = 0 )
Dig2Menu["menu"] = Dig2Menu.menu
Dig2Menu.menu.add_radiobutton(label="PA4-0", variable=DI4, value=0x50, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA4-Z", variable=DI4, value=0, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA4-1", variable=DI4, value=0x51, command=SetDigOut)
Dig2Menu.menu.add_separator()
Dig2Menu.menu.add_radiobutton(label="PA5-0", variable=DI5, value=0x50, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA5-Z", variable=DI5, value=0, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA5-1", variable=DI5, value=0x51, command=SetDigOut)
Dig2Menu.menu.add_separator()
Dig2Menu.menu.add_radiobutton(label="PA6-0", variable=DI6, value=0x50, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA6-Z", variable=DI6, value=0, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA6-1", variable=DI6, value=0x51, command=SetDigOut)
Dig2Menu.menu.add_separator()
Dig2Menu.menu.add_radiobutton(label="PA7-0", variable=DI7, value=0x50, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA7-Z", variable=DI7, value=0, command=SetDigOut)
Dig2Menu.menu.add_radiobutton(label="PA7-1", variable=DI7, value=0x51, command=SetDigOut)
Dig2Menu.pack(side=LEFT, anchor=W)
# input probe wigets
prlab = Label(frame2r, text="Gain Offset Cal")
prlab.pack(side=TOP)
# Input Probes sub frame 
ProbeA = Frame( frame2r )
ProbeA.pack(side=TOP)
gain1lab = Label(ProbeA, text="CA-V")
gain1lab.pack(side=LEFT)
CHAVGainEntry = Entry(ProbeA, width=5)
CHAVGainEntry.pack(side=LEFT)
CHAVGainEntry.delete(0,"end")
CHAVGainEntry.insert(0,1.0)
CHAVOffsetEntry = Entry(ProbeA, width=5)
CHAVOffsetEntry.pack(side=LEFT)
CHAVOffsetEntry.delete(0,"end")
CHAVOffsetEntry.insert(0,0.0)
#
ProbeB = Frame( frame2r )
ProbeB.pack(side=TOP)
gain2lab = Label(ProbeB, text="CB-V")
gain2lab.pack(side=LEFT)
CHBVGainEntry = Entry(ProbeB, width=5)
CHBVGainEntry.pack(side=LEFT)
CHBVGainEntry.delete(0,"end")
CHBVGainEntry.insert(0,1.0)
CHBVOffsetEntry = Entry(ProbeB, width=5)
CHBVOffsetEntry.pack(side=LEFT)
CHBVOffsetEntry.delete(0,"end")
CHBVOffsetEntry.insert(0,0.0)
#
ProbeAI = Frame( frame2r )
ProbeAI.pack(side=TOP)
gainailab = Label(ProbeAI, text="CA-I")
gainailab.pack(side=LEFT)
CHAIGainEntry = Entry(ProbeAI, width=5)
CHAIGainEntry.pack(side=LEFT)
CHAIGainEntry.delete(0,"end")
CHAIGainEntry.insert(0,1.0)
CHAIOffsetEntry = Entry(ProbeAI, width=5)
CHAIOffsetEntry.pack(side=LEFT)
CHAIOffsetEntry.delete(0,"end")
CHAIOffsetEntry.insert(0,0.0)
#
ProbeBI = Frame( frame2r )
ProbeBI.pack(side=TOP)
gainbilab = Label(ProbeBI, text="CB-I")
gainbilab.pack(side=LEFT)
CHBIGainEntry = Entry(ProbeBI, width=5)
CHBIGainEntry.pack(side=LEFT)
CHBIGainEntry.delete(0,"end")
CHBIGainEntry.insert(0,1.0)
CHBIOffsetEntry = Entry(ProbeBI, width=5)
CHBIOffsetEntry.pack(side=LEFT)
CHBIOffsetEntry.delete(0,"end")
CHBIOffsetEntry.insert(0,0.0)
# add ADI logo Don't mess with this bit map data!
ADIlogo = """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"""
logo = PhotoImage(data=ADIlogo)
ADI1 = Label(frame2r, image=logo, anchor= "sw", height=49, width=116, compound="top")
ADI1.pack(side=TOP)

# Bottom Buttons
# Voltage channel A
CHAsb = Spinbox(frame3, width=4, values=CHvpdiv, command=BCHAlevel)
CHAsb.pack(side=LEFT)
CHAsb.delete(0,"end")
CHAsb.insert(0,0.5)
CHAlab = Label(frame3, text="CA V/Div", bg=COLORtrace1)
CHAlab.pack(side=LEFT)

CHAVPosEntry = Entry(frame3, width=5)
CHAVPosEntry.bind("<Return>", BOffsetA)
CHAVPosEntry.pack(side=LEFT)
CHAVPosEntry.delete(0,"end")
CHAVPosEntry.insert(0,2.5)
CHAofflab = Label(frame3, text="CA V Pos", bg=COLORtrace1)
CHAofflab.pack(side=LEFT)
# Current channel A
CHAIsb = Spinbox(frame3, width=4, values=CHipdiv, command=BCHAIlevel)
CHAIsb.pack(side=LEFT)
CHAIsb.delete(0,"end")
CHAIsb.insert(0,50.0)
CHAIlab = Label(frame3, text="CA mA/Div", bg=COLORtrace3)
CHAIlab.pack(side=LEFT)

CHAIPosEntry = Entry(frame3, width=5)
CHAIPosEntry.bind("<Return>", BIOffsetA)
CHAIPosEntry.pack(side=LEFT)
CHAIPosEntry.delete(0,"end")
CHAIPosEntry.insert(0,0.0)
CHAIofflab = Label(frame3, text="CA I Pos", bg=COLORtrace3)
CHAIofflab.pack(side=LEFT)
# Voltage channel B
CHBsb = Spinbox(frame3, width=4, values=CHvpdiv, command=BCHBlevel)
CHBsb.pack(side=LEFT)
CHBsb.delete(0,"end")
CHBsb.insert(0,0.5)
CHBlab = Label(frame3, text="CB V/Div", bg=COLORtrace2)
CHBlab.pack(side=LEFT)

CHBVPosEntry = Entry(frame3, width=5)
CHBVPosEntry.bind("<Return>", BOffsetB)
CHBVPosEntry.pack(side=LEFT)
CHBVPosEntry.delete(0,"end")
CHBVPosEntry.insert(0,2.5)
CHBofflab = Label(frame3, text="CB V Pos", bg=COLORtrace2)
CHBofflab.pack(side=LEFT)
# Current channel B
CHBIsb = Spinbox(frame3, width=4, values=CHipdiv, command=BCHBIlevel)
CHBIsb.pack(side=LEFT)
CHBIsb.delete(0,"end")
CHBIsb.insert(0,50.0)
CHBIlab = Label(frame3, text="CB mA/Div", bg=COLORtrace4)
CHBIlab.pack(side=LEFT)

CHBIPosEntry = Entry(frame3, width=5)
CHBIPosEntry.bind("<Return>", BIOffsetB)
CHBIPosEntry.pack(side=LEFT)
CHBIPosEntry.delete(0,"end")
CHBIPosEntry.insert(0,0.0)
CHBIofflab = Label(frame3, text="CB I Pos", bg=COLORtrace4)
CHBIofflab.pack(side=LEFT)
# ================ Make awg sub window ==========================
subwindow = Toplevel()
subwindow.title("AWG")
#subwindow.call('wm', 'iconphoto', subwindow._w, img)
frame2 = Frame(subwindow, borderwidth=5, relief=RIDGE)
frame2.pack(side=LEFT, expand=1, fill=Y)
frame3 = Frame(subwindow, borderwidth=5, relief=RIDGE)
frame3.pack(side=LEFT, expand=1, fill=X)
# now AWG A
awg1lab = Label(frame2, text="AWG CH A")
awg1lab.pack(side=TOP, pady=5)
# AWG enable sub frame
AWGAMode = IntVar(0)   # AWG A mode variable
AWGATerm = IntVar(0)   # AWG A termination variable
AWGAShape = IntVar(0)  # AWG A Wave shape variable
awg1eb = Frame( frame2 )
awg1eb.pack(side=TOP)
ModeAMenu = Menubutton(awg1eb, text="Mode", relief=RAISED)
ModeAMenu.menu = Menu(ModeAMenu, tearoff = 0 )
ModeAMenu["menu"] = ModeAMenu.menu
ModeAMenu.menu.add_command(label="-Mode-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="SVMI", variable=AWGAMode, value=0)
ModeAMenu.menu.add_radiobutton(label="SIMV", variable=AWGAMode, value=1)
ModeAMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGAMode, value=2)
ModeAMenu.menu.add_radiobutton(label="Split I/O", variable=AWGAMode, value=3)
ModeAMenu.menu.add_separator()
ModeAMenu.menu.add_command(label="-Term-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="Open", variable=AWGATerm, value=0)
ModeAMenu.menu.add_radiobutton(label="To GND", variable=AWGATerm, value=1)
ModeAMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGATerm, value=2)
ModeAMenu.pack(side=LEFT, anchor=W)
ShapeAMenu = Menubutton(awg1eb, text="Shape", relief=RAISED)
ShapeAMenu.menu = Menu(ShapeAMenu, tearoff = 0 )
ShapeAMenu["menu"] = ShapeAMenu.menu
ShapeAMenu.menu.add_radiobutton(label="DC", variable=AWGAShape, value=0)
ShapeAMenu.menu.add_radiobutton(label="Sine", variable=AWGAShape, value=1)
ShapeAMenu.menu.add_radiobutton(label="Triangle", variable=AWGAShape, value=2)
ShapeAMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGAShape, value=3)
ShapeAMenu.menu.add_radiobutton(label="Square", variable=AWGAShape, value=4)
ShapeAMenu.menu.add_radiobutton(label="StairStep", variable=AWGAShape, value=5)
ShapeAMenu.menu.add_radiobutton(label="Custom", variable=AWGAShape, value=6)
ShapeAMenu.pack(side=LEFT, anchor=W)
#
awg1ampl = Frame( frame2 )
awg1ampl.pack(side=TOP)
AWGAAmplEntry = Entry(awg1ampl, width=5)
AWGAAmplEntry.bind("<Return>", BAWGAAmpl)
AWGAAmplEntry.pack(side=LEFT, anchor=W)
AWGAAmplEntry.delete(0,"end")
AWGAAmplEntry.insert(0,0.0)
amp1lab = Label(awg1ampl, text="Min")
amp1lab.pack(side=LEFT, anchor=W)
#
awg1off = Frame( frame2 )
awg1off.pack(side=TOP)
AWGAOffsetEntry = Entry(awg1off, width=5)
AWGAOffsetEntry.bind("<Return>", BAWGAOffset)
AWGAOffsetEntry.pack(side=LEFT, anchor=W)
AWGAOffsetEntry.delete(0,"end")
AWGAOffsetEntry.insert(0,0.0)
off1lab = Label(awg1off, text="Max")
off1lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg1freq = Frame( frame2 )
awg1freq.pack(side=TOP)
AWGAFreqEntry = Entry(awg1freq, width=7)
AWGAFreqEntry.bind("<Return>", BAWGAFreq)
AWGAFreqEntry.pack(side=LEFT, anchor=W)
AWGAFreqEntry.delete(0,"end")
AWGAFreqEntry.insert(0,0.0)
freq1lab = Label(awg1freq, text="Freq")
freq1lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg1phase = Frame( frame2 )
awg1phase.pack(side=TOP)
AWGAPhaseEntry = Entry(awg1phase, width=5)
AWGAPhaseEntry.bind("<Return>", BAWGAPhase)
AWGAPhaseEntry.pack(side=LEFT, anchor=W)
AWGAPhaseEntry.delete(0,"end")
AWGAPhaseEntry.insert(0,0)
phase1lab = Label(awg1phase, text="Phase")
phase1lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg1dc = Frame( frame2 )
awg1dc.pack(side=TOP)
AWGADutyCycleEntry = Entry(awg1dc, width=5)
AWGADutyCycleEntry.bind("<Return>", BAWGADutyCycle)
AWGADutyCycleEntry.pack(side=LEFT, anchor=W)
AWGADutyCycleEntry.delete(0,"end")
AWGADutyCycleEntry.insert(0,50)
duty1lab = Label(awg1dc, text="%")
duty1lab.pack(side=LEFT, anchor=W)
#
AWGSync = IntVar(0) # Sync start both AWG channels
awgsync = Checkbutton(frame2, text="Sync AWG", variable=AWGSync, command=BAWGSync)
awgsync.pack(side=TOP)
# now AWG B
awg2lab = Label(frame3, text="AWG CH B")
awg2lab.pack(side=TOP, pady=5)
# AWG enable sub frame
AWGBMode = IntVar(0)   # AWG B mode variable
AWGBTerm = IntVar(0)   # AWG B termination variable
AWGBShape = IntVar(0)  # AWG B Wave shape variable
awg2eb = Frame( frame3 )
awg2eb.pack(side=TOP)
ModeBMenu = Menubutton(awg2eb, text="Mode", relief=RAISED)
ModeBMenu.menu = Menu(ModeBMenu, tearoff = 0 )
ModeBMenu["menu"] = ModeBMenu.menu
ModeBMenu.menu.add_command(label="-Mode-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="SVMI", variable=AWGBMode, value=0)
ModeBMenu.menu.add_radiobutton(label="SIMV", variable=AWGBMode, value=1)
ModeBMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGBMode, value=2)
ModeBMenu.menu.add_radiobutton(label="Split I/O", variable=AWGBMode, value=3)
ModeBMenu.menu.add_separator()
ModeBMenu.menu.add_command(label="-Term-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="Open", variable=AWGBTerm, value=0)
ModeBMenu.menu.add_radiobutton(label="To GND", variable=AWGBTerm, value=1)
ModeBMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGBTerm, value=2)
ModeBMenu.pack(side=LEFT, anchor=W)
ShapeBMenu = Menubutton(awg2eb, text="Shape", relief=RAISED)
ShapeBMenu.menu = Menu(ShapeBMenu, tearoff = 0 )
ShapeBMenu["menu"] = ShapeBMenu.menu
ShapeBMenu.menu.add_radiobutton(label="DC", variable=AWGBShape, value=0)
ShapeBMenu.menu.add_radiobutton(label="Sine", variable=AWGBShape, value=1)
ShapeBMenu.menu.add_radiobutton(label="Triangle", variable=AWGBShape, value=2)
ShapeBMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGBShape, value=3)
ShapeBMenu.menu.add_radiobutton(label="Square", variable=AWGBShape, value=4)
ShapeBMenu.menu.add_radiobutton(label="StairStep", variable=AWGBShape, value=5)
ShapeBMenu.menu.add_radiobutton(label="Custom", variable=AWGBShape, value=6)
ShapeBMenu.pack(side=LEFT, anchor=W)
#
awg2ampl = Frame( frame3 )
awg2ampl.pack(side=TOP)
AWGBAmplEntry = Entry(awg2ampl, width=5)
AWGBAmplEntry.bind("<Return>", BAWGBAmpl)
AWGBAmplEntry.pack(side=LEFT, anchor=W)
AWGBAmplEntry.delete(0,"end")
AWGBAmplEntry.insert(0,0.0)
amp2lab = Label(awg2ampl, text="Min")
amp2lab.pack(side=LEFT, anchor=W)
#
awg2off = Frame( frame3 )
awg2off.pack(side=TOP)
AWGBOffsetEntry = Entry(awg2off, width=5)
AWGBOffsetEntry.bind("<Return>", BAWGBOffset)
AWGBOffsetEntry.pack(side=LEFT, anchor=W)
AWGBOffsetEntry.delete(0,"end")
AWGBOffsetEntry.insert(0,0.0)
off2lab = Label(awg2off, text="Max")
off2lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg2freq = Frame( frame3 )
awg2freq.pack(side=TOP)
AWGBFreqEntry = Entry(awg2freq, width=7)
AWGBFreqEntry.bind("<Return>", BAWGBFreq)
AWGBFreqEntry.pack(side=LEFT, anchor=W)
AWGBFreqEntry.delete(0,"end")
AWGBFreqEntry.insert(0,0.0)
freq2lab = Label(awg2freq, text="Freq")
freq2lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg2phase = Frame( frame3 )
awg2phase.pack(side=TOP)
AWGBPhaseEntry = Entry(awg2phase, width=5)
AWGBPhaseEntry.bind("<Return>", BAWGBPhase)
AWGBPhaseEntry.pack(side=LEFT, anchor=W)
AWGBPhaseEntry.delete(0,"end")
AWGBPhaseEntry.insert(0,0)
phase2lab = Label(awg2phase, text="Phase")
phase2lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg2dc = Frame( frame3 )
awg2dc.pack(side=TOP)
AWGBDutyCycleEntry = Entry(awg2dc, width=5)
AWGBDutyCycleEntry.bind("<Return>", BAWGBDutyCycle)
AWGBDutyCycleEntry.pack(side=LEFT, anchor=W)
AWGBDutyCycleEntry.delete(0,"end")
AWGBDutyCycleEntry.insert(0,50)
duty2lab = Label(awg2dc, text="%")
duty2lab.pack(side=LEFT, anchor=W)
# ================ Make spectrum sub window ==========================
freqwindow = Toplevel()
freqwindow.title("Spectrum Analyzer")
frame2fr = Frame(freqwindow, borderwidth=5, relief=RIDGE)
frame2fr.pack(side=RIGHT, expand=1, fill=Y)

frame2f = Frame(freqwindow, background="black", borderwidth=5, relief=RIDGE)
frame2f.pack(side=TOP, expand=YES, fill=BOTH)

Freqca = Canvas(frame2f, width=CANVASwidthF, height=CANVASheightF, background=COLORcanvas)
Freqca.pack(side=TOP)

# right side drop down menu buttons
dropmenu = Frame( frame2fr )
dropmenu.pack(side=TOP)
# File menu
Filemenu = Menubutton(dropmenu, text="File", relief=RAISED)
Filemenu.menu = Menu(Filemenu, tearoff = 0 )
Filemenu["menu"] = Filemenu.menu
Filemenu.menu.add_command(label="Save Config", command=BSaveConfig)
Filemenu.menu.add_command(label="Load Config", command=BLoadConfig)
Filemenu.menu.add_command(label="Save Screen", command=BSaveScreenSA)
Filemenu.menu.add_command(label="Save Data", command=BCSVfile)
Filemenu.pack(side=LEFT, anchor=W)
#
SmoothCurves = IntVar(0)
CutDC = IntVar(0)
Optionmenu = Menubutton(dropmenu, text="Options", relief=RAISED)
Optionmenu.menu = Menu(Optionmenu, tearoff = 0 )
Optionmenu["menu"]  = Optionmenu.menu
Optionmenu.menu.add_checkbutton(label='Smooth', variable=SmoothCurves)
Optionmenu.menu.add_checkbutton(label='Cut-DC', variable=CutDC)
Optionmenu.menu.add_command(label="Store trace", command=BSTOREtraceSA)
Optionmenu.menu.add_command(label="Setup", command=BSetup)
Optionmenu.menu.add_command(label="Screen setup", command=BScreensetup)
Optionmenu.pack(side=LEFT, anchor=W)
#
RUNframe = Frame( frame2fr )
RUNframe.pack(side=TOP)
sb = Button(RUNframe, text="Stop", width=4, bg="red", command=BStopSA)
sb.pack(side=LEFT)
rb = Button(RUNframe, text="Run", width=3, bg="green", command=BStartSA)
rb.pack(side=LEFT)
#
SingleShot = IntVar(0) # variable for Single Shot sweeps
Modeframe = Frame( frame2fr )
Modeframe.pack(side=TOP)
Modemenu = Menubutton(Modeframe, text="Mode", relief=RAISED)
Modemenu.menu = Menu(Modemenu, tearoff = 0 )
Modemenu["menu"]  = Modemenu.menu
Modemenu.menu.add_command(label="Normal mode", command=BNormalmode)
Modemenu.menu.add_command(label="Peak hold", command=BPeakholdmode)
Modemenu.menu.add_command(label="Average", command=BAveragemode)
Modemenu.menu.add_checkbutton(label='SingleShot', variable=SingleShot)
Modemenu.pack(side=LEFT)
#
FFTwindow = IntVar(0)   # FFT window function variable
FFTwindmenu = Menubutton(Modeframe, text="FFTwindow", relief=RAISED)
FFTwindmenu.menu = Menu(FFTwindmenu, tearoff = 0 )
FFTwindmenu["menu"]  = FFTwindmenu.menu
FFTwindmenu.menu.add_radiobutton(label='Rectangular window (B=1)', variable=FFTwindow, value=0)
FFTwindmenu.menu.add_radiobutton(label='Cosine window (B=1.24)', variable=FFTwindow, value=1)
FFTwindmenu.menu.add_radiobutton(label='Triangular window (B=1.33)', variable=FFTwindow, value=2)
FFTwindmenu.menu.add_radiobutton(label='Hann window (B=1.5)', variable=FFTwindow, value=3)
FFTwindmenu.menu.add_radiobutton(label='Blackman window (B=1.73)', variable=FFTwindow, value=4)
FFTwindmenu.menu.add_radiobutton(label='Nuttall window (B=2.02)', variable=FFTwindow, value=5)
FFTwindmenu.menu.add_radiobutton(label='Flat top window (B=3.77)', variable=FFTwindow, value=6)
FFTwindmenu.pack(side=LEFT)
#
SamplesMenu = Frame( frame2fr )
SamplesMenu.pack(side=TOP)
b = Button(SamplesMenu, text="-Samples", width=Buttonwidth2, command=Bsamples1)
b.pack(side=LEFT)
b = Button(SamplesMenu, text="+Samples", width=Buttonwidth2, command=Bsamples2)
b.pack(side=LEFT)
#
# Show channels menu
ShowC1_VdB = IntVar(0)   # curves to display variables
ShowC1_P = IntVar(0)
ShowC2_VdB = IntVar(0)
ShowC2_P = IntVar(0)
ShowMarker = IntVar(0)
ShowRA_VdB = IntVar(0)
ShowRA_P = IntVar(0)
ShowRB_VdB = IntVar(0)
ShowRB_P = IntVar(0)
ShowMathSA = IntVar(0)
ShowRMath = IntVar(0)
#
Showmenu = Menubutton(frame2fr, text="Curves", relief=RAISED)
Showmenu.menu = Menu(Showmenu, tearoff = 0 )
Showmenu["menu"] = Showmenu.menu
Showmenu.menu.add_command(label="-Show-", command=donothing)
Showmenu.menu.add_command(label="All", command=BShowCurvesAllSA)
Showmenu.menu.add_command(label="None", command=BShowCurvesNoneSA)
Showmenu.menu.add_checkbutton(label='CA-dBV', variable=ShowC1_VdB)
Showmenu.menu.add_checkbutton(label='CB-dBV', variable=ShowC2_VdB)
Showmenu.menu.add_checkbutton(label='Phase A-B', variable=ShowC1_P)
Showmenu.menu.add_checkbutton(label='Phase B-A', variable=ShowC2_P)
Showmenu.menu.add_checkbutton(label='Marker', variable=ShowMarker)
Showmenu.menu.add_separator()
Showmenu.menu.add_radiobutton(label='None', variable=ShowMathSA, value=0)
Showmenu.menu.add_radiobutton(label='CA-dB - CB-dB', variable=ShowMathSA, value=1)
Showmenu.menu.add_radiobutton(label='CB-dB - CA-dB', variable=ShowMathSA, value=2)
Showmenu.menu.add_separator()
Showmenu.menu.add_checkbutton(label='RA-dBV', variable=ShowRA_VdB)
Showmenu.menu.add_checkbutton(label='RB-dBV', variable=ShowRB_VdB)
Showmenu.menu.add_checkbutton(label='RPhase A-B', variable=ShowRA_P)
Showmenu.menu.add_checkbutton(label='RPhase B-A', variable=ShowRB_P)
Showmenu.menu.add_checkbutton(label='Math', variable=ShowRMath)
Showmenu.pack(side=TOP)
#
Frange = Frame( frame2fr )
Frange.pack(side=TOP)
b1 = Button(Frange, text="Startfreq", width=Buttonwidth2, command=BStartfrequency)
b1.pack(side=LEFT)
b2 = Button(Frange, text="Stopfreq", width=Buttonwidth2, command=BStopfrequency)
b2.pack(side=LEFT)

DBrange = Frame( frame2fr )
DBrange.pack(side=TOP)
b3 = Button(DBrange, text="+dB/div", width=Buttonwidth2, command=BDBdiv2)
b3.pack(side=LEFT)
b4 = Button(DBrange, text="-dB/div", width=Buttonwidth2, command=BDBdiv1)
b4.pack(side=LEFT)

LVBrange = Frame( frame2fr )
LVBrange.pack(side=TOP)
b5 = Button(LVBrange, text="LVL+10", width=Buttonwidth2, command=Blevel4)
b5.pack(side=LEFT)
b6 = Button(LVBrange, text="LVL-10", width=Buttonwidth2, command=Blevel3)
b6.pack(side=LEFT)

LVSrange = Frame( frame2fr )
LVSrange.pack(side=TOP)
b7 = Button(LVSrange, text="LVL+1", width=Buttonwidth2, command=Blevel2)
b7.pack(side=LEFT)
b8 = Button(LVSrange, text="LVL-1", width=Buttonwidth2, command=Blevel1)
b8.pack(side=LEFT)
# sweep generator mode menu buttons
FSweepMode = IntVar(0)
FSweepCont = IntVar(0)
FSweepmenu = Menubutton(frame2fr, text="Sweep Gen", relief=RAISED)
FSweepmenu.menu = Menu(FSweepmenu, tearoff = 0 )
FSweepmenu["menu"]  = FSweepmenu.menu
FSweepmenu.menu.add_radiobutton(label='None', variable=FSweepMode, value=0)
FSweepmenu.menu.add_radiobutton(label='CHA', variable=FSweepMode, value=1)
FSweepmenu.menu.add_radiobutton(label='CHB', variable=FSweepMode, value=2)
FSweepmenu.menu.add_separator()
FSweepmenu.menu.add_command(label='Sweep Steps', command=BSweepsteps)
FSweepmenu.menu.add_separator()
FSweepmenu.menu.add_radiobutton(label='Single', variable=FSweepCont, value=0)
FSweepmenu.menu.add_radiobutton(label='Continuous', variable=FSweepCont, value=1)
FSweepmenu.pack(side=TOP)

ADI2 = Label(frame2fr, image=logo, anchor= "sw", height=49, width=116, compound="top")
ADI2.pack(side=TOP)
#===== Initalize device ======
#
AWGSync.set(1)
AWGAMode.set(2)
AWGBMode.set(2)
ConnectDevice()
# ================ Call main routine ===============================
root.update()               # Activate updated screens
#
# Start sampling
Analog_In()


